const mdpComparimagerElementor = {
    getPercentage: function ( partialValue, totalValue ) {
        return ( 100 * partialValue ) / totalValue;
    },

    comparimagerElementor: function ( wrapperName ) {
        this.imageComparison( wrapperName );

        window.addEventListener( 'resize', () => {
            this.imageComparison( wrapperName );
        } );
    },

    imageComparison: function ( wrapperName ) {
        const $box = document.querySelector( `.${wrapperName} .mdp-comparimager-elementor-box` );
        const $comparisonSlider = document.querySelector( `.${wrapperName} .mdp-comparimager-elementor-comparison-slider` );
        const $imageAfter = document.querySelector( `.${wrapperName} .mdp-comparimager-elementor-img-after` );
        const $imageBefore = document.querySelector( `.${wrapperName} .mdp-comparimager-elementor-img-before` );
        const $sliderHandle = document.querySelector( `.${wrapperName} .mdp-comparimager-elementor-compare-handle` );
        const $sliderImages = $comparisonSlider.querySelectorAll( 'img' );
        const comparisonSliderWidth = $comparisonSlider.clientWidth;
        const comparisonSliderHeight = $comparisonSlider.clientHeight;
        const moveHandle = $box.dataset.moveHandle;
        const linePosition = $box.dataset.linePosition;
        let handleLeftOffset = $sliderHandle.offsetLeft;
        let handleTopOffset = $sliderHandle.offsetTop;

        // set slider images width or height
        $sliderImages.forEach( sliderImage => {
            if ( linePosition === 'vertical' ) {
                sliderImage.style.width = `min(100vw, ${comparisonSliderWidth}px)`;
            } else {
                sliderImage.style.height = `min(100vh, ${comparisonSliderHeight}px)`;
                sliderImage.style.width = '100%';
            }
        } );

        // initial slider values
        if ( linePosition === 'horizontal' ) {
            handleTopOffset = handleTopOffset > comparisonSliderHeight ? comparisonSliderHeight : handleTopOffset;
            const handleTopPercentage = this.getPercentage( handleTopOffset, comparisonSliderHeight );
            $imageAfter.style.height = 100 - handleTopPercentage + '%';
            $imageAfter.style.width = '100%';
        } else {
            handleLeftOffset = handleLeftOffset > comparisonSliderWidth ? comparisonSliderWidth : handleLeftOffset;
            const handleLeftPercentage = this.getPercentage( handleLeftOffset, comparisonSliderWidth );
            $imageAfter.style.width =  100 - handleLeftPercentage + '%';
            $imageAfter.style.height = '100%';
        }

        if ( moveHandle === 'click' ) {
            $comparisonSlider.addEventListener( 'click', moveOnClick );
        } else if ( moveHandle === 'hover' ) {
            $comparisonSlider.addEventListener( 'mousemove', moveOnHover );
        } else if ( moveHandle === 'swipe' ) {
            $sliderHandle.addEventListener( 'mousedown', startDragging, false );
            window.addEventListener( 'mouseup', stopDragging, false );

            $sliderHandle.addEventListener( 'touchstart', startDragging, false );
            $comparisonSlider.addEventListener( 'touchend', stopDragging, false );
            $comparisonSlider.addEventListener( 'touchcancel', stopDragging, false );

            function startDragging( event ) {

                if ( event.type === 'touchstart' ) {
                    $comparisonSlider.addEventListener( 'touchmove', moveOnHover );
                } else {
                    $comparisonSlider.addEventListener( 'mousemove', moveOnHover );
                }
            }


            function stopDragging() {
                $comparisonSlider.removeEventListener( 'touchmove', moveOnHover );
                $comparisonSlider.removeEventListener( 'mousemove', moveOnHover );
            }
        }


        function moveOnClick ( event ) {
            moveOnHover( event );
        }

        function moveOnHover( event ) {
            const leftOffset = $comparisonSlider.getBoundingClientRect().left;
            const topOffset = $comparisonSlider.getBoundingClientRect().top;
            let mouseX = event.clientX - leftOffset;
            let mouseY = event.clientY - topOffset;

            if ( event.type === 'touchmove' && linePosition === 'vertical' ) {
               mouseX = event.touches[0].clientX - leftOffset;
            } else if ( event.type === 'touchmove' && linePosition === 'horizontal' ) {
                mouseY = event.touches[0].clientY - topOffset;
            }

            if ( linePosition === 'vertical' ) {
                if ( mouseX < 0 ) {
                    mouseX = 0;
                } else if ( mouseX > comparisonSliderWidth ) {
                    mouseX = comparisonSliderWidth;
                }
            } else {
                if ( mouseY < 0 ) {
                    mouseY = 0;
                } else if ( mouseY > comparisonSliderHeight ) {
                    mouseY = comparisonSliderHeight;
                }
            }

            if ( linePosition === 'vertical' ) {
                $imageAfter.style.width = `${((1 - mouseX / comparisonSliderWidth) * 100).toFixed( 4 )}%`;
                $sliderHandle.style.left = `${( ( mouseX / comparisonSliderWidth ) * 100 ).toFixed( 4 )}%`;
            } else {
                $imageAfter.style.height = `${100 - (( ( mouseY / comparisonSliderHeight ) * 100 ).toFixed( 4 ))}%`;
                $sliderHandle.style.top = `${( ( mouseY / comparisonSliderHeight ) * 100 ).toFixed( 4 )}%`;
            }

        }

    },
};

jQuery( window ).on( 'elementor/frontend/init', function () {

    elementorFrontend.hooks.addAction( 'frontend/element_ready/global', function( $scope ) {

        if ( $scope[0].classList.contains( 'elementor-widget-mdp-comparimager-elementor' ) ) {
            mdpComparimagerElementor.comparimagerElementor.call( mdpComparimagerElementor, `elementor-element-${$scope[0].dataset.id}` );
        }
    } );

} );