/**
 * Modalier for Elementor
 * Simple modal popup window for Elementor
 * Exclusively on https://1.envato.market/modalier-elementor
 *
 * @encoding        UTF-8
 * @version         1.12.1
 * @copyright       (C) 2018 - 2023 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com)
 * @support         help@merkulov.design
 **/

(function($) {

    window.Modalier = function ( uniqueId, widthDevice, heightDevice ) {

        /**
         * Initialize variables
         */
        function initializeVars() {

            if ( typeof window.mdpModalierInit === 'undefined' ) {

                window.mdpModalierInit = [];

            }

        }

        function strToBool(s) {
            regex=/^\s*(true|1|on)\s*$/i

            return regex.test(s);
        }

        /** Convert to boolean. */
        String.prototype.bool = function() {
            return strToBool(this);
        };

        const $box = document.getElementById( `mdp-modalier-${ uniqueId }` );

        const postID = $box.getAttribute( 'data-post-id' );
        const widgetID = $box.getAttribute( 'data-widget-id' );
        const dataType = $box.getAttribute( 'data-type' );
        const fullscreen = $box.getAttribute( 'data-fullscreen' );
        const typeEvents = $box.getAttribute( 'data-typeEvents' );
        const dataScroll = $box.getAttribute( 'data-modalierScroll' );
        const dataTimeout = $box.getAttribute( 'data-modalierTimeout' );
        const closeViaKeypress = $box.getAttribute( 'data-closeViaKeypress' );
        const displayCloseButton = $box.getAttribute( 'data-displayCloseButton' );
        const closeViaOverlay = $box.getAttribute( 'data-closeViaOverlay' );
        const animation = $box.getAttribute( 'data-animation' );
        const animationSpeed = $box.getAttribute( 'data-animationSpeed' );
        const popupTitle = $box.getAttribute( 'data-popupTitle' );
        const closeText = $box.getAttribute( 'data-closeText' );
        const closeAriaLabel = $box.getAttribute( 'data-closeAriaLabel' );
        const galleryActiveClass = $box.getAttribute( 'data-galleryActiveClass' );
        const outerControls = $box.getAttribute( 'data-outerControls' );
        const background = $box.getAttribute( `data-background-${ uniqueId }` );
        const opacity = $box.getAttribute( 'data-opacity' );
        const openLimit = $box.getAttribute( 'data-openLimit' );
        const contentSource = $box.getAttribute( 'data-contentSource' );

        let popUpClass = '';

        /**
         * Blue and un-blur root elements of the page
         * @param modaalWrapper
         * @param blur
         */
        function blurPage( modaalWrapper, blur ) {

            const rootTag = document.querySelectorAll( `body > *:not(.modaal-wrapper)` );

            // Root elements looping
            for ( let element of rootTag ) {

                if ( element.querySelector( '.modaal-wrapper' ) === null ) {

                    // Don't blur WordPress admin bar
                    if ( element.id === 'wpadminbar' ) {
                        continue
                    }

                    // Don't blur script tags
                    if ( element.tagName === 'SCRIPT' ) {
                        continue
                    }

                    element.style.filter = blur ?
                        `blur(${$box.getAttribute( 'data-blur' )}px)` :
                        'unset';

                }

            }

        }

        /**
         * Init popup but not open it
         * @param popUpClass
         * @param dataType
         */
        function initPopUp( popUpClass, dataType ) {

            // Init popup once
            if ( window.mdpModalierInit.includes( popUpClass ) ) { return; }

            // Prepare modaal type for options
            const modaalType = ( [ 'posts', 'custom-posts', 'woocommerce', 'custom', 'section' ].includes( dataType ) ) ? 'inline' : dataType;

            // Prepare options
            let options = {
                custom_class: `${uniqueId}`,
                type: modaalType,
                mdptype: dataType,
                postid: postID,
                widgetid: widgetID,
                width: widthDevice,
                height: heightDevice,
                background: background,
                overlay_opacity: parseFloat(opacity),
                fullscreen: fullscreen.bool(),
                is_locked: closeViaKeypress.bool(),
                hide_close: displayCloseButton.bool(),
                overlay_close: closeViaOverlay.bool(),
                animation: animation,
                animation_speed: parseInt( animationSpeed ),
                after_callback_delay: 1,
                accessible_title: popupTitle,
                close_text: closeText,
                close_aria_label: closeAriaLabel,
                gallery_active_class: galleryActiveClass,
                outer_controls: outerControls.bool(),

            }

            // Blur options
            if ( null !== $box.getAttribute( 'data-blur' ) ) {

                // Before open callback
                options[ 'after_open' ] = ( modaalWrapper ) => {
                    blurPage( modaalWrapper, true );
                };

                // After close callback
                options[ 'before_close' ] = ( modaalWrapper ) => {
                    blurPage( modaalWrapper, false );
                };

            }

            // Init popup
            $( popUpClass ).modaal( options );

            // Store trigger selector
            window.mdpModalierInit.push( popUpClass );

        }

        /**
         * Open popup
         */
        function openPopup(){
            jQuery( popUpClass ).click();
            sessionStorage.setItem( `mdp-launches-${ uniqueId }`, '1' );
        }

        /** Determines whether to display pop-up windows when scrolling the page vertically. */
        let numberLaunches = sessionStorage.getItem( `mdp-launches-scroll-${ uniqueId }` );
        let scrollBack = 0;
        function scrollEvents( scrollY, openLimit) {

            window.addEventListener("scroll", (event) => {

                let scroll = this.scrollY;

                /** Show a pop-up window if the limit is not exceeded. */
                if ( scrollY <= scroll && parseInt( numberLaunches ) <= openLimit && scrollBack === 1 ){
                    jQuery( popUpClass ).click();
                    numberLaunches = parseInt( numberLaunches ) + 1;
                    sessionStorage.setItem( `mdp-launches-scroll-${ uniqueId }`, numberLaunches  );
                }

                /** Check if we are back in the range where we don't need to display the popup. */
                if( scrollY >= scroll && parseInt( numberLaunches ) <= openLimit ){
                    scrollBack = 1;
                }else{
                    scrollBack = 0;
                }

            });

        }

        /** We launch a pop-up window when scrolling the page. */
        if( 'scroll' === typeEvents ) {
            scrollEvents( parseInt( dataScroll ), openLimit );
        }

        /** Open by timeout */
        if( 'timeout' === typeEvents ){

            /** We launch the pop-up window after a certain time in milliseconds. */
            let dataLaunches = sessionStorage.getItem( `mdp-launches-${ uniqueId }` );

            if ( parseInt( dataLaunches ) !== 1 ){
                setTimeout( openPopup, parseInt(dataTimeout) );
            }

        }

        /** Init Modalier vars */
        initializeVars();

        if ( [ 'ajax', 'image', 'video', 'iframe' ].includes( dataType ) ) {

            popUpClass = `.mdp-modalier-${ uniqueId }`;

        } else if ( [ 'posts', 'custom-posts', 'woocommerce', 'custom', 'section' ].includes( dataType ) ) {

            popUpClass = dataType === 'section' ? `.${ uniqueId }` : `.inline-${ uniqueId }`;

            /** Hide the section by ID. */
            if ( dataType === 'section' ) {
                let classSection = document.querySelector(`.${uniqueId}` );
                classSection.addEventListener('click', function(e) {
                    e.stopPropagation();
                });

                if ( null !== document.getElementById( contentSource ) ) {
                    const $sectionByID = document.getElementById( contentSource );
                    $sectionByID.style.display="none";
                }
            }

        } else {
            return;
        }

        /** Init popup */
        initPopUp( popUpClass, dataType );

    }

})(jQuery);