window.addEventListener( 'elementor/frontend/init', function () {

    window.Sharinger = function () {

        let content = {
            title: document.title,
            text: '',
            image: elementorFrontend.config.post.featuredImage,
            url: location.href,
            width: 640,
            height: 480,
        };

        /**
         * Get social name by class name of the clicked element
         * @param $element
         * @returns {boolean}
         */
        const getNetworkNameFromClass = function( $element ) {

            let social = false;

            $element.classList.forEach( className => {

                if ( className.includes( 'elementor-share-btn_' ) ) {

                    social = className.replace( 'elementor-share-btn_', '' );

                }

            } );

            return social;

        };

        /**
         * Get single network link
         */
        const getNetworkLink = function ( networkName ) {

            const linkTemplates = {
                twitter: `https://twitter.com/intent/tweet?text=${content.title}\x20${content.url}`,
                facebook: `https://www.facebook.com/sharer.php?u=${content.url}`,
                pinterest: `https://www.pinterest.com/pin/create/button/?url=${content.url}&media=${content.image}`,
                linkedin: `https://www.linkedin.com/shareArticle?mini=true&url=${content.url}&title=${content.title}&summary=${content.text}&source=${content.url}`,
                tumblr: `https://tumblr.com/share/link?url=${content.url}`,
                google: `https://plus.google.com/share?url=${content.url}`,
                digg: `https://digg.com/submit?url=${content.url}`,
                reddit: `https://reddit.com/submit?url=${content.url}&title=${content.title}`,
                stumbleupon: `https://www.stumbleupon.com/submit?url=${content.url}`,
                pocket: `https://getpocket.com/edit?url=${content.url}`,
                whatsapp: `https://api.whatsapp.com/send?text=*${content.title}*%0A${content.text}%0A${content.url}`,
                xing: `https://www.xing.com/app/user?op=share&url=${content.url}`,
                print: `javascript:print()`,
                email: `mailto:?subject=${content.title}&body=${content.text}\n${content.url}`,
                telegram: `https://telegram.me/share/url?url=${content.url}&text=${content.text}`,
                skype: `https://web.skype.com/share?url=${content.url}`,
            }

            return linkTemplates[ networkName ];

        };

        const openLink = function( networkName ) {
            let shareWindowParams = '';

            const   shareWindowLeft = ( screen.width / 2 ) - ( content.width / 2 ),
                    shareWindowTop = ( screen.height / 2 ) - ( content.height / 2 );

            shareWindowParams = 'toolbar=0,status=0,width=' + content.width + ',height=' + content.height + ',top=' + shareWindowTop + ',left=' + shareWindowLeft;

            const link = getNetworkLink( networkName ),
                isPlainLink = /^https?:\/\//.test( link ),
                windowName = isPlainLink ? '' : '_self';

            open( link, windowName, shareWindowParams );
        };

        document.querySelectorAll( '.mdp-sharinger' ).forEach( $sharingerContainer => {

            $sharingerContainer.addEventListener( 'click', function ( e ) {

                let networkName = getNetworkNameFromClass( e.target.closest( '.elementor-share-btn' ) );

                openLink( networkName );

            } );

        } );

    };

    window.Sharinger();

} );
