/**
 * Sliper
 * Powerful responsive slider plugin for Elementor
 * Exclusively on https://1.envato.market/sliper-elementor
 *
 * @encoding        UTF-8
 * @version         1.12.1
 * @copyright       (C) 2018 - 2023 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com)
 * @support         help@merkulov.design
 * @license         Envato License https://1.envato.market/KYbje
 **/

/**
 * Sliper Initialization Function
 *
 * @param {string} uniqueId - Unique slipper identifier
 * @constructor
 */
window.Sliper = function () {

    function strToBool(s) {
        let regex = /^\s*(true|1|on)\s*$/i;
        return regex.test(s);
    }

    String.prototype.bool = function() {
        return strToBool(this);
    };

    const sliperItems = document.querySelectorAll( ".mdp-sliper-container" );

    for ( const sliper of sliperItems ) {

        const itemSlider = sliper.querySelector( ".mdp-sliper" );

        const autoplay          = itemSlider.getAttribute( 'data-autoplay' );
        const keyboardN         = itemSlider.getAttribute( 'data-keyboard' );
        const dots              = itemSlider.getAttribute( 'data-dots' );
        const prevNext          = itemSlider.getAttribute( 'data-prevNext' );
        const infiniteScrolling = itemSlider.getAttribute( 'data-infiniteScrolling' );
        const pause             = itemSlider.getAttribute( 'data-pause' );
        const scroll            = itemSlider.getAttribute( 'data-scroll' );
        const friction          = itemSlider.getAttribute( 'data-friction' );
        const attraction        = itemSlider.getAttribute( 'data-attraction' );
        const draggable         = itemSlider.getAttribute( 'data-draggable' );
        const dragThreshold     = itemSlider.getAttribute( 'data-dragThreshold' );
        const rightToLeft       = itemSlider.getAttribute( 'data-rightToLeft' );
        const cellAlign         = itemSlider.getAttribute( 'data-cellAlign' );
        const groupCells        = itemSlider.getAttribute( 'data-groupCells' );
        const initialIndex      = itemSlider.getAttribute( 'data-initialIndex' );
        const watchCss          = itemSlider.getAttribute( 'data-watchCss' );
        const transition        = itemSlider.getAttribute( 'data-transition' );
        const hash              = itemSlider.getAttribute( 'data-hash' );

        // Init Sliper
        let newSliper = new Flickity( itemSlider, {
            accessibility: keyboardN.bool(),
            adaptiveHeight: false,
            autoPlay: autoplay === 'false' ? false : parseInt(autoplay),
            pauseAutoPlayOnHover: pause.bool(),
            cellAlign: cellAlign,
            cellSelector: undefined,
            contain: false,
            draggable: draggable.bool(),
            dragThreshold: dragThreshold === 'false' ? false : parseInt(dragThreshold),
            freeScroll: scroll.bool(),
            selectedAttraction: parseFloat(attraction),
            friction: parseFloat(friction),
            groupCells: parseInt(groupCells),
            initialIndex: parseInt(initialIndex) - 1,
            lazyLoad: false,
            percentPosition: true,
            prevNextButtons: prevNext.bool(),
            pageDots: dots.bool(),
            resize: true,
            rightToLeft: rightToLeft.bool(),
            setGallerySize: true,
            watchCSS: watchCss.bool(),
            wrapAround: infiniteScrolling.bool(),
            hash: hash.bool(),
        } );

        //Hide un-styled content blinking
        itemSlider.removeAttribute( 'style' );

        let $carousel = jQuery( itemSlider );
        let items = itemSlider.querySelectorAll( `.carousel-cell ` );

        let slidePage;

        $carousel.on( 'change.flickity', function( event, index ) {

            for ( let i = 0; i <= parseInt(groupCells) - 1; i++ ) {

                let indexCount;

                if( index === 0 ){
                    indexCount = ( slidePage * parseInt(groupCells) ) + i;
                }else{
                    indexCount = ( ( index - 1) * parseInt(groupCells) ) + i;
                    slidePage = index;
                }

                if( typeof items[indexCount] !== "undefined" ) {
                    items[indexCount].classList.remove( `mdp-${transition}-active` );
                    items[indexCount].classList.add( `mdp-${transition}-inactive` );
                }

            }

            for ( let i = 0; i <= parseInt(groupCells) - 1; i++ ) {

                let indexCount = ( index  * parseInt(groupCells) ) + i;

                if( typeof items[indexCount] !== "undefined" ){
                    items[indexCount].classList.remove( `mdp-${transition}-inactive` );
                    items[indexCount].classList.add( `mdp-${transition}-active` );
                }

            }

        });

    }
}
