/**
 * {{plugin_description}}
 * Exclusively on Envato Market: {{plugin_uri}}
 *
 * @encoding        UTF-8
 * @version         1.12.1
 * @copyright       {{plugin_copyright}}
 * @license         {{license}}
 * @contributors    {{contributors}}
 * @support         {{plugin_support}}
 **/

( function ( $ ) {

    "use strict";

    window.mdpwalkerElementor = function( walker ) {

        /* Render all Walkers or re-render single */
        let wrapper = $( this );
        if ( typeof walker === 'string' ) {
            wrapper = $( '#' + walker );
        }

        /* Get options and panorama */
        let elementPanorama = wrapper.find( '.mdp-panorama' )[ 0 ];
        let elementMap = wrapper.find( '.mdp-map' )[ 0 ];
        let options = wrapper.data();
        let rotationInterval;

        /* Custom Panorama Options */
        const mdpCustomPanorama = function( customPanorama ) {

            let customOptions = {
                location: {
                    pano: options.pano,
                    latLng: new google.maps.LatLng,
                    description: options.description,
                },
                links: [],
                copyright: options.copyrights,
                tiles: {
                    tileSize: new google.maps.Size( 1024, 512 ),
                    worldSize: new google.maps.Size( 2048, 1024 ),
                    heading: options.heading,
                }
            };

            if ( customPanorama === options.pano ) {
                return customOptions;
            }

        }

        /* Rotation tick functions */
        const mdpWalkerRotation = function() {

            /* Get current view angle */
            let oldPov = panorama.getPov();
            let newPov = oldPov;

            /* Set new heading by rotation direction */
            newPov.heading += options.direction * .25;

            /* Rotation */
            if ( options.loop ) {

                /* Infinite rotation */
                panorama.setPov( newPov );

            } else {

                /* Rotation stop position */
                let rotateTo = options.heading + options.direction * options.count * 360;

                if ( oldPov.heading !== rotateTo ) {

                    /* Continue rotating */
                    panorama.setPov( newPov );

                } else {

                    /* Stop rotating */
                    clearInterval( rotationInterval );

                }

            }

        };

        /* Rotation stop function */
        const mdpWalkerRotationStop = function() {

            clearInterval( rotationInterval );
            wrapper.off( 'click', mdpWalkerRotationStop );

        };

        /* Custom Panorama ID options */
        if ( typeof wrapper.data( 'pano' ) === 'string' ) {

            options.pano = wrapper.data( 'pano' );

        /* Google Street View by Latitude and Longitude options */
        } else {

            options.position = {
                lat: wrapper.data( 'lat' ),
                lng: wrapper.data( 'lng' )
            };
            delete options.lat;
            delete options.lng;

        }

        /* View angle options */
        options.pov = {
            heading: wrapper.data( 'heading' ),
            pitch: wrapper.data( 'pitch' ),
            zoom: wrapper.data( 'zoom' )
        };
        delete options.heading;
        delete options.pitch;
        delete options.zoom;

        /* Init Street View Panorama */
        let panorama = new google.maps.StreetViewPanorama( elementPanorama, options );

        /* Init Custom Panorama */
        if ( typeof wrapper.data( 'pano' ) === 'string' ) {
            panorama.registerPanoProvider( mdpCustomPanorama );
        }

        /* Init Map in the Side-by-Side Layout */
        if ( typeof wrapper.data( 'map' ) === 'number' ) {
            let map = new google.maps.Map ( elementMap, { center: options.position, zoom: options.map } );
            map.setStreetView( panorama );
        }

        /* Init Street View Rotation */
        if ( options.rotation ) {

            /* Start rotation */
            let maxSpeed = 50;
            rotationInterval = setInterval( mdpWalkerRotation, maxSpeed - wrapper.data( 'speed' ) );

            /* Stop rotation on click */
            if ( options.click ) {
                wrapper.on( 'click', mdpWalkerRotationStop );
            }

        }

    };

    /** Display Walker on page loaded */
    $( document ).ready( function () {

        $( '.mdp-walker' ).each( mdpwalkerElementor );

    } );

    /** Display Walker in Elementor Pro popup */
    jQuery( document ).on( 'elementor/popup/show', () => {

        $( '.dialog-widget-content .mdp-walker' ).each( mdpwalkerElementor );

    } );

} ( jQuery ) );