const mdpWatcherElementor = {
    watcherElementor: function ( widgetId ) {
        const $box = document.querySelector( `.elementor-element-${widgetId} .mdp-watcher-elementor-box` );
        const features = JSON.parse( $box.dataset.controls );
        const playerBoolSettings = JSON.parse( $box.dataset.playerBoolSettings );
        const isPlaylist = $box.dataset.source === 'playlist';
        const playerControlsSettings = JSON.parse( $box.dataset.playerSettings );
        const showPlaylistByDefault = $box.dataset.showPlaylistByDefault;
        const $video = $box.querySelector( 'video' );

        // Compatibility with twenty twenty theme
        $video.style.height = null;

        function chaptersSwitching( chapterItems, mejs ) {

            chapterItems.forEach( chapterItem => {
                chapterItem.addEventListener( 'click', () => {
                    mejs.setCurrentTime( parseFloat( chapterItem.dataset.chapterTime ) );
                    mejs.play();
                } );
            } );
        }

        function getIndexOfActiveTrack( playlistItems ) {
            let resultIndex = null;

            playlistItems.forEach( ( playlistItem, index ) => {
                if ( playlistItem.classList.contains( 'mejs-playlist-selected' ) ) {
                    resultIndex = index;
                }
            } );

            return resultIndex;
        }

        function playerChapters( mejs ) {
            const chapterItems = $box.querySelectorAll( '.mdp-watcher-elementor-chapter-block' );
            chaptersSwitching( chapterItems, mejs );
        }

        function getActiveChapterWrapper( chaptersWrappers, activeIndex ) {
            let activeChaptersWrapper = null;

            chaptersWrappers.forEach( chaptersWrapper => {
                if ( +chaptersWrapper.dataset.chapterIndex === activeIndex ) {
                    activeChaptersWrapper = chaptersWrapper;
                }
            } );

            return activeChaptersWrapper;
        }

        function resetActiveChapters() {
            const chaptersWrappers = $box.querySelectorAll( '.mdp-watcher-elementor-chapters-wrapper' );
            chaptersWrappers.forEach( chaptersWrapper => {
                chaptersWrapper.style.display = 'none';
            } );
        }

        function playlistChapters( mejs ) {
            const playlistItems = $box.querySelectorAll( '.mejs-playlist-selector-list-item' );
            const chaptersWrappers = $box.querySelectorAll( '.mdp-watcher-elementor-chapters-wrapper' );
            const activeChapterWrapper = getActiveChapterWrapper( chaptersWrappers, getIndexOfActiveTrack( playlistItems ) );
            if ( activeChapterWrapper ) {
                activeChapterWrapper.style.display = 'flex';
                chaptersSwitching( activeChapterWrapper.querySelectorAll('.mdp-watcher-elementor-chapter-block' ), mejs );
            }
        }

        // set player settings
        const playerSettings = {
            features: features,
            customError: function ( media, node ) {
                return `<p>Media error: Format(s) not supported or source(s) not found</p>
                        <p>${node.src}</p>
                        `;
            },
            success: function ( mejs ) {
                // set chapters
                if ( !isPlaylist ) {
                    playerChapters( mejs );
                } else {
                    playlistChapters( mejs );
                    const playlistItems = $box.querySelectorAll( '.mejs-playlist-selector-list-item' );
                    playlistItems.forEach( playlistItem => {
                        playlistItem.addEventListener( 'click', () => {
                            resetActiveChapters();
                            playlistChapters( mejs );
                        } );
                    } );
                }

            }
        };

        // set playlist controls settings
        for ( let playerControlsSetting in playerControlsSettings ) {
            if ( !playerControlsSettings[playerControlsSetting] ) { continue; }
            playerSettings[playerControlsSetting] = playerControlsSettings[playerControlsSetting];
        }

        // set boolean settings
        for ( let key in playerBoolSettings ) {
            if ( playerBoolSettings.hasOwnProperty( key ) ) {
                playerSettings[key] = playerBoolSettings[key] === 'yes';
            }
        }

        // Change to string from array
        mejs.i18n.en['mejs.time-skip-back'] = 'Skip back %1 seconds';
        mejs.i18n.en['mejs.time-jump-forward'] = 'Jump forward %1 seconds';

        if ( isPlaylist ) {
            this.createPlaylist( widgetId, features, playerSettings, showPlaylistByDefault );
        } else {
            this.createVideoPlayer( widgetId, features, playerSettings );
        }

        // Return to default value
        mejs.i18n.en['mejs.time-skip-back'] = ['Skip back 1 second', 'Skip back %1 seconds'];
        mejs.i18n.en['mejs.time-jump-forward'] = ['Jump forward 1 second', 'Jump forward %1 seconds'];


    },

    createPlaylist: function ( widgetId, features, playerSettings, showPlaylistByDefault ) {
        const oldBuildFeatures = MediaElementPlayer.prototype.buildfeatures;

        features.push( 'playlist' );

        // show playlist box by default
        if ( showPlaylistByDefault === 'yes' ) {
            setTimeout( () => {
                const $playlistBox = document.querySelector( `.elementor-element-${widgetId} .mejs-playlist-layer` );

                $playlistBox.classList.remove( 'mejs-playlist-hidden' );
            }, 100 );
        }

        // override build features in mediaelement-migrate
        MediaElementPlayer.prototype.buildfeatures = function ( player, controls, layers, media ) {
            let t = this;
            player.container = document.querySelector( `#${player.id}.mejs-container` );

            for ( let i = 0, total = t.options.features.length; i < total; i++ ) {
                let feature = t.options.features[i];
                if ( t['build' + feature] ) {
                    try {
                        t['build' + feature]( player, controls, layers, media );
                    } catch ( e ) {
                        console.error( 'error building ' + feature, e );
                    }
                }
            }
        };

        new MediaElementPlayer( `mdp-watcher-elementor-player-${widgetId}`, playerSettings );

        // return to default
        MediaElementPlayer.prototype.buildfeatures = oldBuildFeatures;
    },

    createVideoPlayer: function ( widgetId, features, playerSettings ) {

        new MediaElementPlayer( `mdp-watcher-elementor-player-${widgetId}`, playerSettings );
    }
}

/**
 * Init for Front-End
 * @param callback
 */
jQuery( window ).on( 'elementor/frontend/init', function () {

    elementorFrontend.hooks.addAction( 'frontend/element_ready/global', function( $scope ) {

        if ( $scope[0].classList.contains( 'elementor-widget-mdp-watcher-elementor' ) ) {
            mdpWatcherElementor.watcherElementor.call( mdpWatcherElementor, $scope[0].dataset.id );
        }
    } );

} );