/**
 * Worker for Elementor
 * Worker creates a modern and stylish Elementor widget to display business hours.
 * Exclusively on https://1.envato.market/worker-elementor
 *
 * @encoding        UTF-8
 * @version         1.12.1
 * @copyright       (C) 2018 - 2023 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Alexander Khmelnitskiy (info@alexander.khmelnitskiy.ua), Dmitry Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

/**
 * @param window.mdpWorker
 */
window.addEventListener('DOMContentLoaded', (event) => {

    if ( window.mdpWorker === undefined ) { return; }

    let { offset, dateFormat, timeFormat } = window.mdpWorker;
    const now = new Date();
    const locInSec = Math.floor( now / 1000 );
    const utcInSec = locInSec + ( now.getTimezoneOffset() * 60 );
    const wpInSec = utcInSec + parseInt( offset );
    const $dates = document.querySelectorAll( '.mdp-worker-elementor-date' );
    const lang = document.documentElement.lang ? document.documentElement.lang : 'default';
    let wpTime = new Date( wpInSec * 1000 );

    $dates.forEach(( $date) => {

        // Replace time
        const $time = $date.querySelector( '.non-cache-time' );
        if ( $time !== null ) {

            timeFormat = timeFormat.replace( 'a', wpTime.getHours() > 12 ? 'pm' : 'am' ); // am, pm
            timeFormat = timeFormat.replace( 'A', wpTime.getHours() > 12 ? 'PM' : 'AM' ); // AM, PM
            timeFormat = timeFormat.replace( 'g', wpTime.getHours() > 12 ? wpTime.getHours() - 12 : wpTime.getHours() ) // without leading zeros 1–12
            timeFormat = timeFormat.replace( 'h', leadingZero( wpTime.getHours() > 12 ? wpTime.getHours() - 12 : wpTime.getHours() ) ); // with leading zeros 01–12
            timeFormat = timeFormat.replace( 'G', wpTime.getHours() ); // without leading zeros 0-23
            timeFormat = timeFormat.replace( 'H', leadingZero( wpTime.getHours() ) ); // with leading zeros	00-23
            timeFormat = timeFormat.replace( 'i', leadingZero( wpTime.getMinutes() ) ); // with leading zeros 00-59
            timeFormat = timeFormat.replace( 's', leadingZero( wpTime.getSeconds() ) ); // with leading zeros	0-59
            timeFormat = timeFormat.replace( 'T', '' ); // Timezone

            $time.innerHTML = timeFormat;

        }

        // Replace date
        const $day = $date.querySelector( '.non-cache-date' );
        if ( $time !== null ) {

            const getYear = wpTime.getFullYear();

            dateFormat = dateFormat.replace( 'd', leadingZero( wpTime.getDate() ) ); // with leading zeros	01–31
            dateFormat = dateFormat.replace( 'j', wpTime.getDate() ); // without leading zeros	1–31
            dateFormat = dateFormat.replace( 'S', '' ); // The English suffix for the day of the month	st, nd or th in the 1st, 2nd or 15th.
            dateFormat = dateFormat.replace( 'm', leadingZero( wpTime.getMonth() + 1 ) ); // with leading zeros	01–12
            dateFormat = dateFormat.replace( 'n', wpTime.getMonth() + 1 ); // without leading zeros	1–12
            dateFormat = dateFormat.replace( 'Y', wpTime.getFullYear() );  // Numeric, 4 digits	Eg., 1999, 2003
            dateFormat = dateFormat.replace( 'y', getYear.toString().substring( 2 ) ); // Numeric, 2 digits	Eg., 99, 03

            dateFormat = dateFormat.replace( 'F', '%F%' );
            dateFormat = dateFormat.replace( 'M', '%M%' );

            dateFormat = dateFormat.replace( 'l', wpTime.toLocaleString( lang, { weekday: 'long' } ) ); // Full name  (lowercase ‘L’)	Sunday – Saturday
            dateFormat = dateFormat.replace( 'D', wpTime.toLocaleString( lang, { weekday: 'short' } ) ); // Three letter name	Mon – Sun

            dateFormat = dateFormat.replace( '%F%', wpTime.toLocaleString( lang, { month: 'long' } ) ); // Textual full	January – December
            dateFormat = dateFormat.replace( '%M%', wpTime.toLocaleString( lang, { month: 'short' } ) ); // Textual three letters	Jan – Dec

            $day.innerHTML = dateFormat;

        }

    } );

    /**
     * Adding leading zero
     * @param number
     */
    function leadingZero( number ) {

        number = number.toString();
        return number.length < 2 ? `0${number}` : number;

    }

});
