<?php
/**
 * Selection
 * Carefully selected Elementor addons bundle, for building most awesome websites
 * Exclusively on https://1.envato.market/selection
 *
 * @encoding        UTF-8
 * @version         1.12.1
 * @copyright       (C) 2018 - 2023 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Dmitry Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

namespace Merkulove\Selection;

use Merkulove\Selection\Unity\Plugin;
use Merkulove\Selection\Unity\Settings;
use Merkulove\Selection\Unity\TabGeneral;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
	header( 'Status: 403 Forbidden' );
	header( 'HTTP/1.1 403 Forbidden' );
	exit;
}

/**
 * SINGLETON: Settings class used to modify default plugin settings.
 * @since 1.0.0
 **/
final class Config {

	/**
	 * The one true Settings.
	 *
     * @since 1.0.0
     * @access private
	 * @var Config
	 **/
	private static $instance;

    /**
     * Prepare plugin settings by modifying the default one.
     *
     * @since 1.0.0
     * @access public
     *
     * @return void
     **/
    public function prepare_settings() {

        $tabs = Plugin::get_tabs(); // Get default plugin settings.
	    $tabs = $this->set_requirements_checks( $tabs ); // Set System Requirements checks

        // Remove 'Delete plugin, settings and data' option from Uninstall tab.
		unset( $tabs[ 'uninstall' ][ 'fields' ][ 'delete_plugin' ][ 'options' ][ 'plugin+settings+data' ] );

        unset( $tabs[ 'general' ] );
        unset( $tabs[ 'assignments' ] );

        // Widgets tab
        $offset = 0;
        $tabs = array_slice( $tabs, 0, $offset, true ) +
            [ 'widgets' => [
                'enabled'       => true,
                'class'         => TabWidgets::class, // Handler
                'label'         => esc_html__( 'Widgets', 'selection' ),
                'title'         => esc_html__( 'The Selection widgets', 'selection' ),
                'show_title'    => true,
                'icon'          => 'widgets', // Icon for tab
                'fields'        => []
                ],

                'integrations' => [
                    'enabled'       => true,
                    'class'         => TabGeneral::class, // Handler
                    'label'         => esc_html__( 'Integrations', 'selection' ),
                    'title'         => esc_html__( 'Integrations', 'selection' ),
                    'show_title'    => true,
                    'icon'          => 'integration_instructions', // Icon for tab
                    'fields'        => []
                ]
            ] +

            array_slice( $tabs, $offset, NULL, true );

	    // Additional setting for the widgets tab
        $tabs[ 'widgets' ][ 'fields' ][ 'widgets_grid' ] = [
            'type'              => 'custom_type',
            'render'            => [ TabWidgets::get_instance(), 'widgets_additional' ],
            'label'             => '', 'show_label'        => false,
            'description'       => '', 'show_description'  => false,
            'default'           => '',
        ];

        $tabs[ 'integrations' ][ 'fields' ][ 'google_maps_api_key' ] = [
            'type'              => 'text',
            'label'             => esc_html__( 'Google Maps Api key', 'selection' ),
            'show_label'        => true,
            'placeholder'       => esc_html__( 'Google Maps Api key', 'selection' ),
            'description'       => wp_kses_post( 'Set your api key for Google Maps. Visit <a href="https://docs.merkulov.design/how-to-get-google-map-api-key/" target="_blank">Online Documentation</a> for more details.' ),
            'show_description'  => true,
            'default'           => '',
            'attr'              => [
                'class'     => 'mdp-test-class',
                'maxlength' => '4500'
            ]
        ];

        $tabs[ 'integrations' ][ 'fields' ][ 'mdp_synopter_elementor_weather_settings' ] = [
            'type'              => 'text',
            'label'             => esc_html__( 'OpenWeather Key', 'selection' ),
            'show_label'        => true,
            'placeholder'       => esc_html__( 'OpenWeather Key', 'selection' ),
            'description'       => esc_html__( 'You can create key by clicking on the link', 'selection' ) . ' <a href="https://home.openweathermap.org/users/sign_up" target="_blank">Open Weather</a>',
            'show_description'  => true,
            'default'           => '',
            'attr'              => [
                'maxlength' => '250'
            ]
        ];

        // Add Text control
        $tabs['integrations']['fields']['ck_key'] = [
            'type'              => 'text',
            'label'             => esc_html__( 'WooCommerce Consumer key', 'selection' ),
            'show_label'        => true,
            'placeholder'       => esc_html__( 'Enter consumer key', 'selection' ),
            'description'       => esc_html__( 'Get your Consumer key for the Grid widget. Visit', 'selection' ) . ' <a href="https://docs.merkulov.design/how-to-create-api-key-for-woocommerce/" target="_blank">Online Documentation</a>' . esc_html__( ' for more details.', 'selection' ),
            'show_description'  => false,
            'default'           => '',
            'attr'              => [
                'class'     => 'mdp-test-class',
                'maxlength' => '4500'
            ]
        ];

        $tabs['integrations']['fields']['cs_key'] = [
            'type'              => 'text',
            'label'             => esc_html__( 'WooCommerce Consumer secret', 'selection' ),
            'show_label'        => true,
            'placeholder'       => esc_html__( 'Enter consumer secret', 'selection' ),
            'description'       => esc_html__( 'Get your Consumer secret for the Grid widget. Visit', 'selection' ) . ' <a href="https://docs.merkulov.design/how-to-create-api-key-for-woocommerce/" target="_blank">Online Documentation</a>' . esc_html__( ' for more details.', 'selection' ),
            'show_description'  => false,
            'default'           => '',
            'attr'              => [
                'class'     => 'mdp-test-class',
                'maxlength' => '4500'
            ]
        ];

        $tabs['integrations']['fields']['nowpayments_api_key'] = [
            'type'              => 'text',
            'label'             => esc_html__( 'Nowpayments Api key', 'selection' ),
            'show_label'        => true,
            'placeholder'       => esc_html__( 'Nowpayments Api key', 'selection' ),
            'description'       => wp_kses_post( '' ),
            'show_description'  => true,
            'default'           => '',
            'attr'              => [
                'class'     => 'mdp-test-class',
                'maxlength' => '4500'
            ]
        ];

        /** Set updated tabs. */
        Plugin::set_tabs( $tabs );

        /** Refresh settings. */
        Settings::get_instance()->get_options();

    }

	/**
	 * Set System Requirements checks.
	 *
	 * @param array $tabs - All plugin tabs with fields and settings.
	 *
	 * @since  1.0.0
	 * @access public
	 *
	 * @return array
	 **/
	private function set_requirements_checks( $tabs ) {

		/** Set System Requirements. */
		$tabs['status']['reports']['server']['elementor_installed'] = true;
		$tabs['status']['reports']['server']['allow_url_fopen'] = false;
		$tabs['status']['reports']['server']['dom_installed'] = false;
		$tabs['status']['reports']['server']['xml_installed'] = false;
		$tabs['status']['reports']['server']['bcmath_installed'] = false;

		return $tabs;

	}

	/**
	 * Main Settings Instance.
	 * Insures that only one instance of Settings exists in memory at any one time.
	 *
	 * @static
     * @since 1.0.0
     * @access public
     *
	 * @return Config
	 **/
	public static function get_instance() {

		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof self ) ) {

			self::$instance = new self;

		}

		return self::$instance;

	}

}
