<?php /** @noinspection PhpUndefinedClassInspection */
/**
 * Carter for Elementor
 * Customizable cart for Elementor editor
 * Exclusively on https://1.envato.market/carter-elementor
 *
 * @encoding        UTF-8
 * @version         1.12.1
 * @copyright       (C) 2018 - 2023 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Dmitry Merkulov (dmitry@merkulov.design), Cherviakov Vlad (vladchervjakov@gmail.com)
 * @support         help@merkulov.design
 **/

namespace Merkulove\Selection;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
    header( 'Status: 403 Forbidden' );
    header( 'HTTP/1.1 403 Forbidden' );
    exit;
}

use Elementor\Icons_Manager;
use Elementor\Plugin;
use Elementor\Core\Schemes\Color;
use Elementor\Core\Schemes\Typography;
use Elementor\Group_Control_Background;
use Elementor\Group_Control_Border;
use Elementor\Group_Control_Box_Shadow;
use Elementor\Group_Control_Css_Filter;
use Elementor\Group_Control_Typography;
use Exception;
use Elementor\Widget_Base;
use Elementor\Controls_Manager;
use Merkulove\Selection\Unity\Plugin as UnityPlugin;

/** @noinspection PhpUnused */
/**
 * Carter - Custom Elementor Widget.
 **/
class carter_elementor extends Widget_Base {

    /**
     * Use this to sort widgets.
     * A smaller value means earlier initialization of the widget.
     * Can take negative values.
     * Default widgets and widgets from 3rd party developers have 0 $mdp_order
     **/
    public $mdp_order = 1;

    /**
     * Widget base constructor.
     * Initializing the widget base class.
     *
     * @access public
     * @throws Exception If arguments are missing when initializing a full widget instance.
     * @param array      $data Widget data. Default is an empty array.
     * @param array|null $args Optional. Widget default arguments. Default is null.
     *
     * @return void
     **/
    public function __construct( $data = [], $args = null ) {

        parent::__construct( $data, $args );

        wp_register_style(
            'mdp-selection-elementor-admin',
            UnityPlugin::get_url() . 'css/elementor-admin' . UnityPlugin::get_suffix() . '.css',
            [],
            UnityPlugin::get_version()
        );
        wp_register_style(
            'mdp-carter-elementor',
                UnityPlugin::get_url() . 'css/carter-elementor' . UnityPlugin::get_suffix() . '.css',
                    [],
                    UnityPlugin::get_version()
        );
	    wp_register_script(
                'mdp-carter-elementor',
                  UnityPlugin::get_url() . 'js/carter-elementor' . UnityPlugin::get_suffix() . '.js',
                     [ 'jquery' ], UnityPlugin::get_version(),
            true
        );

    }

    /**
     * Return a widget name.
     *
     * @return string
     **/
    public function get_name() {

        return 'mdp-carter-elementor';

    }

    /**
     * Return the widget title that will be displayed as the widget label.
     *
     * @return string
     **/
    public function get_title() {

        return esc_html__( 'Cart', 'selection' );

    }

    /**
     * Set the widget icon.
     *
     * @return string
     */
    public function get_icon() {

        return 'mdp-carter-elementor-widget-icon';

    }

    /**
     * Set the category of the widget.
     *
     * @return array with category names
     **/
    public function get_categories() {

        return [ 'selection-category' ];

    }

    /**
     * Get widget keywords. Retrieve the list of keywords the widget belongs to.
     *
     * @access public
     *
     * @return array Widget keywords.
     **/
    public function get_keywords() {

        return [ 'Merkulove', 'Cart' ];

    }

    /**
     * Get style dependencies.
     * Retrieve the list of style dependencies the widget requires.
     *
     * @access public
     *
     * @return array Widget styles dependencies.
     **/
    public function get_style_depends() {

        return [ 'mdp-carter-elementor', 'mdp-selection-elementor-admin' ];

    }

	/**
	 * Get script dependencies.
	 * Retrieve the list of script dependencies the element requires.
	 *
	 * @access public
     *
	 * @return array Element scripts dependencies.
	 **/
	public function get_script_depends() {

		return [ 'mdp-carter-elementor', 'wc-cart', 'selectWoo' ];

    }

    /**
     * Add the widget controls.
     *
     * @access protected
     * @return void with category names
     **/
    protected function register_controls() {

        /** Content Tab. */
        $this->tab_content();

        /** Style Tab. */
        $this->tab_style();

    }

    /**
     * Add widget controls on Content tab.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function tab_content() {

        /** Content -> Section Content General. */
        $this->section_content_general();

        /** Content -> Section Content Update Cart Button. */
        $this->section_content_update_cart_button();

        /** Content -> Section Content Coupon. */
        $this->section_content_coupon();

        /** Content -> Section Content Totals. */
        $this->section_content_totals();

        /** Content -> Section Content Update Shipping Button. */
        $this->section_content_update_shipping_button();

        /** Content -> Section Content Checkout Button. */
        $this->section_content_checkout_button();

    }

    /**
     * Add widget controls on Style tab.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function tab_style() {

        /** Style -> Section Style Product Table. */
        $this->section_style_product_table();

        /** Style -> Section Style Total. */
        $this->section_style_total();

        /** Style -> Section Style Total Title. */
        $this->section_style_total_title();

        /** Style -> Section Style Product Thumbnail Cell. */
        $this->section_style_product_thumbnail_cell();

        /** Style -> Section Style Product Thumbnail. */
        $this->section_style_product_thumbnail();

        /** Style -> Section Style Product Title Heading. */
        $this->section_style_product_title_heading();

        /** Style -> Section Style Product Title. */
        $this->section_style_product_title();

        /** Style -> Section Style Product Price Heading. */
        $this->section_style_product_price_heading();

        /** Style -> Section Style Product Title. */
        $this->section_style_product_price();

        /** Style -> Section Style Product Quantity Heading. */
        $this->section_style_product_quantity_heading();

        /** Style -> Section Style Product Quantity. */
        $this->section_style_product_quantity();

        /** Style -> Section Style Product Subtotal Heading. */
        $this->section_style_product_subtotal_heading();

        /** Style -> Section Style Product Subtotal. */
        $this->section_style_product_subtotal();

        /** Style -> Section Style Product Remove. */
        $this->section_style_product_remove();

        /** Style -> Section Style Coupon. */
        $this->section_style_coupon();

        /** Style -> Section Style Total Subtotal Heading. */
        $this->create_total_part_style_settings(
            'section_style_total_subtotal_heading',
          'Subtotal heading',
            'cart-subtotal th',
        'totals_show_subtotal',
      ', {{WRAPPER}} .cart-subtotal td:before',
  ', {{WRAPPER}} .cart-subtotal td:hover:before'
        );

        /** Style -> Section Style Total Subtotal. */
        $this->create_total_part_style_settings(
            'section_style_total_subtotal',
        'Subtotal field',
            'cart-subtotal td',
        'totals_show_subtotal'
        );

        /** Style -> Section Style Total Shipping Heading. */
        $this->create_total_part_style_settings(
            'section_style_total_shipping_heading',
            'Shipping heading',
            'woocommerce-shipping-totals th',
            'totals_show_shipping',
          ', {{WRAPPER}} .woocommerce-shipping-totals td:before',
      ', {{WRAPPER}} .woocommerce-shipping-totals td:hover:before'

        );

        /** Style -> Section Style Total Shipping. */
        $this->create_total_part_style_settings(
            'section_style_total_shipping',
        'Shipping field',
            'woocommerce-shipping-totals td',
        'totals_show_shipping'
        );

        /** Style -> Section Style Total Tax Heading. */
        $this->create_total_part_style_settings(
            'section_style_total_tax_heading',
            'Tax heading',
            'tax-total th',
            'totals_show_tax',
         ', {{WRAPPER}} .tax-total td:before',
   ', {{WRAPPER}} .tax-total td:hover:before'

        );

        /** Style -> Section Style Total Tax. */
        $this->create_total_part_style_settings(
            'section_style_total_tax',
        'Tax field',
            'tax-total td',
        'totals_show_tax'
        );

        /** Style -> Section Style Total Field Heading. */
        $this->create_total_part_style_settings(
            'section_style_total_total_heading',
            'Total field heading',
            'order-total th',
            'totals_show_total',
        ', {{WRAPPER}} .order-total td:before',
    ', {{WRAPPER}} .order-total td:hover:before'

        );

        /** Style -> Section Style Total Field. */
        $this->create_total_part_style_settings(
            'section_style_total_total',
        'Total field',
            'order-total td',
        'totals_show_total'
        );

        /** Style -> Section Style Total Fees Heading. */
        $this->create_total_part_style_settings(
            'section_style_total_fees_heading',
            'Fee heading',
            'fee th',
         'totals_show_fees',
        ', {{WRAPPER}} .fee td:before',
    ', {{WRAPPER}} .fee td:hover:before'

        );

        /** Style -> Section Style Total Fees. */
        $this->create_total_part_style_settings(
            'section_style_total_fees',
        'Fee field',
            'fee td',
        'totals_show_fees'
        );

        /** Style -> Section Style Shipping Inputs. */
        $this->create_input_style_settings(
            'section_style_shipping_input',
            'Shipping inputs',
            'shipping-calculator-form input',
              'totals_show_shipping'
        );

        /** Style -> Section Style Coupon Input. */
        $this->create_input_style_settings(
            'section_style_coupon_input',
          'Coupon input',
            'mdp-carter-elementor-coupon-block input',
            'show_coupon'
        );

        /** Style -> Section Style Quantity Input. */
        $this->create_input_style_settings(
            'section_style_quantity_input',
          'Quantity input',
            'product-quantity input',
            '',
            65
        );

        /** Style -> Section Style Shipping Button. */
        $this->create_buttons_controls(
            'section_style_update_cart_button',
            'Update cart button',
            'mdp-carter-elementor-cart-product-table button[name="update_cart"]',
          'auto_update_cart!'
        );

        /** Style -> Section Style Shipping Button. */
        $this->create_buttons_controls(
        'section_style_shipping_button',
     'Update shipping button',
      'woocommerce-shipping-calculator button',
    'totals_show_shipping'
        );

        /** Style -> Section Style Checkout Button. */
        $this->create_buttons_with_icons_styles(
           'section_style_checkout_button',
        'Checkout button',
         'mdp-carter-elementor-box a.button.checkout-button',
       '',
   'checkout_button_enable_icon'
        );

        /** Style -> Section Style Coupon Button. */
        $this->create_buttons_with_icons_styles(
           'section_style_coupon_button',
        'Coupon button',
         'mdp-carter-elementor-coupon-block button',
       'show_coupon',
    'coupon_button_enable_icon'
        );

    }

    /**
     * Get elementor templates.
     *
     * @since 1.0.0
     * @access private
     *
     * @return array
     **/
    private function get_templates() {
        $templates = Plugin::instance()->templates_manager->get_source( 'local' )->get_items();

        $results = [];

        foreach ( $templates as $template ) {
            $results[ $template[ 'template_id' ] ] = $template[ 'title' ];
        }
        return $results;
    }

    /**
     * Create settings for button icon.
     *
     * @param $prefix
     * @return void
     * @since 1.0.0
     * @access private
     */
    private function create_button_icon_settings( $prefix ) {
        $this->add_control(
            $prefix . '_enable_icon',
            [
                'label' => esc_html__( 'Enable button icon', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Yes', 'selection' ),
                'label_off' => esc_html__( 'No', 'selection' ),
                'return_value' => 'yes',
                'default' => 'yes',
            ]
        );

        $this->add_control(
            $prefix . '_icon',
            [
                'label' => esc_html__( 'Button icon', 'selection' ),
                'type' => Controls_Manager::ICONS,
                'default' => [
                    'value' => 'fas fa-star',
                    'library' => 'solid',
                ],
                'condition' => [
                    $prefix . '_enable_icon' => 'yes'
                ]
            ]
        );

        $this->add_control(
            $prefix . '_icon_position',
            [
                'label' => esc_html__( 'Icon position', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'before',
                'options' => [
                    'before'  => esc_html__( 'Left', 'selection' ),
                    'after' => esc_html__( 'Right', 'selection' ),
                    'above-left' => esc_html__( 'Top left', 'selection' ),
                    'above-center' => esc_html__( 'Top center', 'selection' ),
                    'above-right' => esc_html__( 'Top right', 'selection' ),
                    'under-left' => esc_html__( 'Bottom left', 'selection' ),
                    'under-center' => esc_html__( 'Bottom center', 'selection' ),
                    'under-right' => esc_html__( 'Bottom right', 'selection' )
                ],
                'condition' => [
                    $prefix . '_enable_icon' => 'yes'
                ]
            ]
        );
    }

    /**
     * Add widget controls: Content -> General Content Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_content_general() {

        $this->start_controls_section( 'section_content_general', [
            'label' => esc_html__( 'General', 'selection' ),
            'tab'   => Controls_Manager::TAB_CONTENT
        ] );

        $this->add_control(
            'cart_layout',
            [
                'label'     => esc_html__( 'Layout', 'selection' ),
                'type'      => Controls_Manager::SELECT,
                'options'   => [
                    'one-column'    => esc_html__( 'One column', 'selection' ),
                    'two-columns'    => esc_html__( 'Two columns', 'selection' ),
                ],
                'default'   => 'two-columns',
            ]
        );

        $this->add_control(
            'sticky_right_column',
            [
                'label' => esc_html__( 'Sticky right column', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'return_value' => 'yes',
                'default' => 'yes',
                'condition' => [
                    'cart_layout' => 'two-columns'
                ]
            ]
        );

        $this->add_control(
            'show_coupon',
            [
                'label' => esc_html__( 'Show coupon', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'return_value' => 'yes',
                'default' => 'yes',
            ]
        );

        $this->add_control(
            'show_total',
            [
                'label' => esc_html__( 'Show total', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'return_value' => 'yes',
                'default' => 'yes',
            ]
        );

        $this->add_control(
            'auto_update_cart',
            [
                'label' => esc_html__( 'Update cart automatically', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'return_value' => 'yes',
                'default' => 'yes',
            ]
        );

        $this->add_control(
            'customize_empty_cart',
            [
                'label' => esc_html__( 'Customize empty cart', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'return_value' => 'yes',
                'default' => 'no',
            ]
        );

        $templates = $this->get_templates();


        if ( !empty( $templates ) ) {

            $this->add_control(
                'empty_cart_template',
                [
                    'label' => esc_html__('Template', 'selection'),
                    'type' => Controls_Manager::SELECT,
                    'options' => $templates,
                    'condition' => [
                        'customize_empty_cart' => 'yes'
                    ],
                    'default' => 'template-1',
                ]
            );
        } else {
            $this->add_control(
                'template_link',
                [
                    'type' => Controls_Manager::RAW_HTML,
                    'raw' => '<strong>' . esc_html__( 'There are no templates in your site.', 'selection' ) .
                        '</strong><br>' . sprintf( wp_kses_post( 'Go to the <a href="%s" target="_blank">Templates screen</a> to create one.' ),
                            admin_url( 'edit.php?post_type=elementor_library&tabs_group=library' ) ),
                    'separator' => 'after',
                    'content_classes' => 'elementor-panel-alert elementor-panel-alert-info',
                    'condition' => [
                        'customize_empty_cart' => 'yes'
                    ]
                ]
            );
        }

        $this->add_control(
            'empty_cart_text',
            [
                'label' => esc_html__( 'Empty cart text', 'selection' ),
                'type' => Controls_Manager::TEXT,
                'condition' => [
                    'customize_empty_cart!' => 'yes'
                ]
            ]
        );


        $this->end_controls_section();

    }

    /**
     * Add widget controls: Content -> Update Cart Button Content Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_content_update_cart_button() {
        $this->start_controls_section( 'section_content_update_cart_button', [
            'label' => esc_html__( 'Update cart button', 'selection' ),
            'tab' => Controls_Manager::TAB_CONTENT,
            'condition' => [
                'auto_update_cart!' => 'yes'
            ]
        ] );

        $this->add_control(
            'update_cart_button_text',
            [
                'label' => esc_html__( 'Button text', 'selection' ),
                'type' => Controls_Manager::TEXT,
            ]
        );

        $this->add_control(
            'update_cart_button_align',
            [
                'label' => esc_html__( 'Alignment', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'eicon-text-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'eicon-text-align-center',
                    ],
                    'right' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'eicon-text-align-right',
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} td.actions' => 'text-align: {{VALUE}};'
                ],
                'default' => 'center',
                'toggle' => true,
            ]
        );

        $this->add_control(
            'update_cart_hover_animation',
            [
                'label' => esc_html__( 'Button hover animation', 'selection' ),
                'type' => Controls_Manager::HOVER_ANIMATION,
            ]
        );


        $this->end_controls_section();

    }


    /**
     * Add widget controls: Content -> Coupon Content Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_content_coupon() {
        $this->start_controls_section( 'section_content_coupon', [
            'label' => esc_html__( 'Coupon', 'selection' ),
            'tab' => Controls_Manager::TAB_CONTENT,
            'condition' => [
                'show_coupon' => 'yes'
            ]
        ] );

        $this->add_control(
            'coupon_button_text',
            [
                'label' => esc_html__( 'Button text', 'selection' ),
                'type' => Controls_Manager::TEXT,
            ]
        );

        $this->add_control(
            'coupon_button_align',
            [
                'label' => esc_html__( 'Alignment', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'flex-start' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'eicon-text-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'eicon-text-align-center',
                    ],
                    'flex-end' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'eicon-text-align-right',
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-carter-elementor-coupon-block' => 'justify-content: {{VALUE}};'
                ],
                'default' => 'center',
                'toggle' => true,
            ]
        );

        $this->create_button_icon_settings( 'coupon_button' );

        $this->add_control(
            'coupon_hover_animation',
            [
                'label' => esc_html__( 'Button hover animation', 'selection' ),
                'type' => Controls_Manager::HOVER_ANIMATION,
            ]
        );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Content -> Totals Content Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_content_totals() {
        $this->start_controls_section( 'section_content_totals', [
            'label' => esc_html__( 'Totals', 'selection' ),
            'tab' => Controls_Manager::TAB_CONTENT,
            'condition' => [
                'show_total' => 'yes'
            ]
        ] );

        $this->add_control(
            'totals_title_text',
            [
                'label' => esc_html__( 'Title text', 'selection' ),
                'type' => Controls_Manager::TEXT,
            ]
        );

        $this->add_control(
            'total_alignment',
            [
                'label' => esc_html__( 'Alignment', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'eicon-text-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'eicon-text-align-center',
                    ],
                    'right' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'eicon-text-align-right',
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .cart-collaterals .shop_table' => 'text-align: {{VALUE}};',
                    '{{WRAPPER}} .cart-collaterals' => 'text-align: {{VALUE}};'
                ],
                'default' => 'center',
                'toggle' => true,
            ]
        );

        $this->add_control(
            'totals_show_total',
            [
                'label' => esc_html__( 'Show total', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'return_value' => 'yes',
                'default' => 'yes',
            ]
        );

        $this->add_control(
            'totals_show_subtotal',
            [
                'label' => esc_html__( 'Show subtotal', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'return_value' => 'yes',
                'default' => 'yes',
            ]
        );

        $this->add_control(
            'totals_show_tax',
            [
                'label' => esc_html__( 'Show tax', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'return_value' => 'yes',
                'default' => 'yes',
            ]
        );

        $this->add_control(
            'totals_show_fees',
            [
                'label' => esc_html__( 'Show fees', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'return_value' => 'yes',
                'default' => 'yes',
            ]
        );

        $this->add_control(
            'totals_show_shipping',
            [
                'label' => esc_html__( 'Show shipping', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'return_value' => 'yes',
                'default' => 'yes',
            ]
        );


        $this->end_controls_section();

    }

    /**
     * Add widget controls: Content -> Update Shipping Button Content Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_content_update_shipping_button() {
        $this->start_controls_section( 'section_content_update_shipping_button', [
            'label' => esc_html__( 'Update shipping button', 'selection' ),
            'tab' => Controls_Manager::TAB_CONTENT,
            'condition' => [
                'show_total' => 'yes',
                'totals_show_shipping' => 'yes'
            ]
        ] );

        $this->add_control(
            'update_shipping_button_text',
            [
                'label' => esc_html__( 'Button text', 'selection' ),
                'type' => Controls_Manager::TEXT,
            ]
        );

        $this->add_control(
            'update_shipping_button_align',
            [
                'label' => esc_html__( 'Alignment', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'eicon-text-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'eicon-text-align-center',
                    ],
                    'right' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'eicon-text-align-right',
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} #calc_shipping_postcode_field + p' => 'text-align: {{VALUE}};'
                ],
                'default' => 'center',
                'toggle' => true,
            ]
        );

        $this->add_control(
            'update_shipping_hover_animation',
            [
                'label' => esc_html__( 'Button hover animation', 'selection' ),
                'type' => Controls_Manager::HOVER_ANIMATION,
            ]
        );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Content -> Checkout Button Content Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_content_checkout_button() {
        $this->start_controls_section( 'section_content_checkout_button', [
            'label' => esc_html__( 'Checkout button', 'selection' ),
            'tab' => Controls_Manager::TAB_CONTENT,
        ] );

        $this->add_control(
            'checkout_button_text',
            [
                'label' => esc_html__( 'Button text', 'selection' ),
                'type' => Controls_Manager::TEXT,
            ]
        );

        $this->add_control(
            'checkout_button_align',
            [
                'label' => esc_html__( 'Alignment', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'flex-start' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'eicon-text-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'eicon-text-align-center',
                    ],
                    'flex-end' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'eicon-text-align-right',
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .wc-proceed-to-checkout' => 'justify-content: {{VALUE}};'
                ],
                'default' => 'center',
                'toggle' => true,
            ]
        );

        $this->create_button_icon_settings( 'checkout_button' );

        $this->add_control(
            'checkout_hover_animation',
            [
                'label' => esc_html__( 'Button hover animation', 'selection' ),
                'type' => Controls_Manager::HOVER_ANIMATION,
            ]
        );

        $this->end_controls_section();

    }

    /**
     * Method for generating margin padding controls.
     *
     * @param $section_id
     * @param $html_class
     * @param array $default_padding
     * @param array $default_margin
     * @return void
     * @since 1.0.0
     * @access private
     */
    private function generate_margin_padding_controls( $section_id, $html_class, $default_padding = [], $default_margin = [] ) {


        $this->add_responsive_control(
            $section_id.'_margin',
            [
                'label' => esc_html__( 'Margin', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'default' => $default_margin,
                'selectors' => [
                    "{{WRAPPER}} .$html_class" => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}} !important;',
                ],
            ]
        );

        $this->add_responsive_control(
            $section_id.'_padding',
            [
                'label' => esc_html__( 'Padding', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'default' => $default_padding,
                'selectors' => [
                    "{{WRAPPER}} .$html_class" => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );
    }

    /**
     * Function for generating typography and tabs controls.
     *
     * @param $section_id
     * @param $opts
     * @return void
     * @since 1.0.0
     * @access private
     */
    private function generate_typography_tabs_controls( $section_id, $opts = [] ) {
        $style_opts = [
            'html_class' => array_key_exists( 'html_class', $opts ) ?
                $opts['html_class'] : '',
            'additional_html_selector' => array_key_exists( 'additional_html_selector', $opts ) ?
                $opts['additional_html_selector'] : '',
            'additional_html_hover_selector' => array_key_exists( 'additional_html_hover_selector', $opts ) ?
                $opts['additional_html_hover_selector'] : '',
            'active_class' => array_key_exists( 'active_class', $opts ) ?
                $opts['active_class'] : '',
            'include_color' => array_key_exists( 'include_color', $opts ) ?
                $opts['include_color'] : true,
            'include_bg' => array_key_exists( 'include_bg', $opts ) ?
                $opts['include_color'] : true,
            'include_typography' => array_key_exists( 'include_typography', $opts ) ?
                $opts['include_typography'] : true,
            'include_transition' => array_key_exists( 'include_transition', $opts ) ?
                $opts['include_transition'] : true,
            'additional_color' => array_key_exists( 'additional_color', $opts ) ?
                $opts['additional_color'] : false,
            'additional_color_hover' => array_key_exists( 'additional_color_hover', $opts ) ?
                $opts['additional_color_hover'] : false,
            'include_active_tab' => array_key_exists( 'include_active_tab', $opts ) ?
                $opts['include_active_tab'] : false,
            'include_css_filters' => array_key_exists( 'include_css_filters', $opts ) ?
                $opts['include_css_filters'] : false,
            'active_tab_name' => array_key_exists( 'active_tab_name', $opts ) ?
                $opts['active_tab_name'] : 'ACTIVE',
            'color_prefix' => array_key_exists( 'color_prefix', $opts ) ?
                $opts['color_prefix'] : '',
            'color_class' => array_key_exists( 'color_class', $opts ) ?
                $opts['color_class'] : '',
            'color_hover_class' => array_key_exists( 'color_hover_class', $opts ) ?
                $opts['color_hover_class'] : '',
            'color_active_class' => array_key_exists( 'color_active_class', $opts ) ?
                $opts['color_active_class'] : '',
            'color_hover_selector' => array_key_exists( 'color_hover_selector', $opts ) ?
                $opts['color_hover_selector'] : '',
            'additional_color_name' => array_key_exists( 'additional_color_name', $opts ) ?
                $opts['additional_color_name'] : '',
            'additional_color_class' => array_key_exists( 'additional_color_class', $opts ) ?
                $opts['additional_color_class'] : '',
            'additional_color_hover_selectors' => array_key_exists( 'additional_color_hover_selectors', $opts ) ?
                $opts['additional_color_hover_selectors'] : '',
            'additional_color_active_class' => array_key_exists( 'additional_color_active_class', $opts ) ?
                $opts['additional_color_active_class'] : '',
            'additional_transition_selector' => array_key_exists( 'additional_transition_selector', $opts ) ?
                $opts['additional_transition_selector'] : '',
            'typography_class' => array_key_exists( 'typography_class', $opts ) ?
                $opts['typography_class'] : '',
            'additional_border_radius_selector' => array_key_exists( 'additional_border_radius_selector', $opts ) ?
                $opts['additional_border_radius_selector'] : '',
            'additional_border_radius_hover_selector' => array_key_exists( 'additional_border_radius_hover_selector', $opts ) ?
                $opts['additional_border_radius_hover_selector'] : '',
            'color_scheme_default' => array_key_exists( 'color_scheme_default', $opts ) ?
                $opts['color_scheme_default'] : Color::COLOR_3,
            'additional_color_scheme_default' => array_key_exists( 'additional_color_scheme_default', $opts ) ?
                $opts['additional_color_scheme_default'] : Color::COLOR_3,
            'additional_color_default' => array_key_exists( 'additional_color_default', $opts ) ?
                $opts['additional_color_default'] : '',
            'color_default' => array_key_exists( 'color_default', $opts ) ?
                $opts['color_default'] : ''
        ];


        if ( $style_opts['include_typography'] ) {
            $this->add_group_control(
                Group_Control_Typography::get_type(),
                [
                    'name' => $section_id . '_typography',
                    'label' => esc_html__('Typography', 'selection'),
                    'scheme' => Typography::TYPOGRAPHY_1,
                    'selector' => "{{WRAPPER}} .".$style_opts['typography_class'],
                ]
            );
        }

        $this->start_controls_tabs( $section_id.'_style_tabs' );

        $this->start_controls_tab(
            $section_id.'_normal_style_tab',
            ['label' => esc_html__( 'NORMAL', 'selection' )]
        );

        if ( $style_opts['include_color'] ) {
            $this->add_control(
                $section_id . '_normal_text_color',
                [
                    'label' => esc_html__($style_opts['color_prefix'].'Color', 'selection'),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => $style_opts['color_scheme_default'],
                    ],
                    'default' => $style_opts['color_default'],
                    'selectors' => [
                        "{{WRAPPER}} .".$style_opts['color_class'] => 'color: {{VALUE}} !important;',
                    ],
                ]
            );

        }

        if ( $style_opts['additional_color'] ) {
            $this->add_control(
                $section_id . '_' . $style_opts['additional_color_name'] . '_normal_text_color',
                [
                    'label' => esc_html__( $style_opts['additional_color_name'], 'selection' ),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => $style_opts['additional_color_scheme_default'],
                    ],
                    'default' => $style_opts['additional_color_default'],
                    'selectors' => [
                        "{{WRAPPER}} .".$style_opts['additional_color_class'] => 'color: {{VALUE}} !important;',

                    ],
                ]
            );
        }

        if ( $style_opts['include_bg'] ) {

            $this->add_group_control(
                Group_Control_Background::get_type(),
                [
                    'name' => $section_id . '_normal_background',
                    'label' => esc_html__('Background type', 'selection'),
                    'types' => ['classic', 'gradient', 'video'],
                    'selector' => "{{WRAPPER}} .".$style_opts['html_class'] . $style_opts['additional_html_selector'],
                ]
            );

        }

        if ( $style_opts['include_css_filters'] ) {
            $this->add_group_control(
                Group_Control_Css_Filter::get_type(),
                [
                    'name' => 'css_filters_normal',
                    'selector' => '{{WRAPPER}} .'.$style_opts['html_class'],
                ]
            );
        }

        $this->add_control(
            $section_id . '_separate_normal',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => $section_id.'_border_normal',
                'label' => esc_html__( 'Border Type', 'selection' ),
                'selector' => "{{WRAPPER}} .".$style_opts['html_class'] . $style_opts['additional_html_selector'],
            ]
        );

        $this->add_responsive_control(
            $section_id.'_border_radius_normal',
            [
                'label' => esc_html__( 'Border radius', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    "{{WRAPPER}} .".$style_opts['html_class'] . $style_opts['additional_html_selector'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                    $style_opts['additional_border_radius_selector'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};'
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => $section_id.'_box_shadow_normal',
                'label' => esc_html__( 'Box Shadow', 'selection' ),
                'selector' => "{{WRAPPER}} .".$style_opts['html_class'] . $style_opts['additional_html_selector'],
            ]
        );


        $this->end_controls_tab();

        $this->start_controls_tab(
            $section_id.'_hover_style_tab',
            ['label' => esc_html__( 'HOVER', 'selection' )]
        );

        if ( $style_opts['include_color'] ) {
            $this->add_control(
                $section_id . '_hover_color',
                [
                    'label' => esc_html__($style_opts['color_prefix'].'Color', 'selection'),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => $style_opts['color_scheme_default'],
                    ],
                    'default' => $style_opts['color_default'],
                    'selectors' => [
                        "{{WRAPPER}} .".$style_opts['color_hover_class'] . $style_opts['additional_html_hover_selector'] => 'color: {{VALUE}} !important;',
                    ],
                ]
            );
        }

        if ( $style_opts['additional_color_hover'] ) {
            $this->add_control(
                $section_id . '_' . $style_opts['additional_color_name'] . '_hover_text_color',
                [
                    'label' => esc_html__( $style_opts['additional_color_name'], 'selection'),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => $style_opts['additional_color_scheme_default'],
                    ],
                    'default' => $style_opts['additional_color_default'],
                    'selectors' => [
                        $style_opts['additional_color_hover_selectors'] => 'color: {{VAlUE}} !important'
                    ],
                ]
            );
        }

        if ( $style_opts['include_bg'] ) {
            $this->add_group_control(
                Group_Control_Background::get_type(),
                [
                    'name' => $section_id . '_background_hover',
                    'label' => esc_html__('Background type', 'selection'),
                    'types' => ['classic', 'gradient', 'video'],
                    'selector' => "{{WRAPPER}} .".$style_opts['html_class'].":hover" . $style_opts['additional_html_hover_selector'],
                ]
            );
        }

        if ( $style_opts['include_css_filters'] ) {
            $this->add_group_control(
                Group_Control_Css_Filter::get_type(),
                [
                    'name' => 'css_filters_hover',
                    'selector' => '{{WRAPPER}} .'.$style_opts['html_class'].':hover' . $style_opts['additional_html_hover_selector'],
                ]
            );
        }

        if ( $style_opts['include_transition'] ) {
            $this->add_control(
                $section_id.'_hover_transition',
                [
                    'label' => esc_html__( 'Hover transition duration', 'selection' ),
                    'type' => Controls_Manager::SLIDER,
                    'size_units' => [ 's' ],
                    'range' => [
                        's' => [
                            'min' => 0.1,
                            'max' => 5,
                            'step' => 0.1,
                        ],
                    ],
                    'default' => [
                        'unit' => 's',
                        'size' => 0,
                    ],
                    'selectors' => [
                        '{{WRAPPER}} .'.$style_opts['html_class'] => 'transition: color {{SIZE}}{{UNIT}}, background {{SIZE}}{{UNIT}}, box-shadow {{SIZE}}{{UNIT}}, border-radius {{SIZE}}{{UNIT}}, border {{SIZE}}{{UNIT}}, filter {{SIZE}}{{UNIT}}, stroke {{SIZE}}{{UNIT}};',
                        $style_opts['additional_transition_selector'] => 'transition: color {{SIZE}}{{UNIT}}, background {{SIZE}}{{UNIT}}, box-shadow {{SIZE}}{{UNIT}}, border-radius {{SIZE}}{{UNIT}}, border {{SIZE}}{{UNIT}}, filter {{SIZE}}{{UNIT}}, stroke {{SIZE}}{{UNIT}};;'
                    ],
                ]
            );
        }

        $this->add_control(
            $section_id.'_separate_hover',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => $section_id.'_border_hover',
                'label' => esc_html__( 'Border Type', 'selection' ),
                'selector' => "{{WRAPPER}} .".$style_opts['html_class'].":hover" . $style_opts['additional_html_hover_selector'],
            ]
        );

        $this->add_responsive_control(
            $section_id.'_border_radius_hover',
            [
                'label' => esc_html__( 'Border radius', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    "{{WRAPPER}} .".$style_opts['html_class'].":hover" . $style_opts['additional_html_hover_selector'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                    $style_opts['additional_border_radius_hover_selector'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => $section_id.'_box_shadow_hover',
                'label' => esc_html__( 'Box Shadow', 'selection' ),
                'selector' => "{{WRAPPER}} .".$style_opts['html_class'].":hover" . $style_opts['additional_html_hover_selector'],
            ]
        );

        $this->end_controls_tab();

        if ( $style_opts['include_active_tab'] ) {

            $this->start_controls_tab(
                $section_id . '_focus_style_tab',
                ['label' => esc_html__( $style_opts['active_tab_name'], 'selection' )]
            );

            if ( $style_opts['include_color'] ) {
                $this->add_control(
                    $section_id . '_active_color',
                    [
                        'label' => esc_html__( $style_opts['color_prefix'] . 'Color', 'selection' ),
                        'type' => Controls_Manager::COLOR,
                        'scheme' => [
                            'type' => Color::get_type(),
                            'value' => $style_opts['color_scheme_default'],
                        ],
                        'selectors' => [
                            "{{WRAPPER}} ." . $style_opts['color_active_class'] => 'color: {{VALUE}} !important;',
                        ],
                    ]
                );
            }

            if ( $style_opts['additional_color'] ) {
                $this->add_control(
                    $section_id . '_' . $style_opts['additional_color_name'] . '_active_text_color',
                    [
                        'label' => esc_html__( $style_opts['additional_color_name'], 'selection' ),
                        'type' => Controls_Manager::COLOR,
                        'scheme' => [
                            'type' => Color::get_type(),
                            'value' => $style_opts['additional_color_scheme_default'],
                        ],
                        'selectors' => [
                            "{{WRAPPER}} ." . $style_opts['additional_color_active_class'] => 'color: {{VALUE}} !important;',
                        ],
                    ]
                );
            }

            if ( $style_opts['include_bg'] ) {
                $this->add_group_control(
                    Group_Control_Background::get_type(),
                    [
                        'name' => $section_id . '_background_active',
                        'label' => esc_html__( 'Background type', 'selection' ),
                        'types' => ['classic', 'gradient', 'video'],
                        'selector' => "{{WRAPPER}} ." . $style_opts['active_class'],
                    ]
                );
            }

            $this->add_control(
                $section_id . '_separate_active',
                [
                    'type' => Controls_Manager::DIVIDER,
                ]
            );

            $this->add_group_control(
                Group_Control_Border::get_type(),
                [
                    'name' => $section_id . '_border_active',
                    'label' => esc_html__( 'Border Type', 'selection' ),
                    'selector' => "{{WRAPPER}} ." . $style_opts['active_class'],
                ]
            );

            $this->add_responsive_control(
                $section_id . '_border_radius_active',
                [
                    'label' => esc_html__( 'Border radius', 'selection' ),
                    'type' => Controls_Manager::DIMENSIONS,
                    'size_units' => ['px', '%', 'em'],
                    'selectors' => [
                        "{{WRAPPER}} ." . $style_opts['active_class'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                    ],
                ]
            );

            $this->add_group_control(
                Group_Control_Box_Shadow::get_type(),
                [
                    'name' => $section_id . '_box_shadow_active',
                    'label' => esc_html__( 'Box Shadow', 'selection' ),
                    'selector' => "{{WRAPPER}} ." . $style_opts['active_class'],
                ]
            );

            $this->end_controls_tab();

        }


        $this->end_controls_tabs();
    }

    /**
     * Add widget controls: Style -> Section Style Product Thumbnail Cell.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_product_thumbnail_cell() {

        $this->start_controls_section( 'section_style_product_thumbnail_cell', [
            'label' => esc_html__( 'Product thumbnail cell', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->add_responsive_control(
            'section_style_product_thumbnail_cell_padding',
            [
                'label' => esc_html__( 'Padding', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'selectors' => [
                    "{{WRAPPER}} .woocommerce-page .woocommerce-cart-form table.shop_table_responsive td.product-thumbnail" => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->generate_typography_tabs_controls( 'section_style_product_thumbnail_cell', [
            'html_class' => 'mdp-carter-elementor-box td.product-thumbnail',
            'include_typography' => false,
            'include_color' => false,
        ] );


        $this->end_controls_section();

    }


    /**
     * Add widget controls: Style -> Section Style Product Thumbnail.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_product_thumbnail() {

        $this->start_controls_section( 'section_style_product_thumbnail', [
            'label' => esc_html__( 'Product thumbnail', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->add_control(
            'section_style_product_thumbnail_alignment',
            [
                'label' => esc_html__( 'Alignment', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'flex-start' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'eicon-text-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'eicon-text-align-center',
                    ],
                    'flex-end' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'eicon-text-align-right',
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} td.product-thumbnail a' => 'justify-content: {{VALUE}};'
                ],
                'default' => 'center',
                'toggle' => true,
            ]
        );


        $this->add_responsive_control(
            'section_style_product_thumbnail_padding',
            [
                'label' => esc_html__( 'Padding', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'selectors' => [
                    "{{WRAPPER}} td.product-thumbnail img" => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                    "{{WRAPPER}} td.product-thumbnail a" => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};'
                ],
            ]
        );

        $this->add_responsive_control(
            'section_style_product_thumbnail_width',
            [
                'label' => esc_html__( 'Image size', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 100,
                        'step' => 1,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} td.product-thumbnail img' => 'width: {{SIZE}}{{UNIT}} !important; height: {{SIZE}}{{UNIT}} !important;',
                    '{{WRAPPER}} td.product-thumbnail' => 'width: {{SIZE}}{{UNIT}} !important; height: {{SIZE}}{{UNIT}} !important;',
                ],
            ]
        );

        $this->generate_typography_tabs_controls( 'section_style_product_thumbnail', [
            'html_class' => 'mdp-carter-elementor-box td.product-thumbnail img',
            'include_typography' => false,
            'include_color' => false,
        ] );


        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Product Title Heading.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_product_title_heading() {

        $this->start_controls_section( 'section_style_product_title_heading', [
            'label' => esc_html__( 'Product title heading', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->add_control(
            'section_style_product_title_heading_alignment',
            [
                'label' => esc_html__( 'Alignment', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'eicon-text-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'eicon-text-align-center',
                    ],
                    'right' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'eicon-text-align-right',
                    ],
                ],
                'separator' => 'after',
                'selectors' => [
                    '{{WRAPPER}} th.product-name, {{WRAPPER}} td.product-name:before' => 'text-align: {{VALUE}};'
                ],
                'default' => 'center',
                'toggle' => true,
            ]
        );

        $this->add_responsive_control(
            'section_style_product_title_heading_padding',
            [
                'label' => esc_html__( 'Padding', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'selectors' => [
                    "{{WRAPPER}} th.product-name, {{WRAPPER}} td.product-name:before" => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->generate_typography_tabs_controls( 'section_style_product_title_heading', [
            'html_class' => 'mdp-carter-elementor-box th.product-name',
            'additional_html_selector' => ', {{WRAPPER}} .mdp-carter-elementor-box td.product-name:before',
            'additional_html_hover_selector' => ', {{WRAPPER}} .mdp-carter-elementor-box td.product-name:hover:before',
            'include_typography' => true,
            'include_color' => true,
            'typography_class' => 'mdp-carter-elementor-box th.product-name, {{WRAPPER}} td.product-name:before',
            'color_class' => 'mdp-carter-elementor-box th.product-name, {{WRAPPER}} td.product-name:before',
            'color_hover_class' => 'mdp-carter-elementor-box th.product-name:hover, {{WRAPPER}} td.product-name:hover:before',
            'additional_transition_selector' => '{{WRAPPER}} th.product-name, {{WRAPPER}} td.product-name:before'
        ] );


        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Product Title.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_product_title() {

        $this->start_controls_section( 'section_style_product_title', [
            'label' => esc_html__( 'Product title', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->add_control(
            'section_style_product_title_alignment',
            [
                'label' => esc_html__( 'Alignment', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'eicon-text-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'eicon-text-align-center',
                    ],
                    'right' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'eicon-text-align-right',
                    ],
                ],
                'separator' => 'after',
                'selectors' => [
                    '{{WRAPPER}} td.product-name' => 'text-align: {{VALUE}};'
                ],
                'default' => 'center',
                'toggle' => true,
            ]
        );

        $this->add_responsive_control(
            'section_style_product_title_padding',
            [
                'label' => esc_html__( 'Padding', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'selectors' => [
                    "{{WRAPPER}} td.product-name" => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );


        $this->add_responsive_control(
            'section_style_product_title_width',
            [
                'label' => esc_html__( 'Width', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 500,
                        'step' => 1,
                    ],
                ],
                'separator' => 'before',
                'selectors' => [
                    '{{WRAPPER}} td.product-name' => 'width: {{SIZE}}{{UNIT}} !important;',
                ],
            ]
        );

        $this->generate_typography_tabs_controls( 'section_style_product_title', [
            'html_class' => 'mdp-carter-elementor-box td.product-name',
            'include_typography' => true,
            'include_color' => true,
            'additional_color' => true,
            'additional_color_name' => 'Heading color',
            'typography_class' => 'mdp-carter-elementor-box td.product-name a',
            'color_class' => 'mdp-carter-elementor-box td.product-name a',
            'color_hover_class' => 'mdp-carter-elementor-box td.product-name:hover a',
            'additional_transition_selector' => '{{WRAPPER}} td.product-name a'
        ] );


        $this->end_controls_section();

    }


    /**
     * Add widget controls: Style -> Section Style Product Price Heading.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_product_price_heading() {

        $this->start_controls_section( 'section_style_product_price_heading', [
            'label' => esc_html__( 'Product price heading', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->add_control(
            'section_style_product_price_heading_alignment',
            [
                'label' => esc_html__( 'Alignment', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'eicon-text-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'eicon-text-align-center',
                    ],
                    'right' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'eicon-text-align-right',
                    ],
                ],
                'separator' => 'after',
                'selectors' => [
                    '{{WRAPPER}} th.product-price, {{WRAPPER}} td.product-price:before' => 'text-align: {{VALUE}};'
                ],
                'default' => 'center',
                'toggle' => true,
            ]
        );

        $this->add_responsive_control(
            'section_style_product_price_heading_padding',
            [
                'label' => esc_html__( 'Padding', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'selectors' => [
                    "{{WRAPPER}} th.product-price, {{WRAPPER}} td.product-price:before" => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->generate_typography_tabs_controls( 'section_style_product_price_heading', [
            'html_class' => 'mdp-carter-elementor-box th.product-price',
            'additional_html_selector' => ', {{WRAPPER}} .mdp-carter-elementor-box td.product-price:before',
            'additional_html_hover_selector' => ', {{WRAPPER}} .mdp-carter-elementor-box td.product-price:hover:before',
            'include_typography' => true,
            'include_color' => true,
            'typography_class' => 'mdp-carter-elementor-box th.product-price, {{WRAPPER}} td.product-price:before',
            'color_class' => 'mdp-carter-elementor-box th.product-price, {{WRAPPER}} td.product-price:before',
            'color_hover_class' => 'mdp-carter-elementor-box th.product-price:hover, {{WRAPPER}} td.product-price:hover:before',
            'additional_transition_selector' => '{{WRAPPER}} th.product-price, {{WRAPPER}} td.product-price:before'
        ] );


        $this->end_controls_section();

    }

  /**
     * Add widget controls: Style -> Section Style Product Price.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_product_price() {

        $this->start_controls_section( 'section_style_product_price', [
            'label' => esc_html__( 'Product price', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->add_control(
            'section_style_product_price_alignment',
            [
                'label' => esc_html__( 'Alignment', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'eicon-text-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'eicon-text-align-center',
                    ],
                    'right' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'eicon-text-align-right',
                    ],
                ],
                'separator' => 'after',
                'selectors' => [
                    '{{WRAPPER}} td.product-price' => 'text-align: {{VALUE}};'
                ],
                'default' => 'center',
                'toggle' => true,
            ]
        );

        $this->add_responsive_control(
            'section_style_product_price_padding',
            [
                'label' => esc_html__( 'Padding', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'selectors' => [
                    "{{WRAPPER}} td.product-price" => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_responsive_control(
            'section_style_product_price_width',
            [
                'label' => esc_html__( 'Width', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 500,
                        'step' => 1,
                    ],
                ],
                'separator' => 'before',
                'selectors' => [
                    '{{WRAPPER}} td.product-price' => 'width: {{SIZE}}{{UNIT}} !important;',
                ],
            ]
        );

        $this->generate_typography_tabs_controls( 'section_style_product_price', [
            'html_class' => 'mdp-carter-elementor-box td.product-price',
            'include_typography' => true,
            'include_color' => true,
            'typography_class' => 'mdp-carter-elementor-box td.product-price span',
            'color_class' => 'mdp-carter-elementor-box td.product-price span',
            'color_hover_class' => 'mdp-carter-elementor-box td.product-price:hover span',
            'additional_transition_selector' => '{{WRAPPER}} td.product-price span'
        ] );


        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Product Quantity.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_product_quantity_heading() {

        $this->start_controls_section( 'section_style_product_quantity_heading', [
            'label' => esc_html__( 'Product quantity heading', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->add_control(
            'section_style_product_quantity_heading_alignment',
            [
                'label' => esc_html__( 'Alignment', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'eicon-text-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'eicon-text-align-center',
                    ],
                    'right' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'eicon-text-align-right',
                    ],
                ],
                'separator' => 'after',
                'selectors' => [
                    '{{WRAPPER}} th.product-quantity, {{WRAPPER}} td.product-quantity:before' => 'text-align: {{VALUE}};'
                ],
                'default' => 'center',
                'toggle' => true,
            ]
        );

        $this->add_responsive_control(
            'section_style_product_quantity_heading_padding',
            [
                'label' => esc_html__( 'Padding', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'selectors' => [
                    "{{WRAPPER}} th.product-quantity, {{WRAPPER}} td.product-quantity:before" => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->generate_typography_tabs_controls( 'section_style_product_quantity_heading', [
            'html_class' => 'mdp-carter-elementor-box th.product-quantity',
            'additional_html_selector' => ', {{WRAPPER}} .mdp-carter-elementor-box td.product-quantity:before',
            'additional_html_hover_selector' => ', {{WRAPPER}} .mdp-carter-elementor-box td.product-quantity:hover:before',
            'include_typography' => true,
            'include_color' => true,
            'typography_class' => 'mdp-carter-elementor-box th.product-quantity, {{WRAPPER}} td.product-quantity:before',
            'color_class' => 'mdp-carter-elementor-box th.product-quantity, {{WRAPPER}} td.product-quantity:before',
            'color_hover_class' => 'mdp-carter-elementor-box th.product-quantity:hover, {{WRAPPER}} td.product-quantity:hover:before',
            'additional_transition_selector' => '{{WRAPPER}} th.product-quantity, {{WRAPPER}} td.product-quantity:before'
        ] );


        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Product Quantity.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_product_quantity() {

        $this->start_controls_section( 'section_style_product_quantity', [
            'label' => esc_html__( 'Product quantity', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->add_control(
            'section_style_product_quantity_alignment',
            [
                'label' => esc_html__( 'Alignment', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'eicon-text-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'eicon-text-align-center',
                    ],
                    'right' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'eicon-text-align-right',
                    ],
                ],
                'separator' => 'after',
                'selectors' => [
                    '{{WRAPPER}} td.product-quantity' => 'text-align: {{VALUE}};'
                ],
                'default' => 'center',
                'toggle' => true,
            ]
        );

        $this->add_responsive_control(
            'section_style_product_quantity_padding',
            [
                'label' => esc_html__( 'Padding', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'selectors' => [
                    "{{WRAPPER}} td.product-quantity" => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_responsive_control(
            'section_style_product_quantity_width',
            [
                'label' => esc_html__( 'Width', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 500,
                        'step' => 1,
                    ],
                ],
                'separator' => 'before',
                'selectors' => [
                    '{{WRAPPER}} td.product-quantity' => 'width: {{SIZE}}{{UNIT}} !important;',
                ],
            ]
        );

        $this->generate_typography_tabs_controls( 'section_style_product_quantity', [
            'html_class' => 'mdp-carter-elementor-box td.product-quantity',
            'include_typography' => true,
            'include_color' => false,
            'typography_class' => 'mdp-carter-elementor-box td.product-quantity input',
            'additional_transition_selector' => '{{WRAPPER}} td.product-quantity input'
        ] );


        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Product Subtotal Heading.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_product_subtotal_heading() {

        $this->start_controls_section( 'section_style_product_subtotal_heading', [
            'label' => esc_html__( 'Product subtotal heading', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->add_control(
            'section_style_product_subtotal_heading_alignment',
            [
                'label' => esc_html__( 'Alignment', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'eicon-text-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'eicon-text-align-center',
                    ],
                    'right' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'eicon-text-align-right',
                    ],
                ],
                'separator' => 'after',
                'selectors' => [
                    '{{WRAPPER}} th.product-subtotal, {{WRAPPER}} td.product-subtotal:before' => 'text-align: {{VALUE}};'
                ],
                'default' => 'center',
                'toggle' => true,
            ]
        );

        $this->add_responsive_control(
            'section_style_product_subtotal_heading_padding',
            [
                'label' => esc_html__( 'Padding', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'selectors' => [
                    "{{WRAPPER}} th.product-subtotal, {{WRAPPER}} td.product-subtotal:before" => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->generate_typography_tabs_controls( 'section_style_product_subtotal_heading', [
            'html_class' => 'mdp-carter-elementor-box th.product-subtotal',
            'additional_html_selector' => ', {{WRAPPER}} .mdp-carter-elementor-box td.product-subtotal:before',
            'additional_html_hover_selector' => ', {{WRAPPER}} .mdp-carter-elementor-box td.product-subtotal:hover:before',
            'include_typography' => true,
            'include_color' => true,
            'typography_class' => 'mdp-carter-elementor-box th.product-subtotal, {{WRAPPER}} td.product-subtotal:before',
            'color_class' => 'mdp-carter-elementor-box th.product-subtotal, {{WRAPPER}} td.product-subtotal:before',
            'color_hover_class' => 'mdp-carter-elementor-box th.product-subtotal:hover, {{WRAPPER}} td.product-subtotal:hover:before',
            'additional_transition_selector' => '{{WRAPPER}} th.product-subtotal, {{WRAPPER}} td.product-subtotal:before'
        ] );


        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Product Subtotal.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_product_subtotal() {

        $this->start_controls_section( 'section_style_product_subtotal', [
            'label' => esc_html__( 'Product subtotal', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->add_control(
            'section_style_product_subtotal_alignment',
            [
                'label' => esc_html__( 'Alignment', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'eicon-text-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'eicon-text-align-center',
                    ],
                    'right' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'eicon-text-align-right',
                    ],
                ],
                'separator' => 'after',
                'selectors' => [
                    '{{WRAPPER}} td.product-subtotal' => 'text-align: {{VALUE}};'
                ],
                'default' => 'center',
                'toggle' => true,
            ]
        );

        $this->add_responsive_control(
            'section_style_product_subtotal_padding',
            [
                'label' => esc_html__( 'Padding', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'selectors' => [
                    "{{WRAPPER}} td.product-subtotal" => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_responsive_control(
            'section_style_product_subtotal_width',
            [
                'label' => esc_html__( 'Width', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 500,
                        'step' => 1,
                    ],
                ],
                'separator' => 'before',
                'selectors' => [
                    '{{WRAPPER}} td.product-subtotal' => 'width: {{SIZE}}{{UNIT}} !important;',
                ],
            ]
        );

        $this->generate_typography_tabs_controls( 'section_style_product_subtotal', [
            'html_class' => 'mdp-carter-elementor-box td.product-subtotal',
            'include_typography' => true,
            'include_color' => true,
            'typography_class' => 'mdp-carter-elementor-box td.product-subtotal span',
            'color_class' => 'mdp-carter-elementor-box td.product-subtotal span',
            'color_hover_class' => 'mdp-carter-elementor-box td.product-subtotal:hover span',
            'additional_transition_selector' => '{{WRAPPER}} td.product-subtotal span'
        ] );


        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Product Remove.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_product_remove() {

        $this->start_controls_section( 'section_style_product_remove', [
            'label' => esc_html__( 'Product remove', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->add_control(
            'section_style_product_remove_alignment',
            [
                'label' => esc_html__( 'Alignment', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'eicon-text-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'eicon-text-align-center',
                    ],
                    'right' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'eicon-text-align-right',
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} td.product-remove' => 'text-align: {{VALUE}};'
                ],
                'separator' => 'after',
                'default' => 'center',
                'toggle' => true,
            ]
        );

        $this->add_responsive_control(
            'section_style_product_remove_padding',
            [
                'label' => esc_html__( 'Padding', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'selectors' => [
                    "{{WRAPPER}} td.product-remove" => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_responsive_control(
            'section_style_product_remove_width',
            [
                'label' => esc_html__( 'Width', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 500,
                        'step' => 1,
                    ],
                ],
                'separator' => 'before',
                'selectors' => [
                    '{{WRAPPER}} td.product-remove' => 'width: {{SIZE}}{{UNIT}} !important;',
                ],
            ]
        );

        $this->generate_typography_tabs_controls( 'section_style_product_remove', [
            'html_class' => 'mdp-carter-elementor-box td.product-remove',
            'include_typography' => true,
            'include_color' => true,
            'additional_color' => true,
            'additional_color_name' => 'Heading color',
            'typography_class' => 'mdp-carter-elementor-box td.product-remove a',
            'color_class' => 'mdp-carter-elementor-box td.product-remove a',
            'color_hover_class' => 'mdp-carter-elementor-box td.product-remove:hover a',
            'additional_transition_selector' => '{{WRAPPER}} td.product-remove a'
        ] );


        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Total.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_total() {

        $this->start_controls_section( 'section_style_total', [
            'label' => esc_html__( 'Total', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE,
            'condition' => [
                'show_total' => 'yes'
            ]
        ] );

        $this->generate_margin_padding_controls(
            'section_style_total',
            'cart_totals'
        );

        $this->add_responsive_control(
         'section_style_total_button_width',
            [
                'label' => esc_html__( 'Width', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 3000,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 0,
                        'max' => 100,
                        'step' => 1
                    ]
                ],
                'condition' => [
                    'cart_layout' => 'one-column'
                ],
                'selectors' => [
                    '{{WRAPPER}} .cart_totals' => 'width: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->generate_typography_tabs_controls( 'section_style_total', [
            'html_class' => 'cart_totals',
            'include_typography' => false,
            'include_color' => false,
        ] );


        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Total Title.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_total_title() {

        $this->start_controls_section( 'section_style_total_title', [
            'label' => esc_html__( 'Total title', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE,
            'condition' => [
                'show_total' => 'yes'
            ]
        ] );

        $this->generate_margin_padding_controls(
            'section_style_total_title',
            'cart_totals h2'
        );

        $this->generate_typography_tabs_controls( 'section_style_total_title', [
            'html_class' => 'cart_totals h2',
            'include_typography' => true,
            'include_color' => true,
            'typography_class' => 'cart_totals h2',
            'color_class' => 'cart_totals h2',
            'color_hover_class' => 'cart_totals:hover h2',
        ] );


        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Coupon.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_coupon() {

        $this->start_controls_section( 'section_style_coupon', [
            'label' => esc_html__( 'Coupon block', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE,
            'condition' => [
                'show_coupon' => 'yes'
            ]
        ] );

        $this->generate_margin_padding_controls(
                'section_style_coupon',
                'mdp-carter-elementor-coupon-block'
        );

        $this->generate_typography_tabs_controls( 'section_style_coupon', [
            'html_class' => 'mdp-carter-elementor-coupon-block',
            'include_typography' => false,
            'include_color' => false,
        ] );


        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Product Table.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_product_table() {

        $this->start_controls_section( 'section_style_product_table', [
            'label' => esc_html__( 'Product table', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE,
        ] );

        $this->generate_margin_padding_controls(
                'section_style_product_table',
                'mdp-carter-elementor-cart-product-table'
        );

        $this->generate_typography_tabs_controls( 'section_style_product_table', [
            'html_class' => 'mdp-carter-elementor-cart-product-table',
            'include_typography' => false,
            'include_color' => false,
        ] );


        $this->end_controls_section();

    }


    /**
     * Create Shipping Parts styles
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function create_total_part_style_settings( $section_id, $section_name, $part_class, $show_condition, $additional_selector = '', $additional_hover_selector = '' ) {
        $this->start_controls_section( $section_id, [
            'label' => esc_html__( $section_name, 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE,
            'condition' => [
                $show_condition => 'yes',
                'show_total' => 'yes'
            ]
        ] );

        $this->add_responsive_control(
            $section_id . '_padding',
            [
                'label' => esc_html__( 'Padding', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'selectors' => [
                    "{{WRAPPER}} ." . $part_class . $additional_selector => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->generate_typography_tabs_controls( $section_id, [
            'html_class' => $part_class,
            'additional_html_selector' => $additional_selector,
            'additional_html_hover_selector' => $additional_hover_selector,
            'include_typography' => true,
            'include_color' => true,
            'typography_class' => $part_class . $additional_selector,
            'color_class' => $part_class . $additional_selector,
            'color_hover_class' => $part_class . ':hover' . $additional_hover_selector,
            'additional_transition_selector' => '{{WRAPPER}} .' . $part_class . $additional_selector
        ] );

        $this->end_controls_section();
    }

    /**
     * Create styles settings for inputs.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function create_input_style_settings( $section_id, $section_name, $input_class, $show_condition, $default_width = '' ) {
        $section_settings = [
                'label' => esc_html__( $section_name, 'selection' ),
                'tab'   => Controls_Manager::TAB_STYLE
        ];

        if ( $show_condition !== '' ) {
            $section_settings['condition'] = [ $show_condition => 'yes' ];
        }

        $this->start_controls_section( $section_id, $section_settings );

        $this->generate_margin_padding_controls( $section_id, $input_class );

        $width_settings = [
            'label' => esc_html__( 'Width', 'selection' ),
            'type' => Controls_Manager::SLIDER,
            'size_units' => [ 'px', '%' ],
            'range' => [
                'px' => [
                    'min' => 0,
                    'max' => 500,
                    'step' => 1,
                ],
                '%' => [
                    'min' => 0,
                    'max' => 100,
                ],
            ],
            'selectors' => [
                '{{WRAPPER}} .' . $input_class => 'width: {{SIZE}}{{UNIT}};',
            ],
        ];

        if ( $default_width !== '' ) {
            $width_settings['default'] = [
                'unit' => 'px',
                'size' => $default_width
            ];
        }

        $this->add_control($section_id . '_width', $width_settings );

        $this->generate_typography_tabs_controls( $section_id, [
            'html_class' => $input_class,
            'include_typography' => true,
            'include_active_tab' => true,
            'include_color' => true,
            'active_tab_name' => 'FOCUS',
            'active_class' => $input_class . ':focus',
            'color_active_class' => $input_class . ':focus',
            'typography_class' => $input_class,
            'color_class' => $input_class,
            'color_hover_class' => $input_class . ':hover',
            'additional_transition_selector' => '{{WRAPPER}} ' . $input_class
        ] );


        $this->end_controls_section();
    }

    /**
     * Create style controls for buttons with icons.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function create_buttons_with_icons_styles( $section_id, $section_name, $button_class, $show_condition, $show_icon_condition ) {

        $section_settings = [
            'label' => esc_html__( $section_name, 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE,
        ];

        if ( $show_condition !== '' ) {
            $section_settings['condition'] = [ $show_condition => 'yes' ];
        }

        $this->start_controls_section( $section_id, [
            'label' => esc_html__( $section_name, 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE,
            $section_settings
        ] );

        $this->generate_margin_padding_controls(  $section_id, $button_class );

        $this->add_control(
            $section_id . '_icon_size',
            [
                'label' => esc_html__( 'Icon size', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 500,
                        'step' => 1,
                    ]
                ],
                'condition' => [
                    $show_icon_condition => 'yes'
                ],
                'selectors' => [
                    '{{WRAPPER}} .' . $button_class . ' .mdp-carter-elementor-btn-icon' => 'height: {{SIZE}}{{UNIT}}; width: {{SIZE}}{{UNIT}};',
                    '{{WRAPPER}} .' . $button_class . ' .mdp-carter-elementor-btn-icon i' => 'font-size: {{SIZE}}{{UNIT}};',
                    '{{WRAPPER}} .' . $button_class . ' .mdp-carter-elementor-btn-icon svg' => 'height: {{SIZE}}{{UNIT}}; width: {{SIZE}}{{UNIT}};'
                ],
            ]
        );

        $this->add_responsive_control(
            $section_id . '_icon_spacing',
            [
                'label' => esc_html__( 'Icon spacing', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'condition' => [
                    $show_icon_condition => 'yes'
                ],
                'separator' => 'after',
                'selectors' => [
                    "{{WRAPPER}} ." . $button_class . " .mdp-carter-elementor-btn-icon" => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_responsive_control(
            $section_id . '_button_width',
            [
                'label' => esc_html__( 'Button width', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 500,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 1,
                        'max' => 100,
                        'step' => 1
                    ]
                ],
                'selectors' => [
                    '{{WRAPPER}} .' . $button_class => 'width: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->generate_typography_tabs_controls( $section_id, [
            'html_class' => $button_class,
            'include_typography' => true,
            'include_color' => true,
            'additional_color' => true,
            'additional_color_name' => 'Icon color',
            'typography_class' => $button_class,
            'color_class' => $button_class,
            'color_default' => '#fff',
            'additional_color_default' => '#fff',
            'color_hover_class' => $button_class . ':hover',
            'additional_color_class' => $button_class . ' .mdp-carter-elementor-btn-icon',
            'additional_color_hover' => true,
            'additional_color_hover_selectors' => '{{WRAPPER}} .'. $button_class . ':hover .mdp-carter-elementor-btn-icon',
            'additional_transition_selector' => '{{WRAPPER}} .' .$button_class . ' .mdp-carter-elementor-btn-icon'
        ] );


        $this->end_controls_section();

    }

    /**
     * Create style controls for buttons without icons.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function create_buttons_controls( $section_id, $section_name, $button_class, $show_condition = '' ) {
        $this->start_controls_section( $section_id, [
            'label' => esc_html__( $section_name, 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE,
            'condition' => [
                $show_condition => 'yes'
            ]
        ] );

        $this->generate_margin_padding_controls(  $section_id, $button_class );

        $this->add_responsive_control(
            $section_id . '_button_width',
            [
                'label' => esc_html__( 'Button width', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 500,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 1,
                        'max' => 100,
                        'step' => 1
                    ]
                ],
                'selectors' => [
                    '{{WRAPPER}} .' . $button_class => 'width: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->generate_typography_tabs_controls( $section_id, [
            'html_class' => $button_class,
            'include_typography' => true,
            'include_color' => true,
            'typography_class' => $button_class,
            'color_class' => $button_class,
            'color_default' => '#fff',
            'additional_color_default' => '#fff',
            'color_hover_class' => $button_class . ':hover',
            'additional_transition_selector' => '{{WRAPPER}} .' .$button_class . ' .mdp-carter-elementor-btn-icon'
        ] );


        $this->end_controls_section();
    }


    /**
     * Callback for woocommerce_after_cart_table hook.
     *
     * @return string
     * @since 1.0.0
     *
     */
    private function create_coupon_block() {
        $settings = $this->get_settings_for_display();

        $button_text = $settings['coupon_button_text'] !== '' ?
            esc_html__( $settings['coupon_button_text'], 'woocommerce' ) :
            esc_html__( 'Apply coupon', 'woocommerce' );

        $button_icon = sprintf(
            '<span class="mdp-carter-elementor-btn-icon">%s</span>' ,
            $this->create_button_icon( $settings['coupon_button_icon'] )
        );

        $button_icon_position_class = sprintf(
            'mdp-carter-elementor-btn-icon-wrapper mdp-carter-elementor-btn-icon-position-%s',
            esc_attr( $settings['coupon_button_icon_position'] )
        );

        if ( $settings['show_coupon'] === 'yes' ) {
            ?>
            <div class="mdp-carter-elementor-coupon-block coupon">
                <input type="text" name="coupon_code" class="input-text" id="coupon_code" placeholder="Coupon code">
                <button type="submit"
                        class="button elementor-animation-<?php echo esc_attr( $settings['coupon_hover_animation'] ); ?>"
                        name="apply_coupon"
                        value="Apply coupon">
                    <span class="mdp-carter-elementor-btn-content <?php echo $button_icon_position_class ?>">
                        <?php
                            echo $button_icon;
                            echo $button_text;
                        ?>
                    </span>
                </button>
            </div>
            <?php
            do_action( 'woocommerce_cart_coupon' );
        } else {
            return '';
        }
    }

    /**
     * Callback for woocommerce_after_cart_table hook.
     *
     * @return string
     * @since 1.0.0
     *
     */
    public function after_cart_table() {
        return $this->create_coupon_block();
    }


    /**
     * Hide coupon field.
     *
     * @return false
     * @since 1.0.0
     *
     */
    public function show_coupon_field() {
        return false;
    }

    /**
     * Callback for woocommerce_before_cart hook.
     *
     * @return void
     * @since 1.0.0
     *
     */
    public function before_cart_start() {
       $settings = $this->get_settings_for_display();

       echo sprintf(
           '<div class="mdp-carter-elementor-cart-container mdp-elementor-cart-content-layout-%s">
                    <div class="mdp-carter-elementor-cart-product-table">',
                esc_attr( $settings['cart_layout'] )

       );
    }

    /**
     * Callback for woocommerce_before_cart_collaterals hook.
     *
     * @return void
     * @since 1.0.0
     *
     */
    public function before_cart_totals() {
        echo sprintf( '</div><div class="mdp-carter-elementor-cart-product-total">
                            <div class="mdp-carter-elementor-cart-product-total-content">' );
    }


    /**
     * Callback for woocommerce_after_cart hook.
     *
     * @return void
     * @since 1.0.0
     *
     */
    public function after_cart_content() {
        echo sprintf( '</div></div></div>' );
    }

    /**
     * Callback for woocommerce_cart_collaterals hook.
     *
     * @return void
     * @since 1.0.0
     *
     */
    public function remove_totals_block() {
        remove_action( 'woocommerce_cart_collaterals', 'woocommerce_cart_totals', 10 );

        // Add back "Proceed to checkout" button (and hooks)
        echo '<div class="cart_totals">';
        do_action( 'woocommerce_before_cart_totals' );

        echo '<div class="wc-proceed-to-checkout">';
        do_action( 'woocommerce_proceed_to_checkout' );
        echo '</div>';

        do_action( 'woocommerce_after_cart_totals' );
        echo '</div><br clear="all">';
    }

    /**
     * Callback for gettext hook.
     *
     * @return void
     * @since 1.0.0
     *
     */
    public function update_buttons_text( $translated, $text, $domain ) {
        $settings = $this->get_settings_for_display();

        $update_cart_button_text = $settings['update_cart_button_text'];
        $update_shipping_button_text = $settings['update_shipping_button_text'];
        $cart_totals_title = $settings['totals_title_text'];

        if ( is_cart() && $translated === 'Update cart' && $update_cart_button_text !== '' ) {
            $translated = esc_html__( $update_cart_button_text, 'selection' );
        } elseif ( is_cart() && $translated === 'Update' && $update_shipping_button_text !== '' ) {
            $translated = esc_html__( $update_shipping_button_text, 'selection');
        } elseif ( is_cart() && $translated === 'Cart totals' && $cart_totals_title !== '' ) {
            $translated = esc_html__( $cart_totals_title, 'selection' );
        }

        return $translated;
    }

    /**
     * Callback for woocommerce_cart_needs_shipping hook.
     *
     * @return false
     * @since 1.0.0
     *
     */
    function filter_cart_needs_shipping_callback( ){

        return false;
    }

    /**
     * Method for creating cart hooks.
     *
     * @return void
     * @since 1.0.0
     *
     */
    private function create_cart_hooks( $settings ) {
        add_filter( 'woocommerce_coupons_enabled', [$this, 'show_coupon_field'] );
        if ( $settings['show_total'] !== 'yes' ) {
            add_filter( 'woocommerce_cart_collaterals', [$this, 'remove_totals_block'], 9 );
        }

        if ( $settings['totals_show_shipping'] !== 'yes' ) {
            add_filter( 'woocommerce_cart_needs_shipping', [$this, 'filter_cart_needs_shipping_callback'] );
        }


        add_filter( 'gettext', [$this, 'update_buttons_text'], 20, 3 );

        add_action( 'woocommerce_before_cart', [ $this, 'before_cart_start' ] );
        add_action( 'woocommerce_after_cart_table', [ $this, 'after_cart_table' ] );
        add_action( 'woocommerce_before_cart_collaterals', [ $this, 'before_cart_totals' ] );
        add_action( 'woocommerce_after_cart', [ $this, 'after_cart_content' ] );
        add_filter( 'woocommerce_get_cart_url', [ $this, 'woocommerce_get_cart_url' ] );

        if ( $settings['empty_cart_text'] !== '' ) {
            remove_action( 'woocommerce_cart_is_empty', 'wc_empty_cart_message', 10 );
            add_action( 'woocommerce_cart_is_empty', [$this, 'custom_empty_cart_message'], 10 );
        }

        add_action( 'woocommerce_proceed_to_checkout', [$this, 'modify_checkout_button_no_shipping'], 1 );

    }

    /**
     * Method for removing cart hooks.
     *
     * @return void
     * @since 1.0.0
     *
     */
    private function remove_cart_hooks() {
        remove_filter( 'woocommerce_coupons_enabled', [$this, 'show_coupon_field'] );
        remove_filter( 'woocommerce_cart_collaterals', [$this, 'remove_totals_block'], 9 );
        remove_filter( 'woocommerce_cart_needs_shipping', [$this, 'filter_cart_needs_shipping_callback'] );
        remove_filter( 'gettext', [$this, 'update_buttons_text'], 20, 3 );

        remove_action( 'woocommerce_before_cart', [ $this, 'before_cart_start' ] );
        remove_action( 'woocommerce_after_cart_table', [ $this, 'after_cart_table' ] );
        remove_action( 'woocommerce_before_cart_collaterals', [ $this, 'before_cart_totals' ] );
        remove_action( 'woocommerce_after_cart', [ $this, 'after_cart_content' ] );
        remove_action( 'woocommerce_get_cart_url', [ $this, 'woocommerce_get_cart_url' ] );
        remove_action( 'woocommerce_cart_is_empty', [$this, 'custom_empty_cart_message'], 10 );
        remove_action( 'woocommerce_proceed_to_checkout', [$this, 'modify_checkout_button_no_shipping'], 1 );
        remove_action( 'woocommerce_get_cart_url', [ $this, 'woocommerce_get_cart_url' ] );
    }

    /**
     * Callback for woocommerce_proceed_to_checkout hook.
     *
     * @return void
     * @since 1.0.0
     *
     */
    function modify_checkout_button_no_shipping() {
        $settings = $this->get_settings_for_display();

        remove_action(
          'woocommerce_proceed_to_checkout',
            'woocommerce_button_proceed_to_checkout',
             20
        );

        $button_text = $settings['checkout_button_text'] !== '' ?
            esc_html__( $settings['checkout_button_text'], 'woocommerce' ) :
            esc_html__( 'Proceed to checkout', 'woocommerce' );

        $button_icon = sprintf(
                '<span class="mdp-carter-elementor-btn-icon">%s</span>' ,
                        $this->create_button_icon( $settings['checkout_button_icon'] )
        );

        $button_icon_position_class = sprintf(
                'mdp-carter-elementor-btn-icon-wrapper mdp-carter-elementor-btn-icon-position-%s',
                        esc_attr( $settings['checkout_button_icon_position'] )
        );


        $updated_checkout_button = sprintf(
                '<a href="%s" class="mdp-carter-elementor-btn %s %s checkout-button button alt wc-forward">
                            <span class="mdp-carter-elementor-btn-content %s">%s %s</span>
                        </a>',
                esc_url( wc_get_checkout_url() ),
                        'elementor-animation-' . esc_attr( $settings['checkout_hover_animation'] ),
                        $settings['checkout_button_enable_icon'] === 'yes' ? 'mdp-carter-elementor-btn-icon-enabled' : '',
                        $settings['checkout_button_enable_icon'] === 'yes' ? $button_icon_position_class : '',
                        $settings['checkout_button_enable_icon'] === 'yes' ? $button_icon : '',
                        $button_text
        );

        echo $updated_checkout_button;
    }


    /**
     * Callback for woocommerce_get_cart_url hook.
     *
     * @since 1.0.0
     *
     */
    public function woocommerce_get_cart_url( $url ) {
        global $post;

        if ( ! $post ) {
            return $url;
        }

        if ( Plugin::instance()->preview->is_preview_mode() || is_preview() || Plugin::instance()->editor->is_edit_mode() ) {
            return Plugin::instance()->documents->get_current()->get_wp_preview_url();
        }

        return get_permalink( $post->ID );
    }


    /**
     * Callback for woocommerce_cart_is_empty hook.
     *
     * @return void
     * @since 1.0.0
     *
     */
   public function custom_empty_cart_message() {
        $settings = $this->get_settings_for_display();

        $html  = '<p class="cart-empty woocommerce-info">';
        $html .= wp_kses_post( apply_filters(
                'wc_empty_cart_message',
                esc_html__( $settings['empty_cart_text'], 'woocommerce' ) )
        );
        echo $html . '</p>';
    }

    /**
     * Returns template content.
     *
     * @param $template_id
     * @return string
     * @since 1.0.0
     * @access private
     *
     */
    private function get_template_content( $template_id ) {

        if ( $template_id === '' ) { return ''; }

        return Plugin::instance()->frontend->get_builder_content_for_display( $template_id );
    }

    /**
     * Create donation button icon.
     *
     * @return string
     * @since 1.0.0
     * @access private
     *
     */
    private function create_button_icon( $button_icon ) {

        if ( empty( $button_icon ) ) { return ''; }

        return $button_icon['library'] === 'svg' ?
            Icons_Manager::render_uploaded_svg_icon( $button_icon['value'] ) :
            '<i class="'.esc_attr( $button_icon['value'] ).'"></i>';
    }

    /**
     * Render Frontend Output. Generate the final HTML on the frontend.
     *
     * @access protected
     *
     * @return void
     **/
    protected function render() {

        if ( !Caster::get_instance()->is_active_woocommerce() ) {
            esc_html_e( 'Please install Woocommerce plugin!' );
            return;
        }

        if ( !class_exists( 'WooCommerce' ) || !function_exists( 'WC' ) ) {
            return;
        }

	    if ( WC()->cart === NULL ) { return; }

        $settings = $this->get_settings_for_display();
        $this->create_cart_hooks( $settings );

        $auto_update = $settings['auto_update_cart'] === 'yes' ? 'mdp-carter-elementor-auto-update' : '';
        $template_id = isset( $settings['empty_cart_template'] ) ? esc_html( $settings['empty_cart_template'] ) : '';

        $cart_content = WC()->cart->get_cart_contents_count() === 0 && !empty( $this->get_template_content( $template_id ) )
            ? $this->get_template_content( $template_id )
            : do_shortcode( '[woocommerce_cart]' );

        $hide_fees_field_class = $settings['totals_show_fees'] !== 'yes' ? 'mdp-carter-elementor-hide-fees' : '';
        $hide_subtotal_field_class = $settings['totals_show_subtotal'] !== 'yes' ? 'mdp-carter-elementor-hide-subtotal' : '';
        $hide_tax_field_class = $settings['totals_show_tax'] !== 'yes' ? 'mdp-carter-elementor-hide-tax' : '';
        $hide_total_field_class = $settings['totals_show_total'] !== 'yes' ? 'mdp-carter-elementor-hide-total' : '';


        echo sprintf(
            '<!-- Start Carter for Elementor WordPress Plugin -->
                    <div class="mdp-carter-elementor-box %s %s %s %s %s woocommerce-page" 
                    data-widget-id="%s"
                    data-cart-layout="%s"
                    data-sticky-right-column="%s"
                    data-cart-auto-update="%s"
                    data-update-cart-animation="%s"
                    data-update-shipping-animation="%s"
                    >%s</div>
                    <!-- End Carter for Elementor WordPress Plugin -->',
                    $auto_update,
                    $hide_fees_field_class,
                    $hide_subtotal_field_class,
                    $hide_tax_field_class,
                    $hide_total_field_class,
                    esc_attr( $this->get_id() ),
                    esc_attr( $settings['cart_layout'] ),
                    esc_attr( $settings['sticky_right_column'] ),
                    esc_attr( $settings['auto_update_cart'] ),
                    esc_attr( $settings['update_cart_hover_animation'] ),
                    esc_attr( $settings['update_shipping_hover_animation'] ),
                    $cart_content
        );

        $this->remove_cart_hooks();
        ?>
	    <?php

    }

    /**
     * Return link for documentation
     * Used to add stuff after widget
     *
     * @access public
     *
     * @return string
     **/
    public function get_custom_help_url() {

        return 'https://docs.merkulov.design/tag/carter';

    }

}
