<?php /** @noinspection PhpUndefinedClassInspection */
/**
 * Countdowner for Elementor
 * Customizable countdown for Elementor editor
 * Exclusively on https://1.envato.market/countdowner-elementor
 *
 * @encoding        UTF-8
 * @version         1.12.1
 * @copyright       (C) 2018 - 2023 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Cherviakov Vlad (vladchervjakov@gmail.com), Dmitry Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

namespace Merkulove\Selection;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
    header( 'Status: 403 Forbidden' );
    header( 'HTTP/1.1 403 Forbidden' );
    exit;
}

use Exception;
use Elementor\Widget_Base;
use Elementor\Controls_Manager;
use Elementor\Core\Schemes\Typography;
use Elementor\Core\Schemes\Color;
use Elementor\Group_Control_Typography;
use Elementor\Group_Control_Border;
use Elementor\Group_Control_Background;
use Elementor\Group_Control_Box_Shadow;
use Elementor\Group_Control_Text_Shadow;
use Merkulove\Selection\Unity\Plugin as UnityPlugin;

/** @noinspection PhpUnused */
/**
 * Countdowner - Custom Elementor Widget.
 **/
class countdowner_elementor extends Widget_Base {

    /**
     * Use this to sort widgets.
     * A smaller value means earlier initialization of the widget.
     * Can take negative values.
     * Default widgets and widgets from 3rd party developers have 0 $mdp_order
     **/
    public $mdp_order = 1;

    /**
     * Widget base constructor.
     * Initializing the widget base class.
     *
     * @access public
     * @throws Exception If arguments are missing when initializing a full widget instance.
     * @param array      $data Widget data. Default is an empty array.
     * @param array|null $args Optional. Widget default arguments. Default is null.
     *
     * @return void
     **/
    public function __construct( $data = [], $args = null ) {

        parent::__construct( $data, $args );

        wp_register_style(
            'mdp-selection-elementor-admin',
               UnityPlugin::get_url() . 'css/elementor-admin' . UnityPlugin::get_suffix() . '.css',
                   [],
                   UnityPlugin::get_version()
        );
        wp_register_style(
            'mdp-countdowner-elementor',
               UnityPlugin::get_url() . 'css/countdowner-elementor' . UnityPlugin::get_suffix() . '.css',
                   [],
                   UnityPlugin::get_version()
        );
	    wp_register_script(
            'mdp-countdowner-elementor',
               UnityPlugin::get_url() . 'js/countdowner-elementor' . UnityPlugin::get_suffix() . '.js',
                   [ 'jquery', 'moment' ],
                   UnityPlugin::get_version(),
          true
        );

    }

    /**
     * Return a widget name.
     *
     * @return string
     **/
    public function get_name() {

        return 'mdp-countdowner-elementor';

    }

    /**
     * Return the widget title that will be displayed as the widget label.
     *
     * @return string
     **/
    public function get_title() {

        return esc_html__( 'Countdown', 'selection' );

    }

    /**
     * Set the widget icon.
     *
     * @return string
     */
    public function get_icon() {

        return 'mdp-countdowner-elementor-widget-icon';

    }

    /**
     * Set the category of the widget.
     *
     * @return array with category names
     **/
    public function get_categories() {

        return [ 'selection-category' ];

    }

    /**
     * Get widget keywords. Retrieve the list of keywords the widget belongs to.
     *
     * @access public
     *
     * @return array Widget keywords.
     **/
    public function get_keywords() {

        return [ 'Merkulove', 'Countdown' ];

    }

    /**
     * Get style dependencies.
     * Retrieve the list of style dependencies the widget requires.
     *
     * @access public
     *
     * @return array Widget styles dependencies.
     **/
    public function get_style_depends() {

        return [ 'mdp-countdowner-elementor', 'mdp-selection-elementor-admin' ];

    }

	/**
	 * Get script dependencies.
	 * Retrieve the list of script dependencies the element requires.
	 *
	 * @access public
     *
	 * @return array Element scripts dependencies.
	 **/
	public function get_script_depends() {

		return [ 'mdp-countdowner-elementor' ];

    }

    /**
     * Add the widget controls.
     *
     * @access protected
     * @return void with category names
     **/
    protected function register_controls() {

        /** Content Tab. */
        $this->tab_content();

        /** Style Tab. */
        $this->tab_style();

    }

    /**
     * Add widget controls on Content tab.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function tab_content() {

        /** Content -> General Settings Section. */
        $this->section_general_settings();

        /** Content -> Countdown Separator Settings Section. */
        $this->section_separator_settings();

        /** Content -> Countdown Expire Action Settings Section. */
        $this->section_countdown_expire_action_settings();

        /** Content -> Countdown Message Settings Section. */
        $this->section_countdown_message_settings();

        /** Content -> Countdown Button Settings Section. */
        $this->section_countdown_button_settings();

    }

    /**
     * Add widget controls on Style tab.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function tab_style() {

        /** Style -> Section Style Items. */
        $this->section_style_items();

        /** Style -> Section Style Flip Animation Card. */
        $this->section_style_flip_animation_card();

        /** Style -> Section Style Circle Animation. */
        $this->circle_animation_styles();

        /** Style -> Section Style Before Expire Message. */
        $this->before_expire_message_styles();

        /** Style -> Section Style After Expire Message */
        $this->after_expire_message_styles();

        /** Style -> Section Style Separator. */
        $this->separator_styles();

        /** Style -> Section Style Countdown Before Expire Button. */
        $this->countdown_before_expire_button_styles();

        /** Style -> Section Style Countdown After Expire Button. */
        $this->countdown_after_expire_button_styles();

    }

    /**
     * Add widget controls: Content -> General Settings Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_general_settings() {

        $this->start_controls_section( 'section_general_settings', [
            'label' => esc_html__( 'General', 'selection' ),
            'tab'   => Controls_Manager::TAB_CONTENT
        ] );

        $this->add_control(
            'countdown_view',
            [
                'label' => esc_html__( 'View', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'block',
                'options' => [
                    'block'  => esc_html__( 'Block', 'selection' ),
                    'inline-block' => esc_html__( 'Inline', 'selection' ),
                ],
            ]
        );

        $this->add_control(
            'countdown_type',
            [
                'label' => esc_html__( 'Type', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'due-date',
                'options' => [
                    'due-date'  => esc_html__( 'Due Date', 'selection' ),
                    'evergreen-timer' => esc_html__( 'Evergreen Timer', 'selection' ),
                ],
            ]
        );

        $this->add_control(
            'text_align',
            [
                'label' => esc_html__( 'Countdown align', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'flex-start' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'fa fa-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'fa fa-align-center',
                    ],
                    'flex-end' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'fa fa-align-right',
                    ],
                    'space-between' => [
                        'title' => esc_html__( 'Justify', 'selection' ),
                        'icon' => 'fa fa-align-justify',
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-countdowner-elementor--wrapper-digits' => 'justify-content: {{VALUE}}'
                ],
                'default' => 'space-between',
                'toggle' => true,
            ]
        );

        $this->add_control(
            'countdown_due_date',
            [
                'label' => esc_html__( 'Due Date', 'selection' ),
                'type' => Controls_Manager::DATE_TIME,
                'condition' => [
                'countdown_type' => 'due-date'
                ]
            ]
        );

        $this->add_control(
            'countdown_start_date',
            [
                'label' => esc_html__( 'Start Date', 'selection' ),
                'type' => Controls_Manager::DATE_TIME,
                'default' => $this->get_start_time(),
                'condition' => [
                    'countdown_type' => 'due-date',
                    'countdown_animation' => 'circle'
                ]
            ]
        );

        $this->add_control(
            'evergreen_timer_years',
            [
                'label' => esc_html__( 'Years', 'selection' ),
                'type' => Controls_Manager::NUMBER,
                'default' => 0,
                'condition' => [
                     'countdown_type' => 'evergreen-timer'
                ]
            ]
        );

        $this->add_control(
            'evergreen_timer_months',
            [
                'label' => esc_html__( 'Months', 'selection' ),
                'type' => Controls_Manager::NUMBER,
                'default' => 0,
                'condition' => [
                    'countdown_type' => 'evergreen-timer'
                ]
            ]
        );

        $this->add_control(
            'evergreen_timer_weeks',
            [
                'label' => esc_html__( 'Weeks', 'selection' ),
                'type' => Controls_Manager::NUMBER,
                'default' => 0,
                'condition' => [
                    'countdown_type' => 'evergreen-timer'
                ]
            ]
        );

        $this->add_control(
            'evergreen_timer_days',
            [
                'label' => esc_html__( 'Days', 'selection' ),
                'type' => Controls_Manager::NUMBER,
                'default' => 0,
                'condition' => [
                    'countdown_type' => 'evergreen-timer'
                ]
            ]
        );

        $this->add_control(
            'evergreen_timer_hours',
            [
                'label' => esc_html__( 'Hours', 'selection' ),
                'type' => Controls_Manager::NUMBER,
                'default' => 0,
                'condition' => [
                    'countdown_type' => 'evergreen-timer'
                ]
            ]
        );

        $this->add_control(
            'evergreen_timer_minutes',
            [
                'label' => esc_html__( 'Minutes', 'selection' ),
                'type' => Controls_Manager::NUMBER,
                'default' => 1,
                'condition' => [
                    'countdown_type' => 'evergreen-timer'
                ]
            ]
        );
        $this->add_control(
            'evergreen_timer_seconds',
            [
                'label' => esc_html__( 'Seconds', 'selection' ),
                'type' => Controls_Manager::NUMBER,
                'default' => 0,
                'condition' => [
                    'countdown_type' => 'evergreen-timer'
                ]
            ]
        );


        $this->add_control(
            'reset_timer_on_expire',
            [
                'label' => esc_html__( 'Reset on page refresh after expire', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Yes', 'selection' ),
                'label_off' => esc_html__( 'No', 'selection' ),
                'return_value' => 'yes',
                'default' => '',
                'condition' => [
                        'countdown_type' => 'evergreen-timer'
                ]
            ]
        );


        $this->add_control(
            'countdown_time_zone',
            [
                'label' => esc_html__( 'Time Zone', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'label_block' => true,
                'default' => 'wordpress_default',
                'options' => [
                    'wordpress_default'  => esc_html__( 'Wordpress Default', 'selection' ),
                    'user_local_time' => esc_html__( 'User Local Time', 'selection' ),
                ],
            ]
        );

        $this->add_control(
            'time_units',
            [
                'label' => esc_html__( 'Time Units', 'selection' ),
                'type' => Controls_Manager::SELECT2,
                'label_block' => true,
                'multiple' => true,
                'options' => [
                    'years'  => esc_html__( 'Years', 'selection' ),
                    'months' => esc_html__( 'Months', 'selection' ),
                    'weeks' => esc_html__( 'Weeks', 'selection' ),
                    'days' => esc_html__( 'Days', 'selection' ),
                    'hours' => esc_html__( 'Hours', 'selection' ),
                    'minutes' => esc_html__( 'Minutes', 'selection' ),
                    'seconds' => esc_html__( 'Seconds', 'selection' ),
                ],
                'separator' => 'after',
                'default' => [ 'years', 'months', 'weeks', 'days', 'hours', 'minutes', 'seconds' ],
            ]
        );

        $this->add_control(
            'countdown_animation',
            [
                'label' => esc_html__( 'Changing digits animation', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'label_block' => true,
                'default' => 'none',
                'options' => [
                    'none'  => esc_html__( 'None', 'selection' ),
                    'flip' => esc_html__( 'Flip clock', 'selection' ),
                    'circle' => esc_html__( 'Circle', 'selection' ),
                    'fade' => esc_html__( 'Fade', 'selection' ),
                    'roll-digits' => esc_html__( 'Roll digits', 'selection' ),
                    'glowing' => esc_html__( 'Glowing', 'selection' )
                ],
            ]
        );

        $this->add_control(
            'custom_labels_divider',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        $this->add_control(
            'custom_labels',
            [
                'label' => esc_html__( 'Custom labels', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Yes', 'selection' ),
                'label_off' => esc_html__( 'No', 'selection' ),
                'return_value' => 'yes',
                'default' => '',
            ]
        );

        $this->add_control(
            'years_custom_label',
            [
                'label' => esc_html__( 'Years custom label', 'selection' ),
                'type' => Controls_Manager::TEXT,
                'default' => esc_html__( 'Years', 'selection' ),
                'condition' => [
                        'custom_labels' => 'yes'
                ]
            ]
        );

        $this->add_control(
            'months_custom_label',
            [
                'label' => esc_html__( 'Months custom label', 'selection' ),
                'type' => Controls_Manager::TEXT,
                'default' => esc_html__( 'Months', 'selection' ),
                'condition' => [
                        'custom_labels' => 'yes'
                ]
            ]
        );

        $this->add_control(
            'weeks_custom_label',
            [
                'label' => esc_html__( 'Weeks custom label', 'selection' ),
                'type' => Controls_Manager::TEXT,
                'default' => esc_html__( 'Weeks', 'selection' ),
                'condition' => [
                    'custom_labels' => 'yes'
                ]
            ]
        );

        $this->add_control(
            'days_custom_label',
            [
                'label' => esc_html__( 'Days custom label', 'selection' ),
                'type' => Controls_Manager::TEXT,
                'default' => esc_html__( 'Days', 'selection' ),
                'condition' => [
                    'custom_labels' => 'yes'
                ]
            ]
        );

        $this->add_control(
            'hours_custom_label',
            [
                'label' => esc_html__( 'Hours custom label', 'selection' ),
                'type' => Controls_Manager::TEXT,
                'default' => esc_html__( 'Hours', 'selection' ),
                'condition' => [
                    'custom_labels' => 'yes'
                ]
            ]
        );

        $this->add_control(
            'minutes_custom_label',
            [
                'label' => esc_html__( 'Minutes custom label', 'selection' ),
                'type' => Controls_Manager::TEXT,
                'default' => esc_html__( 'Minutes', 'selection' ),
                'condition' => [
                    'custom_labels' => 'yes'
                ]
            ]
        );

        $this->add_control(
            'seconds_custom_label',
            [
                'label' => esc_html__( 'Seconds custom label', 'selection' ),
                'type' => Controls_Manager::TEXT,
                'default' => esc_html__( 'Seconds', 'selection' ),
                'condition' => [
                    'custom_labels' => 'yes'
                ]
            ]
        );

        $this->end_controls_section();

    }

    /**
     * Returns count down start time
     * @return string
     */
    private function get_start_time() {

        // Get times from options
        $times = get_option( 'mdp_countdowner_elementor_start' );

        // Prepare current time
        $current_time = current_time('Y-m-d H:i');
        $start_time = isset( $times[ $this->get_id() ] ) ? $times[ $this->get_id() ] : $current_time;

        // Update current time
        if ( empty( $times ) || isset( $times[ $this->get_id() ] ) ) {

            $times[ $this->get_id() ] = $current_time;
            update_option( 'mdp_countdowner_elementor_start', $times );

        }

        return $start_time;

    }

    /**
     * Add widget controls: Content -> Countdown Message Settings Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_countdown_message_settings() {

        $this->start_controls_section( 'section_countdown_message_settings', [
            'label' => esc_html__( 'Countdown message', 'selection' ),
            'condition' => [
                'expire_action_type' => 'message'
            ],
            'tab'   => Controls_Manager::TAB_CONTENT
        ] );

        $this->add_control(
            'before_message_align',
            [
                'label' => esc_html__( 'Before countdown message align', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'fa fa-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'fa fa-align-center',
                    ],
                    'right' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'fa fa-align-right',
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-countdowner-elementor--wrapper-countdowner-before-expire-message' => 'text-align: {{VALUE}}'
                ],
                'default' => 'center',
                'toggle' => true,
            ]
        );

        $this->add_control(
            'after_message_align',
            [
                'label' => esc_html__( 'After countdown message align', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'fa fa-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'fa fa-align-center',
                    ],
                    'right' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'fa fa-align-right',
                    ],
                ],
                'selectors' => [
                        '{{WRAPPER}} .mdp-countdowner-elementor--wrapper-countdowner-after-expire-message' => 'text-align: {{VALUE}}'
                ],
                'default' => 'center',
                'toggle' => true,
            ]
        );



        $this->add_control(
            'before_countdown_message',
            [
                'label' => esc_html__( 'Before countdown expire message', 'selection' ),
                'type' => Controls_Manager::TEXTAREA,
                'rows' => 10,
                'dynamic' => [
                    'active' => true,
                ],
                'default' => esc_html__( 'Before expire default message text', 'selection' ),
                'placeholder' => esc_html__( 'Type your message text here', 'selection' ),
            ]
        );

        $this->add_control(
            'after_countdown_message',
            [
                'label' => esc_html__( 'After countdown expire message', 'selection' ),
                'type' => Controls_Manager::TEXTAREA,
                'rows' => 10,
                'dynamic' => [
                    'active' => true,
                ],
                'default' => esc_html__( 'After expire default message text', 'selection' ),
                'placeholder' => esc_html__( 'Type your message text here', 'selection' ),
            ]
        );


        $this->end_controls_section();

    }

    /**
     * Add widget controls: Content -> Countdown Button Settings Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_countdown_button_settings() {

        $this->start_controls_section('section_countdown_button_settings', [
            'label' => esc_html__('Countdown button', 'selection'),
            'condition' => [
                'expire_action_type' => 'button'
            ],
            'tab' => Controls_Manager::TAB_CONTENT
        ]);

        $this->add_control(
            'before_button_align',
            [
                'label' => esc_html__( 'Before expire button align', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'fa fa-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'fa fa-align-center',
                    ],
                    'right' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'fa fa-align-right',
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-countdowner-elementor--wrapper-before-button-wrapper' => 'text-align: {{VALUE}}'
                ],
                'default' => 'center',
                'toggle' => true,
            ]
        );

        $this->add_control(
            'after_button_align',
            [
                'label' => esc_html__( 'After expire button align', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'fa fa-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'fa fa-align-center',
                    ],
                    'right' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'fa fa-align-right',
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-countdowner-elementor--wrapper-after-button-wrapper' => 'text-align: {{VALUE}}'
                ],
                'default' => 'center',
                'toggle' => true,
            ]
        );


        $this->add_control(
            'before_countdown_text_button',
            [
                'label' => esc_html__( 'Before countdown text', 'selection' ),
                'label_block' => true,
                'dynamic' => [
                    'active' => true,
                ],
                'type' => Controls_Manager::TEXT,
                'default' => esc_html__( 'Before expire text', 'selection' ),
                'placeholder' => esc_html__( 'Type message text here', 'selection' ),
            ]
        );


        $this->add_control(
            'after_countdown_text_button',
            [
                'label' => esc_html__( 'After countdown text', 'selection' ),
                'label_block' => true,
                'dynamic' => [
                    'active' => true,
                ],
                'type' => Controls_Manager::TEXT,
                'default' => esc_html__( 'After expire text', 'selection' ),
                'placeholder' => esc_html__( 'Type message text here', 'selection' ),
            ]
        );

        $this->add_control(
            'countdown_before_expire_button_link',
            [
                'label' => esc_html__( 'Countdown before expire button link', 'selection' ),
                'type' => Controls_Manager::URL,
                'placeholder' => esc_html__( 'https://your-link.com', 'selection' ),
                'show_external' => true,
                'default' => [
                    'url' => '',
                    'is_external' => true,
                    'nofollow' => true,
                ],
            ]
        );

        $this->add_control(
            'countdown_after_expire_button_link',
            [
                'label' => esc_html__( 'Countdown after expire button link', 'selection' ),
                'type' => Controls_Manager::URL,
                'placeholder' => esc_html__( 'https://your-link.com', 'selection' ),
                'show_external' => true,
                'default' => [
                    'url' => '',
                    'is_external' => true,
                    'nofollow' => true,
                ],
            ]
        );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Content -> Countdown Separator Settings Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_separator_settings() {

        $this->start_controls_section('section_separator_settings', [
            'label' => esc_html__('Separator', 'selection'),
            'tab' => Controls_Manager::TAB_CONTENT
        ]);

        $this->add_control(
            'show_digits_separator',
            [
                'label' => esc_html__( 'Show digits separator', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'separator'=> 'before',
                'label_on' => esc_html__( 'Show', 'selection' ),
                'label_off' => esc_html__( 'Hide', 'selection' ),
                'return_value' => 'yes',
                'default' => 'yes',
            ]
        );

        $this->add_control(
            'digits_separator',
            [
                'label' => esc_html__( 'Digits separator', 'selection' ),
                'default' => esc_html__( ':', 'selection' ),
                'type' => Controls_Manager::TEXT,
            ]
        );


        $this->end_controls_section();

    }

    /**
     * Add widget controls: Content -> Countdown Expire Action Settings Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_countdown_expire_action_settings() {

        $this->start_controls_section( 'section_expire_action_settings', [
            'label' => esc_html__( 'Expire action', 'selection' ),
            'tab'   => Controls_Manager::TAB_CONTENT
        ] );

        $this->add_control(
            'expire_action_type',
            [
                'label'     => esc_html__( 'Expire type', 'selection' ),
                'type'      => Controls_Manager::SELECT,
                'options'   => [
                    'none'    => esc_html__( 'None', 'selection' ),
                    'message'    => esc_html__( 'Message', 'selection' ),
                    'redirection-link'    => esc_html__( 'Redirection Link', 'selection' ),
                    'hide' => esc_html__( 'Hide', 'selection' ),
                    'button' => esc_html__( 'Button', 'selection' )
                ],
                'default'   => 'none',
                'separator' => 'none',
            ]
        );

        $this->add_control(
            'redirection_link',
            [
                'label' => esc_html__( 'Link', 'selection' ),
                'type' => Controls_Manager::URL,
                'placeholder' => esc_html__( 'https://your-link.com', 'selection' ),
                'show_external' => true,
                'condition' => [
                        'expire_action_type' => 'redirection-link'
                ],
                'default' => [
                    'url' => '',
                    'is_external' => true,
                    'nofollow' => true,
                ],
                'description' => esc_html__( 'Your page will be redirected only on frontend' )
            ]
        );

        $this->end_controls_section();

    }

    /**
     * Function for generating margin padding controls.
     *
     * @param $section_id
     * @param $html_class
     * @return void
     * @since 1.0.0
     * @access private
     *
     */
    private function generate_margin_padding_controls( $section_id, $html_class ) {
        $this->add_responsive_control(
            $section_id.'_margin',
            [
                'label' => esc_html__( 'Margin', 'tiktoker-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'selectors' => [
                    "{{WRAPPER}} .$html_class" => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_responsive_control(
            $section_id.'_padding',
            [
                'label' => esc_html__( 'Padding', 'tiktoker-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'separator' => 'after',
                'selectors' => [
                    "{{WRAPPER}} .$html_class" => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );
    }

    /**
     * Function for generating typography and tabs controls.
     *
     * @param $section_id
     * @param $html_class
     * @param bool $transition_duration
     * @return void
     * @since 1.0.0
     * @access private
     */
    private function generate_typography_tabs_controls( $section_id, $html_class, $transition_duration = false ) {

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name' => $section_id.'_typography',
                'label' => esc_html__( 'Typography', 'selection' ),
                'scheme' =>  Typography::TYPOGRAPHY_1,
                'selector' => "{{WRAPPER}} .$html_class",
            ]
        );

        $this->start_controls_tabs( $section_id.'_style_tabs' );

        $this->start_controls_tab( $section_id.'_normal_style_tab', ['label' => esc_html__( 'NORMAL', 'selection' )] );

        $this->add_control(
            $section_id.'_normal_text_color',
            [
                'label' => esc_html__( 'Color', 'selection' ),
                'type' => Controls_Manager::COLOR,
                'scheme' => [
                    'type' => Color::get_type(),
                    'value' => Color::COLOR_3,
                ],
                'selectors' => [
                    "{{WRAPPER}} .$html_class" => 'color: {{VALUE}}',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Background::get_type(),
            [
                'name' => $section_id.'_normal_background',
                'label' => esc_html__( 'Background type', 'selection' ),
                'types' => [ 'classic', 'gradient', 'video' ],
                'selector' => "{{WRAPPER}} .$html_class",
            ]
        );

        $this->add_control(
            $section_id.'_separate_normal',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => $section_id.'_border_normal',
                'label' => esc_html__( 'Border Type', 'selection' ),
                'selector' => "{{WRAPPER}} .$html_class",
            ]
        );

        $this->add_responsive_control(
            $section_id.'_border_radius_normal',
            [
                'label' => esc_html__( 'Border radius', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    "{{WRAPPER}} .$html_class" => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => $section_id.'_box_shadow_normal',
                'label' => esc_html__( 'Box Shadow', 'selection' ),
                'selector' => "{{WRAPPER}} .$html_class",
            ]
        );


        $this->end_controls_tab();

        $this->start_controls_tab( $section_id.'_hover_style_tab', ['label' => esc_html__( 'HOVER', 'selection' )] );

        $this->add_control(
            $section_id.'_hover_color',
            [
                'label' => esc_html__( 'Color', 'selection' ),
                'type' => Controls_Manager::COLOR,
                'scheme' => [
                    'type' => Color::get_type(),
                    'value' => Color::COLOR_3,
                ],
                'selectors' => [
                    "{{WRAPPER}} .$html_class:hover" => 'color: {{VALUE}}',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Background::get_type(),
            [
                'name' => $section_id.'_background_hover',
                'label' => esc_html__( 'Background type', 'selection' ),
                'types' => [ 'classic', 'gradient', 'video' ],
                'selector' => "{{WRAPPER}} .$html_class:hover",
            ]
        );

        $this->add_control(
            $section_id.'_hover_transition',
            [
                'label' => esc_html__( 'Hover transition duration', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 's' ],
                'range' => [
                    's' => [
                        'min' => 0.1,
                        'max' => 5,
                        'step' => 0.1,
                    ],
                ],
                'default' => [
                    'unit' => 's',
                    'size' => 0,
                ],
                'selectors' => [
                    '{{WRAPPER}} .'.$html_class => 'transition: all {{SIZE}}{{UNIT}};',
                ],
            ]
        );


        $this->add_control(
            $section_id.'_separate_hover',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => $section_id.'_border_hover',
                'label' => esc_html__( 'Border Type', 'selection' ),
                'selector' => "{{WRAPPER}} .$html_class:hover",
            ]
        );

        $this->add_responsive_control(
            $section_id.'_border_radius_hover',
            [
                'label' => esc_html__( 'Border radius', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    "{{WRAPPER}} .$html_class:hover" => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => $section_id.'_box_shadow_hover',
                'label' => esc_html__( 'Box Shadow', 'selection' ),
                'selector' => "{{WRAPPER}} .$html_class:hover",
            ]
        );

        $this->end_controls_tab();

        $this->end_controls_tabs();
    }

    /**
     * Add widget controls: Style -> Section Style Items.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_items() {

        $this->start_controls_section( 'section_style_items', [
            'label' => esc_html__( 'Items', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->generate_margin_padding_controls( 'section_style_items', 'mdp-countdowner-elementor--wrapper-item' );

        $this->add_responsive_control(
            'section_style_items_container_width',
            [
                'label' => esc_html__( 'Container width', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 1,
                        'max' => 500,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 1,
                        'max' => 100,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-countdowner-elementor--wrapper-item' => 'width: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->add_responsive_control(
            'section_style_items_space_between_items',
            [
                'label' => esc_html__( 'Space between items', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 1,
                        'max' => 500,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 1,
                        'max' => 100,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} div.mdp-countdowner-elementor--wrapper-item' => 'margin-right: {{SIZE}}{{UNIT}};',
                    '{{WRAPPER}} .mdp-countdowner-elementor--wrapper-digits div:last-of-type' => 'margin-right: 0;'
                ],
            ]
        );

        $this->add_control(
            'section_style_items_separate_typography',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name' => 'section_style_digits_typography',
                'label' => esc_html__( 'Digits typography', 'selection' ),
                'scheme' =>  Typography::TYPOGRAPHY_1,
                'selector' => "{{WRAPPER}} .mdp-countdowner-elementor--wrapper-item-val, {{WRAPPER}} .mdp-countdowner-elementor--wrapper-digit::before, {{WRAPPER}} .mdp-countdowner-elementor--wrapper-digit::after, {{WRAPPER}} .mdp-countdowner-elementor--wrapper-digit-face-front, {{WRAPPER}} .mdp-countdowner-elementor--wrapper-digit-face-back",
            ]
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name' => 'section_style_label_typography',
                'label' => esc_html__( 'Labels typography', 'selection' ),
                'scheme' =>  Typography::TYPOGRAPHY_1,
                'selector' => "{{WRAPPER}} .mdp-countdowner-elementor--wrapper-item-label",
            ]
        );

        $this->start_controls_tabs( 'section_style_items_style_tabs' );

        $this->start_controls_tab( 'section_style_items_normal_style_tab', ['label' => esc_html__( 'NORMAL', 'selection' )] );

        $this->add_control(
            'section_style_items_normal_digits_color',
            [
                'label' => esc_html__( 'Digits color', 'selection' ),
                'type' => Controls_Manager::COLOR,
                'scheme' => [
                    'type' => Color::get_type(),
                    'value' => Color::COLOR_3,
                ],
                'selectors' => [
                    "{{WRAPPER}} .mdp-countdowner-elementor--wrapper-item-val, {{WRAPPER}} .mdp-countdowner-elementor--wrapper-digit::before, {{WRAPPER}} .mdp-countdowner-elementor--wrapper-digit::after, {{WRAPPER}} .mdp-countdowner-elementor--wrapper-digit-face-front, {{WRAPPER}} .mdp-countdowner-elementor--wrapper-digit-face-back" => 'color: {{VALUE}}',
                ],
            ]
        );

        $this->add_control(
            'section_style_items_normal_label_color',
            [
                'label' => esc_html__( 'Label color', 'selection' ),
                'type' => Controls_Manager::COLOR,
                'scheme' => [
                    'type' => Color::get_type(),
                    'value' => Color::COLOR_3,
                ],
                'selectors' => [
                    "{{WRAPPER}} .mdp-countdowner-elementor--wrapper-item-label" => 'color: {{VALUE}}',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Background::get_type(),
            [
                'name' => 'section_style_items_normal_background',
                'label' => esc_html__( 'Background type', 'selection' ),
                'types' => [ 'classic', 'gradient', 'video' ],
                'selector' => "{{WRAPPER}} .mdp-countdowner-elementor--wrapper-item",
            ]
        );

        $this->add_control(
            'section_style_items_separate_normal',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => 'section_style_items_border_normal',
                'label' => esc_html__( 'Border Type', 'selection' ),
                'selector' => "{{WRAPPER}} .mdp-countdowner-elementor--wrapper-item",
            ]
        );

        $this->add_responsive_control(
            'section_style_items_border_radius_normal',
            [
                'label' => esc_html__( 'Border radius', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    "{{WRAPPER}} .mdp-countdowner-elementor--wrapper-item" => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => 'section_style_items_box_shadow_normal',
                'label' => esc_html__( 'Box Shadow', 'selection' ),
                'selector' => "{{WRAPPER}} .mdp-countdowner-elementor--wrapper-item",
            ]
        );


        $this->end_controls_tab();

        $this->start_controls_tab( 'section_style_items_hover_style_tab', ['label' => esc_html__( 'HOVER', 'selection' )] );

        $this->add_control(
            'section_style_items_hover_digits_color',
            [
                'label' => esc_html__( 'Digit color', 'selection' ),
                'type' => Controls_Manager::COLOR,
                'scheme' => [
                    'type' => Color::get_type(),
                    'value' => Color::COLOR_3,
                ],
                'selectors' => [
                    "{{WRAPPER}} .mdp-countdowner-elementor--wrapper-item:hover .mdp-countdowner-elementor--wrapper-item-val, {{WRAPPER}} .mdp-countdowner-elementor--wrapper-item:hover .mdp-countdowner-elementor--wrapper-digit::before, {{WRAPPER}} .mdp-countdowner-elementor--wrapper-item:hover .mdp-countdowner-elementor--wrapper-digit::after, {{WRAPPER}} .mdp-countdowner-elementor--wrapper-item:hover .mdp-countdowner-elementor--wrapper-digit .mdp-countdowner-elementor--wrapper-digit-face-front, {{WRAPPER}} .mdp-countdowner-elementor--wrapper-item:hover .mdp-countdowner-elementor--wrapper-digit .mdp-countdowner-elementor--wrapper-digit-face-back" => 'color: {{VALUE}}',
                ],
            ]
        );

        $this->add_control(
            'section_style_items_hover_label_color',
            [
                'label' => esc_html__( 'Label color', 'selection' ),
                'type' => Controls_Manager::COLOR,
                'scheme' => [
                    'type' => Color::get_type(),
                    'value' => Color::COLOR_3,
                ],
                'selectors' => [
                    "{{WRAPPER}} .mdp-countdowner-elementor--wrapper-item:hover .mdp-countdowner-elementor--wrapper-item-label" => 'color: {{VALUE}}',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Background::get_type(),
            [
                'name' => 'section_style_items_background_hover',
                'label' => esc_html__( 'Background type', 'selection' ),
                'types' => [ 'classic', 'gradient', 'video' ],
                'selector' => "{{WRAPPER}} .mdp-countdowner-elementor--wrapper-item:hover",
            ]
        );


        $this->add_control(
            'section_style_items_separate_hover',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => 'section_style_items_border_hover',
                'label' => esc_html__( 'Border Type', 'selection' ),
                'selector' => "{{WRAPPER}} .mdp-countdowner-elementor--wrapper-item:hover",
            ]
        );

        $this->add_responsive_control(
            'section_style_items_border_radius_hover',
            [
                'label' => esc_html__( 'Border radius', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    "{{WRAPPER}} .mdp-countdowner-elementor--wrapper-item:hover" => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => 'section_style_items_box_shadow_hover',
                'label' => esc_html__( 'Box Shadow', 'selection' ),
                'selector' => "{{WRAPPER}} .mdp-countdowner-elementor--wrapper-item:hover",
            ]
        );

        $this->end_controls_tab();

        $this->end_controls_tabs();


        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Flip Animation card.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_flip_animation_card() {
        $this->start_controls_section( 'section_style_flip_animation_card', [
            'label' => esc_html__( 'Flip animation digit block', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE,
            'condition' => [
                'countdown_animation' => 'flip'
            ]
        ] );

        $this->add_responsive_control(
            'section_style_flip_animation_card_width',
            [
                'label' => esc_html__( 'Digit width', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 1,
                        'max' => 500,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 1,
                        'max' => 100,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-countdowner-elementor--wrapper-digit' => 'width: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->add_responsive_control(
            'section_style_flip_animation_card_height',
            [
                'label' => esc_html__( 'Digit height', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 1,
                        'max' => 500,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 1,
                        'max' => 100,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-countdowner-elementor--wrapper-digit' => 'height: {{SIZE}}{{UNIT}};',
                ],
            ]
        );


        $this->start_controls_tabs( 'section_style_flip_card_style_tabs' );

        $this->start_controls_tab( 'section_style_flip_card_normal_style_tab', ['label' => esc_html__( 'NORMAL', 'selection' )] );

        $this->add_control(
            'section_style_flip_card_normal_separator_color',
            [
                'label' => esc_html__( 'Separator color', 'selection' ),
                'type' => Controls_Manager::COLOR,
                'scheme' => [
                    'type' => Color::get_type(),
                    'value' => Color::COLOR_3,
                ],
                'selectors' => [
                    "{{WRAPPER}} .mdp-countdowner-elementor--wrapper-digit:before, .mdp-countdowner-elementor--wrapper-digit-face-back" => 'border-top-color: {{VALUE}} !important',
                    "{{WRAPPER}} .mdp-countdowner-elementor--wrapper-digit:after, .mdp-countdowner-elementor--wrapper-digit-face-front" => 'border-bottom-color: {{VALUE}} !important'
                ],
            ]
        );


        $this->add_group_control(
            Group_Control_Background::get_type(),
            [
                'name' => 'section_style_flip_card_normal_background',
                'label' => esc_html__( 'Background type', 'selection' ),
                'types' => [ 'classic', 'gradient', 'video' ],
                'selector' => "{{WRAPPER}} .mdp-countdowner-elementor--wrapper-digit::before, {{WRAPPER}} .mdp-countdowner-elementor--wrapper-digit::after, {{WRAPPER}} .mdp-countdowner-elementor--wrapper-digit-face-front, {{WRAPPER}} .mdp-countdowner-elementor--wrapper-digit-face-back",
            ]
        );

        $this->add_control(
            'section_style_flip_card_separate_normal',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => 'section_style_flip_card_border_normal',
                'label' => esc_html__( 'Border Type', 'selection' ),
                'selector' => "{{WRAPPER}} .mdp-countdowner-elementor--wrapper-digit:after, {{WRAPPER}} .mdp-countdowner-elementor--wrapper-digit-face-front, {{WRAPPER}} .mdp-countdowner-elementor--wrapper-digit:before, {{WRAPPER}} .mdp-countdowner-elementor--wrapper-digit-face-back ",
            ]
        );

        $this->add_responsive_control(
            'section_style_flip_card_border_radius_normal',
            [
                'label' => esc_html__( 'Border radius', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    "{{WRAPPER}} .mdp-countdowner-elementor--wrapper-digit" => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                    "{{WRAPPER}} .mdp-countdowner-elementor--wrapper-digit:after, {{WRAPPER}} .mdp-countdowner-elementor--wrapper-digit-face-front" => 'border-top-left-radius: {{TOP}}{{UNIT}}; border-top-right-radius: {{RIGHT}}{{UNIT}};',
                    "{{WRAPPER}} .mdp-countdowner-elementor--wrapper-digit:before, {{WRAPPER}} .mdp-countdowner-elementor--wrapper-digit-face-back" => 'border-bottom-right-radius: {{BOTTOM}}{{UNIT}}; border-bottom-left-radius: {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => 'section_style_flip_card_box_shadow_normal',
                'label' => esc_html__( 'Box Shadow', 'selection' ),
                'selector' => "{{WRAPPER}} .mdp-countdowner-elementor--wrapper-digit",
            ]
        );


        $this->end_controls_tab();

        $this->start_controls_tab( 'section_style_flip_card_hover_style_tab', ['label' => esc_html__( 'HOVER', 'selection' )] );

        $this->add_control(
            'section_style_flip_card_hover_separator_color',
            [
                'label' => esc_html__( 'Separator color', 'selection' ),
                'type' => Controls_Manager::COLOR,
                'scheme' => [
                    'type' => Color::get_type(),
                    'value' => Color::COLOR_3,
                ],
                'selectors' => [
                    "{{WRAPPER}} .mdp-countdowner-elementor--wrapper-digit:hover:before, {{WRAPPER}} .mdp-countdowner-elementor--wrapper-digit:hover .mdp-countdowner-elementor--wrapper-digit-face-back" => 'border-top-color: {{VALUE}} !important',
                    "{{WRAPPER}} .mdp-countdowner-elementor--wrapper-digit:hover:after, {{WRAPPER}} .mdp-countdowner-elementor--wrapper-digit:hover .mdp-countdowner-elementor--wrapper-digit-face-front" => 'border-bottom-color: {{VALUE}} !important'
                ],
            ]
        );


        $this->add_group_control(
            Group_Control_Background::get_type(),
            [
                'name' => 'section_style_flip_card_background_hover',
                'label' => esc_html__( 'Background type', 'selection' ),
                'types' => [ 'classic', 'gradient', 'video' ],
                'selector' => "{{WRAPPER}} .mdp-countdowner-elementor--wrapper-digit:hover::before, {{WRAPPER}} .mdp-countdowner-elementor--wrapper-digit:hover::after, {{WRAPPER}} .mdp-countdowner-elementor--wrapper-digit:hover .mdp-countdowner-elementor--wrapper-digit-face-front, {{WRAPPER}} .mdp-countdowner-elementor--wrapper-digit:hover .mdp-countdowner-elementor--wrapper-digit-face-back",
            ]
        );


        $this->add_control(
            'section_style_flip_card_separate_hover',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => 'section_style_flip_card_border_hover',
                'label' => esc_html__( 'Border Type', 'selection' ),
                'selector' => "{{WRAPPER}} .mdp-countdowner-elementor--wrapper-digit:hover::before, {{WRAPPER}} .mdp-countdowner-elementor--wrapper-digit:hover::after, {{WRAPPER}} .mdp-countdowner-elementor--wrapper-digit:hover .mdp-countdowner-elementor--wrapper-digit-face-front, {{WRAPPER}} .mdp-countdowner-elementor--wrapper-digit:hover .mdp-countdowner-elementor--wrapper-digit-face-back",
            ]
        );

        $this->add_responsive_control(
            'section_style_flip_card_border_radius_hover',
            [
                'label' => esc_html__( 'Border radius', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    "{{WRAPPER}} .mdp-countdowner-elementor--wrapper-digit:hover" => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                    "{{WRAPPER}} .mdp-countdowner-elementor--wrapper-digit:hover:after, {{WRAPPER}} .mdp-countdowner-elementor--wrapper-digit:hover .mdp-countdowner-elementor--wrapper-digit-face-front" => 'border-top-left-radius: {{TOP}}{{UNIT}}; border-top-right-radius: {{RIGHT}}{{UNIT}};',
                    "{{WRAPPER}} .mdp-countdowner-elementor--wrapper-digit:hover:before, {{WRAPPER}} .mdp-countdowner-elementor--wrapper-digit:hover .mdp-countdowner-elementor--wrapper-digit-face-back" => 'border-bottom-right-radius: {{BOTTOM}}{{UNIT}}; border-bottom-left-radius: {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => 'section_style_flip_card_box_shadow_hover',
                'label' => esc_html__( 'Box Shadow', 'selection' ),
                'selector' => "{{WRAPPER}} .mdp-countdowner-elementor--wrapper-digit:hover",
            ]
        );

        $this->end_controls_tab();

        $this->end_controls_tabs();

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> circle animation.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function circle_animation_styles() {
        $this->start_controls_section( 'section_styles_circle_animation', [
            'label' => esc_html__( 'Circle animation', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE,
            'condition' => [
                'countdown_animation' => 'circle'
            ]
        ] );

        $this->add_control(
            'section_style_circle_animation_offset_radius',
            [
                'label' => esc_html__( 'Radius', 'selection' ),
                'type' => Controls_Manager::NUMBER,
                'default' => 56,
            ]
        );

        $this->add_control(
            'section_style_circle_animation_line_width',
            [
                'label' => esc_html__( 'Circle line width', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 1,
                        'max' => 500,
                        'step' => 1,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-countdowner-circle-animation' => 'stroke-width: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->add_control(
            'rounded_stroke',
            [
                'label' => esc_html__( 'Rounded stroke linecap', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Yes', 'selection' ),
                'label_off' => esc_html__( 'No', 'selection' ),
                'return_value' => 'yes',
                'default' => 'yes',
            ]
        );

        $this->add_control(
            'section_style_circle_animation_line_width_divider',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        $this->add_control(
            'section_style_circle_animation_circle_color',
            [
                'label' => esc_html__( 'Circle color', 'selection' ),
                'type' => Controls_Manager::COLOR,
                'scheme' => [
                    'type' => Color::get_type(),
                    'value' => Color::COLOR_3,
                ],
                'default' => '#000',
                'selectors' => [
                    "{{WRAPPER}} .mdp-countdowner-circle-animation" => 'stroke: {{VALUE}}',
                ],
            ]

        );

        $this->end_controls_section();
    }


    /**
     * Add widget controls: Style -> Section After Expire Message Styles.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function after_expire_message_styles() {

        $this->start_controls_section( 'section_styles_after_expire_message', [
            'label' => esc_html__( 'After expire message', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE,
            'condition' => [
                'expire_action_type' => 'message'
            ]
        ] );

        $this->generate_margin_padding_controls( 'section_styles_after_expire_message', 'mdp-countdowner-elementor--wrapper-countdowner-after-expire-message' );

        $this->generate_typography_tabs_controls( 'section_styles_after_expire_message', 'mdp-countdowner-elementor--wrapper-countdowner-after-expire-message' );

        $this->end_controls_section();
    }

    /**
     * Add widget controls: Style -> Section Before Expire Message Styles.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function before_expire_message_styles() {

        $this->start_controls_section( 'section_styles_before_expire_message', [
            'label' => esc_html__( 'Before expire message', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE,
            'condition' => [
                'expire_action_type' => 'message'
            ]
        ] );

        $this->generate_margin_padding_controls( 'section_styles_before_expire_message', 'mdp-countdowner-elementor--wrapper-countdowner-before-expire-message' );

        $this->generate_typography_tabs_controls( 'section_styles_before_expire_message', 'mdp-countdowner-elementor--wrapper-countdowner-before-expire-message' );

        $this->end_controls_section();
    }

    /**
     * Add widget controls: Style -> Section Separator Styles.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function separator_styles() {

        $this->start_controls_section( 'section_styles_separator', [
            'label' => esc_html__( 'Separator', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE,
            'condition' => [
                    'show_digits_separator' => 'yes'
            ]
        ] );

        $this->generate_margin_padding_controls( 'section_styles_separator', 'mdp-countdowner-elementor-digits-separator' );


        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name' => 'section_styles_typography',
                'label' => esc_html__( 'Typography', 'selection' ),
                'scheme' =>  Typography::TYPOGRAPHY_1,
                'selector' => "{{WRAPPER}} .mdp-countdowner-elementor-digits-separator",
            ]
        );

        $this->start_controls_tabs( 'section_styles_separator_style_tabs' );

        $this->start_controls_tab( 'section_styles_separator_normal_style_tab', ['label' => esc_html__( 'NORMAL', 'selection' )] );

        $this->add_control(
            'section_styles_separator_normal_text_color',
            [
                'label' => esc_html__( 'Color', 'selection' ),
                'type' => Controls_Manager::COLOR,
                'scheme' => [
                    'type' => Color::get_type(),
                    'value' => Color::COLOR_3,
                ],
                'selectors' => [
                    "{{WRAPPER}} .mdp-countdowner-elementor-digits-separator" => 'color: {{VALUE}}',
                ],
            ]

        );

        $this->add_group_control(
            Group_Control_Text_Shadow::get_type(),
            [
                'name' => 'separator_normal_text_shadow',
                'label' => esc_html__( 'Text Shadow', 'selection' ),
                'selector' => '{{WRAPPER}} .mdp-countdowner-elementor-digits-separator',
            ]
        );

        $this->end_controls_tab();

        $this->start_controls_tab( 'section_styles_separator_hover_style_tab', ['label' => esc_html__( 'HOVER', 'selection' )] );

        $this->add_control(
            'section_styles_separator_hover_color',
            [
                'label' => esc_html__( 'Color', 'selection' ),
                'type' => Controls_Manager::COLOR,
                'scheme' => [
                    'type' => Color::get_type(),
                    'value' => Color::COLOR_3,
                ],
                'selectors' => [
                    "{{WRAPPER}} .mdp-countdowner-elementor-digits-separator:hover" => 'color: {{VALUE}}',
                ],
            ]
        );


        $this->add_group_control(
            Group_Control_Text_Shadow::get_type(),
            [
                'name' => 'separator_hover_text_shadow',
                'label' => esc_html__( 'Text Shadow', 'selection' ),
                'selector' => '{{WRAPPER}} .mdp-countdowner-elementor-digits-separator:hover',
            ]
        );

        $this->end_controls_tab();

        $this->end_controls_tabs();


        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Countdown Before Expire Button Styles.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function countdown_before_expire_button_styles() {

        $this->start_controls_section( 'section_styles_countdown_before_expire_button', [
            'label' => esc_html__( 'Countdown before expire button', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE,
            'condition' => [
                'expire_action_type' => 'button'
            ]
        ] );

        $this->generate_margin_padding_controls( 'section_styles_countdown_before_expire_button', 'mdp-countdowner-elementor--wrapper-before-button' );

        $this->generate_typography_tabs_controls( 'section_styles_countdown_before_expire_button', 'mdp-countdowner-elementor--wrapper-before-button', true );

        $this->end_controls_section();
    }

    /**
     * Add widget controls: Style -> Section Countdown After Expire Button Styles.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function countdown_after_expire_button_styles() {

        $this->start_controls_section( 'section_styles_countdown_after_expire_button', [
            'label' => esc_html__( 'Countdown after expire button', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE,
            'condition' => [
                'expire_action_type' => 'button'
            ]
        ] );

        $this->generate_margin_padding_controls( 'section_styles_countdown_after_expire_button', 'mdp-countdowner-elementor--wrapper-after-button' );

        $this->generate_typography_tabs_controls( 'section_styles_countdown_after_expire_button', 'mdp-countdowner-elementor--wrapper-after-button', true );

        $this->end_controls_section();
    }

    /**
     * Digits block.
     *
     * @access private
     *
     * @param $settings
     *
     * @return void
     */
    private function countdown_digits( $settings ) {
        ?>
        <div class="mdp-countdowner-elementor--wrapper-digits mdp-countdowner-elementor--wrapper-digits-<?php esc_attr_e( $settings['countdown_view'] ); ?>  <?php if( $settings['countdown_animation'] === 'circle' ): ?> mdp-countdowner-elementor--wrapper-animation-circle <?php endif; ?>  <?php if ($settings['countdown_animation'] === 'flip'): ?>mdp-countdowner-elementor--wrapper-digits-flip<?php endif; ?>">
            <?php if ( in_array( 'years', $settings['time_units'] ) ): ?>
                <div class="mdp-countdowner-elementor--wrapper-item <?php if ( $settings['countdown_animation'] === 'flip' ): ?>mdp-countdowner-elementor--wrapper-item-flip<?php endif; ?> mdp-countdowner-elementor--wrapper-item-year">
                <?php if( $settings['countdown_animation'] === 'flip' ): ?>
                    <div class="mdp-countdowner-elementor--wrapper-digit-wrapper">
                        <div class="mdp-countdowner-elementor--wrapper-digit mdp-countdowner-elementor--wrapper-digit-years">
                           <div class="mdp-countdowner-elementor--wrapper-digit-card">
                             <div class="mdp-countdowner-elementor--wrapper-digit-face mdp-countdowner-elementor--wrapper-digit-face-front mdp-countdowner-elementor--wrapper-digit-face-front-years"></div>
                             <div class="mdp-countdowner-elementor--wrapper-digit-face mdp-countdowner-elementor--wrapper-digit-face-back mdp-countdowner-elementor--wrapper-digit-face-back-years"></div>
                        </div>
                     </div>
                    </div>
                    <div class="mdp-countdowner-elementor--wrapper-item-label mdp-countdowner-elementor--wrapper-item-option mdp-countdowner-elementor--wrapper-item-option-<?php esc_attr_e( $settings['countdown_view'] ); ?>  mdp-countdowner-elementor--wrapper-item-years-label"><?php $settings['custom_labels'] === 'yes' ? esc_html_e( $settings['years_custom_label'] ) : esc_html_e( 'Years' ) ?></div>
                <?php else: ?>
                <?php if( $settings['countdown_animation'] === 'circle' ): ?>
                    <svg class="mdp-countdowner-elementor-circle-animation-svg mdp-countdowner-elementor-circle-animation-svg-<?php esc_attr_e( $settings['countdown_view'] ); ?>">
                        <g>
                            <circle class="mdp-countdowner-circle-animation mdp-countdowner-circle-animation-years" <?php if ( $settings['rounded_stroke'] === 'yes' ): ?>style="stroke-linecap: round"<?php endif; ?> r="<?php $settings['section_style_circle_animation_offset_radius'] !== '' ? esc_attr_e( $settings['section_style_circle_animation_offset_radius'] ) : esc_html_e( '0' ); ?>" cy="<?php $settings['section_style_circle_animation_offset_radius'] !== '' ? esc_attr_e( $settings['section_style_circle_animation_offset_radius'] ) : esc_html_e( '0' ); ?>" cx="<?php $settings['section_style_circle_animation_offset_radius'] !== '' ? esc_attr_e( $settings['section_style_circle_animation_offset_radius'] ) : esc_html_e( '0' ); ?>">
                        </g>
                    </svg>
                <?php endif; ?>
                    <div class="mdp-countdowner-elementor--wrapper-item-val mdp-countdowner-elementor--wrapper-item-option mdp-countdowner-elementor--wrapper-item-option-<?php esc_attr_e( $settings['countdown_view'] ); ?> mdp-countdowner-elementor--wrapper-item-year-val">0</div>
                    <div class="mdp-countdowner-elementor--wrapper-item-label mdp-countdowner-elementor--wrapper-item-option mdp-countdowner-elementor--wrapper-item-option-<?php esc_attr_e( $settings['countdown_view'] ); ?> mdp-countdowner-elementor--wrapper-item-year-label"><?php $settings['custom_labels'] === 'yes' ? esc_html_e( $settings['years_custom_label'] ) : esc_html_e( 'Years' ) ?></div>
                <?php endif; ?>
                </div>
                <?php if ( $settings['show_digits_separator'] === 'yes' ): ?>
                    <span class="mdp-countdowner-elementor-digits-separator"><?php esc_html_e( $settings['digits_separator'] ); ?></span>
                <?php endif; ?>
            <?php endif; ?>
            <?php if ( in_array( 'months', $settings['time_units'] ) ): ?>
                <div class="mdp-countdowner-elementor--wrapper-item <?php if ($settings['countdown_animation'] === 'flip'): ?>mdp-countdowner-elementor--wrapper-item-flip<?php endif; ?> mdp-countdowner-elementor--wrapper-item-month">
                <?php if( $settings['countdown_animation'] === 'flip' ): ?>
                <div class="mdp-countdowner-elementor--wrapper-digit-wrapper">
                        <div class="mdp-countdowner-elementor--wrapper-digit mdp-countdowner-elementor--wrapper-digit-months">
                           <div class="mdp-countdowner-elementor--wrapper-digit-card">
                             <div class="mdp-countdowner-elementor--wrapper-digit-face mdp-countdowner-elementor--wrapper-digit-face-front mdp-countdowner-elementor--wrapper-digit-face-front-months"></div>
                             <div class="mdp-countdowner-elementor--wrapper-digit-face mdp-countdowner-elementor--wrapper-digit-face-back mdp-countdowner-elementor--wrapper-digit-face-back-months"></div>
                        </div>
                     </div>
                </div>
                    <div class="mdp-countdowner-elementor--wrapper-item-label mdp-countdowner-elementor--wrapper-item-option mdp-countdowner-elementor--wrapper-item-option-<?php esc_attr_e( $settings['countdown_view'] ); ?> mdp-countdowner-elementor--wrapper-item-months-label"><?php $settings['custom_labels'] === 'yes' ? esc_html_e( $settings['months_custom_label'] ) : esc_html_e( 'Months' ) ?></div>
                <?php else: ?>
                <?php if( $settings['countdown_animation'] === 'circle' ): ?>
                    <svg class="mdp-countdowner-elementor-circle-animation-svg mdp-countdowner-elementor-circle-animation-svg-<?php esc_attr_e( $settings['countdown_view'] ); ?>">
                        <g>
                            <circle class="mdp-countdowner-circle-animation mdp-countdowner-circle-animation-months" <?php if ( $settings['rounded_stroke'] === 'yes' ): ?>style="stroke-linecap: round"<?php endif; ?> r="<?php $settings['section_style_circle_animation_offset_radius'] !== '' ? esc_attr_e( $settings['section_style_circle_animation_offset_radius'] ) : esc_html_e( '0' ); ?>" cy="<?php $settings['section_style_circle_animation_offset_radius'] !== '' ? esc_attr_e( $settings['section_style_circle_animation_offset_radius'] ) : esc_html_e( '0' ); ?>" cx="<?php $settings['section_style_circle_animation_offset_radius'] !== '' ? esc_attr_e( $settings['section_style_circle_animation_offset_radius'] ) : esc_html_e( '0' ); ?>">
                        </g>
                    </svg>
                <?php endif; ?>
                    <div class="mdp-countdowner-elementor--wrapper-item-val mdp-countdowner-elementor--wrapper-item-option mdp-countdowner-elementor--wrapper-item-option-<?php esc_attr_e( $settings['countdown_view'] ); ?> mdp-countdowner-elementor--wrapper-item-month-val">0</div>
                    <div class="mdp-countdowner-elementor--wrapper-item-label mdp-countdowner-elementor--wrapper-item-option mdp-countdowner-elementor--wrapper-item-option-<?php esc_attr_e( $settings['countdown_view'] ); ?> mdp-countdowner-elementor--wrapper-item-month-label"><?php $settings['custom_labels'] === 'yes' ? esc_html_e( $settings['months_custom_label'] ) : esc_html_e( 'Months' ) ?></div>
                <?php endif; ?>
                </div>
                <?php if ( $settings['show_digits_separator'] === 'yes' ): ?>
                    <span class="mdp-countdowner-elementor-digits-separator"><?php esc_html_e( $settings['digits_separator'] ); ?></span>
                <?php endif; ?>
            <?php endif; ?>
            <?php if ( in_array( 'weeks', $settings['time_units'] ) ): ?>
                <div class="mdp-countdowner-elementor--wrapper-item <?php if ($settings['countdown_animation'] === 'flip'): ?>mdp-countdowner-elementor--wrapper-item-flip<?php endif; ?> mdp-countdowner-elementor--wrapper-item-weeks">
                <?php if( $settings['countdown_animation'] === 'flip' ): ?>
                 <div class="mdp-countdowner-elementor--wrapper-digit-wrapper">
                        <div class="mdp-countdowner-elementor--wrapper-digit mdp-countdowner-elementor--wrapper-digit-weeks">
                           <div class="mdp-countdowner-elementor--wrapper-digit-card">
                             <div class="mdp-countdowner-elementor--wrapper-digit-face mdp-countdowner-elementor--wrapper-digit-face-front mdp-countdowner-elementor--wrapper-digit-face-front-weeks"></div>
                             <div class="mdp-countdowner-elementor--wrapper-digit-face mdp-countdowner-elementor--wrapper-digit-face-back mdp-countdowner-elementor--wrapper-digit-face-back-weeks"></div>
                        </div>
                     </div>
                 </div>
                     <div class="mdp-countdowner-elementor--wrapper-item-label mdp-countdowner-elementor--wrapper-item-option mdp-countdowner-elementor--wrapper-item-option-<?php esc_attr_e( $settings['countdown_view'] ); ?> mdp-countdowner-elementor--wrapper-item-weeks-label"><?php $settings['custom_labels'] === 'yes' ? esc_html_e( $settings['weeks_custom_label'] ) : esc_html_e( 'Weeks' ) ?></div>
                <?php else: ?>
                <?php if( $settings['countdown_animation'] === 'circle' ): ?>
                    <svg class="mdp-countdowner-elementor-circle-animation-svg mdp-countdowner-elementor-circle-animation-svg-<?php esc_attr_e( $settings['countdown_view'] ); ?>">
                        <g>
                            <circle class="mdp-countdowner-circle-animation mdp-countdowner-circle-animation-weeks" <?php if ( $settings['rounded_stroke'] === 'yes' ): ?>style="stroke-linecap: round"<?php endif; ?> r="<?php $settings['section_style_circle_animation_offset_radius'] !== '' ? esc_attr_e( $settings['section_style_circle_animation_offset_radius'] ) : esc_html_e( '0' ); ?>" cy="<?php $settings['section_style_circle_animation_offset_radius'] !== '' ? esc_attr_e( $settings['section_style_circle_animation_offset_radius'] ) : esc_html_e( '0' ); ?>" cx="<?php $settings['section_style_circle_animation_offset_radius'] !== '' ? esc_attr_e( $settings['section_style_circle_animation_offset_radius'] ) : esc_html_e( '0' ); ?>">
                        </g>
                    </svg>
                <?php endif; ?>
                    <div class="mdp-countdowner-elementor--wrapper-item-val mdp-countdowner-elementor--wrapper-item-option mdp-countdowner-elementor--wrapper-item-option-<?php esc_attr_e( $settings['countdown_view'] ); ?> mdp-countdowner-elementor--wrapper-item-weeks-val">0</div>
                    <div class="mdp-countdowner-elementor--wrapper-item-label mdp-countdowner-elementor--wrapper-item-option mdp-countdowner-elementor--wrapper-item-option-<?php esc_attr_e( $settings['countdown_view'] ); ?> mdp-countdowner-elementor--wrapper-item-weeks-label"><?php $settings['custom_labels'] === 'yes' ? esc_html_e( $settings['weeks_custom_label'] ) : esc_html_e( 'Weeks' ) ?></div>
                    <?php endif; ?>
                </div>
                <?php if ( $settings['show_digits_separator'] === 'yes' ): ?>
                    <span class="mdp-countdowner-elementor-digits-separator"><?php esc_html_e( $settings['digits_separator'] ); ?></span>
                <?php endif; ?>
            <?php endif; ?>
            <?php if ( in_array( 'days', $settings['time_units'] ) ): ?>
                <div class="mdp-countdowner-elementor--wrapper-item <?php if ($settings['countdown_animation'] === 'flip'): ?>mdp-countdowner-elementor--wrapper-item-flip<?php endif; ?> mdp-countdowner-elementor--wrapper-item-days">
                <?php if( $settings['countdown_animation'] === 'flip' ): ?>
                    <div class="mdp-countdowner-elementor--wrapper-digit-wrapper">
                        <div class="mdp-countdowner-elementor--wrapper-digit mdp-countdowner-elementor--wrapper-digit-days">
                           <div class="mdp-countdowner-elementor--wrapper-digit-card">
                             <div class="mdp-countdowner-elementor--wrapper-digit-face mdp-countdowner-elementor--wrapper-digit-face-front mdp-countdowner-elementor--wrapper-digit-face-front-days"></div>
                             <div class="mdp-countdowner-elementor--wrapper-digit-face mdp-countdowner-elementor--wrapper-digit-face-back mdp-countdowner-elementor--wrapper-digit-face-back-days"></div>
                        </div>
                     </div>
                    </div>
                    <div class="mdp-countdowner-elementor--wrapper-item-label mdp-countdowner-elementor--wrapper-item-option mdp-countdowner-elementor--wrapper-item-option-<?php esc_attr_e( $settings['countdown_view'] ); ?> mdp-countdowner-elementor--wrapper-item-days-label"><?php $settings['custom_labels'] === 'yes' ? esc_html_e( $settings['days_custom_label'] ) : esc_html_e( 'Days' ) ?></div>
                <?php else: ?>
                <?php if( $settings['countdown_animation'] === 'circle' ): ?>
                    <svg class="mdp-countdowner-elementor-circle-animation-svg mdp-countdowner-elementor-circle-animation-svg-<?php esc_attr_e( $settings['countdown_view'] ); ?>">
                        <g>
                            <circle class="mdp-countdowner-circle-animation mdp-countdowner-circle-animation-days" <?php if ( $settings['rounded_stroke'] === 'yes' ): ?>style="stroke-linecap: round"<?php endif; ?> r="<?php $settings['section_style_circle_animation_offset_radius'] !== '' ? esc_attr_e( $settings['section_style_circle_animation_offset_radius'] ) : esc_html_e( '0' ); ?>" cy="<?php $settings['section_style_circle_animation_offset_radius'] !== '' ? esc_attr_e( $settings['section_style_circle_animation_offset_radius'] ) : esc_html_e( '0' ); ?>" cx="<?php $settings['section_style_circle_animation_offset_radius'] !== '' ? esc_attr_e( $settings['section_style_circle_animation_offset_radius'] ) : esc_html_e( '0' ); ?>">
                        </g>
                    </svg>
                <?php endif; ?>
                    <div class="mdp-countdowner-elementor--wrapper-item-val mdp-countdowner-elementor--wrapper-item-option mdp-countdowner-elementor--wrapper-item-option-<?php esc_attr_e( $settings['countdown_view'] ); ?> mdp-countdowner-elementor--wrapper-item-days-val">0</div>
                    <div class="mdp-countdowner-elementor--wrapper-item-label mdp-countdowner-elementor--wrapper-item-option mdp-countdowner-elementor--wrapper-item-option-<?php esc_attr_e( $settings['countdown_view'] ); ?> mdp-countdowner-elementor--wrapper-item-days-label"><?php $settings['custom_labels'] === 'yes' ? esc_html_e( $settings['days_custom_label'] ) : esc_html_e( 'Days' ) ?></div>
                    <?php endif; ?>
                </div>
                <?php if ( $settings['show_digits_separator'] === 'yes' ): ?>
                    <span class="mdp-countdowner-elementor-digits-separator"><?php esc_html_e( $settings['digits_separator'] ); ?></span>
                <?php endif; ?>
            <?php endif; ?>
            <?php if ( in_array( 'hours', $settings['time_units'] ) ): ?>
                <div class="mdp-countdowner-elementor--wrapper-item <?php if ($settings['countdown_animation'] === 'flip'): ?>mdp-countdowner-elementor--wrapper-item-flip<?php endif; ?> mdp-countdowner-elementor--wrapper-item-hours">
                <?php if( $settings['countdown_animation'] === 'flip' ): ?>
                 <div class="mdp-countdowner-elementor--wrapper-digit-wrapper">
                        <div class="mdp-countdowner-elementor--wrapper-digit mdp-countdowner-elementor--wrapper-digit-hours">
                           <div class="mdp-countdowner-elementor--wrapper-digit-card">
                             <div class="mdp-countdowner-elementor--wrapper-digit-face mdp-countdowner-elementor--wrapper-digit-face-front mdp-countdowner-elementor--wrapper-digit-face-front-hours"></div>
                             <div class="mdp-countdowner-elementor--wrapper-digit-face mdp-countdowner-elementor--wrapper-digit-face-back mdp-countdowner-elementor--wrapper-digit-face-back-hours"></div>
                        </div>
                     </div>
                 </div>
                 <div class="mdp-countdowner-elementor--wrapper-item-label mdp-countdowner-elementor--wrapper-item-option mdp-countdowner-elementor--wrapper-item-option-<?php esc_attr_e( $settings['countdown_view'] ); ?> mdp-countdowner-elementor--wrapper-item-hours-label"><?php $settings['custom_labels'] === 'yes' ? esc_html_e( $settings['hours_custom_label'] ) : esc_html_e( 'Hours' ) ?></div>
                <?php else: ?>
                <?php if( $settings['countdown_animation'] === 'circle' ): ?>
                    <svg class="mdp-countdowner-elementor-circle-animation-svg mdp-countdowner-elementor-circle-animation-svg-<?php esc_attr_e( $settings['countdown_view'] ); ?>">
                        <g>
                            <circle class="mdp-countdowner-circle-animation mdp-countdowner-circle-animation-hours" <?php if ( $settings['rounded_stroke'] === 'yes' ): ?>style="stroke-linecap: round"<?php endif; ?> r="<?php $settings['section_style_circle_animation_offset_radius'] !== '' ? esc_attr_e( $settings['section_style_circle_animation_offset_radius'] ) : esc_html_e( '0' ); ?>" cy="<?php $settings['section_style_circle_animation_offset_radius'] !== '' ? esc_attr_e( $settings['section_style_circle_animation_offset_radius'] ) : esc_html_e( '0' ); ?>" cx="<?php $settings['section_style_circle_animation_offset_radius'] !== '' ? esc_attr_e( $settings['section_style_circle_animation_offset_radius'] ) : esc_html_e( '0' ); ?>">
                        </g>
                    </svg>
                <?php endif; ?>
                    <div class="mdp-countdowner-elementor--wrapper-item-val mdp-countdowner-elementor--wrapper-item-option mdp-countdowner-elementor--wrapper-item-option-<?php esc_attr_e( $settings['countdown_view'] ); ?> mdp-countdowner-elementor--wrapper-item-hours-val">0</div>
                    <div class="mdp-countdowner-elementor--wrapper-item-label mdp-countdowner-elementor--wrapper-item-option mdp-countdowner-elementor--wrapper-item-option-<?php esc_attr_e( $settings['countdown_view'] ); ?> mdp-countdowner-elementor--wrapper-item-hours-label"><?php $settings['custom_labels'] === 'yes' ? esc_html_e( $settings['hours_custom_label'] ) : esc_html_e( 'Hours' ) ?></div>
                    <?php endif; ?>
                </div>
                <?php if ( $settings['show_digits_separator'] === 'yes' ): ?>
                    <span class="mdp-countdowner-elementor-digits-separator"><?php esc_html_e( $settings['digits_separator'] ); ?></span>
                <?php endif; ?>
            <?php endif; ?>
            <?php if ( in_array( 'minutes', $settings['time_units'] ) ): ?>
                <div class="mdp-countdowner-elementor--wrapper-item <?php if ($settings['countdown_animation'] === 'flip'): ?>mdp-countdowner-elementor--wrapper-item-flip<?php endif; ?> mdp-countdowner-elementor--wrapper-item-minutes">
                   <?php if( $settings['countdown_animation'] === 'flip' ): ?>
                    <div class="mdp-countdowner-elementor--wrapper-digit-wrapper">
                        <div class="mdp-countdowner-elementor--wrapper-digit mdp-countdowner-elementor--wrapper-digit-minutes">
                           <div class="mdp-countdowner-elementor--wrapper-digit-card">
                             <div class="mdp-countdowner-elementor--wrapper-digit-face mdp-countdowner-elementor--wrapper-digit-face-front mdp-countdowner-elementor--wrapper-digit-face-front-minutes"></div>
                             <div class="mdp-countdowner-elementor--wrapper-digit-face mdp-countdowner-elementor--wrapper-digit-face-back mdp-countdowner-elementor--wrapper-digit-face-back-minutes"></div>
                        </div>
                     </div>
                     </div>
                      <div class="mdp-countdowner-elementor--wrapper-item-label mdp-countdowner-elementor--wrapper-item-option mdp-countdowner-elementor--wrapper-item-option-<?php esc_attr_e( $settings['countdown_view'] ); ?> mdp-countdowner-elementor--wrapper-item-minutes-label"><?php $settings['custom_labels'] === 'yes' ? esc_html_e( $settings['minutes_custom_label'] ) : esc_html_e( 'Minutes' ) ?></div>
                    <?php else: ?>
                    <?php if( $settings['countdown_animation'] === 'circle' ): ?>
                       <svg class="mdp-countdowner-elementor-circle-animation-svg mdp-countdowner-elementor-circle-animation-svg-<?php esc_attr_e( $settings['countdown_view'] ); ?>">
                           <g>
                               <circle class="mdp-countdowner-circle-animation mdp-countdowner-circle-animation-minutes" <?php if ( $settings['rounded_stroke'] === 'yes' ): ?>style="stroke-linecap: round"<?php endif; ?> r="<?php $settings['section_style_circle_animation_offset_radius'] !== '' ? esc_attr_e( $settings['section_style_circle_animation_offset_radius'] ) : esc_html_e( '0' ); ?>" cy="<?php $settings['section_style_circle_animation_offset_radius'] !== '' ? esc_attr_e( $settings['section_style_circle_animation_offset_radius'] ) : esc_html_e( '0' ); ?>" cx="<?php $settings['section_style_circle_animation_offset_radius'] !== '' ? esc_attr_e( $settings['section_style_circle_animation_offset_radius'] ) : esc_html_e( '0' ); ?>">
                           </g>
                       </svg>
                    <?php endif; ?>
                    <div class="mdp-countdowner-elementor--wrapper-item-val mdp-countdowner-elementor--wrapper-item-option mdp-countdowner-elementor--wrapper-item-option-<?php esc_attr_e( $settings['countdown_view'] ); ?> mdp-countdowner-elementor--wrapper-item-minutes-val">0</div>
                    <div class="mdp-countdowner-elementor--wrapper-item-label mdp-countdowner-elementor--wrapper-item-option mdp-countdowner-elementor--wrapper-item-option-<?php esc_attr_e( $settings['countdown_view'] ); ?> mdp-countdowner-elementor--wrapper-item-minutes-label"><?php $settings['custom_labels'] === 'yes' ? esc_html_e( $settings['minutes_custom_label'] ) : esc_html_e( 'Minutes' ) ?></div>
                    <?php endif; ?>
                </div>
                <?php if ($settings['show_digits_separator'] === 'yes'): ?>
                    <span class="mdp-countdowner-elementor-digits-separator"><?php esc_html_e( $settings['digits_separator'] ); ?></span>
                <?php endif; ?>
            <?php endif; ?>
            <?php if ( in_array( 'seconds', $settings['time_units'] ) ): ?>
                <div class="mdp-countdowner-elementor--wrapper-item <?php if ($settings['countdown_animation'] === 'flip'): ?>mdp-countdowner-elementor--wrapper-item-flip<?php endif; ?> mdp-countdowner-elementor--wrapper-item-seconds">
                    <?php if( $settings['countdown_animation'] === 'flip' ): ?>
                     <div class="mdp-countdowner-elementor--wrapper-digit-wrapper">
                        <div class="mdp-countdowner-elementor--wrapper-digit mdp-countdowner-elementor--wrapper-digit-seconds">
                           <div class="mdp-countdowner-elementor--wrapper-digit-card mdp-countdowner-elementor--wrapper-digit-card">
                             <div class="mdp-countdowner-elementor--wrapper-digit-face mdp-countdowner-elementor--wrapper-digit-face-front mdp-countdowner-elementor--wrapper-digit-face-front-seconds"></div>
                             <div class="mdp-countdowner-elementor--wrapper-digit-face mdp-countdowner-elementor--wrapper-digit-face-back mdp-countdowner-elementor--wrapper-digit-face-back-seconds"></div>
                        </div>
                     </div>
                     </div>
                      <div class="mdp-countdowner-elementor--wrapper-item-label mdp-countdowner-elementor--wrapper-item-option mdp-countdowner-elementor--wrapper-item-option-<?php esc_attr_e( $settings['countdown_view'] ); ?> mdp-countdowner-elementor--wrapper-item-seconds-label"><?php $settings['custom_labels'] === 'yes' ? esc_html_e( $settings['seconds_custom_label'] ) : esc_html_e( 'Seconds' ) ?></div>
                     <?php else: ?>
                    <?php if( $settings['countdown_animation'] === 'circle' ): ?>
                     <svg class="mdp-countdowner-elementor-circle-animation-svg mdp-countdowner-elementor-circle-animation-svg-<?php esc_attr_e( $settings['countdown_view'] ); ?>">
                          <g>
                              <circle class="mdp-countdowner-circle-animation mdp-countdowner-circle-animation-seconds" <?php if ( $settings['rounded_stroke'] === 'yes' ): ?>style="stroke-linecap: round"<?php endif; ?> r="<?php $settings['section_style_circle_animation_offset_radius'] !== '' ? esc_attr_e( $settings['section_style_circle_animation_offset_radius'] ) : esc_html_e( '0' ); ?>" cy="<?php $settings['section_style_circle_animation_offset_radius'] !== '' ? esc_attr_e( $settings['section_style_circle_animation_offset_radius'] ) : esc_html_e( '0' ); ?>" cx="<?php $settings['section_style_circle_animation_offset_radius'] !== '' ? esc_attr_e( $settings['section_style_circle_animation_offset_radius'] ) : esc_html_e( '0' ); ?>">
                          </g>
                     </svg>
                    <?php endif; ?>
                        <div class="mdp-countdowner-elementor--wrapper-item-val mdp-countdowner-elementor--wrapper-item-option mdp-countdowner-elementor--wrapper-item-option-<?php esc_attr_e( $settings['countdown_view'] ); ?> mdp-countdowner-elementor--wrapper-item-seconds-val">0</div>
                        <div class="mdp-countdowner-elementor--wrapper-item-label mdp-countdowner-elementor--wrapper-item-option mdp-countdowner-elementor--wrapper-item-option-<?php esc_attr_e( $settings['countdown_view'] ); ?> mdp-countdowner-elementor--wrapper-item-seconds-label"><?php $settings['custom_labels'] === 'yes' ? esc_html_e( $settings['seconds_custom_label'] ) : esc_html_e( 'Seconds' ) ?></div>
                    <?php endif; ?>
                </div>
                <?php if ($settings['show_digits_separator'] === 'yes'): ?>
                    <span class="mdp-countdowner-elementor-digits-separator"><?php esc_html_e( $settings['digits_separator'] ); ?></span>
                <?php endif; ?>
            <?php endif; ?>
        </div>
        <?php
    }

    /**
     * Count interval for evergreen timer.
     *
     * @access private
     *
     * @param $settings
     *
     * @return float|int
     */
    private function count_interval_for_evergreen_timer( $settings ) {
        $evergreen_years = $settings['evergreen_timer_years'] * YEAR_IN_SECONDS;
        $evergreen_months = $settings['evergreen_timer_months'] * MONTH_IN_SECONDS;
        $evergreen_weeks = $settings['evergreen_timer_weeks'] * WEEK_IN_SECONDS;
        $evergreen_days = $settings['evergreen_timer_days'] * DAY_IN_SECONDS;
        $evergreen_hours = $settings['evergreen_timer_hours'] * HOUR_IN_SECONDS;
        $evergreen_minutes = $settings['evergreen_timer_minutes'] * MINUTE_IN_SECONDS;
        $evergreen_seconds = $settings['evergreen_timer_seconds'];

        return $evergreen_years + $evergreen_months + $evergreen_weeks + $evergreen_days + $evergreen_hours + $evergreen_minutes + $evergreen_seconds;
    }

    /**
     * Render Frontend Output. Generate the final HTML on the frontend.
     *
     * @access protected
     *
     * @return void
     **/
    protected function render() {

        $settings = $this->get_settings_for_display();

        $time_units_string = implode( ',', $settings['time_units'] );

        $current_wp_time = current_time( 'Y m d H i s' );


        ?>
        <!-- Start Countdowner for Elementor WordPress Plugin -->
        <div class="mdp-countdowner-elementor-box"
             <?php if( $settings['countdown_type'] === 'evergreen-timer' ): ?>
             data-evergreen-seconds="<?php echo esc_attr( $this->count_interval_for_evergreen_timer( $settings ) ); ?>"
             <?php endif; ?>
             data-countdown-type="<?php echo esc_attr( $settings['countdown_type'] ); ?>"
             data-start="<?php echo esc_attr( $settings['countdown_start_date'] ); ?>"
             data-time="<?php echo  esc_attr( $settings['countdown_due_date'] ); ?>"
             data-units="<?php echo esc_attr( $time_units_string ); ?>"
             data-animation="<?php echo esc_attr( $settings['countdown_animation'] ); ?>"
             data-expire-action = "<?php echo esc_attr( $settings['expire_action_type'] ); ?>"
             data-wp-time = "<?php echo esc_attr( $current_wp_time ); ?>"
             data-timezone = "<?php echo esc_attr( $settings['countdown_time_zone'] ); ?>"
             data-evergreen-years = "<?php echo esc_attr( $settings['evergreen_timer_years'] ); ?>"
             data-redirect-link = "<?php echo esc_attr( $settings['redirection_link']['url'] ); ?>"
             data-countdown-view = "<?php echo esc_attr( $settings['countdown_view'] ); ?>"
             data-editor-mode = "<?php echo esc_attr( \Elementor\Plugin::$instance->editor->is_edit_mode() ); ?>"
             data-radius = "<?php echo esc_attr( $settings['section_style_circle_animation_offset_radius'] ); ?>"
             <?php if( $settings['countdown_type'] === 'evergreen-timer' ): ?>
             data-reset-timer-on-expire = "<?php echo esc_attr( $settings['reset_timer_on_expire'] ); ?>"
             <?php endif; ?>
            >
            <?php if( $settings['expire_action_type'] === 'message' ): ?>

                <div class="mdp-countdowner-elementor--wrapper-countdowner-after-expire-message" style="display: none">
                    <?php echo wp_kses_post( $settings['after_countdown_message'] ); ?>
                </div>
            <?php endif; ?>

            <?php if( $settings['expire_action_type'] === 'button' ): ?>
            <div class="mdp-countdowner-elementor--wrapper-after-button-wrapper">
                <a href="<?php echo esc_url( $settings['countdown_after_expire_button_link']['url'] ) ?>" class="mdp-countdowner-elementor--wrapper-after-button mdp-countdowner-elementor--wrapper-btn" style="display: none">
                    <?php esc_html_e( $settings['after_countdown_text_button'] ); ?>
                </a>
            </div>
            <?php endif; ?>

            <div class="mdp-countdowner-elementor--wrapper">
                <?php if( $settings['expire_action_type'] === 'message' ): ?>
                    <div class="mdp-countdowner-elementor--wrapper-countdowner-before-expire-message">
                        <?php echo wp_kses_post( $settings['before_countdown_message'] ); ?>
                    </div>
                <?php endif; ?>

                <?php $this->countdown_digits( $settings ) ?>

                <?php if( $settings['expire_action_type'] === 'button' ): ?>
                    <div class="mdp-countdowner-elementor--wrapper-before-button-wrapper">
                        <a href="<?php echo esc_url( $settings['countdown_before_expire_button_link']['url'] ) ?>" class="mdp-countdowner-elementor--wrapper-before-button mdp-countdowner-elementor--wrapper-btn" ><?php esc_html_e( $settings['before_countdown_text_button'] ); ?></a>
                    </div>
                <?php endif; ?>
            </div>
        </div>
        <script>
            var wpGlobalTime = '<?php echo esc_attr( $current_wp_time ); ?>';
        </script>
        <!-- End Countdowner for Elementor WordPress Plugin -->
        <?php  if ( is_admin() ) {
            $widget_hash = substr( hash( 'ripemd160', date('l jS \of F Y h:i:s A') ), rand( 0 , 20 ), 3 ) . rand( 11 , 99 ); ?>
            <script>
                try {
                    countdownerReady<?php esc_attr_e( $widget_hash ); ?>( mdpCountdowner.addCountdowner.bind( mdpCountdowner ) );
                } catch ( msg ) {
                    const countdownerReady<?php esc_attr_e( $widget_hash ); ?> = ( callback ) => {
                        'loading' !== document.readyState ?
                            callback() :
                            document.addEventListener( 'DOMContentLoaded', callback );
                    };
                    countdownerReady<?php esc_attr_e( $widget_hash ); ?>( mdpCountdowner.addCountdowner.bind( mdpCountdowner ) );
                }
            </script>
	    <?php
        }
    }

    /**
     * Return link for documentation
     * Used to add stuff after widget
     *
     * @access public
     *
     * @return string
     **/
    public function get_custom_help_url() {

        return 'https://docs.merkulov.design/tag/countdowner';

    }

}
