<?php /** @noinspection PhpUndefinedClassInspection */
/**
 * Highlighter for Elementor
 * Highlighted headings widget for Elementor
 * Exclusively on https://1.envato.market/highlighter-elementor
 *
 * @encoding        UTF-8
 * @version         {{plugins_version}}
 * @copyright       (C) 2018 - 2023 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Cherviakov Vlad (vladchervjakov@gmail.com), Dmitry Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

namespace Merkulove\Selection;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
    header( 'Status: 403 Forbidden' );
    header( 'HTTP/1.1 403 Forbidden' );
    exit;
}

use Exception;
use Elementor\Controls_Manager;
use Elementor\Group_Control_Background;
use Elementor\Group_Control_Text_Shadow;
use Elementor\Group_Control_Typography;
use Elementor\Widget_Base;
use Merkulove\Selection\Unity\Plugin as UnityPlugin;

/** @noinspection PhpUnused */
/**
 * Highlighter - Custom Elementor Widget.
 **/
class highlighter_elementor extends Widget_Base {

    /**
     * Use this to sort widgets.
     * A smaller value means earlier initialization of the widget.
     * Can take negative values.
     * Default widgets and widgets from 3rd party developers have 0 $mdp_order
     **/
    public $mdp_order = 1;

    /**
     * Widget base constructor.
     * Initializing the widget base class.
     *
     * @access public
     * @throws Exception If arguments are missing when initializing a full widget instance.
     * @param array      $data Widget data. Default is an empty array.
     * @param array|null $args Optional. Widget default arguments. Default is null.
     *
     * @return void
     **/
    public function __construct( $data = [], $args = null ) {

        parent::__construct( $data, $args );

        wp_register_style( 'mdp-selection-elementor-admin', UnityPlugin::get_url() . 'css/elementor-admin' . UnityPlugin::get_suffix() . '.css', [], UnityPlugin::get_version() );
        wp_register_style( 'mdp-highlighter-elementor', UnityPlugin::get_url() . 'css/highlighter-elementor' . UnityPlugin::get_suffix() . '.css', [], UnityPlugin::get_version() );

    }

    /**
     * Return a widget name.
     *
     * @return string
     **/
    public function get_name() {

        return 'mdp-highlighter-elementor';

    }

    /**
     * Return the widget title that will be displayed as the widget label.
     *
     * @return string
     **/
    public function get_title() {

        return esc_html__( 'Text Highlight', 'selection' );

    }

    /**
     * Set the widget icon.
     *
     * @return string
     */
    public function get_icon() {

        return 'mdp-highlighter-elementor-widget-icon';

    }

    /**
     * Set the category of the widget.
     *
     * @return array with category names
     **/
    public function get_categories() {

        return [ 'selection-category' ];

    }

    /**
     * Get widget keywords. Retrieve the list of keywords the widget belongs to.
     *
     * @access public
     *
     * @return array Widget keywords.
     **/
    public function get_keywords() {

        return [ 'Merkulove', 'Text Highlight', 'Animation' ];

    }

    /**
     * Get style dependencies.
     * Retrieve the list of style dependencies the widget requires.
     *
     * @access public
     *
     * @return array Widget styles dependencies.
     **/
    public function get_style_depends() {

        return [ 'mdp-highlighter-elementor', 'mdp-selection-elementor-admin' ];

    }

	/**
	 * Get script dependencies.
	 * Retrieve the list of script dependencies the element requires.
	 *
	 * @access public
     *
	 * @return array Element scripts dependencies.
	 **/
	public function get_script_depends() {

		return [];

    }

    /**
     * Section Highlighter in the Content Tab
     */
    private function content_section_highlighter() {

        /** Highlighter section */
        $this->start_controls_section( 'content_section_highlighter', [ 'label' => esc_html__( 'Text Highlight', 'selection' ) ] );

        /** Highlighter animation. */
        $this->add_control(
            'animation_header',
            [
                'label' => esc_html__( 'Animation', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'options' => [
                    'circle' => 'Circle',
                    'ellipse' => 'Ellipse',
                    'frame' => 'Frame',
                    'wave' => 'Wave',
                    'left-shading' => 'Left shading',
                    'right-shading' => 'Right shading',
                    'underline' => 'Underline',
                    'overline' => 'Overline',
                    'wrap' => 'Wrap',
                    'diagonal-bottom' => 'Diagonal from bottom',
                    'diagonal-top' => 'Diagonal from top',
                    'strikethrough' => 'Strikethrough',
                    'cross' => 'Crossing',
                    'fill' => 'Fill',
                    'double-underline' => 'Double underline',
                    'zigzag' => 'ZigZag',
                    'dash' => 'Dash',
                    'curl' => 'Curl',
                    'curl-wave' => 'Curl-wave',
                    'scallop' => 'Scallop',
                ],
                'default' => 'circle',
                'separator' => 'after'
            ]
        );

        /** Highlighter animation delay */
        $this->add_control(
            'highlighter_delay',
            [
                'label' => esc_html__( 'Animation delay(s)', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 's' ],
                'range' => [
                    's' => [
                        'min' => 0.1,
                        'max' => 10,
                        'step' => 0.1,
                    ],
                ],
                'default' => [
                    'unit' => 's',
                    'size' => 0,
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-highlighter-svg path' => 'animation-delay: {{SIZE}}{{UNIT}};',
                    '{{WRAPPER}} .mdp-highlighter-svg path:nth-child(2)' => 'animation-delay: calc( 1s + {{SIZE}}{{UNIT}} );',
                ],
            ]
        );

        /** Highlighter animation duration */
        $this->add_control(
            'highlighter_duration',
            [
                'label' => esc_html__( 'Scene duration(s)', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 's' ],
                'range' => [
                    's' => [
                        'min' => 0.1,
                        'max' => 30,
                        'step' => 0.1,
                    ],
                ],
                'default' => [
                    'unit' => 's',
                    'size' => 5,
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-highlighter-svg path' => 'animation-duration: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        /** Highlighter lifetime */
        $this->add_control(
            'highlighter_lifetime',
            [
                'label' => esc_html__( 'Animation duration(%)', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ '%' ],
                'range' => [
                    '%' => [
                        'min' => 0,
                        'max' => 100,
                        'step' => 10,
                    ],
                ],
                'default' => [
                    'unit' => '%',
                    'size' => 50,
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-highlighter-svg path' => 'animation-name: mdp-highlighter-{{SIZE}};',
                ],
                'separator' => 'after'
            ]
        );

        /** Highlighter viewport animation */
        $this->add_control(
            'highlighter_viewport',
            [
                'label' => __( 'Animate on viewport', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => __( 'On', 'selection' ),
                'label_off' => __( 'Off', 'selection' ),
                'return_value' => 'on',
                'default' => 'off',
            ]
        );

        /** Highlighter reverse animation */
        $this->add_control(
            'highlighter_reverse',
            [
                'label' => __( 'Reverse animation', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => __( 'On', 'selection' ),
                'label_off' => __( 'Off', 'selection' ),
                'return_value' => 'on',
                'default' => 'off',
            ]
        );

        /** Highlighter loop */
        $this->add_control(
            'highlighter_loop',
            [
                'label' => __( 'Loop animation', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => __( 'On', 'selection' ),
                'label_off' => __( 'Off', 'selection' ),
                'return_value' => 'on',
                'default' => 'on',
            ]
        );

        /** Highlighter animation iteration count */
        $this->add_control(
            'highlighter_iteration',
            [
                'label' => esc_html__( 'Iteration count', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 1,
                        'max' => 10,
                        'step' => 1,
                    ],
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 1,
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-highlighter-svg path' => 'animation-iteration-count: {{SIZE}};',
                ],
                'condition' => [ 'highlighter_loop' => '' ],
                'separator' => 'after'
            ]
        );

        /** End Highlighter section */
        $this->end_controls_section();

    }

    /**
     * Section Highlighter in the Style tab
     */
    private function style_section_highlighter() {

        /** Highlighter section */
        $this->start_controls_section( 'style_section_highlighter', [ 'label' => esc_html__( 'Text Highlight', 'selection' ), 'tab' => Controls_Manager::TAB_STYLE ] );

        /** Highlighter thickness */
        $this->add_responsive_control(
            'highlighter_thickness',
            [
                'label' => esc_html__( 'Line thickness', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%', 'em', 'rem' ],
                'range' => [
                    'px' => [
                        'min' => 1,
                        'max' => 100,
                        'step' => 1,
                    ],
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 8,
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-highlighter-svg path' => 'stroke-width: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        /** Highlighter width */
        $this->add_responsive_control(
            'highlighter_width',
            [
                'label' => esc_html__( 'Width', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%', 'em', 'rem' ],
                'range' => [
                    '%' => [
                        'min' => 1,
                        'max' => 100,
                        'step' => 1,
                    ],
                ],
                'default' => [
                    'unit' => '%',
                    'size' => 100,
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-highlighter-svg' => 'width: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        /** Highlighter height */
        $this->add_responsive_control(
            'highlighter_height',
            [
                'label' => esc_html__( 'Height', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%', 'em', 'rem' ],
                'range' => [
                    '%' => [
                        'min' => 1,
                        'max' => 200,
                        'step' => 1,
                    ],
                ],
                'default' => [
                    'unit' => '%',
                    'size' => 115,
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-highlighter-svg' => 'height: {{SIZE}}{{UNIT}};',
                ],
                'separator' => 'after'
            ]
        );

        /** Highlighter scale */
        $this->add_responsive_control(
            'highlighter_scale',
            [
                'label' => esc_html__( 'Scale', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'scale' ],
                'range' => [
                    'scale' => [
                        'min' => 0.1,
                        'max' => 10,
                        'step' => 0.1,
                    ],
                ],
                'default' => [
                    'unit' => 'scale',
                    'size' => 1,
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-highlighter-svg' => 'transform: translate( {{highlighter_offset_x.SIZE}}{{highlighter_offset_x.UNIT}}, {{highlighter_offset_y.SIZE}}{{highlighter_offset_y.UNIT}} ) rotate( {{SIZE}}{{UNIT}} ) scale( {{SIZE}} );',
                ],
            ]
        );

        /** Highlighter rotation */
        $this->add_responsive_control(
            'highlighter_rotate',
            [
                'label' => esc_html__( 'Rotate', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'deg' ],
                'range' => [
                    'deg' => [
                        'min' => -180,
                        'max' => 180,
                        'step' => 1,
                    ],
                ],
                'default' => [
                    'unit' => 'deg',
                    'size' => 0,
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-highlighter-svg' => 'transform: translate( {{highlighter_offset_x.SIZE}}{{highlighter_offset_x.UNIT}}, {{highlighter_offset_y.SIZE}}{{highlighter_offset_y.UNIT}} ) rotate( {{SIZE}}{{UNIT}} ) scale( {{highlighter_scale.SIZE}} );',
                ],
            ]
        );

        /** Highlighter offset X */
        $this->add_responsive_control(
            'highlighter_offset_x',
            [
                'label' => esc_html__( 'Offset X', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ '%', 'px' ],
                'range' => [
                    '%' => [
                        'min' => -100,
                        'max' => 100,
                        'step' => 1,
                    ],
                    'px' => [
                        'min' => -500,
                        'max' => 500,
                        'step' => 1,
                    ],
                ],
                'default' => [
                    'unit' => '%',
                    'size' => 0,
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-highlighter-svg' => 'transform: translate( {{SIZE}}{{UNIT}}, {{highlighter_offset_y.SIZE}}{{highlighter_offset_y.UNIT}} ) rotate( {{highlighter_rotate.SIZE}}{{highlighter_rotate.UNIT}} ) scale( {{highlighter_scale.SIZE}} );',
                ],
            ]
        );

        /** Highlighter offset Y */
        $this->add_responsive_control(
            'highlighter_offset_y',
            [
                'label' => esc_html__( 'Offset Y', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ '%', 'px' ],
                'range' => [
                    '%' => [
                        'min' => -100,
                        'max' => 100,
                        'step' => 1,
                    ],
                    'px' => [
                        'min' => -500,
                        'max' => 500,
                        'step' => 1,
                    ],
                ],
                'default' => [
                    'unit' => '%',
                    'size' => 0,
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-highlighter-svg' => 'transform: translate( {{highlighter_offset_x.SIZE}}{{highlighter_offset_x.UNIT}}, {{SIZE}}{{UNIT}} ) rotate( {{highlighter_rotate.SIZE}}{{highlighter_rotate.UNIT}} ) scale( {{highlighter_scale.SIZE}} );',
                ],
                'separator' => 'after'
            ]
        );

        /** Highlighter color */
        $this->add_control(
            'highlighter_color',
            [
                'label' => esc_html__( 'Highlighter color', 'selection' ),
                'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .mdp-highlighter-svg path' => 'stroke: {{VALUE}}',
                ],
            ]
        );

        /** Highlighter blend mode */
        $this->add_control(
            'highlighter_blend',
            [
                'label' => esc_html__( 'Blend mode', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'options' => [
                    'normal' => 'Normal',
                    'multiply' => 'Multiply',
                    'screen' => 'Screen',
                    'overlay' => 'Overlay',
                    'darken' => 'Darken',
                    'lighten' => 'Lighten',
                    'color-dodge' => 'Color-dodge',
                    'color-burn' => 'Color-burn',
                    'hard-light' => 'Hard-light',
                    'soft-light' => 'Soft-light',
                    'difference' => 'Difference',
                    'exclusion' => 'Exclusion',
                    'hue' => 'Hue',
                    'saturation' => 'Saturation',
                    'color' => 'Color',
                    'luminosity' => 'Luminosity'
                ],
                'default' => 'normal',
                'selectors' => [
                    '{{WRAPPER}} .mdp-highlighter-svg' => 'mix-blend-mode: {{VALUE}};',
                ],
            ]
        );

        /** Highlighter bring to front */
        $this->add_control(
            'highlighter_foreground',
            [
                'label' => __( 'Foreground', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => __( 'On', 'selection' ),
                'label_off' => __( 'Off', 'selection' ),
                'return_value' => 'on',
                'default' => 'off',
            ]
        );

        /** Highlighter edges shape */
        $this->add_control(
            'highlighter_round',
            [
                'label' => __( 'Round edges', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => __( 'On', 'selection' ),
                'label_off' => __( 'Off', 'selection' ),
                'return_value' => 'on',
                'default' => 'off',
            ]
        );

        /** End Highlighter section */
        $this->end_controls_section();

    }

    /**
     * Add the widget controls.
     *
     * @since 1.0.0
     * @access protected
     *
     * @return void with category names
     **/
    protected function register_controls() {

        $this->content_section_highlighter();

        /** Header tab. */
        $this->start_controls_section( 'section_header', [ 'label' => esc_html__( 'Header', 'selection' ) ] );

        /** Header text. */
        $this->add_control(
            'header_text',
            [
                'label' => esc_html__( 'Text', 'selection' ),
                'type' => Controls_Manager::TEXTAREA,
                'rows' => 10,
                'dynamic' => ['active' => true],
                'default' => esc_html__( '{Title} for Elementor.', 'selection' ),
                'placeholder' => esc_html__( 'Title', 'selection' ),
                'description' => esc_html__( 'The text to be animated is enclosed in braces. Example: Text before {Highlighted Text} text after.', 'selection' ),
            ]
        );

        /** HTML Tag. */
        $this->add_control(
            'header_tag',
            [
                'label' => esc_html__( 'HTML Tag', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'options' => [
                    'h1' => 'H1',
                    'h2' => 'H2',
                    'h3' => 'H3',
                    'h4' => 'H4',
                    'h5' => 'H5',
                    'h6' => 'H6',
                    'div' => 'div',
                    'span' => 'span',
                    'p' => 'p',
                ],
                'default' => 'h2',
            ]
        );

        /** Heading link. */
        $this->add_control(
            'heading_link',
            [
                'label' => esc_html__( 'Link', 'selection' ),
                'type' => Controls_Manager::URL,
                'placeholder' => esc_html__( 'https://codecanyon.net/user/merkulove', 'selection' ),
                'show_external' => true,
                'default' => [
                    'url' => '',
                    'is_external' => false,
                    'nofollow' => false,
                ],
            ]
        );

        /** Header line. */
        $this->add_control(
            'header_line',
            [
                'label' => esc_html__( 'Line', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Show', 'selection' ),
                'label_off' => esc_html__( 'Hide', 'selection' ),
                'return_value' => 'yes',
                'separator' => 'after'
            ]
        );

        /** Split header. */
        $this->add_control(
            'header_split',
            [
                'label' => esc_html__( 'Split header', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Show', 'selection' ),
                'label_off' => esc_html__( 'Hide', 'selection' ),
                'return_value' => 'yes',
            ]
        );

        /** Split text. */
        $this->add_control(
            'split_text',
            [
                'label' => esc_html__( 'Split text', 'selection' ),
                'type' => Controls_Manager::TEXTAREA,
                'rows' => 10,
                'dynamic' => ['active' => true],
                'default' => esc_html__( 'Split text', 'selection' ),
                'placeholder' => esc_html__( 'Split text', 'selection' ),
                'condition'  => ['header_split' => 'yes']
            ]
        );

        /** Show Subheader. */
        $this->add_control(
            'show_subheader',
            [
                'label' => esc_html__( 'Subheader', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Show', 'selection' ),
                'label_off' => esc_html__( 'Hide', 'selection' ),
                'return_value' => 'yes',
            ]
        );

        /** Show Content Header. */
        $this->add_control(
            'show_content_header',
            [
                'label' => esc_html__( 'Additional Header', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Show', 'selection' ),
                'label_off' => esc_html__( 'Hide', 'selection' ),
                'return_value' => 'yes',
            ]
        );

        /** End section content. */
        $this->end_controls_section();

        /** Subheader. */
        $this->start_controls_section( 'section_subheader',
            [
                'label' => esc_html__( 'Subheader', 'selection' ),
                'condition'   => ['show_subheader' => 'yes' ]
            ] );

        /** Header line. */
        $this->add_control(
            'subheader_line',
            [
                'label' => esc_html__( 'Line', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Show', 'selection' ),
                'label_off' => esc_html__( 'Hide', 'selection' ),
                'return_value' => 'yes',
                'separator' => 'after'
            ]
        );

        /** Subheader bottom. */
        $this->add_control(
            'header_bottom',
            [
                'label' => esc_html__( 'Subheader bottom', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Show', 'selection' ),
                'label_off' => esc_html__( 'Hide', 'selection' ),
                'return_value' => 'yes',
            ]
        );

        /** Subheader. */
        $this->add_control(
            'sub_heading',
            [
                'label' => esc_html__( 'Text', 'selection' ),
                'type' => Controls_Manager::TEXTAREA,
                'rows' => 10,
                'dynamic' => ['active' => true],
                'separator' => 'before',
                'default' => esc_html__( 'Subheader', 'selection' ),
                'placeholder' => esc_html__( 'Subheader', 'selection' ),
            ]
        );

        /** End section content. */
        $this->end_controls_section();

        /** Content header. */
        $this->start_controls_section( 'section_content',
            [
                'label' => esc_html__( 'Additional Header', 'selection' ),
                'condition'   => ['show_content_header' => 'yes' ]
            ] );

        /** Sub heading. */
        $this->add_control(
            'text_content_heading',
            [
                'label' => esc_html__( 'Text', 'selection' ),
                'type' => Controls_Manager::TEXTAREA,
                'rows' => 10,
                'dynamic' => ['active' => true],
                'separator' => 'before',
                'default' => esc_html__( 'Additional Header', 'selection' ),
                'placeholder' => esc_html__( 'Additional Header', 'selection' ),
            ]
        );

        /** End section content. */
        $this->end_controls_section();

        $this->style_section_highlighter();

        /** Style header tab. */
        $this->start_controls_section( 'style_header', [ 'label' => esc_html__( 'Header', 'selection' ), 'tab' => Controls_Manager::TAB_STYLE, ] );

        $this->start_controls_tabs('tabs_header_style');

        /** Text style tab. */
        $this->start_controls_tab(
            'tab_style_text',
            [
                'label' => esc_html__('Text style', 'selection')
            ]
        );

        /** Color. */
        $this->add_control(
            'color_header',
            [
                'label' => esc_html__( 'Color', 'selection' ),
                'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .mdp-typography-default-header' => 'color: {{VALUE}}',
                ],
            ]
        );

        /** Typography. */
        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name' => 'header_typography',
                'label' => esc_html__( 'Typography', 'selection' ),
                'selector' => '{{WRAPPER}} .mdp-typography-header',
            ]
        );

        /** Shadow. */
        $this->add_group_control(
            Group_Control_Text_Shadow::get_type(),
            [
                'name' => 'header_shadow',
                'label' => esc_html__( 'Shadow', 'selection' ),
                'selector' => '{{WRAPPER}} .mdp-shadow-header',
            ]
        );

        /** Alignment. */
        $this->add_responsive_control(
            'header_align',
            [
                'label' => esc_html__( 'Alignment', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'fa fa-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'fa fa-align-center',
                    ],
                    'right' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'fa fa-align-right',
                    ],
                ],
                'default' => 'center',
                'selectors' => [
                    '{{WRAPPER}} .mdp-header-align' => 'text-align: {{header_align}};',
                ],
                'toggle' => true,
                'label_block' => false,
            ]
        );

        /** Text animation. */
        $this->add_control(
            'text_animation_header',
            [
                'label' => __( 'Highlighted text', 'selection' ),
                'type' => Controls_Manager::HEADING,
                'separator' => 'before',
            ]
        );

        /** Text animation color. */
        $this->add_control(
            'text_animation_color',
            [
                'label' => esc_html__( 'Color', 'selection' ),
                'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .mdp-highlighter-wrapper' => '-webkit-text-fill-color: {{VALUE}}',
                ],
            ]
        );

        /** Typography. */
        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name' => 'text_animation_typography',
                'label' => esc_html__( 'Typography', 'selection' ),
                'selector' => '{{WRAPPER}} .mdp-highlighter-wrapper',
            ]
        );

        /** Shadow. */
        $this->add_group_control(
            Group_Control_Text_Shadow::get_type(),
            [
                'name' => 'text_animation_shadow',
                'label' => esc_html__( 'Shadow', 'selection' ),
                'selector' => '{{WRAPPER}} .mdp-highlighter-wrapper',
            ]
        );

        /** Custom width */
        $this->add_control(
            'text_animation_custom',
            [
                'label' => esc_html__( 'Auto-width', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'On', 'selection' ),
                'label_off' => esc_html__( 'Off', 'selection' ),
                'return_value' => 'yes',
                'default' => 'yes'
            ]
        );

        /** Box width. */
        $this->add_responsive_control(
            'text_animation_width',
            [
                'label' => esc_html__( 'Element width', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 500,
                        'step' => 1,
                    ]
                ],
                'default' => [ 'unit' => 'px', 'size' => 300, ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-highlighter-wrapper' => 'width: {{SIZE}}{{UNIT}};',
                ],
                'condition' => ['text_animation_custom' => '' ]
            ]
        );

        /** Margin. */
        $this->add_responsive_control(
            'text_animation_margin_all',
            [
                'label' => esc_html__( 'Margin', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => [ 'desktop', 'tablet', 'mobile' ],
                'desktop_default' => [
                    'top' => '',
                    'right' => '',
                    'bottom' => '',
                    'left' => '',
                    'unit' => 'px',
                ],
                'tablet_default' => [
                    'top' => '',
                    'right' => '',
                    'bottom' => '',
                    'left' => '',
                    'unit' => 'px',
                ],
                'mobile_default' => [
                    'top' => '',
                    'right' => '',
                    'bottom' => '',
                    'left' => '',
                    'unit' => 'px',
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-highlighter-wrapper' => 'margin: {{top}}{{unit}} {{right}}{{unit}} {{bottom}}{{unit}} {{left}}{{unit}};',
                ],
            ]
        );

        /** Alignment. */
        $this->add_responsive_control(
            'text_animation_align',
            [
                'label' => esc_html__( 'Alignment', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'fa fa-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'fa fa-align-center',
                    ],
                    'right' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'fa fa-align-right',
                    ],
                ],
                'default' => 'center',
                'selectors' => [
                    '{{WRAPPER}} .mdp-highlighter-wrapper' => 'text-align: {{header_align}};',
                ],
                'toggle' => true,
                'label_block' => false,
            ]
        );

        $this->end_controls_tab();

        /** Box tab. */
        $this->start_controls_tab(
            'tab_style_box',
            [
                'label' => esc_html__('Box style', 'selection')
            ]
        );

        $this->add_group_control(
            Group_Control_Background::get_type(),
            [
                'name' => 'header_background',
                'label' => esc_html__( 'Background', 'selection' ),
                'types' => [ 'classic', 'gradient' ],
                'selector' => '{{WRAPPER}} .mdp-header-before, {{WRAPPER}} .mdp-header-after, {{WRAPPER}} .mdp-highlighter-middle',
            ]
        );

        /** Background effect. */
        $this->add_control(
            'header_background_effect',
            [
                'label' => esc_html__( 'Show background effect', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Show', 'selection' ),
                'label_off' => esc_html__( 'Hide', 'selection' ),
                'return_value' => 'yes',
            ]
        );

        /** Border type. */
        $this->add_control(
            'header_border_type',
            [
                'label' => esc_html__( 'Border type', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'none',
                'options' => [
                    'none'  => esc_html__( 'None', 'selection' ),
                    'double' => esc_html__( 'Double', 'selection' ),
                    'dashed' => esc_html__( 'Dashed', 'selection' ),
                    'dotted' => esc_html__( 'Dotted', 'selection' ),
                    'solid' => esc_html__( 'Solid', 'selection' ),
                    'grove' => esc_html__( 'Grove', 'selection' ),
                ],
                'separator' => 'before',
                'selectors' => [
                    '{{WRAPPER}} .mdp-typography-default-header' => 'border-style: {{VALUE}}',
                ],
            ]
        );

        /** Border color. */
        $this->add_control(
            'border_color_header',
            [
                'label' => esc_html__( 'Border color', 'selection' ),
                'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .mdp-typography-default-header' => 'border-color: {{VALUE}}',
                ],
                'conditions' => [
                    'terms' => [
                        [
                            'name' => 'header_border_type',
                            'operator' => '!in',
                            'value' => [
                                'none',
                            ],
                        ],
                    ],
                ]
            ]
        );

        /** Border padding. */
        $this->add_responsive_control(
            'header_border_padding',
            [
                'label' => esc_html__( 'Border padding', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => [ 'desktop', 'tablet', 'mobile' ],
                'desktop_default' => [
                    'top' => '',
                    'right' => '',
                    'bottom' => '',
                    'left' => '',
                    'unit' => 'px',
                ],
                'tablet_default' => [
                    'top' => '',
                    'right' => '',
                    'bottom' => '',
                    'left' => '',
                    'unit' => 'px',
                ],
                'mobile_default' => [
                    'top' => '',
                    'right' => '',
                    'bottom' => '',
                    'left' => '',
                    'unit' => 'px',
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-typography-default-header' => 'padding: {{top}}{{unit}} {{right}}{{unit}} {{bottom}}{{unit}} {{left}}{{unit}};',
                ],
                'conditions' => [
                    'terms' => [
                        [
                            'name' => 'header_border_type',
                            'operator' => '!in',
                            'value' => [
                                'none',
                            ],
                        ],
                    ],
                ]
            ]
        );

        /** Border width. */
        $this->add_responsive_control(
            'header_border_width',
            [
                'label' => esc_html__( 'Border width', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => [ 'desktop', 'tablet', 'mobile' ],
                'desktop_default' => [
                    'top' => '',
                    'right' => '',
                    'bottom' => '',
                    'left' => '',
                    'unit' => 'px',
                ],
                'tablet_default' => [
                    'top' => '',
                    'right' => '',
                    'bottom' => '',
                    'left' => '',
                    'unit' => 'px',
                ],
                'mobile_default' => [
                    'top' => '',
                    'right' => '',
                    'bottom' => '',
                    'left' => '',
                    'unit' => 'px',
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-typography-default-header' => 'border-width: {{top}}{{unit}} {{right}}{{unit}} {{bottom}}{{unit}} {{left}}{{unit}};',
                ],
                'conditions' => [
                    'terms' => [
                        [
                            'name' => 'header_border_type',
                            'operator' => '!in',
                            'value' => [
                                'none',
                            ],
                        ],
                    ],
                ]
            ]
        );

        /** Border radius. */
        $this->add_responsive_control(
            'header_border_radius',
            [
                'label' => esc_html__( 'Border radius', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => [ 'desktop', 'tablet', 'mobile' ],
                'desktop_default' => [
                    'top' => '',
                    'right' => '',
                    'bottom' => '',
                    'left' => '',
                    'unit' => 'px',
                ],
                'tablet_default' => [
                    'top' => '',
                    'right' => '',
                    'bottom' => '',
                    'left' => '',
                    'unit' => 'px',
                ],
                'mobile_default' => [
                    'top' => '',
                    'right' => '',
                    'bottom' => '',
                    'left' => '',
                    'unit' => 'px',
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-typography-default-header' => 'border-radius: {{top}}{{unit}} {{right}}{{unit}} {{bottom}}{{unit}} {{left}}{{unit}};',
                ],
                'conditions' => [
                    'terms' => [
                        [
                            'name' => 'header_border_type',
                            'operator' => '!in',
                            'value' => [
                                'none',
                            ],
                        ],
                    ],
                ]
            ]
        );

        $this->end_controls_tab();

        $this->end_controls_tabs();

        /** Line style. */
        $this->add_control(
            'line_style',
            [
                'label' => esc_html__( 'Line style', 'selection' ),
                'type' => Controls_Manager::HEADING,
                'separator' => 'before',
                'condition'   => ['header_line' => 'yes' ]
            ]
        );

        /** Line position. */
        $this->add_control(
            'header_position',
            [
                'label' => esc_html__( 'Position', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'bottom',
                'options' => [
                    'after'  => esc_html__( 'After', 'selection' ),
                    'before' => esc_html__( 'Before', 'selection' ),
                    'bottom' => esc_html__( 'Bottom', 'selection' ),
                    'after_before' => esc_html__( 'After and Before', 'selection' ),
                ],
                'condition'   => ['header_line' => 'yes' ]
            ]
        );

        /** Line color. */
        $this->add_control(
            'line_color_header',
            [
                'label' => esc_html__( 'Color', 'selection' ),
                'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .line:after' => 'background-color: {{VALUE}};',
                ],
                'condition'   => ['header_line' => 'yes' ]
            ]
        );

        /** Line width. */
        $this->add_responsive_control(
            'line_width_header',
            [
                'label' => esc_html__( 'Width', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'default' => [ 'unit' => 'px', 'size' => 25, ],
                'selectors' => [
                    '{{WRAPPER}} .line:after' => 'width: {{SIZE}}{{UNIT}};',
                ],
                'condition'   => ['header_line' => 'yes' ]
            ]
        );

        /** Line height. */
        $this->add_responsive_control(
            'line_height_header',
            [
                'label' => esc_html__( 'Height', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'default' => [ 'unit' => 'px', 'size' => 2, ],
                'selectors' => [
                    '{{WRAPPER}} .line:after' => 'height: {{SIZE}}{{UNIT}};',
                ],
                'condition'   => ['header_line' => 'yes' ]
            ]
        );

        /** Line spacing. */
        $this->add_responsive_control(
            'line_spacing_header',
            [
                'label' => esc_html__( 'Spacing', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'default' => [ 'unit' => 'px', 'size' => 10, ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-header-line-left' => 'margin-right: {{SIZE}}{{UNIT}};',
                    '{{WRAPPER}} .mdp-header-line-right' => 'margin-left: {{SIZE}}{{UNIT}};',
                    '{{WRAPPER}} .mdp-header-line-bottom' => 'margin-top: {{SIZE}}{{UNIT}};',
                ],
                'condition'   => ['header_line' => 'yes' ]
            ]
        );

        /** End section style header. */
        $this->end_controls_section();

        /** Style Subheader tab. */
        $this->start_controls_section( 'style_subheader',
            [
                'label' => esc_html__( 'Subheader', 'selection' ), 'tab' => Controls_Manager::TAB_STYLE,
                'condition'   => ['show_subheader' => 'yes' ]
            ] );

        /** Color. */
        $this->add_control(
            'subheader',
            [
                'label' => esc_html__( 'Color', 'selection' ),
                'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .mdp-color-sub-header' => 'color: {{VALUE}};',
                ],
            ]
        );

        /** Typography. */
        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name' => 'subheader_typography',
                'label' => esc_html__( 'Typography', 'selection' ),
                'selector' => '{{WRAPPER}} .mdp-typography-subheader',
            ]
        );

        /** Shadow. */
        $this->add_group_control(
            Group_Control_Text_Shadow::get_type(),
            [
                'name' => 'subheader_shadow',
                'label' => esc_html__( 'Shadow', 'selection' ),
                'selector' => '{{WRAPPER}} .mdp-shadow-subheader',
            ]
        );

        /** Alignment. */
        $this->add_responsive_control(
            'subheader_align',
            [
                'label' => esc_html__( 'Alignment', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'fa fa-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'fa fa-align-center',
                    ],
                    'right' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'fa fa-align-right',
                    ],
                ],
                'default' => 'center',
                'selectors' => [
                    '{{WRAPPER}} .mdp-sub-header-align' => 'text-align: {{subheader_align}};',
                ],
                'toggle' => true,
                'label_block' => false,
            ]
        );

        /** Line style. */
        $this->add_control(
            'sub_line_style',
            [
                'label' => esc_html__( 'Line style', 'selection' ),
                'type' => Controls_Manager::HEADING,
                'separator' => 'before',
                'condition'   => ['subheader_line' => 'yes' ]
            ]
        );

        /** Position. */
        $this->add_control(
            'subheader_position',
            [
                'label' => esc_html__( 'Position', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'bottom',
                'options' => [
                    'after'  => esc_html__( 'After', 'selection' ),
                    'before' => esc_html__( 'Before', 'selection' ),
                    'bottom' => esc_html__( 'Bottom', 'selection' ),
                    'after_before' => esc_html__( 'After and Before', 'selection' ),
                ],
                'condition'   => ['subheader_line' => 'yes' ]
            ]
        );

        /** Line color. */
        $this->add_control(
            'line_color_subheader',
            [
                'label' => esc_html__( 'Color', 'selection' ),
                'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .sub-line:after' => 'background-color: {{VALUE}};'
                ],
                'condition'   => ['subheader_line' => 'yes' ]
            ]
        );

        /** Line width. */
        $this->add_responsive_control(
            'line_width_subheader',
            [
                'label' => esc_html__( 'Width', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'default' => [ 'unit' => 'px', 'size' => 25, ],
                'selectors' => [
                    '{{WRAPPER}} .sub-line:after' => 'width: {{SIZE}}{{UNIT}};',
                ],
                'condition'   => ['subheader_line' => 'yes' ]
            ]
        );

        /** Line height. */
        $this->add_responsive_control(
            'line_height_subheader',
            [
                'label' => esc_html__( 'Height', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'default' => [ 'unit' => 'px', 'size' => 2, ],
                'selectors' => [
                    '{{WRAPPER}} .sub-line:after' => 'height: {{SIZE}}{{UNIT}};',
                ],
                'condition'   => ['subheader_line' => 'yes' ]
            ]
        );

        /** Line spacing. */
        $this->add_responsive_control(
            'line_spacing_subheader',
            [
                'label' => esc_html__( 'Spacing', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'default' => [ 'unit' => 'px', 'size' => 0, ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-subheader-line-left' => 'margin-right: {{SIZE}}{{UNIT}};',
                    '{{WRAPPER}} .mdp-subheader-line-right' => 'margin-left: {{SIZE}}{{UNIT}};',
                    '{{WRAPPER}} .mdp-subheader-line-bottom' => 'margin-top: {{SIZE}}{{UNIT}};',
                ],
                'condition'   => ['subheader_line' => 'yes' ]
            ]
        );

        /** End section style Subheader. */
        $this->end_controls_section();

        /** Style content tab. */
        $this->start_controls_section( 'style_content',
            [
                'label' => esc_html__( 'Additional Header', 'selection' ), 'tab' => Controls_Manager::TAB_STYLE,
                'condition'   => ['show_content_header' => 'yes' ]
            ] );

        /** Color. */
        $this->add_control(
            'content_color_header',
            [
                'label' => esc_html__( 'Color', 'selection' ),
                'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .mdp-color-content-header' => 'color: {{VALUE}}',
                ],
            ]
        );

        /** Typography. */
        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name' => 'content_header_typography',
                'label' => esc_html__( 'Typography', 'selection' ),
                'selector' => '{{WRAPPER}} .mdp-typography-content-header',
            ]
        );

        /** Shadow. */
        $this->add_group_control(
            Group_Control_Text_Shadow::get_type(),
            [
                'name' => 'content_shadow',
                'label' => esc_html__( 'Shadow', 'selection' ),
                'selector' => '{{WRAPPER}} .mdp-shadow-content',
            ]
        );

        /** Alignment. */
        $this->add_responsive_control(
            'test_advanced_align',
            [
                'label' => esc_html__( 'Alignment', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'fa fa-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'fa fa-align-center',
                    ],
                    'right' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'fa fa-align-right',
                    ],
                ],
                'default' => 'center',
                'selectors' => [
                    '{{WRAPPER}} .mdp-text-content-align' => 'text-align: {{subheader_align}};',
                ],
                'toggle' => true,
                'label_block' => false,
            ]
        );

        /** Transform origin */
        $this->add_control(
            'advanced_transform_origin',
            [
                'label' => esc_html__( 'Transform origin', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'center center',
                'options' => [
                    'top center'  => esc_html__( 'Top Center', 'selection' ),
                    'top left' => esc_html__( 'Top Left', 'selection' ),
                    'top right' => esc_html__( 'Top Right', 'selection' ),
                    'center center' => esc_html__( 'Center Center', 'selection' ),
                    'center left' => esc_html__( 'Center Left', 'selection' ),
                    'center right' => esc_html__( 'Center Right', 'selection' ),
                    'bottom center' => esc_html__( 'Bottom Center', 'selection' ),
                    'bottom left' => esc_html__( 'Bottom Left', 'selection' ),
                    'bottom right' => esc_html__( 'Bottom Right', 'selection' ),
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-typography-content-header' => 'transform-origin: {{advanced_transform_origin}};',
                ],
            ]
        );

        /** Rotate. */
        $this->add_responsive_control(
            'line_rotate_advanced',
            [
                'label' => esc_html__( 'Rotate', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'range' => [
                    'deg' => [
                        'min' => 0,
                        'max' => 360,
                        'step' => 1
                    ],
                ],
                'default' => [ 'unit' => 'deg', 'size' => 0, ],
                'selectors' => [
                    '(desktop){{WRAPPER}} .mdp-typography-content-header' => 'transform: translate({{line_xoffset_advanced.SIZE}}{{line_xoffset_advanced.UNIT}}, {{line_yoffset_advanced.SIZE}}{{line_yoffset_advanced.UNIT}}) rotate({{line_rotate_advanced.SIZE}}{{line_rotate_advanced.UNIT}});',
                    '(tablet){{WRAPPER}} .mdp-typography-content-header' => 'transform: translate({{line_xoffset_advanced_tablet.SIZE}}{{line_xoffset_advanced.UNIT}}, {{line_yoffset_advanced_tablet.SIZE}}{{line_yoffset_advanced.UNIT}}) rotate({{line_rotate_advanced.SIZE}}{{line_rotate_advanced.UNIT}});',
                    '(mobile){{WRAPPER}} .mdp-typography-content-header' => 'transform: translate({{line_xoffset_advanced_mobile.SIZE}}{{line_xoffset_advanced.UNIT}}, {{line_yoffset_advanced_mobile.SIZE}}{{line_yoffset_advanced.UNIT}}) rotate({{line_rotate_advanced.SIZE}}{{line_rotate_advanced.UNIT}});',
                ],
            ]
        );

        /** X Offset. */
        $this->add_responsive_control(
            'line_xoffset_advanced',
            [
                'label' => esc_html__( 'X Offset', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%', 'vh' ],
                'range' => [
                    'px' => [
                        'min' => -500,
                        'max' => 500,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => -100,
                        'max' => 100,
                        'step' => 1,
                    ]
                ],
                'default' => [
                    'unit' => '%',
                    'size' => 0,
                ],
                'selectors' => [
                    '(desktop){{WRAPPER}} .mdp-typography-content-header' => 'transform: translate({{line_xoffset_advanced.SIZE}}{{line_xoffset_advanced.UNIT}}, {{line_yoffset_advanced.SIZE}}{{line_yoffset_advanced.UNIT}}) rotate({{line_rotate_advanced.SIZE}}{{line_rotate_advanced.UNIT}});',
                    '(tablet){{WRAPPER}} .mdp-typography-content-header' => 'transform: translate({{line_xoffset_advanced_tablet.SIZE}}{{line_xoffset_advanced.UNIT}}, {{line_yoffset_advanced_tablet.SIZE}}{{line_yoffset_advanced.UNIT}}) rotate({{line_rotate_advanced.SIZE}}{{line_rotate_advanced.UNIT}});',
                    '(mobile){{WRAPPER}} .mdp-typography-content-header' => 'transform: translate({{line_xoffset_advanced_mobile.SIZE}}{{line_xoffset_advanced.UNIT}}, {{line_yoffset_advanced_mobile.SIZE}}{{line_yoffset_advanced.UNIT}}) rotate({{line_rotate_advanced.SIZE}}{{line_rotate_advanced.UNIT}});',
                ],
            ]
        );

        /** Y Offset. */
        $this->add_responsive_control(
            'line_yoffset_advanced',
            [
                'label' => esc_html__( 'Y Offset', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%', 'vh' ],
                'range' => [
                    'px' => [
                        'min' => -500,
                        'max' => 500,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => -100,
                        'max' => 100,
                        'step' => 1,
                    ]
                ],
                'default' => [
                    'unit' => '%',
                    'size' => -100,
                ],
                'selectors' => [
                    '(desktop){{WRAPPER}} .mdp-typography-content-header' => 'transform: translate({{line_xoffset_advanced.SIZE}}{{line_xoffset_advanced.UNIT}}, {{line_yoffset_advanced.SIZE}}{{line_yoffset_advanced.UNIT}}) rotate({{line_rotate_advanced.SIZE}}{{line_rotate_advanced.UNIT}});',
                    '(tablet){{WRAPPER}} .mdp-typography-content-header' => 'transform: translate({{line_xoffset_advanced_tablet.SIZE}}{{line_xoffset_advanced.UNIT}}, {{line_yoffset_advanced_tablet.SIZE}}{{line_yoffset_advanced.UNIT}}) rotate({{line_rotate_advanced.SIZE}}{{line_rotate_advanced.UNIT}});',
                    '(mobile){{WRAPPER}} .mdp-typography-content-header' => 'transform: translate({{line_xoffset_advanced_mobile.SIZE}}{{line_xoffset_advanced.UNIT}}, {{line_yoffset_advanced_mobile.SIZE}}{{line_yoffset_advanced.UNIT}}) rotate({{line_rotate_advanced.SIZE}}{{line_rotate_advanced.UNIT}});',
                ],
            ]
        );

        /** End section style content. */
        $this->end_controls_section();

    }

    /**
     * Split header string to before, middle and after and wrap each part in span tag
     * @param $svg - SVG path for animation
     *
     * @return void
     */
    public function highlighter( $svg ) {

        /** @var $settings - Settings from Elementor */
        $settings = $this->get_settings_for_display();
        $strHTML = '';

        if ( ! isset( $settings ) ) { return ''; }

        /** @var $header_phrase - Array of header parts divide by { and } */
        $header_phrase = preg_split( "/([{}])/", $settings["header_text"] );

        /** @var $svg_class - String of classes for SVG */
        $svg_class = "mdp-highlighter-svg";
        if ( $settings[ "highlighter_round" ] === "on" ) { $svg_class .= " mdp-highlighter-round"; }
        if ( $settings[ "highlighter_foreground" ] === "on" ) { $svg_class .= " mdp-highlighter-foreground"; }
        if ( $settings[ "highlighter_loop" ] === "on" ) { $svg_class .= " mdp-highlighter-loop"; }
        if ( $settings[ "highlighter_reverse" ] === "on" ) { $svg_class .= " mdp-highlighter-reverse"; }

//			if ( $header_phrase !== false ) {
//
//			    /** Header before */
//			    if ( strlen( $header_phrase[ 0 ] ) > 0 ) {
//
//				    $strHTML = '<span class="mdp-header-before">' . $header_phrase[ 0 ] . '</span>';
//
//                }
//
//				$strHTML .= '<span class="mdp-highlighter-wrapper mdp-highlighter-' . $this->get_id() . '">';
//				$strHTML .= '<span class="mdp-highlighter-middle">' . $header_phrase[ 1 ] . '</span>';
//				$strHTML .= '<svg class="' . $svg_class . '" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 500 150" preserveAspectRatio="none">';
//
//				if ( is_array( $svg ) ) {
//					foreach( $svg as $path) {
//						$strHTML .= '<path d="' . $path . '"></path>';
//					}
//				} else {
//					$strHTML .= '<path d="' . $svg . '"></path>';
//				}
//
//				$strHTML .= '</svg>';
//				$strHTML .= '</span> ';
//
//				/** Header after */
//				if ( strlen( end( $header_phrase ) ) > 0 ) {
//
//					$strHTML .= '<span class="mdp-header-after">' . end( $header_phrase ) . '</span>';
//
//                }
//
//
//			} else {
//
//				$strHTML = $settings["header_text"];
//
//			}

//			return $strHTML;

        /** Highlighter markup */
        if ( $header_phrase !== false ) {

            /** Header before */
            if ( strlen( $header_phrase[0] ) > 0 ) {
                ?><span class="mdp-header-before"><?php echo wp_filter_nohtml_kses( $header_phrase[0] ) ?></span><?php
            }

            ?>
            <span class="mdp-highlighter-wrapper mdp-highlighter-<?php echo esc_attr( $this->get_id() ) ?>">
                    <span class="mdp-highlighter-middle"><?php echo wp_filter_nohtml_kses( $header_phrase[ 1 ] ) ?></span>
                    <svg class="<?php echo esc_attr( $svg_class ) ?>" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 500 150" preserveAspectRatio="none">
                    <?php

                    if ( is_array( $svg ) ) {
                        foreach( $svg as $path) {
                            ?><path d="<?php echo esc_attr( $path ) ?>"></path><?php
                        }
                    } else {
                        ?><path d="<?php echo esc_attr( $svg ) ?>"></path><?php
                    }

                    ?>
                    </svg>
				</span>
            <?php

            /** Header after */
            if ( strlen( end( $header_phrase ) ) > 0 ) {
                ?><span class="mdp-header-after"><?php echo wp_filter_nohtml_kses( end( $header_phrase ) ) ?></span><?php
            }

        }

    }

    /**
     * Render Frontend Output. Generate the final HTML on the frontend.
     *
     * @since 1.0.0
     * @access protected
     **/
    protected function render() {

        /** We get all the values from the admin panel. */
        $settings = $this->get_settings_for_display();

        /** We write the section style class to a variable. */
        $globalClass = 'mdp-' . $this->get_id();

        /** Heading link. */
        $url = $settings['heading_link']['url'];
        $target = $settings['heading_link']['is_external'] ? ' target="_blank"' : '';
        $nofollow = $settings['heading_link']['nofollow'] ? ' rel="nofollow"' : '';

        /** SVG paths */
        $svg = [
            'ellipse' => 'M131.44,135.53c110.44,14,182.44,13.47,274.39,10.78,11.09-.33,28.51-4.39,38.63-8.95,8.71-3.93,20.94-9.63,27.63-16.45,9.42-9.58,11.85-11.14,19.12-22.44,8-12.39,10.43-40.21,2.06-52.33-3.61-5.23-10.38-14.32-16.14-17C444,13.82,457.88,13.8,369.71,6.48c-72.07-6-144.55-3.15-216.86-3.55-37.41-.2-80,.8-114.54,15.08-15,6.21-34.23,24.3-36,40.49-4.23,39.5,9.72,49.34,20.48,59.24s24.71,15.49,38.48,20.39c45.56,16.19,131.15,8.91,178.13,2',
            'wave' => 'M2,132.5c9.68-1.31,19.66-2.44,29.44-1.3,17.11,2,31.32,10.56,48.45,12.47,32.86,3.67,62.1-18,95.14-15.24,25.73,2.18,45.62,18.76,71.52,19.33,30.57.68,54.53-21.28,85-19.17,19.78,1.37,36,14.79,55.86,15.25,18,.42,33-10.2,49.74-14.44,16.34-4.13,40.58-4.46,60.94,3.47',
            'fill' => 'M7.27,47.28q185.64,2.52,371.18,9.37c34.75,1.28,69.62,2.69,103.87,8.71-147.1,16.91-296.58.46-443.6,18.1C29.22,84.6,19.66,85.9,10.61,89l130.85,9.31c115,8.19,230.11,16.38,345.36,14.54',
            'diagonal-bottom' => 'M1,120c236.92-16.81,264.37-55.25,498-80',
            'diagonal-top' => 'M1,40c210.71,17.49,260.73,51.39,498,72.14',
            'strikethrough' => 'M1,82.44C238.22,69,265.37,89.15,499,72.68',
            'frame' => 'M6.35,6.79C5.71,51.62,12.73,101.14,5,142.29c164.2,10.41,331.12,1.48,492.63-2.34-11.92-42-10.59-86.93-1.85-129.73C426.18,5.76,349.49,2.14,279.71,2.51c-76.11.4-152.38,4.78-228.49,5.18',
            'left-shading' => 'M4.09,0c.69,14,.16,44.09-.54,58.06C2.8,73,2.26,83.57,2.39,98.48c0,5.57-.21,11.15-.48,16.72L.67,141.09C14,100.68,10.24,50.05,12.25,7.56c7.55,40.84-1.62,94.2,4.29,135.32',
            'right-shading' => 'M495.71,4.92c-.34,12.16-4.68,29.32-5,41.48-.33,11.61-.66,23.28-2.79,34.7-1.24,6.64-3.08,13.17-3.92,19.87-.88,7-.66,14-1.46,21-.74,6.49,7.64,18.07,8.92,24.48-.75-20-11.51-45-10.9-65,.61-20.46,2.65-54-.07-74.31,12.6,38.49,8.46,98.89,16.69,138.55',
            'underline' => 'M3.25,146.33c70.24-29.46,214.46-12.34,290.5-7.89,72.91,4.27,97.74,6.32,203,8.46',
            'overline' => 'M3.25,9.46c88.53.57,142.23-7,290.5-4.89,73,1,97.74,3.32,203,5.46',
            'cross' => [ 'M4.31,116.61C84.55,103,105.62,98.85,186.36,88.54S422.25,66.68,497.54,35.76', 'M5.93,42.35C97.69,49.68,175.44,71.08,267.2,78.41c37.57,3,193.73,23.56,224.24,48.57' ],
            'wrap' => [ 'M3.61,146.51c20.69-7.89,85.86-7.56,108-7.93,88.78-1.52,127.92-2.39,212.84-.38,55.11,1.31,122.57,4.29,170.09,7.09', 'M9.7,10c133,7.76,266.32,6.32,399.52,4.88,28.59-.31,57.44-.65,85.32-7' ],
            'double-underline' => [ 'M3.61,137.51c20.69-7.89,104.17-11.4,126.31-11.78,88.78-1.52,109.61,1.46,194.53,3.47,55.11,1.31,122.57,4.29,170.09,7.09', 'M3.61,148.73c187.93-24,299.59.07,423.76,0'],
            'zigzag' => 'M3,132.66c99.83-12.18,200.7-14.27,301.27-14.34l190.46-.13c-19.54,7-40.37,8.58-61,10.48-50,4.6-100.17,6.51-150.32,7.34-61.67,1-123.34-4.09-185-3.33-100,.11,39.83,11.19,60.08,12,101.33,4.07,202.94,4.13,304-3.84',
            'dash' => 'M7.26,146.19a292.14,292.14,0,0,1,44.55-1.94m27.47-2.06,47-1.08m41.8,1,53.38,2m72,1.08a469.3,469.3,0,0,0-47.28-2m64.69,1.27q36.78-.92,73.5-3.33m97,3.37a609,609,0,0,0-69.55-.92',
            'curl' => 'M3,127.56c0,72.44,69.36-72.44,69,0s72-72.44,72,0,72-72.44,72,0,72-72.44,72,0,72.81-72.44,72.81,0,70.78-72.44,70.78,0,63-72.44,63,0',
            'curl-wave' => 'M3,134.56c0,30.54,69,0,69,0s72-32.2,72,0,72,0,72,0,71.69-32.2,72,0,72.81,0,72.81,0,70.68-32.2,70.78,0,63,0,63,0',
            'scallop' => 'M7.88,138.45,69,129.18a2.61,2.61,0,0,1,3,2.58V145.3a2.61,2.61,0,0,0,3.3,2.52L141,129.64a2.61,2.61,0,0,1,3.3,2.52v13.13a2.61,2.61,0,0,0,3.31,2.52l65.08-18.16a2.61,2.61,0,0,1,3.31,2.51V145.3a2.61,2.61,0,0,0,3.3,2.52L285,129.64a2.61,2.61,0,0,1,3.3,2.52V145.3a2.61,2.61,0,0,0,3.31,2.51L357,129.65a2.6,2.6,0,0,1,3.3,2.51v13.13a2.61,2.61,0,0,0,3.31,2.52l65.08-18.16a2.61,2.61,0,0,1,3.31,2.51v13a2.6,2.6,0,0,0,3.4,2.48l55.84-17.83a2.61,2.61,0,0,1,3.4,2.49v12.18',
            'circle' => 'M224.19,1.86S10.68-6.34,1.53,77.56s313,70.35,313,70.35,195.41,7.43,183.1-69.74S250,4.86,250,4.86',
        ];

        $this->add_render_attribute( 'sub_heading', 'class', 'mdp-sub-heading-content mdp-typography-subheader mdp-shadow-subheader mdp-color-sub-header' );

        /** Use the gradient style class where necessary. */
        $this->add_render_attribute(
            [
                'header_text' => [
                    'class' => [
                        'mdp-typography-header mdp-typography-default-header mdp-shadow-header mdp-heading-margin',
                        $settings[ "header_background_effect" ] === 'yes' ? 'mdp-header-background-effect' : '',
                    ],
                ],
            ]
        );

        $this->add_render_attribute(
            [
                'split_text' => [
                    'class' => [
                        'mdp-typography-header mdp-typography-default-header mdp-shadow-header mdp-heading-margin mdp-highlighter-split-text',
                        $settings["header_background_effect"] === 'yes' ? 'mdp-header-background-effect' : '',
                    ],
                ],
            ]
        );

        ?>

        <div id="<?php esc_attr_e( $globalClass ); ?>" >

            <?php if ( !wp_is_mobile() ): ?>
                <div class="mdp-content-header mdp-color-content-header mdp-text-content-align mdp-text-content-margin mdp-shadow-content">
                    <div class="mdp-typography-content-header">
                        <?php esc_html_e( $settings["text_content_heading"] ); ?>
                    </div>
                </div>
            <?php endif ?>

            <?php if ( $settings["show_subheader"] === 'yes' and  $settings["header_bottom"] === '' ): ?>

                <div class="mdp-sub-heading mdp-sub-header-align mdp-subheading-margin">
                    <div class="mdp-sub-header-box">
                        <div <?php echo $this->get_render_attribute_string( 'sub_heading' ); ?>>
                            <?php esc_html_e( $settings["sub_heading"] ); ?>
                        </div>

                        <?php if ( $settings["subheader_line"] === 'yes' ): ?>

                            <?php if ( $settings["subheader_position"] === 'before' or $settings["subheader_position"] === 'after_before' ): ?>
                                <div class="sub-line mdp-subheader-line-left mdp-typography-subheader"></div>
                            <?php endif ?>

                            <?php if ( $settings["subheader_position"] === 'after' or $settings["subheader_position"] === 'after_before' ): ?>
                                <div class="sub-line mdp-subheader-line-right mdp-typography-subheader"></div>
                            <?php endif ?>

                            <?php if ( $settings["subheader_position"] === 'bottom' ): ?>
                                <div class="sub-line mdp-subheader-line-bottom"></div>
                            <?php endif ?>

                        <?php endif ?>

                    </div>
                </div>

            <?php endif ?>

            <<?php esc_attr_e( $settings["header_tag"] ); ?>  class="mdp-header-text mdp-header-align">

            <?php if (!empty($url) ): ?>
            <a href="<?php esc_html_e( $url ); ?>"
               class="mdp-color-header" <?php esc_attr_e( $target ); ?> <?php esc_attr_e( $nofollow ); ?>>
                <?php endif; ?>

                <span class="mdp-sub-header-box">
                        <span <?php echo $this->get_render_attribute_string( 'header_text' ); ?>>
		                    <?php $this->highlighter( $svg[ $settings["animation_header"] ] ); ?>
                        </span>

                        <?php if ( $settings["header_line"] === 'yes' ): ?>

                            <?php if ( $settings["header_position"] === 'before' or $settings["header_position"] === 'after_before' ): ?>
                                <div class="line mdp-header-line-left mdp-typography-header"></div>
                            <?php endif ?>

                            <?php if ( $settings["header_position"] === 'after' or $settings["header_position"] === 'after_before' ): ?>
                                <div class="line mdp-header-line-right mdp-typography-header"></div>
                            <?php endif ?>

                            <?php if ( $settings["header_position"] === 'bottom' ): ?>
                                <div class="line mdp-header-line-bottom"></div>
                            <?php endif ?>

                        <?php endif ?>

                    </span>

                <?php if ( $settings["header_split"] === 'yes' ): ?>
                    <span <?php echo $this->get_render_attribute_string( 'split_text' ); ?>>
                            <?php esc_html_e( $settings["split_text"] ); ?>
                        </span>
                <?php endif ?>

                <?php if (!empty($url) ): ?>
            </a>
        <?php endif; ?>

        </<?php esc_attr_e( $settings["header_tag"] ); ?>>

        <?php if ( $settings["show_subheader"] === 'yes' and  $settings["header_bottom"] === 'yes' ): ?>

            <div class="mdp-sub-heading mdp-sub-header-align mdp-subheading-margin">
                <div class="mdp-sub-header-box">
                    <div <?php echo $this->get_render_attribute_string( 'sub_heading' ); ?>>
                        <?php esc_html_e( $settings["sub_heading"] ); ?>
                    </div>

                    <?php if ( $settings["subheader_line"] === 'yes' ): ?>

                        <?php if ( $settings["subheader_position"] === 'before' or $settings["subheader_position"] === 'after_before' ): ?>
                            <div class="sub-line mdp-subheader-line-left mdp-typography-subheader"></div>
                        <?php endif ?>

                        <?php if ( $settings["subheader_position"] === 'after' or $settings["subheader_position"] === 'after_before' ): ?>
                            <div class="sub-line mdp-subheader-line-right mdp-typography-subheader"></div>
                        <?php endif ?>

                        <?php if ( $settings["subheader_position"] === 'bottom' ): ?>
                            <div class="sub-line mdp-subheader-line-bottom"></div>
                        <?php endif ?>

                    <?php endif ?>

                </div>
            </div>

        <?php endif ?>

        </div>

        <?php if ( $settings[ "highlighter_viewport" ] === 'on' ): ?>

            <!--suppress JSUnresolvedFunction -->
            <script>

                jQuery( document ).ready( function ( $ ) {

                    const mdpHighlighter = $( '.mdp-highlighter-wrapper' );

                    /**
                     * Check if widget in viewport and play pause animation
                     * @param widget - widget element
                     * @return {boolean|boolean}
                     */
                    function mdpHighlighterViewport( widget ) {

                        let elementTop = widget.offset().top;
                        let elementBottom = elementTop + widget.outerHeight();
                        let viewportTop = $( window ).scrollTop();
                        let viewportBottom = viewportTop + $( window ).height();

                        if ( elementBottom > viewportTop && elementTop < viewportBottom ) {

                            widget.removeClass( 'mdp-highlighter-pause' );

                        } else {

                            widget.addClass( 'mdp-highlighter-pause' );

                        }

                    }

                    /** Check thr position of a widget in viewport on init */
                    mdpHighlighter.each( function () {
                        mdpHighlighterViewport( $( this ) );
                    } );

                    /** Check thr position of a widget in viewport on init */
                    $( window ).scroll( function() {
                        mdpHighlighter.each( function () {
                            mdpHighlighterViewport( $( this ) )
                        } );
                    });

                });

            </script>

        <?php endif;

    }

    /**
     * Return link for documentation.
     *
     * Used to add stuff after widget.
     *
     * @since 1.0.0
     * @access public
     **/
    public function get_custom_help_url() {
        return 'https://docs.merkulov.design/tag/highlighter/';
    }

}
