<?php /** @noinspection PhpUndefinedClassInspection */
/**
 * Modalier for Elementor
 * Simple modal popup window for Elementor
 * Exclusively on https://1.envato.market/modalier-elementor
 *
 * @encoding        UTF-8
 * @version         1.12.1
 * @copyright       (C) 2018 - 2023 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com)
 * @support         help@merkulov.design
 **/

namespace Merkulove\Selection;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
    header( 'Status: 403 Forbidden' );
    header( 'HTTP/1.1 403 Forbidden' );
    exit;
}


use Exception;
use Elementor\Plugin;
use Elementor\Widget_Base;
use Elementor\Controls_Manager;
use Elementor\Repeater;
use Elementor\Group_Control_Background;
use Elementor\Group_Control_Text_Shadow;
use Elementor\Group_Control_Typography;
use Elementor\Core\Schemes\Typography;
use Elementor\Core\Schemes\Color;
use Elementor\Group_Control_Border;
use Elementor\Group_Control_Box_Shadow;
use Elementor\Group_Control_Css_Filter;
use Elementor\Utils;
use Elementor\Group_Control_Image_Size;
use Merkulove\Selection\Unity\Plugin as UnityPlugin;

/** @noinspection PhpUnused */
/**
 * Modalier - Custom Elementor Widget.
 **/
class modalier_elementor extends Widget_Base {

    /**
     * Use this to sort widgets.
     * A smaller value means earlier initialization of the widget.
     * Can take negative values.
     * Default widgets and widgets from 3rd party developers have 0 $mdp_order
     **/
    public $mdp_order = 1;

	/**
	 * Widget Scripts.
	 *
	 * @string version
	 * @since 1.0.0
	 **/
	public $scripts = [];

    /**
     * Widget base constructor.
     * Initializing the widget base class.
     *
     * @access public
     * @throws Exception If arguments are missing when initializing a full widget instance.
     * @param array      $data Widget data. Default is an empty array.
     * @param array|null $args Optional. Widget default arguments. Default is null.
     *
     * @return void
     **/
    public function __construct( $data = [], $args = null ) {

        parent::__construct( $data, $args );

        wp_register_style(
    'mdp-selection-elementor-admin',
        UnityPlugin::get_url() . 'css/elementor-admin' . UnityPlugin::get_suffix() . '.css',
            [],
            UnityPlugin::get_version()
        );
        wp_register_style(
    'mdp-modaal',
        UnityPlugin::get_url() . 'css/modaal' . UnityPlugin::get_suffix() . '.css',
            [],
            UnityPlugin::get_version(),
     "screen"
        );
        wp_register_style(
    'mdp-modalier',
        UnityPlugin::get_url() . 'css/modalier' . UnityPlugin::get_suffix() . '.css',
            [],
            UnityPlugin::get_version(),
    "screen"
        );

        wp_register_script(
    'mdp-modaal',
        UnityPlugin::get_url() . 'js/modaal' . UnityPlugin::get_suffix() . '.js',
            [ 'jquery' ],
            UnityPlugin::get_version(),
    true
        );
        wp_register_script(
    'mdp-modalier',
        UnityPlugin::get_url() . 'js/modalier' . UnityPlugin::get_suffix() . '.js',
            [ 'mdp-modaal' ],
            UnityPlugin::get_version(),
    true
        );
    }

    /**
     * Return a widget name.
     *
     * @return string
     **/
    public function get_name() {
        return 'mdp-modalier-elementor';
    }

    /**
     * Return the widget title that will be displayed as the widget label.
     *
     * @return string
     **/
    public function get_title() {
        return esc_html__( 'Modal Popup', 'selection' );
    }

    /**
     * Set the widget icon.
     *
     * @return string
     */
    public function get_icon() {
        return 'mdp-modalier-elementor-widget-icon';
    }

    /**
     * Set the category of the widget.
     *
     * @return array with category names
     **/
    public function get_categories() {
        return [ 'selection-category' ];
    }

    /**
     * Get widget keywords. Retrieve the list of keywords the widget belongs to.
     *
     * @access public
     *
     * @return array Widget keywords.
     **/
    public function get_keywords() {
        return [ 'Merkulove', 'Modal Popup' ];
    }

    /**
     * Get style dependencies.
     * Retrieve the list of style dependencies the widget requires.
     *
     * @access public
     *
     * @return array Widget styles dependencies.
     **/
    public function get_style_depends() {
        return [ 'mdp-modaal', 'mdp-modalier', 'mdp-selection-elementor-admin' ];
    }

	/**
	 * Get script dependencies.
	 * Retrieve the list of script dependencies the element requires.
	 *
	 * @access public
     *
	 * @return array Element scripts dependencies.
	 **/
	public function get_script_depends() {

		/**
		 * Editor Mode.
		 * In the edit mode, get_script_depends() does not have access to get_settings()
		 * or get_settings_for_display() functions.
		 * So, the solution would be, enqueue all the scripts for edit and preview mode,
		 * whereas use conditional enqueue for the frontend.
		 **/
		if (
			Plugin::$instance->editor->is_edit_mode() ||
			Plugin::$instance->preview->is_preview_mode()
		) {
		    return [ '' ];
		}

		return [
			'mdp-modaal',
			'mdp-modalier'
		];

	}

	/**
	 * @return array
	 */
	public function get_post_types() {

		$wp_pt = get_post_types( array(
			'public' => 1,
		) );
		$options = array();

		/** Add available post categories. */
		$exceptions = ['post', 'page', 'e-landing-page', 'elementor_library', 'product'];
		foreach( $wp_pt as $post_type ){
			if( !in_array( $post_type, $exceptions, true ) ) {
				$options[] = $post_type;
			}
		}

		return $options;
	}

	/**
	 * Group controllers for setting styles.
	 *
	 * @param $name_Section - The name of the styles section.
	 * @param string $css_class - Specifies one or more class names for an element.
	 * @param array $condition - Interconnected controllers.
	 * @param array $display - Determines which controllers to display. {
	 *      @type $key 'margin'                 => (boolean) - Use margin.
	 *      @type $key 'padding'                => (boolean) - Use padding.
	 *      @type $key 'color'                  => (boolean) - Use color.
	 *      @type $key 'typography'             => (boolean) - Use typography.
	 *      @type $key 'text-shadow'            => (boolean) - Use text shadow.
	 *      @type $key 'alignment'              => (boolean) - Use alignment.
	 *      @type $key 'alignment-justify'      => (boolean) - Use alignment justify.
	 *      @type $key 'alignment-horizontal'   => (boolean) - Use alignment horizontal.
	 *      @type $key 'alignment-vertical'     => (boolean) - Use alignment vertical.
	 *      @type $key 'background'             => (boolean) - Use background.
	 *      @type $key 'border'                 => (boolean) - Use border.
	 *      @type $key 'border-radius'          => (boolean) - Use border radius.
	 *      @type $key 'box-shadow'             => (boolean) - Use box shadow.
	 *      @type $key 'css-filter'             => (boolean) - Use css filter.
	 *
	 * }
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @return void
	 */
	public function get_settings_style_group( $name_Section,
        $css_class, $condition = [], $display = [], $separator = [ 'after', 'before', 'before'] ) {

		/** Margin. */
		if ( isset($display['margin']) && $display['margin'] === true ) {
			$this->add_responsive_control(
				'modalier_' . $name_Section . '_margin',
				[
					'label'      => esc_html__( 'Margin', 'selection' ),
					'type'       => Controls_Manager::DIMENSIONS,
					'size_units' => [ 'px', '%', 'em' ],
					'selectors'  => [
						'{{WRAPPER}} ' . $css_class => 'margin: {{top}}{{unit}} {{right}}{{unit}} {{bottom}}{{unit}} {{left}}{{unit}};',
					],
					'default' => [
				            'top' => 0,
                            'right' => 0,
                            'bottom' => 0,
                            'left' => 0,
                            'unit' => 'px',
			        ],
					'toggle'     => true,
					'condition'  => $condition,
				]
			);
		}

		/** Padding. */
		if ( isset($display['padding']) && $display['padding'] === true ) {
			$this->add_responsive_control(
				'modalier_' . $name_Section . '_padding',
				[
					'label'      => esc_html__( 'Padding', 'selection' ),
					'type'       => Controls_Manager::DIMENSIONS,
					'size_units' => [ 'px', '%', 'em' ],
					'selectors'  => [
						'{{WRAPPER}} ' . $css_class => 'padding: {{top}}{{unit}} {{right}}{{unit}} {{bottom}}{{unit}} {{left}}{{unit}};',
					],
					'default' => [
						'top' => 0,
						'right' => 0,
						'bottom' => 0,
						'left' => 0,
						'unit' => 'px',
					],
					'toggle'     => true,
					'separator'  => $separator[0],
					'condition'  => $condition,
				]
			);
		}

		/** Text Color. */
		if ( isset($display['color']) && $display['color'] === true ) {
			$this->add_control(
				'modalier_' . $name_Section . '_text_color',
				[
					'label'     => esc_html__( 'Color', 'selection' ),
					'type'      => Controls_Manager::COLOR,
					'selectors' => [
						'{{WRAPPER}} ' . $css_class => 'color: {{VALUE}}',
					],
					'condition' => $condition,
				]
			);
		}

		/** Typography */
		if ( isset($display['typography']) && $display['typography'] === true ) {
			$this->add_group_control(
				Group_Control_Typography::get_type(),
				[
					'name'      => 'modalier_' . $name_Section . '_typography',
					'label'     => esc_html__( 'Typography', 'selection' ),
					'scheme'    => Typography::TYPOGRAPHY_1,
					'selector'  => '{{WRAPPER}} ' . $css_class,
					'condition' => $condition,
				]
			);
		}

		/** Text Shadow. */
		if ( isset($display['text-shadow']) && $display['text-shadow'] === true ) {
			$this->add_group_control(
				Group_Control_Text_Shadow::get_type(),
				[
					'name'      => 'modalier_' . $name_Section . '_text_shadow',
					'label'     => esc_html__( 'Text Shadow', 'selection' ),
					'selector'  => '{{WRAPPER}} ' . $css_class,
					'condition' => $condition,
				]
			);
		}

		/** Alignment horizontal. */
		if ( isset($display['alignment-horizontal']) && $display['alignment-horizontal'] === true ) {
			$this->add_responsive_control(
				'modalier_' . $name_Section . '_horizontal_align',
				[
					'label'     => esc_html__( 'Horizontal Position', 'selection' ),
					'type'      => Controls_Manager::CHOOSE,
					'options'   => [
						'flex-start'    => [
							'title' => esc_html__( 'Left', 'selection' ),
							'icon'  => 'fa fa-align-left',
						],
						'center'  => [
							'title' => esc_html__( 'Center', 'selection' ),
							'icon'  => 'fa fa-align-center',
						],
						'flex-end'   => [
							'title' => esc_html__( 'Right', 'selection' ),
							'icon'  => 'fa fa-align-right',
						],
					],
					'default'   => '',
					'selectors' => [
						'{{WRAPPER}} ' . $css_class => 'justify-content: {{header_align}};',
						'{{WRAPPER}} .mdp-modalier-box-content' => 'width: 65%;',
					],
					'toggle'    => true,
					'condition' => $condition,
				]
			);
		}

		/** Alignment vertical. */
		if ( isset($display['alignment-vertical']) && $display['alignment-vertical'] === true ) {
			$this->add_responsive_control(
				'modalier_' . $name_Section . '_vertical_align',
				[
					'label'     => esc_html__( 'Vertical Position', 'selection' ),
					'type'      => Controls_Manager::CHOOSE,
					'options'   => [
						'flex-start'    => [
							'title' => esc_html__( 'Top', 'selection' ),
							'icon'  => 'eicon-v-align-top',
						],
						'center'  => [
							'title' => esc_html__( 'Center', 'selection' ),
							'icon'  => 'eicon-v-align-middle',
						],
						'flex-end'   => [
							'title' => esc_html__( 'Bottom', 'selection' ),
							'icon'  => 'eicon-v-align-bottom',
						],
					],
					'default'   => 'center',
					'selectors' => [
						'{{WRAPPER}} ' . $css_class => 'align-items: {{value}};',
					],
					'toggle'    => true,
					'condition' => $condition,
				]
			);
		}

		/** Alignment. */
		if ( isset($display['alignment']) && $display['alignment'] === true ) {
			$this->add_responsive_control(
				'modalier_' . $name_Section . '_align',
				[
					'label'     => esc_html__( 'Alignment', 'selection' ),
					'type'      => Controls_Manager::CHOOSE,
					'options'   => [
						'left'    => [
							'title' => esc_html__( 'Left', 'selection' ),
							'icon'  => 'eicon-text-align-left',
						],
						'center'  => [
							'title' => esc_html__( 'Center', 'selection' ),
							'icon'  => 'eicon-text-align-center',
						],
						'right'   => [
							'title' => esc_html__( 'Right', 'selection' ),
							'icon'  => 'eicon-text-align-right',
						],
					],
					'default'   => 'center',
					'selectors' => [
						'{{WRAPPER}} ' . $css_class => 'text-align: {{value}};',
					],
					'toggle'    => true,
					'condition' => $condition,
				]
			);
		}

		/** Alignment justify. */
		if ( isset($display['alignment-justify']) && $display['alignment-justify'] === true ) {
			$this->add_responsive_control(
				'modalier_' . $name_Section . '_align_justify',
				[
					'label'     => esc_html__( 'Alignment', 'selection' ),
					'type'      => Controls_Manager::CHOOSE,
					'options'   => [
						'left'    => [
							'title' => esc_html__( 'Left', 'selection' ),
							'icon'  => 'fa fa-align-left',
						],
						'center'  => [
							'title' => esc_html__( 'Center', 'selection' ),
							'icon'  => 'fa fa-align-center',
						],
						'right'   => [
							'title' => esc_html__( 'Right', 'selection' ),
							'icon'  => 'fa fa-align-right',
						],
						'justify' => [
							'title' => esc_html__( 'Justify', 'selection' ),
							'icon'  => 'fa fa-align-justify',
						],
					],
					'default'   => 'center',
					'selectors' => [
						'{{WRAPPER}} ' . $css_class => 'text-align: {{header_align}};',
					],
					'toggle'    => true,
					'condition' => $condition,
				]
			);
		}

		/** Background. */
		if ( isset($display['background']) && $display['background'] === true ) {
			$this->add_group_control(
				Group_Control_Background::get_type(),
				[
					'name'      => 'modalier_' . $name_Section . '_background',
					'label'     => esc_html__( 'Background', 'selection' ),
					'types'     => [ 'classic', 'gradient' ],
					'separator' => $separator[1],
					'selector'  => '{{WRAPPER}} ' . $css_class,
					'condition' => $condition,
				]
			);
		}

		/** Border. */
		if ( isset($display['border']) && $display['border'] === true ) {
			$this->add_group_control(
				Group_Control_Border::get_type(),
				[
					'name'      => 'modalier_' . $name_Section . '_border',
					'label'     => esc_html__( 'Border', 'selection' ),
					'selector'  => '{{WRAPPER}} ' . $css_class,
					'separator' => 'before',
					'condition' => $condition,
				]
			);
		}

		/** Border Radius. */
		if ( isset($display['border-radius']) && $display['border-radius'] === true ) {
			$this->add_responsive_control(
				'modalier_' . $name_Section . '_border_radius',
				[
					'label'      => esc_html__( 'Border Radius', 'selection' ),
					'type'       => Controls_Manager::DIMENSIONS,
					'size_units' => [ 'px', '%', 'em' ],
					'selectors'  => [
						'{{WRAPPER}} ' . $css_class => 'border-radius: {{top}}{{unit}} {{right}}{{unit}} {{bottom}}{{unit}} {{left}}{{unit}};',
					],
					'default' => [
						'top' => 0,
						'right' => 0,
						'bottom' => 0,
						'left' => 0,
						'unit' => 'px',
					],
					'separator'  => $separator[2],
					'toggle'     => true,
					'condition'  => $condition,
				]
			);
		}

		/** Box Shadow. */
		if ( isset($display['box-shadow']) && $display['box-shadow'] === true ) {
			$this->add_group_control(
				Group_Control_Box_Shadow::get_type(),
				[
					'name'      => 'modalier_' . $name_Section . '_box_shadow',
					'label'     => esc_html__( 'Box Shadow', 'selection' ),
					'selector'  => '{{WRAPPER}} ' . $css_class,
					'condition' => $condition,
				]
			);
		}

		/** CSS filter. */
		if ( isset($display['css-filter']) && $display['css-filter'] === true ) {
			$this->add_group_control(
				Group_Control_Css_Filter::get_type(),
				[
					'name'     => 'modalier_' . $name_Section . '_css_filter',
					'label'    => esc_html__( 'CSS filter', 'selection' ),
					'selector' => '{{WRAPPER}} ' . $css_class,
					'condition' => $condition,
				]
			);
		}

	}

	/**
	 * We write the existing categories of posts into an array.
	 */
	public function get_post_category() {

		$wp_cat = get_categories('&hide_empty=0');

		/** If no category is selected.  */
		$options['select_category'] = 'Select category';

		/** Add available post categories. */
		foreach( $wp_cat as $cat_item ){
			$options[$cat_item->slug] =  $cat_item->name;
		}

		return $options;
	}

	/**
	 * We write the existing product categories into an array.
	 */
	public function get_products_category() {

		$wp_cat = get_categories( array(
			'taxonomy'     => 'product_cat',
			'orderby'      => 'name',
			'pad_counts'   => false,
			'hierarchical' => 1,
			'hide_empty'   => false
		) );

		/** If no category is selected.  */
		$options['select_category'] = 'Select category';

		/** Add available product categories. */
		foreach( $wp_cat as $cat_item ){
			$options[$cat_item->slug] =  $cat_item->name;
		}

		return $options;
	}

	/**
	 * Returns an array of available products.
	 */
	public function get_product_array() {

	    $product_list = [];

		/** We get an array with post data. */
		$product = get_posts( [ 'post_type'   => 'product' ] );

		/** We count the number of product item in the post type. */
		$quantity_product = count( $product );

		if( $quantity_product > 0 ){
			for( $i = 0; $i < $quantity_product; $i++) {
				$product_list[ $product[$i]->ID ] = $product[$i]->post_title;
			}
		}

	    return $product_list;
	}

	/**
	 * Returns an array of available posts.
	 */
	public function get_post_array() {

		$post_list = [];

		/** We get an array with post data. */
		$posts = get_posts();

		/** We count the number of post item in the post type. */
		$quantity_posts = count( $posts );

		if( $quantity_posts > 0 ) {
			for( $i = 0; $i < $quantity_posts; $i ++ ) {
				$post_list[ $posts[ $i ]->ID ] = $posts[ $i ]->post_title;
			}
		}

		return $post_list;

	}

	/**
	 * @return array
	 */
	public function get_post_taxonomies() {

		$taxonomies = get_taxonomies( array(
			'public'   => true,
			'_builtin' => false
		) );

		/** Add default option */
		$options = array();
        $exceptions = ['product_cat', 'product_tag', 'product_shipping_class' ];
		foreach( $taxonomies as $taxonomy ) {
		    if( !in_array( $taxonomy, $exceptions, true ) ){
			    $options[] = $taxonomy;
            }
		}

		return $options;

	}

	/**
	 * @return array
	 */
	public function get_post_taxonomy_terms() {

		$taxonomies = get_taxonomies( array(
			'public'   => true,
			'_builtin' => false
		) );

		/** Add default option */
		$options = array();

		foreach( $taxonomies as $taxonomy ) {

			$terms = get_terms( $taxonomy, array(
				'hide_empty' => false,
			) );

			foreach( $terms as $term ) {
                if( $term->taxonomy !== 'product_cat' ){
	                $options[$term->term_id] = $term->slug;
                }
			}

		}

		return $options;

	}

	/**
	 *  Returns a list of custom posts.
     *
	 */
	public function get_custom_post_array() {

        $post_types         = $this->get_post_types();
		$taxonomies         = $this->get_post_taxonomies();
		$post_list = [];

		$posts = [];
        for( $i = 0, $iMax = count( $taxonomies ); $i < $iMax; $i++){
	        $args = array(
		        'post_type'   => $post_types,
	        );

	        /** We get an array with post data. */
	        $posts[] = get_posts( $args );
        }

		$all_posts = [];
		array_walk_recursive($posts, function ($item, $key) use (&$all_posts) {
			$all_posts[] = $item;
		});

		/** We count the number of post item in the post type. */
		$quantity_posts = count( $all_posts );

		if( $quantity_posts > 0 ) {
			for( $i = 0; $i < $quantity_posts; $i ++ ) {
				$post_list[ $all_posts[ $i ]->ID ] = $all_posts[ $i ]->post_title;
			}
		}

		return $post_list;

	}

	/**
	 *  TAB: Content.
	 */

	/**
	 * General TAB.
	 */
	public function get_general_section_content() {

		$this->start_controls_section(
            'general_content',
            [
                'label' => esc_html__( 'General', 'selection' ),
                'tab'   => Controls_Manager::TAB_CONTENT
            ]
        );

            /** Source */
            $this->add_control(
                'general_source',
                [
                    'label' => esc_html__( 'Source', 'selection' ),
                    'type' => Controls_Manager::SELECT,
                    'default' => 'custom',
                    'options' => [
                        'posts' => esc_html__( 'Posts', 'selection' ),
                        'custom-posts' => esc_html__( 'Custom posts', 'selection' ),
                        'woocommerce' => esc_html__( 'WooCommerce', 'selection' ),
                        'custom'  => esc_html__( 'Content', 'selection' ),
                        'ajax'  => esc_html__( 'AJAX content', 'selection' ),
                        'single-image'  => esc_html__( 'Single Image', 'selection' ),
                        'image-gallery'  => esc_html__( 'Image Gallery', 'selection' ),
                        'video'  => esc_html__( 'Video', 'selection' ),
                        'iframe'  => esc_html__( 'Iframe', 'selection' ),
                        'section' => esc_html__( 'Section', 'selection' ),
                    ],
                ]
            );

            /** Posts. */
            $this->add_control(
                'general_item_posts',
                [
                    'label' => esc_html__( 'Posts', 'selection' ),
                    'type' => Controls_Manager::SELECT2,
                    'label_block' => true,
                    'multiple' => true,
                    'options' => $this->get_post_array(),
                    'condition' => [ 'general_source' => 'posts' ],
                ]
            );

            /** Custom posts. */
            $this->add_control(
                'general_item_custom_posts',
                [
                    'label' => esc_html__( 'Custom posts', 'selection' ),
                    'type' => Controls_Manager::SELECT2,
                    'label_block' => true,
                    'multiple' => true,
                    'options' => $this->get_custom_post_array(),
                    'condition' => [ 'general_source' => 'custom-posts' ],
                ]
            );

            /** Product */
            $this->add_control(
                'general_item_product',
                [
                    'label' => esc_html__( 'Product', 'selection' ),
                    'type' => Controls_Manager::SELECT2,
                    'label_block' => true,
                    'multiple' => true,
                    'options' => $this->get_product_array(),
                    'condition' => [ 'general_source' => 'woocommerce' ],
                ]
            );

            /** Link post */
            $this->add_control(
                'general_link_post_text',
                [
                    'label' => esc_html__( 'Link post', 'selection' ),
                    'type' => Controls_Manager::TEXT,
                    'label_block' => true,
                    'dynamic'     => ['active' => true],
                    'default' => esc_html__( 'Link post', 'selection' ),
                    'placeholder' => esc_html__( 'Link post', 'selection' ),
                    'separator' => 'after',
                    'condition' => [ 'general_source' => ['posts', 'custom-posts', 'woocommerce'] ]
                ]
            );

            /** Type events. */
            $this->add_control(
                'general_type_events',
                [
                    'label' => esc_html__( 'Type events', 'selection' ),
                    'type' => Controls_Manager::SELECT,
                    'default' => 'button',
                    'options' => [
                        'button'  => esc_html__( 'Button', 'selection' ),
                        'scroll'  => esc_html__( 'Scroll', 'selection' ),
                        'timeout'  => esc_html__( 'Timeout', 'selection' ),
                    ],
                    'condition' => [ 'general_source!' => ['single-image', 'image-gallery'] ],
                ]
            );

            /** Section ID. */
            $this->add_control(
                'general_section_id',
                [
                    'label' => esc_html__( 'Section ID', 'selection' ),
                    'type' => Controls_Manager::TEXT,
                    'label_block' => true,
                    'dynamic'     => ['active' => true],
                    'placeholder' => esc_html__( 'Section ID', 'selection' ),
                    'condition' => [ 'general_source' => 'section' ]
                ]
            );

            /** Button text */
            $this->add_control(
                'general_button_text',
                [
                    'label' => esc_html__( 'Button text', 'selection' ),
                    'type' => Controls_Manager::TEXT,
                    'label_block' => true,
                    'dynamic'     => ['active' => true],
                    'default' => esc_html__( 'Button text', 'selection' ),
                    'placeholder' => esc_html__( 'Button', 'selection' ),
                    'condition' => [ 'general_source!' => ['single-image', 'image-gallery' ], 'general_type_events' => 'button']
                ]
            );

            /** Scroll */
            $this->add_control(
                'general_scroll',
                [
                    'label'           => esc_html__( 'Scroll', 'selection' ),
                    'type'            => Controls_Manager::SLIDER,
                    'size_units'      => [ 'px' ],
                    'range'           => [
                        'px' => [
                            'min'  => 0,
                            'step' => 1,
                        ],
                    ],
                    'default' => [
                        'unit' => 'px',
                        'size' => 350,
                    ],
                    'description' => esc_html__( 'Show a modal window by scrolling the specified number of pixels in height.', 'selection' ),
                    'condition' => [ 'general_source!' => 'single-image', 'general_type_events' => 'scroll' ]
                ]
            );

            /** Popup open limit */
            $this->add_control(
                'navigation_popup_open_limit',
                [
                    'label'           => esc_html__( 'Popup open limit', 'selection' ),
                    'type'            => Controls_Manager::SLIDER,
                    'size_units'      => [ 'px' ],
                    'range'           => [
                        'px' => [
                            'min'  => 0,
                            'step' => 1,
                        ],
                    ],
                    'default' => [
                        'unit' => 'px',
                        'size' => 1,
                    ],
                    'condition' => [ 'general_source!' => 'single-image', 'general_type_events' => 'scroll' ],
                ]
            );

            /** Timeout */
            $this->add_control(
                'general_timeout',
                [
                    'label'           => esc_html__( 'Timeout', 'selection' ),
                    'type'            => Controls_Manager::SLIDER,
                    'size_units'      => [ 'px' ],
                    'range'           => [
                        'px' => [
                            'min'  => 0,
                            'step' => 100,
                            'max' => 10000
                        ],
                    ],
                    'default' => [
                        'unit' => 'px',
                        'size' => 1000,
                    ],
                    'description' => esc_html__( 'Measured in milliseconds', 'selection' ),
                    'condition' => [ 'general_source!' => 'single-image', 'general_type_events' => 'timeout' ]
                ]
            );

		$this->end_controls_section();

    }

	/**
	 * Navigation TAB.
	 */
	public function get_navigation_section_content() {

		$this->start_controls_section(
			'navigation_content',
			[
				'label' => esc_html__( 'Navigation', 'selection' ),
				'tab'   => Controls_Manager::TAB_CONTENT
			]
		);

            /** Opening animation. */
            $this->add_control(
                'navigation_animation',
                [
                    'label' => esc_html__( 'Opening animation', 'selection' ),
                    'type' => Controls_Manager::SELECT,
                    'default' => 'none',
                    'options' => [
                        'none'  => esc_html__( 'None', 'selection' ),
                        'fade'  => esc_html__( 'Fade', 'selection' ),
                    ],
                ]
            );

            /** Animation speed. */
            $this->add_control(
                'navigation_animation_speed',
                [
                    'label'           => esc_html__( 'Animation speed', 'selection' ),
                    'type'            => Controls_Manager::SLIDER,
                    'size_units'      => [ 'px' ],
                    'range'           => [
                        'px' => [
                            'min'  => 0,
                            'step' => 1,
                        ],
                    ],
                    'default' => [
                        'unit' => 'px',
                        'size' => 300,
                    ],
                    'description' => esc_html__( 'Measured in milliseconds', 'selection' ),
                    'condition' => [ 'navigation_animation' => 'fade' ]
                ]
            );

            /** Forbidden to close. */
            $this->add_control(
                'navigation_close_keypress',
                [
                    'label' => esc_html__( 'Forbidden to close', 'selection' ),
                    'type' => Controls_Manager::SWITCHER,
                    'label_on' => esc_html__( 'Yes', 'selection' ),
                    'label_off' => esc_html__( 'No', 'selection' ),
                    'return_value' => 'yes',
                ]
            );

            /** Display close button. */
            $this->add_control(
                'navigation_close_button',
                [
                    'label' => esc_html__( 'Display close button', 'selection' ),
                    'type' => Controls_Manager::SWITCHER,
                    'label_on' => esc_html__( 'Yes', 'selection' ),
                    'label_off' => esc_html__( 'No', 'selection' ),
                    'return_value' => 'yes',
                    'default' => 'yes',
                    'condition' => [ 'navigation_close_keypress!' => 'yes' ]
                ]
            );

            /** Close via overlay. */
            $this->add_control(
                'navigation_close_overlay',
                [
                    'label' => esc_html__( 'Close via overlay', 'selection' ),
                    'type' => Controls_Manager::SWITCHER,
                    'label_on' => esc_html__( 'Yes', 'selection' ),
                    'label_off' => esc_html__( 'No', 'selection' ),
                    'return_value' => 'yes',
                    'default' => 'yes',
                    'condition' => [ 'navigation_close_keypress!' => 'yes' ]
                ]
            );

            /** Popup title */
            $this->add_control(
                'navigation_popup_title',
                [
                    'label' => esc_html__( 'Popup title', 'modalier-domain' ),
                    'type' => Controls_Manager::TEXT,
                    'dynamic'     => ['active' => true],
                    'placeholder' => esc_html__( 'Popup title', 'modalier-domain' ),
                    'description' => esc_html__( 'Set the aria-label attribute value used for Accessibility purposes.', 'modalier-domain' ),
                ]
            );

            /** Close text */
            $this->add_control(
                'navigation_close_text',
                [
                    'label' => esc_html__( 'Close text', 'modalier-domain' ),
                    'type' => Controls_Manager::TEXT,
                    'dynamic'     => ['active' => true],
                    'placeholder' => esc_html__( 'Close text', 'modalier-domain' ),
	                'description' => esc_html__( 'String for close button text. Available for localisation and alternative languages to be used.', 'modalier-domain' ),
                ]
            );

            /** Close aria label */
            $this->add_control(
                'navigation_close_labelt',
                [
                    'label' => esc_html__( 'Close aria label', 'modalier-domain' ),
                    'type' => Controls_Manager::TEXT,
                    'dynamic'     => ['active' => true],
                    'placeholder' => esc_html__( 'Close aria label', 'modalier-domain' ),
                    'description' => esc_html__( 'String for close button aria-label attribute.Available for localisation and alternative languages to be used.', 'modalier-domain' ),
                ]
            );

            /** Gallery active class */
            $this->add_control(
                'navigation_gallery_active_class',
                [
                    'label' => esc_html__( 'Gallery active class', 'modalier-domain' ),
                    'type' => Controls_Manager::TEXT,
                    'dynamic'     => ['active' => true],
                    'placeholder' => esc_html__( 'Gallery active class', 'modalier-domain' ),
                    'condition' => [ 'general_source' => 'image-gallery' ]
                ]
            );

            /** Next/prev controls outside. */
            $this->add_control(
                'navigation_controls_outside',
                [
                    'label' => esc_html__( 'Next/prev controls outside', 'selection' ),
                    'type' => Controls_Manager::SWITCHER,
                    'label_on' => esc_html__( 'Yes', 'selection' ),
                    'label_off' => esc_html__( 'No', 'selection' ),
                    'condition' => [ 'general_source' => 'image-gallery' ]
                ]
            );

		$this->end_controls_section();

    }

	/**
	 * Layout TAB.
	 */
	public function get_layout_section_content() {

		$this->start_controls_section(
			'layout_content',
			[
				'label' => esc_html__( 'Layout', 'selection' ),
				'tab'   => Controls_Manager::TAB_CONTENT,
				'condition' => [ 'general_source' => [ 'posts', 'custom-posts', 'woocommerce', 'custom' ] ],
			]
		);

            $layout = new Repeater();

            /** Item. */
            $layout->add_control(
                'layout_item',
                [
                    'label'   => esc_html__( 'Item', 'selection' ),
                    'type'    => Controls_Manager::SELECT,
                    'default' => 'title',
                    'options' => [
                        'image'       => esc_html__( 'Image', 'selection' ),
                        'title'       => esc_html__( 'Title', 'selection' ),
                        'description' => esc_html__( 'Description', 'selection' ),
                        'price'       => esc_html__( 'Price', 'selection' ),
                        'button'      => esc_html__( 'Button', 'selection' ),
                    ],
                ]
            );

            /** Priority. */
            $this->add_control(
                'layout_priority',
                [
                    'label'       => esc_html__( 'Priority', 'selection' ),
                    'type'        => Controls_Manager::REPEATER,
                    'fields'      => $layout->get_controls(),
                    'default'     => [
                        [ 'layout_item' => 'image' ],
                        [ 'layout_item' => 'title' ],
                        [ 'layout_item' => 'description' ],
                    ],
                    'title_field' => 'Layout item  - {{{layout_item}}}'
                ]
            );

		$this->end_controls_section();

    }

	/**
	 * Content TAB.
	 */
	public function get_section_content() {

		$this->start_controls_section(
			'section_content',
			[
				'label' => esc_html__( 'Content', 'selection' ),
				'tab'   => Controls_Manager::TAB_CONTENT,
				'condition' => [ 'general_source' => [ 'custom', 'ajax', 'single-image', 'image-gallery', 'video', 'iframe' ] ]
			]
		);

		    /** Gallery. */
            $this->add_control(
                'content_gallery',
                [
                    'label' => esc_html__( 'Gallery', 'selection' ),
                    'type' => Controls_Manager::GALLERY,
                    'default' => [],
                    'condition' => [ 'general_source' => 'image-gallery' ]
                ]
            );

		    /** Image */
            $this->add_control(
                'content_image',
                [
                    'label' => esc_html__( 'Image', 'selection' ),
                    'type' => Controls_Manager::MEDIA,
                    'default' => [
                        'url' => Utils::get_placeholder_image_src(),
                    ],
                    'conditions' => [
	                    'relation' => 'or',
	                    'terms' => [
		                    [
			                    'name' => 'general_source',
			                    'operator' => '===',
			                    'value' => 'single-image'
		                    ],
		                    [
			                    'name' => 'general_source',
			                    'operator' => '===',
			                    'value' => 'custom'
		                    ]
	                    ]
                    ]
                ]
            );

            /** Title */
            $this->add_control(
                'content_title',
                [
                    'label' => esc_html__( 'Title', 'selection' ),
                    'type' => Controls_Manager::TEXT,
                    'label_block' => true,
                    'dynamic'     => ['active' => true],
                    'default' => esc_html__( 'Header text', 'selection' ),
                    'placeholder' => esc_html__( 'Title', 'selection' ),
                    'condition' => [ 'general_source' => 'custom' ]
                ]
            );

            /** Description */
            $this->add_control(
                'content_description',
                [
                    'label' => esc_html__( 'Description', 'selection' ),
                    'type' => Controls_Manager::WYSIWYG,
                    'default' => esc_html__( 'Default description', 'selection' ),
                    'placeholder' => esc_html__( 'Type your description here', 'selection' ),
                    'condition' => [ 'general_source' => 'custom' ]
                ]
            );

            /** Button Text */
            $this->add_control(
                'content_button_text',
                [
                    'label' => esc_html__( 'Button Text', 'selection' ),
                    'type' => Controls_Manager::TEXT,
                    'label_block' => true,
                    'dynamic'     => ['active' => true],
                    'placeholder' => esc_html__( 'Button Text', 'selection' ),
                    'condition' => [ 'general_source' => 'custom' ]
                ]
            );

            /** Button link. */
            $this->add_control(
                'content_button_link',
                [
                    'label' => esc_html__( 'Link', 'selection' ),
                    'type' => Controls_Manager::URL,
                    'placeholder' => esc_html__( 'https://codecanyon.net/user/merkulove', 'selection' ),
                    'show_external' => true,
                    'default' => [
                        'url' => '',
                        'is_external' => true,
                        'nofollow' => true,
                    ],
                    'condition' => [ 'general_source' => 'custom' ]
                ]
            );

            /** AJAX content link. */
            $this->add_control(
                'content_ajax_link',
                [
                    'label' => esc_html__( 'AJAX content link', 'selection' ),
                    'type' => Controls_Manager::TEXT,
                    'label_block' => true,
                    'dynamic'     => ['active' => true],
                    'description' => esc_html__( 'Internal pages of the site', 'selection' ),
                    'placeholder' => esc_html__( 'AJAX content link', 'selection' ),
                    'condition' => [ 'general_source' => 'ajax' ]
                ]
            );

            /** Video */
            $this->add_control(
                'content_video_link',
                [
                    'label' => esc_html__( 'Video link', 'selection' ),
                    'type' => Controls_Manager::TEXT,
                    'label_block' => true,
                    'dynamic'     => ['active' => true],
                    'placeholder' => esc_html__( 'Video link', 'selection' ),
                    'description' => esc_html__( 'https://www.youtube.com/embed/ID or https://player.vimeo.com/video/ID where the ID at the end is your unique video id', 'selection' ),
                    'condition' => [ 'general_source' => 'video' ]
                ]
            );

            /** iframe. */
            $this->add_control(
                'content_iframe_link',
                [
                    'label' => esc_html__( 'Iframe link', 'selection' ),
                    'type' => Controls_Manager::TEXT,
                    'label_block' => true,
                    'dynamic'     => ['active' => true],
                    'placeholder' => esc_html__( 'Iframe link', 'selection' ),
                    'condition' => [ 'general_source' => 'iframe' ]
                ]
            );

		$this->end_controls_section();

    }

	/**
	 *  TAB: Style.
	 */

	/**
	 * Modal styles TAB.
	 */
	public function get_modal_section_styles() {

		$this->start_controls_section( 'modal_style', [
			'label' => esc_html__( 'Modal', 'selection' ),
			'tab'   => Controls_Manager::TAB_STYLE
		] );

            /** Fullscreen Mode. */
            $this->add_control(
                'navigation_fullscreen_mode',
                [
                    'label' => esc_html__( 'Fit fullscreen', 'selection' ),
                    'type' => Controls_Manager::SWITCHER,
                    'label_on' => esc_html__( 'Yes', 'selection' ),
                    'label_off' => esc_html__( 'No', 'selection' ),
                    'condition' => [
                        'general_source!' => ['video', 'single-image' ]
                    ],
                    'return_value' => 'yes',
                ]
            );

            /** Width. */
            $this->add_responsive_control(
                'modal_style_width',
                [
                    'label'           => esc_html__( 'Width(px)', 'selection' ),
                    'type'            => Controls_Manager::SLIDER,
                    'size_units'      => [ 'px' ],
                    'range'           => [
                        'px' => [
                            'min'  => 1,
                            'step' => 1,
                            'max' => 3000
                        ],
                    ],
                    'desktop_default' => [
	                    'size' => 700,
	                    'unit' => 'px',
                    ],
                    'tablet_default' => [
	                    'size' => 600,
	                    'unit' => 'px',
                    ],
                    'mobile_default' => [
	                    'size' => 320,
	                    'unit' => 'px',
                    ],
                    'selectors'  => [
	                    '{{WRAPPER}} .modaal-video-container'  => 'width: {{size}}{{unit}}',
                    ],
                    'condition' => [
	                    'navigation_fullscreen_mode!' => 'yes',
	                    'general_source!' => 'single-image'
                    ],
                ]
            );

            /** Height. */
            $this->add_responsive_control(
                'modal_style_height',
                [
                    'label'           => esc_html__( 'Height(px)', 'selection' ),
                    'type'            => Controls_Manager::SLIDER,
                    'size_units'      => [ 'px' ],
                    'range'           => [
                        'px' => [
                            'min'  => 1,
                            'step' => 1,
                            'max' => 1500
                        ],
                    ],
                    'desktop_default' => [
	                    'size' => 500,
	                    'unit' => 'px',
                    ],
                    'tablet_default' => [
	                    'size' => 450,
	                    'unit' => 'px',
                    ],
                    'mobile_default' => [
	                    'size' => 400,
	                    'unit' => 'px',
                    ],
                    'selectors'  => [
	                    '{{WRAPPER}} .modaal-video-container'  => 'height: {{size}}{{unit}}',
                    ],
                    'condition' => [
	                    'navigation_fullscreen_mode!' => 'yes',
	                    'general_source!' => 'single-image'
                    ],
                ]
            );

            /** Overlay */
            $this->add_control(
                'modal_style_header',
                [
                    'label' => esc_html__( 'Overlay', 'selection' ),
                    'type' => Controls_Manager::HEADING,
                    'separator' => 'after',
                ]
            );

            /** Background */
            $this->add_control(
                'modal_style_overlay_color',
                [
                    'label' => esc_html__( 'Background', 'selection' ),
                    'type' => Controls_Manager::COLOR,
                    'default' => '#000000',
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => Color::COLOR_1,
                    ],
                ]
            );

            /** Overlay opacity */
            $this->add_control(
                'modal_style_overlay_opacity',
                [
                    'label'           => esc_html__( 'Overlay opacity', 'selection' ),
                    'type'            => Controls_Manager::SLIDER,
                    'size_units'      => [ 'px'],
                    'range'           => [
                        'px' => [
                            'min'  => 0.1,
                            'step' => 0.1,
                            'max' => 1
                        ],
                    ],
                    'default' => [
                        'unit' => 'px',
                        'size' => 0.7,
                    ],
                ]
            );

            /** Blur Content */
            $this->add_control(
                'modal_style_overlay_blur',
                [
                    'label' => esc_html__( 'Blur Content', 'selection' ),
                    'type' => Controls_Manager::SWITCHER,
                    'label_on' => esc_html__( 'Yes', 'selection' ),
                    'label_off' => esc_html__( 'No', 'selection' ),
                ]
            );

            /** Filter blur. */
            $this->add_control(
                'modal_style_overlay_filter_blur',
                [
                    'label'           => esc_html__( 'Filter blur', 'selection' ),
                    'type'            => Controls_Manager::SLIDER,
                    'size_units'      => ['px'],
                    'range'           => [
                        'px' => [
                            'min'  => 1,
                            'step' => 1,
                            'max' => 10
                        ],
                    ],
                    'default' => [
                        'unit' => 'px',
                        'size' => 4,
                    ],
                    'condition' => ['modal_style_overlay_blur' => 'yes']
                ]
            );

		$this->add_control(
			'modal_style_heading',
			[
				'label' => esc_html__( 'Style Modal', 'selection' ),
                'type' => Controls_Manager::HEADING,
                'separator' => 'before',
			]
		);

        $class = '.modaal-container';
		if( Plugin::$instance->editor->is_edit_mode() === true ){
			$class = '.mdp-modalier-box';
        }

		$this->add_responsive_control(
			'modalier_modal_style_padding',
			[
				'label'      => esc_html__( 'Padding', 'selection' ),
				'type'       => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'em' ],
				'selectors'  => [
					'{{WRAPPER}} .modaal-container' . $class => 'padding: {{top}}{{unit}} {{right}}{{unit}} {{bottom}}{{unit}} {{left}}{{unit}};',
				],
				'default' => [
					'top' => 0,
					'right' => 0,
					'bottom' => 0,
					'left' => 0,
					'unit' => 'px',
				],
				'toggle'     => true,
			]
		);

		$this->get_settings_style_group(
			'modal_style',
			$class,
			[],
			[
				'background' => true,
				'border' => true,
				'border-radius' => true,
				'box-shadow' => true,
			]
		);

		$this->end_controls_section();

	}

	/**
	 * Item styles TAB.
	 */
	public function get_item_section_styles() {

	    $this->start_controls_section( 'item_style', [
			'label' => esc_html__( 'Item', 'selection' ),
			'tab'   => Controls_Manager::TAB_STYLE,
			'condition' => [ 'general_source' => ['posts', 'custom-posts', 'woocommerce'] ]
		] );

            /** Width. */
            $this->add_responsive_control(
                'item_style_width',
                [
                    'label'           => esc_html__( 'Width', 'selection' ),
                    'type'            => Controls_Manager::SLIDER,
                    'size_units'      => [ 'px', '%'],
                    'range'           => [
                        '%' => [
                            'min'  => 1,
                            'step' => 1,
                        ],
                    ],
                    'default' => [
                        'unit' => '%',
                        'size' => 49,
                    ],
                    'selectors'  => [
                        '{{WRAPPER}} .mdp-container-item'  => 'width: {{size}}{{unit}}',
                    ],
                ]
            );

            $this->get_settings_style_group(
                'item_style',
                '.mdp-container-item',
                [],
                [
	                'margin' => true,
	                'padding' => true,
	                'alignment' => true,
	                'background' => true,
	                'border' => true,
	                'border-radius' => true,
	                'box-shadow' => true,
                ]
            );

		$this->end_controls_section();

	}

	/**
	 * Image styles TAB.
	 */
	public function get_image_section_styles() {

		$this->start_controls_section( 'image_style', [
			'label' => esc_html__( 'Image', 'selection' ),
			'tab'   => Controls_Manager::TAB_STYLE,
			'condition' => [ 'general_source' => ['posts', 'custom-posts', 'woocommerce', 'custom', 'single-image', 'image-gallery'] ]
		] );

            /** Width. */
            $this->add_responsive_control(
                'image_style_width',
                [
                    'label'           => esc_html__( 'Width', 'selection' ),
                    'type'            => Controls_Manager::SLIDER,
                    'size_units'      => [ 'px', '%'],
                    'range'           => [
                        'px' => [
                            'min'  => 1,
                            'step' => 1,
                        ],
                    ],
                    'default' => [
                        'unit' => '%',
                        'size' => 100,
                    ],
                    'selectors'  => [
                        '{{WRAPPER}} .mdp-modalier-image,'.
                        '.modaal-wrapper.mdp-modalier-wrapper-'.$this->get_id().' .mdp-modalier-image' => 'width: {{size}}{{unit}}',
                        '{{WRAPPER}} .gallery-thumb img,'.
                        '.modaal-wrapper.mdp-modalier-wrapper-'.$this->get_id().' .gallery-thumb img' => 'width: {{size}}{{unit}}',
                        '{{WRAPPER}} .mdp-modalier-image-box img' => 'width: {{size}}{{unit}}'
                    ],
                ]
            );

            /** Type image size. */
            $this->add_control(
                'image_style_type_size',
                [
                    'label' => esc_html__( 'Type image size', 'selection' ),
                    'type' => Controls_Manager::SELECT,
                    'default' => 'large',
                    'options' => [
                        'thumbnail'  => esc_html__( 'Thumbnail', 'selection' ),
                        'medium'  => esc_html__( 'Medium', 'selection' ),
                        'large'  => esc_html__( 'Large', 'selection' ),
                        'full'  => esc_html__( 'Full', 'selection' ),
                    ],
                    'conditions' => [
	                    'relation' => 'or',
	                    'terms' => [
		                    [
			                    'name' => 'general_source',
			                    'operator' => '==',
			                    'value' => 'posts'
		                    ],
		                    [
			                    'name' => 'general_source',
			                    'operator' => '==',
			                    'value' => 'woocommerce'
		                    ]
	                    ]
                    ]
                ]
            );

            $this->add_group_control(
                Group_Control_Image_Size::get_type(),
                [
                    'name' => 'thumbnail',
                    'exclude' => [ 'custom' ],
                    'include' => [],
                    'default' => 'large',
                    'condition' => [ 'general_source' => 'single-image' ]
                ]
            );

            /** Float right. */
            $this->add_control(
                'image_style_float',
                [
                    'label'     => esc_html__( 'Alignment', 'selection' ),
                    'type'      => Controls_Manager::CHOOSE,
                    'options'   => [
                        'flex-start'    => [
                            'title' => esc_html__( 'Left', 'selection' ),
                            'icon'  => 'fa fa-align-left',
                        ],
                        'center'  => [
                            'title' => esc_html__( 'Center', 'selection' ),
                            'icon'  => 'fa fa-align-center',
                        ],
                        'flex-end'   => [
                            'title' => esc_html__( 'Right', 'selection' ),
                            'icon'  => 'fa fa-align-right',
                        ],
                    ],
                    'default'   => 'center',
                    'selectors' => [
                        '{{WRAPPER}} .mdp-modalier-image-box,'.
                        '{{WRAPPER}} .modaal-wrapper .mdp-modalier-image-box' => 'justify-content: {{VALUE}};',
                    ],
                    'conditions' => [
	                    'terms' => [
		                    [
			                    'name' => 'general_source',
			                    'operator' => 'in',
			                    'value' => [
				                    'posts',
				                    'custom-posts',
				                    'woocommerce',
				                    'custom',
			                    ]
		                    ]
	                    ]
                    ]
                ]
            );

            /** Alignment. */
            $this->add_control(
                'image_style_align_gallery',
                [
                    'label'     => esc_html__( 'Alignment', 'selection' ),
                    'type'      => Controls_Manager::CHOOSE,
                    'options'   => [
                        'left'    => [
                            'title' => esc_html__( 'Left', 'selection' ),
                            'icon'  => 'fa fa-align-left',
                        ],
                        'center'  => [
                            'title' => esc_html__( 'Center', 'selection' ),
                            'icon'  => 'fa fa-align-center',
                        ],
                        'right'   => [
                            'title' => esc_html__( 'Right', 'selection' ),
                            'icon'  => 'fa fa-align-right',
                        ],
                    ],
                    'default'   => 'center',
                    'selectors' => [
	                    '{{WRAPPER}} .mdp-modalier-box' => 'text-align: {{VALUE}};'
                    ],
                    'conditions' => [
	                    'terms' => [
		                    [
			                    'name' => 'general_source',
			                    'operator' => 'in',
			                    'value' => [
				                    'single-image',
				                    'image-gallery'
			                    ]
		                    ]
	                    ]
                    ]
                ]
            );

            $this->get_settings_style_group(
                'image_style',
                '.mdp-modalier-image, {{WRAPPER}} .gallery-thumb img,'.
                '.modaal-wrapper.mdp-modalier-wrapper-'.$this->get_id().' .mdp-modalier-image,'.
                '.modaal-wrapper.mdp-modalier-wrapper-'.$this->get_id().' .gallery-thumb img,'.
                '{{WRAPPER}} .mdp-modalier-image-box',
                [],
                [
                    'margin' => true,
                    'padding' => true,
                    'background' => true,
                ]
            );

            $this->get_settings_style_group(
                'image_style',
                '.mdp-modalier-image, {{WRAPPER}} .gallery-thumb img,'.
                '.modaal-wrapper.mdp-modalier-wrapper-'.$this->get_id().' .mdp-modalier-image,'.
                '.modaal-wrapper.mdp-modalier-wrapper-'.$this->get_id().' .gallery-thumb img,'.
                '{{WRAPPER}} .mdp-modalier-image-box img',
                [],
                [
                    'border' => true,
                    'border-radius' => true,
                    'box-shadow' => true,
                    'css-filter' => true
                ]
            );

		$this->end_controls_section();

	}

	/**
	 * Title styles TAB.
	 */
	public function get_title_section_styles() {

		$this->start_controls_section( 'title_style', [
			'label' => esc_html__( 'Title', 'selection' ),
			'tab'   => Controls_Manager::TAB_STYLE,
			'condition' => [ 'general_source' => ['posts', 'custom-posts', 'woocommerce', 'custom'] ]
		] );

            /** Width. */
            $this->add_responsive_control(
                'title_style_width',
                [
                    'label'           => esc_html__( 'Width', 'selection' ),
                    'type'            => Controls_Manager::SLIDER,
                    'size_units'      => [ 'px', '%'],
                    'range'           => [
                        'px' => [
                            'min'  => 1,
                            'step' => 1,
                        ],
                    ],
                    'default' => [
                        'unit' => '%',
                        'size' => 100,
                    ],
                    'selectors'  => [
                        '{{WRAPPER}} .mdp-modalier-title' => 'width: {{size}}{{unit}}; max-width: {{size}}{{unit}};',
                    ],
                ]
            );

            $this->get_settings_style_group(
                'title_style',
	            '.mdp-modalier-title',
                [],
                [
                    'margin' => true,
                    'padding' => true,
                    'color' => true,
                    'typography' => true,
                    'text-shadow' => true,
                    'alignment' => true,
                    'background' => true,
                    'border' => true,
                    'border-radius' => true,
                    'box-shadow' => true,
                ]
            );

		$this->end_controls_section();

	}

	/**
	 * Description styles TAB.
	 */
	public function get_description_section_styles() {

		$this->start_controls_section( 'description_style', [
			'label' => esc_html__( 'Description', 'selection' ),
			'tab'   => Controls_Manager::TAB_STYLE,
			'condition' => [ 'general_source' => ['posts', 'custom-posts', 'woocommerce', 'custom'] ]
		] );

            /** Width. */
            $this->add_responsive_control(
                'description_style_width',
                [
                    'label'           => esc_html__( 'Width', 'selection' ),
                    'type'            => Controls_Manager::SLIDER,
                    'size_units'      => [ 'px', '%'],
                    'range'           => [
                        'px' => [
                            'min'  => 1,
                            'step' => 1,
                        ],
                    ],
                    'default' => [
                        'unit' => '%',
                        'size' => 100,
                    ],
                    'selectors'  => [
                        '{{WRAPPER}} .mdp-modalier-description,'.
                        '.modaal-wrapper.mdp-modalier-wrapper-'.$this->get_id().' .mdp-modalier-description,'.
                        '.modaal-wrapper.mdp-modalier-wrapper-'.$this->get_id().' .mdp-modalier-description > *' => 'width: {{size}}{{unit}}',
                    ],
                ]
            );

            $this->get_settings_style_group(
                'description_style',
                '.mdp-modalier-description,'.
                '.modaal-wrapper.mdp-modalier-wrapper-'.$this->get_id().' .mdp-modalier-description,'.
                '.modaal-wrapper.mdp-modalier-wrapper-'.$this->get_id().' .mdp-modalier-description > *',
                [],
                [
                    'margin' => true,
                    'padding' => true,
                    'color' => true,
                    'typography' => true,
                    'text-shadow' => true,
                    'alignment-justify' => true,
                    'background' => true,
                    'border' => true,
                    'border-radius' => true,
                    'box-shadow' => true,
                ]
            );

		$this->end_controls_section();

	}


	/**
	 *  Link styles TAB.
	 */
	public function get_link_section_styles() {

		$this->start_controls_section( 'link_style', [
			'label' => esc_html__( 'Post Link', 'selection' ),
			'tab'   => Controls_Manager::TAB_STYLE,
			'condition' => [ 'general_source' => ['posts', 'custom-posts', 'woocommerce', 'custom'] ]
		] );

            /** Width. */
            $this->add_responsive_control(
                'link_style_width',
                [
                    'label'           => esc_html__( 'Width', 'selection' ),
                    'type'            => Controls_Manager::SLIDER,
                    'size_units'      => [ 'px', '%'],
                    'range'           => [
                        'px' => [
                            'min'  => 1,
                            'step' => 1,
                        ],
                    ],
                    'default' => [
                        'unit' => '%',
                        'size' => 100,
                    ],
                    'selectors'  => [
                        '{{WRAPPER}} .mdp-modalier-link, '.
                        '.modaal-wrapper.mdp-modalier-wrapper-'.$this->get_id().' .mdp-modalier-link' => 'width: {{size}}{{unit}}',
                    ],
                ]
            );

            $this->get_settings_style_group(
                'link_options_style',
                '.mdp-modalier-box-link, '.
                '.modaal-wrapper.mdp-modalier-wrapper-'.$this->get_id().' .mdp-modalier-box-link',
                [],
                [
                    'margin' => true,
                ]
            );

            $this->get_settings_style_group(
                'link_options_style',
                '.mdp-modalier-box-link a, '.
                '.modaal-wrapper.mdp-modalier-wrapper-'.$this->get_id().' .mdp-modalier-box-link a',
                [],
                [
                    'padding' => true,
                ]
            );

            $this->get_settings_style_group(
                'link_options_style',
                '.mdp-modalier-link, '.
                '.modaal-wrapper.mdp-modalier-wrapper-'.$this->get_id().' .mdp-modalier-link',
                [],
                [
                    'color' => true,
                    'typography' => true,
                    'text-shadow' => true,
                ]
            );

            $this->get_settings_style_group(
                'link_options_style',
                '.mdp-modalier-box-link, '.
                '.modaal-wrapper.mdp-modalier-wrapper-'.$this->get_id().' .mdp-modalier-box-link',
                [],
                [
                    'alignment' => true,
                ]
            );

            $this->get_settings_style_group(
                'link_options_style',
                '.mdp-modalier-link, '.
                '.modaal-wrapper.mdp-modalier-wrapper-'.$this->get_id().' .mdp-modalier-link',
                [],
                [
                    'background' => true,
                    'border' => true,
                    'border-radius' => true,
                    'box-shadow' => true,
                ]
            );

		$this->end_controls_section();

	}

	/**
	 * Price styles TAB.
	 */
	public function get_price_section_styles() {

		$this->start_controls_section( 'price_style', [
			'label' => esc_html__( 'Price', 'selection' ),
			'tab'   => Controls_Manager::TAB_STYLE,
			'condition' => [ 'general_source' => 'woocommerce' ]
		] );

            /** Width. */
            $this->add_responsive_control(
                'price_style_width',
                [
                    'label'           => esc_html__( 'Width', 'selection' ),
                    'type'            => Controls_Manager::SLIDER,
                    'size_units'      => [ 'px', '%'],
                    'range'           => [
                        'px' => [
                            'min'  => 1,
                            'step' => 1,
                        ],
                    ],
                    'default' => [
                        'unit' => '%',
                        'size' => 100,
                    ],
                    'selectors'  => [
                        '{{WRAPPER}} .mdp-modalier-price' => 'width: {{size}}{{unit}}',
                    ],
                ]
            );

            $this->get_settings_style_group(
                'price_style',
                '.mdp-modalier-price',
                [],
                [
                    'margin' => true,
                ]
            );

            $this->get_settings_style_group(
                'price_style',
                '.mdp-modalier-price del, {{WRAPPER}} .mdp-modalier-price > span',
                [],
                [
                    'padding' => true,
                ]
            );

            $this->get_settings_style_group(
                'price_style',
                '.mdp-modalier-price',
                [],
                [
                    'alignment' => true,
                ]
            );

            $this->get_settings_style_group(
                'price_style',
                '.mdp-modalier-price del, {{WRAPPER}} .mdp-modalier-price > span',
                [],
                [
                    'color' => true,
                    'typography' => true,
                    'text-shadow' => true,
                    'background' => true,
                    'border' => true,
                    'border-radius' => true,
                    'box-shadow' => true,
                ]
            );

            /** Discount header. */
            $this->add_control(
                'price_discount_style_header',
                [
                    'label' => esc_html__( 'Discount', 'selection' ),
                    'type' => Controls_Manager::HEADING,
                    'separator' => 'before',
                ]
            );

            $this->get_settings_style_group(
                'price_discount_style',
                '.mdp-modalier-price ins',
                [],
                [
                    'padding' => true,
                ]
            );

            $this->get_settings_style_group(
                'price_discount_style',
                '.mdp-modalier-price ins',
                [],
                [
                    'color' => true,
                    'typography' => true,
                    'text-shadow' => true,
                    'background' => true,
                    'border' => true,
                    'border-radius' => true,
                    'box-shadow' => true,
                ]
            );

		$this->end_controls_section();

	}

	/**
	 * Close button TAB.
	 */
	public function get_close_button_styles() {

	    $this->start_controls_section( 'close_button_style', [
			'label' => esc_html__( 'Close button', 'selection' ),
			'tab'   => Controls_Manager::TAB_STYLE,
		] );

		$this->add_control(
			'close_button_size_header',
			[
				'label' => esc_html__( 'Button size', 'selection' ),
				'type' => Controls_Manager::HEADING,
				'separator' => 'before',
			]
		);

		/** Width. */
		$this->add_responsive_control(
			'close_button_width',
			[
				'label'           => esc_html__( 'Width(px)', 'selection' ),
				'type'            => Controls_Manager::SLIDER,
				'size_units'      => [ 'px' ],
				'range'           => [
					'px' => [
						'min'  => 1,
						'step' => 1,
					],
				],
				'default' => [
					'unit' => 'px',
					'size' => 50,
				],
				'selectors' => [
					'{{WRAPPER}} .modaal-close' => 'width: {{SIZE}}{{UNIT}};',
				],
			]
		);

		/** Height. */
		$this->add_responsive_control(
			'close_button_h',
			[
				'label'           => esc_html__( 'Height(px)', 'selection' ),
				'type'            => Controls_Manager::SLIDER,
				'size_units'      => [ 'px' ],
				'range'           => [
					'px' => [
						'min'  => 1,
						'step' => 1,
					],
				],
				'default' => [
					'unit' => 'px',
					'size' => 50,
				],
				'selectors' => [
					'{{WRAPPER}} .modaal-close' => 'height: {{SIZE}}{{UNIT}};',
				],
			]
		);

		/**
		 * Border radius
		 */
		$this->add_responsive_control(
			'close_button_border_radius',
			[
				'label'           => esc_html__( 'Border radius', 'selection' ),
				'type'            => Controls_Manager::SLIDER,
				'size_units'      => [ 'px', '%'],
				'range'           => [
					'%' => [
						'min'  => 1,
						'step' => 1,
					],
				],
				'default' => [
					'unit' => '%',
					'size' => 100,
				],
				'selectors' => [
					'{{WRAPPER}} .modaal-close' => 'border-radius: {{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->add_control(
			'close_button_position_header',
			[
				'label' => esc_html__( 'Button position ', 'selection' ),
				'type' => Controls_Manager::HEADING,
				'separator' => 'before',
			]
		);

		/**
		 * Top
		 */
		$this->add_responsive_control(
			'close_button_top_position',
			[
				'label'           => esc_html__( 'Top(px)', 'selection' ),
				'type'            => Controls_Manager::SLIDER,
				'size_units'      => [ 'px' ],
				'range'           => [
					'px' => [
						'min'  => 1,
						'step' => 1,
					],
				],
				'default' => [
					'unit' => 'px',
					'size' => 20,
				],
				'selectors' => [
					'{{WRAPPER}} .modaal-close' => 'top: {{SIZE}}{{UNIT}};',
				],
			]
		);

		/**
		 * Right
		 */
		$this->add_responsive_control(
			'close_button_right_position',
			[
				'label'           => esc_html__( 'Right(px)', 'selection' ),
				'type'            => Controls_Manager::SLIDER,
				'size_units'      => [ 'px' ],
				'range'           => [
					'px' => [
						'min'  => 1,
						'step' => 1,
					],
				],
				'default' => [
					'unit' => 'px',
					'size' => 20,
				],
                'selectors' => [
                    '{{WRAPPER}} .modaal-close' => 'right: {{SIZE}}{{UNIT}};',
                ],
			]
		);

		$this->add_control(
			'close_button_icon_size_header',
			[
				'label' => esc_html__( 'Icon size', 'selection' ),
				'type' => Controls_Manager::HEADING,
				'separator' => 'before',
			]
		);

		/**
		 * Width
		 */
		$this->add_responsive_control(
			'close_button_top_size_width',
			[
				'label'           => esc_html__( 'Width(px)', 'selection' ),
				'type'            => Controls_Manager::SLIDER,
				'size_units'      => [ 'px' ],
				'range'           => [
					'px' => [
						'min'  => 1,
						'step' => 1,
					],
				],
				'default' => [
					'unit' => 'px',
					'size' => 4,
				],
				'selectors' => [
					'{{WRAPPER}} .modaal-close:before' => 'width: {{SIZE}}{{UNIT}};',
					'{{WRAPPER}} .modaal-close:after' => 'width: {{SIZE}}{{UNIT}};',
				],
			]
		);

		/**
		 * Height
		 */
		$this->add_responsive_control(
			'close_button_top_size_height',
			[
				'label'           => esc_html__( 'Height(px)', 'selection' ),
				'type'            => Controls_Manager::SLIDER,
				'size_units'      => [ 'px' ],
				'range'           => [
					'px' => [
						'min'  => 1,
						'step' => 1,
					],
				],
				'default' => [
					'unit' => 'px',
					'size' => 22,
				],
				'selectors' => [
					'{{WRAPPER}} .modaal-close:before' => 'height: {{SIZE}}{{UNIT}};',
					'{{WRAPPER}} .modaal-close:after' => 'height: {{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->add_control(
			'close_button_icon_position_header',
			[
				'label' => esc_html__( 'Icon position', 'selection' ),
				'type' => Controls_Manager::HEADING,
				'separator' => 'before',
			]
		);

		/**
		 * Top
		 */
		$this->add_responsive_control(
			'close_button_top_icon_position',
			[
				'label'           => esc_html__( 'Top(px)', 'selection' ),
				'type'            => Controls_Manager::SLIDER,
				'size_units'      => [ 'px' ],
				'range'           => [
					'px' => [
						'min'  => 1,
						'step' => 1,
					],
				],
				'default' => [
					'unit' => 'px',
					'size' => 14,
				],
                'selectors' => [
                    '{{WRAPPER}} .modaal-close:before' => 'top: {{SIZE}}{{UNIT}};',
                    '{{WRAPPER}} .modaal-close:after' => 'top: {{SIZE}}{{UNIT}};',
                ],
			]
		);

		/**
		 * Left
		 */
		$this->add_responsive_control(
			'close_button_left_icon_position',
			[
				'label'           => esc_html__( 'Left(px)', 'selection' ),
				'type'            => Controls_Manager::SLIDER,
				'size_units'      => [ 'px' ],
				'range'           => [
					'px' => [
						'min'  => 1,
						'step' => 1,
					],
				],
				'default' => [
					'unit' => 'px',
					'size' => 23,
				],
				'selectors' => [
					'{{WRAPPER}} .modaal-close:before' => 'left: {{SIZE}}{{UNIT}};',
					'{{WRAPPER}} .modaal-close:after' => 'left: {{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->add_control(
			'close_button_header',
			[
				'type' => Controls_Manager::HEADING,
				'separator' => 'before',
			]
		);

		$this->start_controls_tabs( 'tabs_close_button' );

            /** Content tab. */
            $this->start_controls_tab(
                'tab_close_button_normal',
                [
                    'label' => esc_html__('NORMAL', 'selection')
                ]
            );

                $this->add_control(
                    'modalier_close_button_style_normal_color',
                    [
                        'label' => esc_html__( 'Color icon', 'selection' ),
                        'type' => Controls_Manager::COLOR,
                        'default' => '#000000',
                        'selectors' => [
	                        '{{WRAPPER}} .modaal-close:before' => 'background-color: {{VALUE}}',
                            '{{WRAPPER}} .modaal-close:after' => 'background-color: {{VALUE}}'
                        ],
                    ]
                );

                /**
                 *  Background color
                 */
                $this->add_control(
                    'modalier_close_button_style_normal_background_color',
                    [
                        'label' => esc_html__( 'Background color', 'selection' ),
                        'type' => Controls_Manager::COLOR,
                        'default' => '#ffffff',
                        'selectors' => [
	                        '{{WRAPPER}} .modaal-close' => 'background-color: {{VALUE}}'
                        ],
                    ]
                );

            $this->end_controls_tab();

            /** Content tab. */
            $this->start_controls_tab(
                'tab_close_button_hover',
                [
                    'label' => esc_html__('HOVER', 'selection')
                ]
            );

                $this->add_control(
                    'modalier_close_button_style_hover_color',
                    [
                        'label' => esc_html__( 'Color icon', 'selection' ),
                        'type' => Controls_Manager::COLOR,
                        'default' => '#ffffff',
                        'selectors' => [
	                        '{{WRAPPER}} .modaal-close:hover:before' => 'background-color: {{VALUE}}',
	                        '{{WRAPPER}} .modaal-close:hover:after' => 'background-color: {{VALUE}}'
                        ],
                    ]
                );

                /**
                 *  Background color
                 */
                $this->add_control(
                    'modalier_close_button_style_hover_background_color',
                    [
                        'label' => esc_html__( 'Background color', 'selection' ),
                        'type' => Controls_Manager::COLOR,
                        'default' => '#000000',
                        'selectors' => [
	                        '{{WRAPPER}} .modaal-close:hover' => 'background-color: {{VALUE}}'
                        ],
                    ]
                );

		    $this->end_controls_tab();

		$this->end_controls_tabs();

		$this->end_controls_section();

	}

	/**
	 * Button styles TAB.
	 */
	public function get_button_section_styles() {

		$this->start_controls_section( 'button_style', [
			'label' => esc_html__( 'Open button', 'selection' ),
			'tab'   => Controls_Manager::TAB_STYLE,
			'condition' => [
                            'general_type_events' => 'button',
                            'general_source!' => [ 'single-image', 'image-gallery' ]
            ]
		] );

		/** Width. */
		$this->add_responsive_control(
			'button_style_width',
			[
				'label'           => esc_html__( 'Width', 'selection' ),
				'type'            => Controls_Manager::SLIDER,
				'size_units'      => [ 'px', '%'],
				'range'           => [
					'px' => [
						'min'  => 1,
						'step' => 1,
					],
				],
				'default' => [
					'unit' => '%',
					'size' => 100,
				],
				'selectors'  => [
					'{{WRAPPER}} .mpd-modalier-button' => 'width: {{size}}{{unit}};',
				],
			]
		);

		$this->get_settings_style_group(
			'button_style',
			'.mpd-modalier-button',
			[],
			[
				'margin' => true,
				'padding' => true,
				'typography' => true,
				'text-shadow' => true,
			]
		);

		$this->add_control(
			'modalier_button_style_alignment',
			[
				'label' => esc_html__( 'Alignment', 'selection' ),
				'type' => Controls_Manager::CHOOSE,
				'options' => [
					'left' => [
						'title' => esc_html__( 'Left', 'selection' ),
						'icon' => 'fa fa-align-left',
					],
					'center' => [
						'title' => esc_html__( 'Center', 'selection' ),
						'icon' => 'fa fa-align-center',
					],
					'right' => [
						'title' => esc_html__( 'Right', 'selection' ),
						'icon' => 'fa fa-align-right',
					],
				],
				'default' => 'center',
				'toggle' => true,
			]
		);

		$this->get_settings_style_group(
			'button_style',
			'.mpd-modalier-button',
			[],
			[
				'border-radius' => true,
			]
		);

		$this->add_control(
			'button_item_header',
			[
				'type' => Controls_Manager::HEADING,
				'separator' => 'before',
			]
		);

		$this->start_controls_tabs( 'tabs_slides_button' );

		/** Content tab. */
		$this->start_controls_tab(
			'tab_normal',
			[
				'label' => esc_html__('NORMAL', 'selection')
			]
		);

		$this->add_control(
			'modalier_button_style_normal_text_color',
			[
				'label' => esc_html__( 'Color', 'selection' ),
				'type' => Controls_Manager::COLOR,
				'default' => '#FF0000',
				'selectors' => [
					'{{WRAPPER}} .mpd-modalier-button' => 'color: {{VALUE}}',
				],
			]
		);

		$this->get_settings_style_group(
			'button_style_normal',
			'.mpd-modalier-button',
			[],
			[
				'background' => true,
				'border' => true,
				'box-shadow' => true,
			],
            [ 'none', 'none', 'none' ]
		);

		$this->end_controls_tab();

		/** Content tab. */
		$this->start_controls_tab(
			'tab_hover',
			[
				'label' => esc_html__('HOVER', 'selection')
			]
		);

		$this->add_control(
			'modalier_button_style_hover_text_color',
			[
				'label' => esc_html__( 'Color', 'selection' ),
				'type' => Controls_Manager::COLOR,
				'default' => '#0016FF',
				'selectors' => [
					'{{WRAPPER}} .mpd-modalier-button:hover' => 'color: {{VALUE}}',
				],
			]
		);

		$this->get_settings_style_group(
			'button_style_hover',
			'.mpd-modalier-button:hover',
			[],
			[
				'background' => true,
				'border' => true,
				'box-shadow' => true,
			],
			[ 'none', 'none', 'none' ]
		);

		$this->end_controls_tab();

		$this->end_controls_tabs();

		$this->end_controls_section();

	}

	/**
     * Add the widget controls.
     *
     * @access protected
     * @return void with category names
     **/
    protected function register_controls() {

	    /**
	     *  TAB: Content.
	     */

	    /** General section. */
	    $this->get_general_section_content();

	    /** Navigation section. */
	    $this->get_navigation_section_content();

	    /** Layout section. */
	    $this->get_layout_section_content();

	    /** Content section. */
	    $this->get_section_content();

	    /**
	     *  TAB: Style.
	     */

	    /** Modal styles section. */
	    $this->get_modal_section_styles();

	    /** Item styles section. */
	    $this->get_item_section_styles();

	    /** Image styles section. */
	    $this->get_image_section_styles();

	    /** Title styles section. */
	    $this->get_title_section_styles();

	    /** Description styles section. */
	    $this->get_description_section_styles();

	    /** Button styles section. */
	    $this->get_link_section_styles();

	    /** Price styles section. */
	    $this->get_price_section_styles();

	    /** Close button styles */
	    $this->get_close_button_styles();

	    /** Button styles section. */
	    $this->get_button_section_styles();

    }

	/**
	 * We return an array with the names of the layouts.
	 *
	 * @param $settings - we get an array with all the data.
	 *
	 * @return array
	 */
	private function get_LayoutSortable( $settings ) {

		/** We get all the data on the lists of layouts. */
		$layout = $settings['layout_priority'];

		/** An array in which the names of the layouts and their width will be stored. */
		$res = array();

		/** We write the names of the layouts and their width into an array. */
        if( isset($layout) ) {
	        foreach ( $layout as $key => $val ) {

		        $res[] = [ 'name' => $val[ 'layout_item' ] ];
	        }
        }

		return $res;

	}

	/**
	 * Displays image for custom slide.
	 *
	 * @param $settings
	 */
	public function print_custom_image( $settings ) {
        $image_html = '<div class="mdp-modalier-image-box"><img src="%s" alt=""></div>';
        echo sprintf( $image_html, esc_url($settings['content_image']['url']) );
	}

	/**
	 * Displays title for custom slide.
	 *
	 * @param $settings - slide settings.
	 */
	public function print_custom_title( $settings ) {
        $title_html = '<h3 class="mdp-modalier-title">%s</h3>';
        echo sprintf( $title_html, wp_kses_post( $settings['content_title'], 'post' ) );
	}

	/**
	 *  Displays description for custom slide.
	 *
	 * @param $settings - slide settings.
	 */
	public function print_custom_description( $settings ) {
        $description_html = '<div class="mdp-modalier-description">%s</div>';
        echo sprintf( $description_html, wp_kses_post( $settings['content_description'], 'post' ) );
	}

	/**
	 * Displays button for custom slide.
	 *
	 * @param $settings - slide settings.
	 */
	public function print_custom_button( $settings ) {
		$target = $settings['content_button_link']['is_external'] ? ' target="_blank"' : '';
		$nofollow = $settings['content_button_link']['nofollow'] ? ' rel="nofollow"' : '';

		if( !empty($settings['content_button_text']) && empty($settings['content_button_link']['url']) ){
			echo sprintf(
				'<p class="mdp-modalier-box-link"><span class="mdp-modalier-link">%s</span></p>',
				esc_html($settings['content_button_text'])
			);
        }

		if( !empty($settings['content_button_link']['url']) ){
			echo sprintf(
        '<p class="mdp-modalier-box-link"><a href="%s" %s %s class="mdp-modalier-link">%s</a></p>',
				esc_url($settings['content_button_link']['url']),
				esc_attr($target),
				esc_attr($nofollow),
				esc_html($settings['content_button_text'])
			);
		}
	}

	/**
	 * @param $settings
	 * @param $display_edit
	 * @param $LayoutSortable
	 * @param $display_none
	 */
	public function print_custom_source( $settings, $display_edit, $LayoutSortable, $display_none ) {
		echo sprintf('<div id="inline-a%s" style="%s">', $this->get_id(), $display_edit );

            /** We list the layouts in the given order. */
            foreach ( $LayoutSortable as $val ) {

                switch ( $val[ 'name' ] ) {

                    case 'image':
                        $this->print_custom_image( $settings );
                        break;

                    case 'title':
                        $this->print_custom_title( $settings );
                        break;

                    case 'description':
                        $this->print_custom_description( $settings );
                        break;

                    case 'button':
                        $this->print_custom_button( $settings );
                        break;

                    default:
                        echo '';

                }

            }
		?>
        </div>
    <?php
		echo sprintf(
			'<div style="clear:both; text-align: %s;"><a href="#inline-a%s" '.
			'class="mpd-modalier-button inline-a%s %s">%s</a></div>',
			esc_attr($settings['modalier_button_style_alignment']),
			esc_attr($this->get_id()),
			esc_attr($this->get_id()),
			esc_attr($display_none),
			esc_html($settings['general_button_text'])
		);
	}

	/**
	 * @param $settings
	 */
	public function print_image_gallery_source( $settings ) {
		foreach ( $settings['content_gallery'] as $image ) {
			$link_html = '<a  href="javascript:void(0);" data-modaal-content-source="%s" data-group="gallery" class="mdp-modalier-a%s modaal-image gallery-thumb">'.
			             '<img src="%s" alt=""></a>';
			echo  sprintf( $link_html, esc_url($image['url']), esc_attr($this->get_id()), esc_url($image['url']) );
		}
	}

	/**
	 * @param $settings
	 * @param $display_none
	 */
	public function print_video_source( $settings, $display_none ) {
		$video_html = '<div style="text-align: %s;"><a href="%s" '.
		              'class="mpd-modalier-button mdp-modalier-a%s %s">%s</a></div>';
		echo sprintf(
			$video_html,
			esc_attr($settings['modalier_button_style_alignment']),
			esc_url($settings['content_video_link']),
			esc_attr($this->get_id()),
			esc_attr($display_none),
			esc_html($settings['general_button_text'])
		);
    }

	/**
	 * @param $settings
	 * @param $display_none
	 */
	public function print_iframe_source( $settings, $display_none ) {
		$iframe_html = '<div style="text-align: %s;"><a href="%s" '.
		               'class="mpd-modalier-button mdp-modalier-a%s %s">%s</a></div>';
		echo sprintf(
			$iframe_html,
			esc_attr($settings['modalier_button_style_alignment']),
			esc_url($settings['content_iframe_link']),
			esc_attr($this->get_id()),
			esc_attr($display_none),
			esc_html($settings['general_button_text'])
		);
    }

	/**
	 * @param $settings
	 * @param $display_none
	 */
	public function print_ajax_source( $settings, $display_none ) {
		$ajax_html = '<div style="text-align: %s;"><a href="%s" '.
		             'class="mpd-modalier-button mdp-modalier-a%s %s">%s</a></div>';
		echo sprintf(
			$ajax_html,
			esc_attr($settings['modalier_button_style_alignment']),
			esc_url($settings['content_ajax_link']),
			esc_attr($this->get_id()),
			esc_attr($display_none),
			esc_html($settings['general_button_text'])
		);
    }

	/**
	 */
	public function print_single_image_source() {
		$settings_img = $this->get_settings_for_display();

		$img = Group_Control_Image_Size::get_attachment_image_html(
			$settings_img,
			'thumbnail',
			'content_image'
		);
		$img_src = explode(" ", $img);

		$url = str_replace( ['src="', '"'], "", $img_src[3]);
		if( strpos( $img_src[1], 'width') === false){
			$url = str_replace( ['src="', '"'], "", $img_src[1]);
		}

		$single_image_html = '<i class="mdp-visually-hidden">Visibility hidden</i><div data-modaal-content-source="%s" '.
		                     'class="mdp-modalier-a%s gallery-thumb"><img src="%s" alt=""></div>';
		echo sprintf( $single_image_html, esc_url($url), esc_attr($this->get_id()), esc_url($url) );
    }

	/**
	 * @param $settings
	 * @param $display_none
	 */
	public function print_section_source( $settings, $display_none ) {
		$section_html = '<div style="text-align: %s;">'.
                        '<a href="#%s" class="mpd-modalier-button %s %s">%s</a></div>';
		echo sprintf(
			$section_html,
			esc_attr($settings['modalier_button_style_alignment']),
			esc_attr($settings['general_section_id']),
			esc_attr($settings['general_section_id']),
			esc_attr($display_none),
			esc_html($settings['general_button_text'])
		);
    }

	/**
	 * Custom: We transfer data to create a slider.
	 *
	 * @param $settings - Global settings.
	 * @param $LayoutSortable - An array of slides layouts.
	 */
	public function get_custom_info( $settings, $LayoutSortable ) {

	    $display_none = '';
	    if( 'scroll' === $settings['general_type_events'] ||
            'timeout' === $settings['general_type_events'] ){ $display_none = 'mdp-display-none'; }

		$display_edit = 'display: none;';
		if( Plugin::$instance->editor->is_edit_mode() === true ){ $display_edit = 'display: block;'; }

		switch ( $settings['general_source'] ) {
			case 'custom':
				$this->print_custom_source( $settings, $display_edit, $LayoutSortable, $display_none );
				break;

			case 'image-gallery':
				$this->print_image_gallery_source( $settings );
				break;

			case 'video':
				$this->print_video_source( $settings, $display_none );
				break;

			case 'iframe':
				$this->print_iframe_source( $settings, $display_none );
				break;

			case 'ajax':
				$this->print_ajax_source( $settings, $display_none );
				break;

			case 'single-image':
				$this->print_single_image_source();
				break;

			case 'section':
				$this->print_section_source( $settings, $display_none );
				break;
		}

	}

	/**
	 * @param $data
	 * @param $settings
	 */
	public function print_posts_image( $data, $settings ) {

		$image_url = get_the_post_thumbnail_url( $data, $settings['image_style_type_size'] );

		if( empty($image_url) ){ $image_url =  Utils::get_placeholder_image_src(); }

        echo sprintf( '<div class="mdp-modalier-image-box"><img src="%s" alt=""></div>', esc_url($image_url) );
	}

	/**
	 * @param $data
	 */
	public function print_posts_title( $data ) {
		echo sprintf('<h3 class="mdp-modalier-title">%s</h3>', wp_kses_post( get_the_title( $data ), 'post' ) );
	}

	/**
	 * @param $data
	 */
	public function print_posts_description( $data ) {
		$excerpt = get_the_excerpt($data);
	    if( !empty($excerpt ) ) {
		    echo sprintf('<p class="mdp-modalier-description">%s</p>', esc_html($excerpt) );
        }
	}

	/**
	 * @param $data
	 * @param $settings
	 */
	public function print_posts_button( $data, $settings ) {
        $button_html = '<p class="mdp-modalier-box-link"><a href="%s" class="mdp-modalier-link">%s</a></p>';
        echo sprintf(
                $button_html,
                esc_url(get_post_permalink( $data )),
                esc_html($settings['general_link_post_text'])
        );
	}

	/**
	 * @param $data
	 * @param $settings
	 */
	public function print_posts_price( $data, $settings ) {
	    $product = wc_get_product( $data );
		if( 'woocommerce' === $settings['general_source'] ){
			echo sprintf(
				'<p class="mdp-modalier-price">%s</p>',
				wp_kses_post( $product->get_price_html(), 'post' )
			);
        }
	}

	/**
	 * @param $data
	 * @param $settings
	 * @param $LayoutSortable
	 */
	public function get_post_item( $data, $settings, $LayoutSortable ) {

		/** We list the layouts in the given order. */
		foreach ($LayoutSortable as $val) {

			switch ( $val['name'] ) {
				case 'image':
					$this->print_posts_image( $data, $settings );
					break;

				case 'title':
					$this->print_posts_title( $data, $settings );
					break;

				case 'description':
					$this->print_posts_description( $data );
					break;

				case 'price':
					$this->print_posts_price( $data, $settings );
					break;

				case 'button':
					$this->print_posts_button( $data, $settings );
					break;

				default:
					echo '';
			}

		}

	}

	/**
	 * @param $settings
	 * @param $LayoutSortable
	 */
	public function get_posts_info( $settings, $LayoutSortable ) {

		$posts = $settings['general_item_posts'];
		if ( $settings['general_source'] === 'custom-posts' ) { $posts = $settings[ 'general_item_custom_posts' ]; }

		$display_edit = 'display: none;';
		if( Plugin::$instance->editor->is_edit_mode() === true ){ $display_edit = 'display: block;'; }

		if( '' !== $posts ) {
			$quantity_posts = count( $posts ) - 1;

			$display_none = '';
			if( 'scroll' === $settings[ 'general_type_events' ] ||
                'timeout' === $settings[ 'general_type_events' ] ) { $display_none = 'mdp-display-none'; } ?>

            <div id="inline-a<?php echo esc_attr( $this->get_id() ); ?>" style="<?php echo esc_attr($display_edit); ?>">
				<?php
				for ( $i = 0; $i <= $quantity_posts; $i ++ ) { ?>
                    <div class="mdp-container-item">
						<?php $this->get_post_item( $posts[ $i ], $settings, $LayoutSortable ); ?>
                    </div>
					<?php
				} ?>
            </div>

			<?php
            echo sprintf(
            '<div style="clear:both; text-align: %s;"><a href="#inline-a%s" '.
                    'class="mpd-modalier-button inline-a%s %s">%s</a></div>',
	                esc_attr($settings['modalier_button_style_alignment']),
	                esc_attr($this->get_id()),
	                esc_attr($this->get_id()),
	                esc_attr($display_none),
	                esc_html($settings['general_button_text'])
            );
		}

	}

	/**
	 * @param $settings
	 * @param $LayoutSortable
	 */
	public function get_woocommerce_info( $settings, $LayoutSortable ) {

		/** We get an array with post data. */
		$product = $settings['general_item_product'];

		$display_edit = 'display: none;';
		if( \Elementor\Plugin::$instance->editor->is_edit_mode() === true ){
			$display_edit = 'display: block;';
		}

		$display_none = '';
		if( 'scroll' === $settings['general_type_events'] ||
            'timeout' === $settings['general_type_events'] ){ $display_none = 'mdp-display-none'; }

		if( '' !== $product ):

			/** We count the number of imagebox item in the category. */
			$quantity_product = count( $product ) - 1;
        ?>

        <div id="inline-a<?php echo esc_attr( $this->get_id() ); ?>" style="<?php echo esc_attr( $display_edit ); ?>">
            <?php
            for ( $i = 0; $i <= $quantity_product; $i ++ ) { ?>
                <div class="mdp-container-item">
                    <?php $this->get_post_item( $product[$i], $settings, $LayoutSortable );  ?>
                </div>
                <?php
            } ?>
        </div>

    <?php
        echo sprintf(
            '<div style="clear:both; text-align: %s;" ><a href="#inline-a%s" '.
                    'class="mpd-modalier-button inline-a%s %s">%s</a></div>',
	                esc_attr($settings['modalier_button_style_alignment']),
	                esc_attr($this->get_id()),
	                esc_attr($this->get_id()),
	                esc_attr($display_none),
	                esc_html($settings['general_button_text'])
        );

        endif;
	}

	/**
	 *
	 */
	public function get_type_modaal( $settings ) {

        $type_modaal = $settings['general_source'];

	    if( 'single-image' === $settings['general_source'] ||
           'image-gallery' === $settings['general_source'] ){ $type_modaal = 'image'; }

	    return $type_modaal;
	}

    /**
     * Render Frontend Output. Generate the final HTML on the frontend.
     *
     * @access protected
     *
     * @return void
     **/
    protected function render() {

	    /** Get all the values from the admin panel. */
	    $settings = $this->get_settings_for_display();

	    /** We return an array with the names of the layouts and their width. */
	    $LayoutSortable = $this->get_LayoutSortable( $settings );

	    if( Plugin::$instance->editor->is_edit_mode() === true
            && $settings['general_type_events'] !== "button"
            && !in_array( $settings[ 'general_source'], ['single-image','image-gallery']) ){
	        echo esc_html__('Modalier section', 'selection');
        }

	    $jsID = !empty($settings[ 'general_section_id' ]) ? $settings[ 'general_section_id' ] : 'a'.$this->get_id();

	    /** Check if variables are empty. */
	    $modalierScroll     = !empty($settings['general_scroll']['size']) ?
                                        $settings['general_scroll']['size'] : 0;
	    $modalierTimeout    = !empty($settings['general_timeout']['size']) ?
                                        $settings['general_timeout']['size'] : 0;
	    $animationSpeed     = !empty($settings['navigation_animation_speed']['size']) ?
                                        $settings['navigation_animation_speed']['size'] : 0;
	    $width              = !empty($settings['modal_style_width']['size']) ?
                                        $settings['modal_style_width']['size'] : 700;
	    $width_tablet       = !empty($settings['modal_style_width_tablet']['size']) ?
                                        $settings['modal_style_width_tablet']['size'] : 600;
	    $width_mobile       = !empty($settings['modal_style_width_mobile']['size']) ?
                                        $settings['modal_style_width_mobile']['size'] : 320;
	    $height             = !empty($settings['modal_style_height']['size']) ?
                                        $settings['modal_style_height']['size'] : 500;
	    $height_tablet      = !empty($settings['modal_style_height_tablet']['size']) ?
                                        $settings['modal_style_height_tablet']['size'] : 450;
	    $height_mobile      = !empty($settings['modal_style_height_mobile']['size']) ?
                                        $settings['modal_style_height_mobile']['size'] : 400;
	    $opacity            = !empty($settings['modal_style_overlay_opacity']['size']) ?
                                        $settings['modal_style_overlay_opacity']['size'] : 0.5;
	    $openLimit          = !empty($settings['navigation_popup_open_limit']['size']) ?
                                        $settings['navigation_popup_open_limit']['size'] : 0;
	    $filterBlur         = !empty($settings['modal_style_overlay_filter_blur']['size']) ?
                                        $settings['modal_style_overlay_filter_blur']['size'] : 0.7;
	    ?>

        <div id="mdp-modalier-<?php echo esc_attr( $jsID ); ?>"
             class="mdp-modalier-container-<?php echo esc_attr( $this->get_id() ); ?> mdp-modalier-box"
             data-post-id="<?php esc_attr_e( get_the_ID() ); ?>"
             data-widget-id="<?php esc_attr_e( $this->get_id() ); ?>"
             data-type="<?php echo esc_attr( $this->get_type_modaal( $settings ) ); ?>"
             data-typeEvents="<?php echo esc_attr($settings['general_type_events']); ?>"
             data-modalierScroll="<?php echo esc_attr($modalierScroll); ?>"
             data-modalierTimeout="<?php echo esc_attr($modalierTimeout); ?>"
             data-fullscreen="<?php echo ( 'yes' === $settings['navigation_fullscreen_mode'] ) ? 'true' : 'false'; ?>"
             data-animation="<?php echo esc_attr($settings['navigation_animation']); ?>"
             data-animationSpeed="<?php echo esc_attr( $animationSpeed ); ?>"
             data-closeViaKeypress="<?php echo ( 'yes' === $settings['navigation_close_keypress'] ) ? 'true' : 'false'; ?>"
             data-displayCloseButton="<?php echo ( 'yes' === $settings['navigation_close_button'] ) ? 'false' : 'true'; ?>"
             data-closeViaOverlay="<?php echo ( 'yes' === $settings['navigation_close_overlay'] ) ? 'true' : 'false'; ?>"
             data-popupTitle="<?php echo esc_attr($settings['navigation_popup_title']); ?>"
             data-closeText="<?php echo esc_attr($settings['navigation_close_text']); ?>"
             data-closeAriaLabel="<?php echo esc_attr($settings['navigation_close_labelt']); ?>"
             data-galleryActiveClass="<?php echo esc_attr($settings['navigation_gallery_active_class']); ?>"
             data-outerControls="<?php echo ( 'yes' === $settings['navigation_controls_outside'] ) ? 'true' : 'false'; ?>"
             data-width="<?php echo esc_attr($width); ?>"
             data-height="<?php echo esc_attr($height); ?>"
             data-background-<?php echo esc_attr( $jsID ); ?>="<?php echo esc_attr($settings['modal_style_overlay_color']); ?>"
             data-opacity="<?php echo esc_attr($opacity); ?>"
             data-openLimit="<?php echo esc_attr( $openLimit ); ?>"
             data-contentSource="<?php echo esc_attr($settings['general_section_id']); ?>"
             <?php echo 'yes' === $settings[ 'modal_style_overlay_blur' ] ? 'data-blur="' . esc_attr($filterBlur) . '"' : '' ?>
        >

        <?php
            switch ($settings['general_source']){

                case 'custom':
                case 'ajax':
                case 'single-image':
                case 'image-gallery':
                case 'video':
                case 'iframe':
                case 'section':
                    $this->get_custom_info( $settings, $LayoutSortable );
                    break;

	            case 'custom-posts':
                case 'posts':
                    $this->get_posts_info( $settings, $LayoutSortable );
                    break;

                case 'woocommerce':
                    $this->get_woocommerce_info($settings, $LayoutSortable);
                    break;

                default:
	                echo esc_html__( "The source of information was not found.", 'selection' );

            }

        ?>
        </div>

        <?php if( Plugin::$instance->editor->is_edit_mode() !== true ): ?>

        <script>
            document.addEventListener('DOMContentLoaded', function () {

                let sectionID = document.querySelectorAll( '#<?php echo esc_attr( $jsID); ?>' );
                for( let i = 0; sectionID.length - 1 >= i; i++ ){
                    let classContains = sectionID[i].classList.contains( '<?php echo esc_attr( $jsID); ?>');
                    if( classContains !== undefined  ){
                        sectionID[i].classList.remove( '<?php echo esc_attr( $jsID); ?>' );
                    }
                }

                <?php if( $settings['general_type_events'] === 'timeout' ): ?>
                sessionStorage.setItem( 'mdp-launches-<?php esc_attr_e( $jsID ); ?>', 0 );
                <?php endif; ?>

	            <?php if( $settings['general_type_events'] === 'scroll' ): ?>
                sessionStorage.setItem( 'mdp-launches-scroll-<?php esc_attr_e( $jsID ); ?>', 1 );
	            <?php endif; ?>

                var observer_<?php esc_attr_e( $this->get_id() ); ?> = new MutationObserver( list => {

                    let widthDevice, heightDevice;

                    if( list[0].target.attributes[1] !== undefined ){
                        let newDevice = list[0].target.attributes[1].nodeValue;

                        if( newDevice === 'desktop' ){
                            widthDevice  = <?php echo esc_attr( $width ); ?>;
                            heightDevice = <?php echo esc_attr( $height ); ?>;
                        }else if( newDevice === 'tablet' ){
                            widthDevice  = <?php echo esc_attr( $width_tablet ); ?>;
                            heightDevice = <?php echo esc_attr( $height_tablet ); ?>;
                        } else if( newDevice === 'mobile' ){
                            widthDevice  = <?php echo esc_attr( $width_mobile ); ?>;
                            heightDevice = <?php echo esc_attr( $height_mobile ); ?>;
                        }

                        window.Modalier( '<?php echo esc_attr( $jsID); ?>', widthDevice, heightDevice );

                    }

                } );

                observer_<?php esc_attr_e( $this->get_id() ); ?>.observe( document.body, { attributes: true });

            } );

        </script>

        <?php endif;
    }

    /**
     * Return link for documentation
     * Used to add stuff after widget
     *
     * @access public
     *
     * @return string
     **/
    public function get_custom_help_url() { return 'https://docs.merkulov.design/tag/modalier'; }

}
