<?php /** @noinspection PhpUndefinedClassInspection */
/**
 * Motionger for Elementor
 * Awesome animated headings widget for Elementor.
 * Exclusively on https://1.envato.market/motionger-elementor
 *
 * @encoding        UTF-8
 * @version         1.12.1
 * @copyright       (C) 2018 - 2023 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Cherviakov Vlad (vladchervjakov@gmail.com), Dmitry Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

namespace Merkulove\Selection;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
    header( 'Status: 403 Forbidden' );
    header( 'HTTP/1.1 403 Forbidden' );
    exit;
}

use Exception;
use Elementor\Group_Control_Background;
use Elementor\Group_Control_Text_Shadow;
use Elementor\Group_Control_Typography;
use Elementor\Widget_Base;
use Elementor\Controls_Manager;
use Merkulove\Selection\Unity\Plugin as UnityPlugin;

/** @noinspection PhpUnused */
/**
 * Motionger - Custom Elementor Widget.
 **/
class motionger_elementor extends Widget_Base {

    /**
     * Use this to sort widgets.
     * A smaller value means earlier initialization of the widget.
     * Can take negative values.
     * Default widgets and widgets from 3rd party developers have 0 $mdp_order
     **/
    public $mdp_order = 1;

    /**
     * Widget base constructor.
     * Initializing the widget base class.
     *
     * @access public
     * @throws Exception If arguments are missing when initializing a full widget instance.
     * @param array      $data Widget data. Default is an empty array.
     * @param array|null $args Optional. Widget default arguments. Default is null.
     *
     * @return void
     **/
    public function __construct( $data = [], $args = null ) {

        parent::__construct( $data, $args );

        wp_register_style( 'mdp-selection-elementor-admin', UnityPlugin::get_url() . 'css/elementor-admin' . UnityPlugin::get_suffix() . '.css', [], UnityPlugin::get_version() );
        wp_register_style( 'mdp-motionger-elementor', UnityPlugin::get_url() . 'css/motionger-elementor' . UnityPlugin::get_suffix() . '.css', [], UnityPlugin::get_version() );

    }

    /**
     * Return a widget name.
     *
     * @return string
     **/
    public function get_name() {

        return 'mdp-motionger-elementor';

    }

    /**
     * Return the widget title that will be displayed as the widget label.
     *
     * @return string
     **/
    public function get_title() {

        return esc_html__( 'Animated Heading', 'selection' );

    }

    /**
     * Set the widget icon.
     *
     * @return string
     */
    public function get_icon() {

        return 'mdp-motionger-elementor-widget-icon';

    }

    /**
     * Set the category of the widget.
     *
     * @return array with category names
     **/
    public function get_categories() {

        return [ 'selection-category' ];

    }

    /**
     * Get widget keywords. Retrieve the list of keywords the widget belongs to.
     *
     * @access public
     *
     * @return array Widget keywords.
     **/
    public function get_keywords() {

        return [ 'Merkulove', 'Animated Heading', 'Animation', 'heading', 'title' ];

    }

    /**
     * Get style dependencies.
     * Retrieve the list of style dependencies the widget requires.
     *
     * @access public
     *
     * @return array Widget styles dependencies.
     **/
    public function get_style_depends() {

        return [ 'mdp-motionger-elementor', 'mdp-selection-elementor-admin' ];

    }

	/**
	 * Get script dependencies.
	 * Retrieve the list of script dependencies the element requires.
	 *
	 * @access public
     *
	 * @return array Element scripts dependencies.
	 **/
	public function get_script_depends() {

		return [];

    }

    /**
     * Add the widget controls.
     *
     * @since 1.0.0
     * @access protected
     *
     * @return void with category names
     **/
    protected function register_controls() {

        /** Header tab. */
        $this->start_controls_section( 'section_header', [ 'label' => esc_html__( 'Header', 'selection' ) ] );

        /** Animation. */
        $this->add_control(
            'animation_header',
            [
                'label' => esc_html__( 'Animation', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'options' => [
                    'motionger-rise' => 'Rice',
                    'curl' => 'Curl',
                    'jalousie' => 'Jalousie',
                    'motionger-drop-in' => 'Drop',
                    'ripple' => 'Ripple',
                    'motionger-slide-right' => 'Slide Right',
                    'motionger-slide-left' => 'Slide Left',
                    'motionger-slide-down' => 'Slide Down',
                    'motionger-slide-up' => 'Slide Up',
                    'motionger-bounce' => 'Bounce',
                    'motionger-fade' => 'Fade',
                    'motionger-flip-x' => 'Flip X',
                    'motionger-flip-y' => 'Flip Y',
                    'motionger-jello' => 'Jello',
                    'motionger-zoom' => 'Zoom',
                    'typing' => 'Typing'
                ],
                'default' => 'motionger-rise',
            ]
        );

        /** Animation speed. */
        $this->add_responsive_control(
            'text_animation_speed',
            [
                'label' => esc_html__( 'Word change time', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'range' => [
                    'ms' => [
                        'min' => 100,
                        'max' => 10000,
                        'step' => 100,
                    ],
                ],
                'default' => [ 'unit' => 'ms', 'size' => 2000, ],
            ]
        );

        /** Letter animation speed. */
        $this->add_responsive_control(
            'letter_animation_speed',
            [
                'label' => esc_html__( 'Letter animation time', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'range' => [
                    'ms' => [
                        'min' => 50,
                        'max' => 1000,
                        'step' => 50,
                    ],
                ],
                'default' => [ 'unit' => 'ms', 'size' => 150, ],
                'conditions' => [
                    'terms' => [
                        [
                            'name' => 'animation_header',
                            'operator' => 'in',
                            'value' => [
                                'curl',
                                'jalousie',
                                'ripple',
                                'typing'
                            ],
                        ],
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .motionger-typing-active' => 'animation-duration: {{SIZE}}{{UNIT}};',
                ],

            ]
        );

        /** Header text. */
        $this->add_control(
            'header_text',
            [
                'label' => esc_html__( 'Text', 'selection' ),
                'type' => Controls_Manager::TEXTAREA,
                'rows' => 10,
                'default' => esc_html__( 'Motionger {modern | best | popular} header.', 'selection' ),
                'dynamic' => ['active' => true],
                'placeholder' => esc_html__( 'Animated Heading', 'selection' ),
                'description' => esc_html__( 'The text to be animated is enclosed in braces. Through a comma indicate the text to be animated. Example: Text before {Text One | Text Two} text after.', 'selection' ),
                'separator' => 'before'
            ]
        );

        /** Header line. */
        $this->add_control(
            'header_line',
            [
                'label' => esc_html__( 'Line', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Show', 'selection' ),
                'label_off' => esc_html__( 'Hide', 'selection' ),
                'return_value' => 'yes',
                'separator' => 'after'
            ]
        );

        /** Split header. */
        $this->add_control(
            'header_split',
            [
                'label' => esc_html__( 'Split header', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Show', 'selection' ),
                'label_off' => esc_html__( 'Hide', 'selection' ),
                'return_value' => 'yes',
            ]
        );

        /** Split text. */
        $this->add_control(
            'split_text',
            [
                'label' => esc_html__( 'Split text', 'selection' ),
                'type' => Controls_Manager::TEXTAREA,
                'rows' => 10,
                'default' => esc_html__( 'Split text', 'selection' ),
                'dynamic' => ['active' => true],
                'placeholder' => esc_html__( 'Split text', 'selection' ),
                'condition'  => ['header_split' => 'yes']
            ]
        );

        /** HTML Tag. */
        $this->add_control(
            'header_tag',
            [
                'label' => esc_html__( 'HTML Tag', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'options' => [
                    'h1' => 'H1',
                    'h2' => 'H2',
                    'h3' => 'H3',
                    'h4' => 'H4',
                    'h5' => 'H5',
                    'h6' => 'H6',
                    'div' => 'div',
                    'span' => 'span',
                    'p' => 'p',
                ],
                'default' => 'h2',
            ]
        );

        /** Heading link. */
        $this->add_control(
            'heading_link',
            [
                'label' => esc_html__( 'Link', 'selection' ),
                'type' => Controls_Manager::URL,
                'placeholder' => esc_html__( 'https://codecanyon.net/user/merkulove', 'selection' ),
                'show_external' => true,
                'default' => [
                    'url' => '',
                    'is_external' => false,
                    'nofollow' => false,
                ],
            ]
        );

        /** Show Subheader. */
        $this->add_control(
            'show_subheader',
            [
                'label' => esc_html__( 'Subheader', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Show', 'selection' ),
                'label_off' => esc_html__( 'Hide', 'selection' ),
                'return_value' => 'yes',
            ]
        );

        /** Show Content Header. */
        $this->add_control(
            'show_content_header',
            [
                'label' => esc_html__( 'Additional Header', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Show', 'selection' ),
                'label_off' => esc_html__( 'Hide', 'selection' ),
                'return_value' => 'yes',
            ]
        );

        /** End section content. */
        $this->end_controls_section();

        /** Subheader. */
        $this->start_controls_section( 'section_subheader',
            [
                'label' => esc_html__( 'Subheader', 'selection' ),
                'condition'   => ['show_subheader' => 'yes' ]
            ] );

        /** Header line. */
        $this->add_control(
            'subheader_line',
            [
                'label' => esc_html__( 'Line', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Show', 'selection' ),
                'label_off' => esc_html__( 'Hide', 'selection' ),
                'return_value' => 'yes',
                'separator' => 'after'
            ]
        );

        /** Subheader bottom. */
        $this->add_control(
            'header_bottom',
            [
                'label' => esc_html__( 'Subheader bottom', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Show', 'selection' ),
                'label_off' => esc_html__( 'Hide', 'selection' ),
                'return_value' => 'yes',
            ]
        );

        /** Subheader. */
        $this->add_control(
            'sub_heading',
            [
                'label' => esc_html__( 'Text', 'selection' ),
                'type' => Controls_Manager::TEXTAREA,
                'rows' => 10,
                'separator' => 'before',
                'dynamic' => ['active' => true],
                'default' => esc_html__( 'Subheader', 'selection' ),
                'placeholder' => esc_html__( 'Subheader', 'selection' ),
            ]
        );

        /** End section content. */
        $this->end_controls_section();

        /** Content header. */
        $this->start_controls_section( 'section_content',
            [
                'label' => esc_html__( 'Additional Header', 'selection' ),
                'condition'   => ['show_content_header' => 'yes' ]
            ] );

        /** Sub heading. */
        $this->add_control(
            'text_content_heading',
            [
                'label' => esc_html__( 'Text', 'selection' ),
                'type' => Controls_Manager::TEXTAREA,
                'rows' => 10,
                'separator' => 'before',
                'dynamic' => ['active' => true],
                'default' => esc_html__( 'Additional Header', 'selection' ),
                'placeholder' => esc_html__( 'Additional Header', 'selection' ),
            ]
        );

        /** End section content. */
        $this->end_controls_section();

        /** Style header tab. */
        $this->start_controls_section( 'style_header', [ 'label' => esc_html__( 'Header', 'selection' ), 'tab' => Controls_Manager::TAB_STYLE, ] );

        $this->start_controls_tabs('tabs_header_style');

        /** Text style tab. */
        $this->start_controls_tab(
            'tab_style_text',
            [
                'label' => esc_html__('Text style', 'selection')
            ]
        );

        /** Color. */
        $this->add_control(
            'color_header',
            [
                'label' => esc_html__( 'Color', 'selection' ),
                'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .mdp-typography-default-header' => 'color: {{VALUE}}',
                ],
            ]
        );

        /** Typography. */
        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name' => 'header_typography',
                'label' => esc_html__( 'Typography', 'selection' ),
                'selector' => '{{WRAPPER}} .mdp-typography-header',
            ]
        );

        /** Shadow. */
        $this->add_group_control(
            Group_Control_Text_Shadow::get_type(),
            [
                'name' => 'header_shadow',
                'label' => esc_html__( 'Shadow', 'selection' ),
                'selector' => '{{WRAPPER}} .mdp-shadow-header',
            ]
        );

        /** Alignment. */
        $this->add_responsive_control(
            'header_align',
            [
                'label' => esc_html__( 'Alignment', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'fa fa-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'fa fa-align-center',
                    ],
                    'right' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'fa fa-align-right',
                    ],
                ],
                'default' => 'center',
                'selectors' => [
                    '{{WRAPPER}} .mdp-header-align' => 'text-align: {{header_align}};',
                ],
                'toggle' => true,
                'label_block' => false,
            ]
        );

        /** Text animation. */
        $this->add_control(
            'text_animation_header',
            [
                'label' => __( 'Animated text', 'selection' ),
                'type' => Controls_Manager::HEADING,
                'separator' => 'before',
            ]
        );

        /** Text animation color. */
        $this->add_control(
            'text_animation_color',
            [
                'label' => esc_html__( 'Color', 'selection' ),
                'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .mdp-motionger-wrapper' => '-webkit-text-fill-color: {{VALUE}}',
                ],
            ]
        );

        /** Typography. */
        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name' => 'text_animation_typography',
                'label' => esc_html__( 'Typography', 'selection' ),
                'selector' => '{{WRAPPER}} .mdp-motionger-wrapper',
            ]
        );

        /** Shadow. */
        $this->add_group_control(
            Group_Control_Text_Shadow::get_type(),
            [
                'name' => 'text_animation_shadow',
                'label' => esc_html__( 'Shadow', 'selection' ),
                'selector' => '{{WRAPPER}} .mdp-motionger-wrapper',
            ]
        );

        /** Custom width */
        $this->add_control(
            'text_animation_custom',
            [
                'label' => esc_html__( 'Auto-width', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'On', 'selection' ),
                'label_off' => esc_html__( 'Off', 'selection' ),
                'return_value' => 'yes',
                'default' => 'yes'
            ]
        );

        /** Box width. */
        $this->add_responsive_control(
            'text_animation_width',
            [
                'label' => esc_html__( 'Element width', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 500,
                        'step' => 1,
                    ]
                ],
                'default' => [ 'unit' => 'px', 'size' => 300, ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-motionger-wrapper' => 'width: {{SIZE}}{{UNIT}};',
                ],
                'condition' => ['text_animation_custom' => '' ]
            ]
        );

        /** Margin. */
        $this->add_responsive_control(
            'text_animation_margin_all',
            [
                'label' => esc_html__( 'Margin', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => [ 'desktop', 'tablet', 'mobile' ],
                'desktop_default' => [
                    'top' => '',
                    'right' => '',
                    'bottom' => '',
                    'left' => '',
                    'unit' => 'px',
                ],
                'tablet_default' => [
                    'top' => '',
                    'right' => '',
                    'bottom' => '',
                    'left' => '',
                    'unit' => 'px',
                ],
                'mobile_default' => [
                    'top' => '',
                    'right' => '',
                    'bottom' => '',
                    'left' => '',
                    'unit' => 'px',
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-motionger-wrapper' => 'margin: {{top}}{{unit}} {{right}}{{unit}} {{bottom}}{{unit}} {{left}}{{unit}};',
                ],
            ]
        );

        /** Alignment. */
        $this->add_responsive_control(
            'text_animation_align',
            [
                'label' => esc_html__( 'Alignment', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'fa fa-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'fa fa-align-center',
                    ],
                    'right' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'fa fa-align-right',
                    ],
                ],
                'default' => 'center',
                'selectors' => [
                    '{{WRAPPER}} .mdp-motionger-wrapper' => 'text-align: {{header_align}};',
                ],
                'toggle' => true,
                'label_block' => false,
            ]
        );

        $this->end_controls_tab();

        /** Box tab. */
        $this->start_controls_tab(
            'tab_style_box',
            [
                'label' => esc_html__('Box style', 'selection')
            ]
        );

        $this->add_group_control(
            Group_Control_Background::get_type(),
            [
                'name' => 'header_background',
                'label' => esc_html__( 'Background', 'selection' ),
                'types' => [ 'classic', 'gradient' ],
                'selector' => '{{WRAPPER}} .mdp-header-before, {{WRAPPER}} .mdp-header-after, {{WRAPPER}} .mdp-motionger-letter-animation span, {{WRAPPER}} .mdp-motionger-word-animation',
            ]
        );

        /** Background effect. */
        $this->add_control(
            'header_background_effect',
            [
                'label' => esc_html__( 'Show background effect', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Show', 'selection' ),
                'label_off' => esc_html__( 'Hide', 'selection' ),
                'return_value' => 'yes',
            ]
        );

        /** Border type. */
        $this->add_control(
            'header_border_type',
            [
                'label' => esc_html__( 'Border type', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'none',
                'options' => [
                    'none'  => esc_html__( 'None', 'selection' ),
                    'double' => esc_html__( 'Double', 'selection' ),
                    'dashed' => esc_html__( 'Dashed', 'selection' ),
                    'dotted' => esc_html__( 'Dotted', 'selection' ),
                    'solid' => esc_html__( 'Solid', 'selection' ),
                    'grove' => esc_html__( 'Grove', 'selection' ),
                ],
                'separator' => 'before',
                'selectors' => [
                    '{{WRAPPER}} .mdp-typography-default-header' => 'border-style: {{VALUE}}',
                ],
            ]
        );

        /** Border color. */
        $this->add_control(
            'border_color_header',
            [
                'label' => esc_html__( 'Border color', 'selection' ),
                'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .mdp-typography-default-header' => 'border-color: {{VALUE}}',
                ],
                'conditions' => [
                    'terms' => [
                        [
                            'name' => 'header_border_type',
                            'operator' => '!in',
                            'value' => [
                                'none',
                            ],
                        ],
                    ],
                ]
            ]
        );

        /** Border padding. */
        $this->add_responsive_control(
            'header_border_padding',
            [
                'label' => esc_html__( 'Border padding', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => [ 'desktop', 'tablet', 'mobile' ],
                'desktop_default' => [
                    'top' => '',
                    'right' => '',
                    'bottom' => '',
                    'left' => '',
                    'unit' => 'px',
                ],
                'tablet_default' => [
                    'top' => '',
                    'right' => '',
                    'bottom' => '',
                    'left' => '',
                    'unit' => 'px',
                ],
                'mobile_default' => [
                    'top' => '',
                    'right' => '',
                    'bottom' => '',
                    'left' => '',
                    'unit' => 'px',
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-typography-default-header' => 'padding: {{top}}{{unit}} {{right}}{{unit}} {{bottom}}{{unit}} {{left}}{{unit}};',
                ],
                'conditions' => [
                    'terms' => [
                        [
                            'name' => 'header_border_type',
                            'operator' => '!in',
                            'value' => [
                                'none',
                            ],
                        ],
                    ],
                ]
            ]
        );

        /** Border width. */
        $this->add_responsive_control(
            'header_border_width',
            [
                'label' => esc_html__( 'Border width', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => [ 'desktop', 'tablet', 'mobile' ],
                'desktop_default' => [
                    'top' => '',
                    'right' => '',
                    'bottom' => '',
                    'left' => '',
                    'unit' => 'px',
                ],
                'tablet_default' => [
                    'top' => '',
                    'right' => '',
                    'bottom' => '',
                    'left' => '',
                    'unit' => 'px',
                ],
                'mobile_default' => [
                    'top' => '',
                    'right' => '',
                    'bottom' => '',
                    'left' => '',
                    'unit' => 'px',
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-typography-default-header' => 'border-width: {{top}}{{unit}} {{right}}{{unit}} {{bottom}}{{unit}} {{left}}{{unit}};',
                ],
                'conditions' => [
                    'terms' => [
                        [
                            'name' => 'header_border_type',
                            'operator' => '!in',
                            'value' => [
                                'none',
                            ],
                        ],
                    ],
                ]
            ]
        );

        /** Border radius. */
        $this->add_responsive_control(
            'header_border_radius',
            [
                'label' => esc_html__( 'Border radius', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => [ 'desktop', 'tablet', 'mobile' ],
                'desktop_default' => [
                    'top' => '',
                    'right' => '',
                    'bottom' => '',
                    'left' => '',
                    'unit' => 'px',
                ],
                'tablet_default' => [
                    'top' => '',
                    'right' => '',
                    'bottom' => '',
                    'left' => '',
                    'unit' => 'px',
                ],
                'mobile_default' => [
                    'top' => '',
                    'right' => '',
                    'bottom' => '',
                    'left' => '',
                    'unit' => 'px',
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-typography-default-header' => 'border-radius: {{top}}{{unit}} {{right}}{{unit}} {{bottom}}{{unit}} {{left}}{{unit}};',
                ],
                'conditions' => [
                    'terms' => [
                        [
                            'name' => 'header_border_type',
                            'operator' => '!in',
                            'value' => [
                                'none',
                            ],
                        ],
                    ],
                ]
            ]
        );

        $this->end_controls_tab();

        $this->end_controls_tabs();

        /** Line style. */
        $this->add_control(
            'line_style',
            [
                'label' => esc_html__( 'Line style', 'selection' ),
                'type' => Controls_Manager::HEADING,
                'separator' => 'before',
                'condition'   => ['header_line' => 'yes' ]
            ]
        );

        /** Line position. */
        $this->add_control(
            'header_position',
            [
                'label' => esc_html__( 'Position', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'bottom',
                'options' => [
                    'after'  => esc_html__( 'After', 'selection' ),
                    'before' => esc_html__( 'Before', 'selection' ),
                    'bottom' => esc_html__( 'Bottom', 'selection' ),
                    'after_before' => esc_html__( 'After and Before', 'selection' ),
                ],
                'condition'   => ['header_line' => 'yes' ]
            ]
        );

        /** Line color. */
        $this->add_control(
            'line_color_header',
            [
                'label' => esc_html__( 'Color', 'selection' ),
                'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .line:after' => 'background-color: {{VALUE}};',
                ],
                'condition'   => ['header_line' => 'yes' ]
            ]
        );

        /** Line width. */
        $this->add_responsive_control(
            'line_width_header',
            [
                'label' => esc_html__( 'Width', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'default' => [ 'unit' => 'px', 'size' => 25, ],
                'selectors' => [
                    '{{WRAPPER}} .line:after' => 'width: {{SIZE}}{{UNIT}};',
                ],
                'condition'   => ['header_line' => 'yes' ]
            ]
        );

        /** Line height. */
        $this->add_responsive_control(
            'line_height_header',
            [
                'label' => esc_html__( 'Height', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'default' => [ 'unit' => 'px', 'size' => 2, ],
                'selectors' => [
                    '{{WRAPPER}} .line:after' => 'height: {{SIZE}}{{UNIT}};',
                ],
                'condition'   => ['header_line' => 'yes' ]
            ]
        );

        /** Line spacing. */
        $this->add_responsive_control(
            'line_spacing_header',
            [
                'label' => esc_html__( 'Spacing', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'default' => [ 'unit' => 'px', 'size' => 10, ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-header-line-left' => 'margin-right: {{SIZE}}{{UNIT}};',
                    '{{WRAPPER}} .mdp-header-line-right' => 'margin-left: {{SIZE}}{{UNIT}};',
                    '{{WRAPPER}} .mdp-header-line-bottom' => 'margin-top: {{SIZE}}{{UNIT}};',
                ],
                'condition'   => ['header_line' => 'yes' ]
            ]
        );

        /** End section style header. */
        $this->end_controls_section();

        /** Style Subheader tab. */
        $this->start_controls_section( 'style_subheader',
            [
                'label' => esc_html__( 'Subheader', 'selection' ), 'tab' => Controls_Manager::TAB_STYLE,
                'condition'   => ['show_subheader' => 'yes' ]
            ] );

        /** Color. */
        $this->add_control(
            'subheader',
            [
                'label' => esc_html__( 'Color', 'selection' ),
                'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .mdp-color-sub-header' => 'color: {{VALUE}};',
                ],
            ]
        );

        /** Typography. */
        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name' => 'subheader_typography',
                'label' => esc_html__( 'Typography', 'selection' ),
                'selector' => '{{WRAPPER}} .mdp-typography-subheader',
            ]
        );

        /** Shadow. */
        $this->add_group_control(
            Group_Control_Text_Shadow::get_type(),
            [
                'name' => 'subheader_shadow',
                'label' => esc_html__( 'Shadow', 'selection' ),
                'selector' => '{{WRAPPER}} .mdp-shadow-subheader',
            ]
        );

        /** Alignment. */
        $this->add_responsive_control(
            'subheader_align',
            [
                'label' => esc_html__( 'Alignment', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'fa fa-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'fa fa-align-center',
                    ],
                    'right' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'fa fa-align-right',
                    ],
                ],
                'default' => 'center',
                'selectors' => [
                    '{{WRAPPER}} .mdp-sub-header-align' => 'text-align: {{subheader_align}};',
                ],
                'toggle' => true,
                'label_block' => false,
            ]
        );

        /** Line style. */
        $this->add_control(
            'subline_style',
            [
                'label' => esc_html__( 'Line style', 'selection' ),
                'type' => Controls_Manager::HEADING,
                'separator' => 'before',
                'condition'   => ['subheader_line' => 'yes' ]
            ]
        );

        /** Position. */
        $this->add_control(
            'subheader_position',
            [
                'label' => esc_html__( 'Position', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'bottom',
                'options' => [
                    'after'  => esc_html__( 'After', 'selection' ),
                    'before' => esc_html__( 'Before', 'selection' ),
                    'bottom' => esc_html__( 'Bottom', 'selection' ),
                    'after_before' => esc_html__( 'After and Before', 'selection' ),
                ],
                'condition'   => ['subheader_line' => 'yes' ]
            ]
        );

        /** Line color. */
        $this->add_control(
            'line_color_subheader',
            [
                'label' => esc_html__( 'Color', 'selection' ),
                'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .sub-line:after' => 'background-color: {{VALUE}};'
                ],
                'condition'   => ['subheader_line' => 'yes' ]
            ]
        );

        /** Line width. */
        $this->add_responsive_control(
            'line_width_subheader',
            [
                'label' => esc_html__( 'Width', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'default' => [ 'unit' => 'px', 'size' => 25, ],
                'selectors' => [
                    '{{WRAPPER}} .sub-line:after' => 'width: {{SIZE}}{{UNIT}};',
                ],
                'condition'   => ['subheader_line' => 'yes' ]
            ]
        );

        /** Line height. */
        $this->add_responsive_control(
            'line_height_subheader',
            [
                'label' => esc_html__( 'Height', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'default' => [ 'unit' => 'px', 'size' => 2, ],
                'selectors' => [
                    '{{WRAPPER}} .sub-line:after' => 'height: {{SIZE}}{{UNIT}};',
                ],
                'condition'   => ['subheader_line' => 'yes' ]
            ]
        );

        /** Line spacing. */
        $this->add_responsive_control(
            'line_spacing_subheader',
            [
                'label' => esc_html__( 'Spacing', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'default' => [ 'unit' => 'px', 'size' => 0, ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-subheader-line-left' => 'margin-right: {{SIZE}}{{UNIT}};',
                    '{{WRAPPER}} .mdp-subheader-line-right' => 'margin-left: {{SIZE}}{{UNIT}};',
                    '{{WRAPPER}} .mdp-subheader-line-bottom' => 'margin-top: {{SIZE}}{{UNIT}};',
                ],
                'condition'   => ['subheader_line' => 'yes' ]
            ]
        );

        /** End section style Subheader. */
        $this->end_controls_section();

        /** Style content tab. */
        $this->start_controls_section( 'style_content',
            [
                'label' => esc_html__( 'Additional Header', 'selection' ), 'tab' => Controls_Manager::TAB_STYLE,
                'condition'   => ['show_content_header' => 'yes' ]
            ] );

        /** Color. */
        $this->add_control(
            'content_color_header',
            [
                'label' => esc_html__( 'Color', 'selection' ),
                'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .mdp-color-content-header' => 'color: {{VALUE}}',
                ],
            ]
        );

        /** Typography. */
        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name' => 'content_header_typography',
                'label' => esc_html__( 'Typography', 'selection' ),
                'selector' => '{{WRAPPER}} .mdp-typography-content-header',
            ]
        );

        /** Shadow. */
        $this->add_group_control(
            Group_Control_Text_Shadow::get_type(),
            [
                'name' => 'content_shadow',
                'label' => esc_html__( 'Shadow', 'selection' ),
                'selector' => '{{WRAPPER}} .mdp-shadow-content',
            ]
        );

        /** Alignment. */
        $this->add_responsive_control(
            'test_advanced_align',
            [
                'label' => esc_html__( 'Alignment', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'fa fa-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'fa fa-align-center',
                    ],
                    'right' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'fa fa-align-right',
                    ],
                ],
                'default' => 'center',
                'selectors' => [
                    '{{WRAPPER}} .mdp-text-content-align' => 'text-align: {{subheader_align}};',
                ],
                'toggle' => true,
                'label_block' => false,
            ]
        );

        /** Transform origin */
        $this->add_control(
            'advanced_transform_origin',
            [
                'label' => esc_html__( 'Transform origin', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'center center',
                'options' => [
                    'top center'  => esc_html__( 'Top Center', 'selection' ),
                    'top left' => esc_html__( 'Top Left', 'selection' ),
                    'top right' => esc_html__( 'Top Right', 'selection' ),
                    'center center' => esc_html__( 'Center Center', 'selection' ),
                    'center left' => esc_html__( 'Center Left', 'selection' ),
                    'center right' => esc_html__( 'Center Right', 'selection' ),
                    'bottom center' => esc_html__( 'Bottom Center', 'selection' ),
                    'bottom left' => esc_html__( 'Bottom Left', 'selection' ),
                    'bottom right' => esc_html__( 'Bottom Right', 'selection' ),
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-typography-content-header' => 'transform-origin: {{advanced_transform_origin}};',
                ],
            ]
        );

        /** Rotate. */
        $this->add_responsive_control(
            'line_rotate_advanced',
            [
                'label' => esc_html__( 'Rotate', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'range' => [
                    'deg' => [
                        'min' => 0,
                        'max' => 360,
                        'step' => 1
                    ],
                ],
                'default' => [ 'unit' => 'deg', 'size' => 0, ],
                'selectors' => [
                    '(desktop){{WRAPPER}} .mdp-typography-content-header' => 'transform: translate({{line_xoffset_advanced.SIZE}}{{line_xoffset_advanced.UNIT}}, {{line_yoffset_advanced.SIZE}}{{line_yoffset_advanced.UNIT}}) rotate({{line_rotate_advanced.SIZE}}{{line_rotate_advanced.UNIT}});',
                    '(tablet){{WRAPPER}} .mdp-typography-content-header' => 'transform: translate({{line_xoffset_advanced_tablet.SIZE}}{{line_xoffset_advanced.UNIT}}, {{line_yoffset_advanced_tablet.SIZE}}{{line_yoffset_advanced.UNIT}}) rotate({{line_rotate_advanced.SIZE}}{{line_rotate_advanced.UNIT}});',
                    '(mobile){{WRAPPER}} .mdp-typography-content-header' => 'transform: translate({{line_xoffset_advanced_mobile.SIZE}}{{line_xoffset_advanced.UNIT}}, {{line_yoffset_advanced_mobile.SIZE}}{{line_yoffset_advanced.UNIT}}) rotate({{line_rotate_advanced.SIZE}}{{line_rotate_advanced.UNIT}});',
                ],
            ]
        );

        /** X Offset. */
        $this->add_responsive_control(
            'line_xoffset_advanced',
            [
                'label' => esc_html__( 'X Offset', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'range' => [
                    'px' => [
                        'min' => -500,
                        'max' => 500,
                        'step' => 1,
                    ]
                ],
                'default' => [ 'unit' => 'px', 'size' => 0, ],
                'selectors' => [
                    '(desktop){{WRAPPER}} .mdp-typography-content-header' => 'transform: translate({{line_xoffset_advanced.SIZE}}{{line_xoffset_advanced.UNIT}}, {{line_yoffset_advanced.SIZE}}{{line_yoffset_advanced.UNIT}}) rotate({{line_rotate_advanced.SIZE}}{{line_rotate_advanced.UNIT}});',
                    '(tablet){{WRAPPER}} .mdp-typography-content-header' => 'transform: translate({{line_xoffset_advanced_tablet.SIZE}}{{line_xoffset_advanced.UNIT}}, {{line_yoffset_advanced_tablet.SIZE}}{{line_yoffset_advanced.UNIT}}) rotate({{line_rotate_advanced.SIZE}}{{line_rotate_advanced.UNIT}});',
                    '(mobile){{WRAPPER}} .mdp-typography-content-header' => 'transform: translate({{line_xoffset_advanced_mobile.SIZE}}{{line_xoffset_advanced.UNIT}}, {{line_yoffset_advanced_mobile.SIZE}}{{line_yoffset_advanced.UNIT}}) rotate({{line_rotate_advanced.SIZE}}{{line_rotate_advanced.UNIT}});',
                ],
            ]
        );

        /** Y Offset. */
        $this->add_responsive_control(
            'line_yoffset_advanced',
            [
                'label' => esc_html__( 'Y Offset', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'range' => [
                    'px' => [
                        'min' => -500,
                        'max' => 500,
                        'step' => 1,
                    ]
                ],
                'default' => [ 'unit' => 'px', 'size' => 0, ],
                'selectors' => [
                    '(desktop){{WRAPPER}} .mdp-typography-content-header' => 'transform: translate({{line_xoffset_advanced.SIZE}}{{line_xoffset_advanced.UNIT}}, {{line_yoffset_advanced.SIZE}}{{line_yoffset_advanced.UNIT}}) rotate({{line_rotate_advanced.SIZE}}{{line_rotate_advanced.UNIT}});',
                    '(tablet){{WRAPPER}} .mdp-typography-content-header' => 'transform: translate({{line_xoffset_advanced_tablet.SIZE}}{{line_xoffset_advanced.UNIT}}, {{line_yoffset_advanced_tablet.SIZE}}{{line_yoffset_advanced.UNIT}}) rotate({{line_rotate_advanced.SIZE}}{{line_rotate_advanced.UNIT}});',
                    '(mobile){{WRAPPER}} .mdp-typography-content-header' => 'transform: translate({{line_xoffset_advanced_mobile.SIZE}}{{line_xoffset_advanced.UNIT}}, {{line_yoffset_advanced_mobile.SIZE}}{{line_yoffset_advanced.UNIT}}) rotate({{line_rotate_advanced.SIZE}}{{line_rotate_advanced.UNIT}});',
                ],
            ]
        );

        /** End section style content. */
        $this->end_controls_section();

    }

    /**
     * Convert the expression to the title bar.
     *
     * @param $header - contains the title text.
     *
     * @return string
     *
     */
    public function get_customHeaderEasy( $header ) {

        /** We get all the values from the admin panel. */
        $settings = $this->get_settings_for_display();

        /** Contains a string to be animated. */
        $strAnimation = '';

        /** Determine the length of the string. */
        $headerLength = iconv_strlen($header);

        /** Count the number of characters to the opening curly bracket. */
        $posLeft = strripos( $header, "{") + 1;

        /** Count the number of characters to the closing curly bracket. */
        $posRight = strripos( $header, "}") - 1;

        if( strripos( $header, "{") !== false ){

            /** We cut out only the fragment that will participate in the animation. */
            $removeBefore = stristr( $header, '{');
            $removeAfter = stristr( $removeBefore, '}', true);
            $strAnimation = substr($removeAfter, 1);

            /** Break the animation string and put it in an array. */
            $arrayAnimation = explode(" | ", $strAnimation);

            /** Wrap the title text in tags. */
            $strHTML = '<span class="mdp-header-before">'.substr( $header, 0, $posLeft-1).'</span>';
            $strHTML .= '<span class="mdp-motionger-wrapper mdp-motionger-box-' . $settings["animation_header"] . ' mdp-motionger-wrapper-' . $this->get_id() . '">';
            foreach( $arrayAnimation as $val ){
                $strHTML .= '<span class="mdp-motionger-word-animation mdp-' . $settings["animation_header"] . '-active-header mdp-' . $settings["animation_header"] . '-inactive">'.$val.'</span>';
            }
            $strHTML .= '</span> ';
            $strHTML .= '<span  class="mdp-header-after">'.substr( $header, $posRight+2, $headerLength ).'</span>';

        }else{
            $strHTML = $header;
        }

        return $strHTML;
    }

    public function get_customHeaderHard( $header ) {

        /** We get all the values from the admin panel. */
        $settings = $this->get_settings_for_display();

        /** Determine the length of the string. */
        $headerLength = iconv_strlen($header);

        /** Count the number of characters to the opening curly bracket. */
        $posLeft = strripos( $header, "{") + 1;

        /** Count the number of characters to the closing curly bracket. */
        $posRight = strripos( $header, "}") - 1;

        if( strripos( $header, "{") !== false ){
            /** We cut out only the fragment that will participate in the animation. */
            $removeBefore = stristr( $header, '{');
            $removeAfter = stristr( $removeBefore, '}', true);
            $strAnimation = substr($removeAfter, 1);

            /** Break the animation string and put it in an array. */
            $arrayAnimation = explode(" | ", $strAnimation);

            /** Wrap the title text in tags. */
            $strHTML = '<span class="mdp-header-before">'.substr( $header, 0, $posLeft-1).'</span>';
            $strHTML .= '<span class="mdp-motionger-wrapper mdp-motionger-box-' . $settings["animation_header"] . ' mdp-motionger-wrapper-' . $this->get_id() . '">';
            foreach( $arrayAnimation as $val ){

                $word_split = preg_split('//u', $val, -1, PREG_SPLIT_NO_EMPTY);

                $strHTML .= '<span class="mdp-motionger-letter-animation mdp-' . $settings["animation_header"] . '-active-header mdp-' . $settings["animation_header"] . '-inactive">';

                foreach ( $word_split as $split_text){

                    $split_text = ( $split_text !== ' ' ) ? mb_convert_encoding( $split_text, 'UTF-8', 'UTF-8' ) : '&nbsp;';
                    $strHTML .= '<span class="mdp-split-word mdp-' . $settings["animation_header"] . '-text">' . $split_text . '</span>';

                }

                $strHTML .= '</span>';

            }
            $strHTML .= '</span> ';
            $strHTML .= '<span  class="mdp-header-after">'.substr( $header, $posRight+2, $headerLength ).'</span>';
        } else {
            $strHTML = $header;
        }

        return $strHTML;

    }

    /**
     * Render Frontend Output. Generate the final HTML on the frontend.
     *
     * @since 1.0.0
     * @access protected
     **/
    protected function render() {

        /** We get all the values from the admin panel. */
        $settings = $this->get_settings_for_display();

        /** We write the section style class to a variable. */
        $globalClass = 'mdp-' . $this->get_id();

        /** Heading link. */
        $url = $settings['heading_link']['url'];
        $target = $settings['heading_link']['is_external'] ? ' target="_blank"' : '';
        $nofollow = $settings['heading_link']['nofollow'] ? ' rel="nofollow"' : '';

        $this->add_render_attribute( 'sub_heading', 'class', 'mdp-sub-heading-content mdp-typography-subheader mdp-shadow-subheader mdp-color-sub-header' );

        /** Use the gradient style class where necessary. */
        $this->add_render_attribute(
            [
                'header_text' => [
                    'class' => [
                        'mdp-typography-header mdp-typography-default-header mdp-shadow-header mdp-heading-margin',
                        $settings[ "header_background_effect" ] === 'yes' ? 'mdp-header-background-effect' : '',
                    ],
                ],
            ]
        );

        $this->add_render_attribute(
            [
                'split_text' => [
                    'class' => [
                        'mdp-typography-header mdp-typography-default-header mdp-shadow-header mdp-heading-margin mdp-motionger-split-text',
                        $settings["header_background_effect"] === 'yes' ? 'mdp-header-background-effect' : '',
                    ],
                ],
            ]
        );

        ?>

        <div id="<?php esc_attr_e( $globalClass ); ?>" >

            <?php if ( !wp_is_mobile() ): ?>
                <div class="mdp-content-header mdp-color-content-header mdp-text-content-align mdp-text-content-margin mdp-shadow-content">
                    <div class="mdp-typography-content-header">
                        <?php esc_html_e( $settings["text_content_heading"] ); ?>
                    </div>
                </div>
            <?php endif ?>

            <?php if ( $settings["show_subheader"] === 'yes' and  $settings["header_bottom"] === '' ): ?>

                <div class="mdp-sub-heading mdp-sub-header-align mdp-subheading-margin">
                    <div class="mdp-sub-header-box">
                        <div <?php echo $this->get_render_attribute_string( 'sub_heading' ); ?>>
                            <?php esc_html_e( $settings["sub_heading"] ); ?>
                        </div>

                        <?php if ( $settings["subheader_line"] === 'yes' ): ?>

                            <?php if ( $settings["subheader_position"] === 'before' or $settings["subheader_position"] === 'after_before' ): ?>
                                <div class="sub-line mdp-subheader-line-left mdp-typography-subheader"></div>
                            <?php endif ?>

                            <?php if ( $settings["subheader_position"] === 'after' or $settings["subheader_position"] === 'after_before' ): ?>
                                <div class="sub-line mdp-subheader-line-right mdp-typography-subheader"></div>
                            <?php endif ?>

                            <?php if ( $settings["subheader_position"] === 'bottom' ): ?>
                                <div class="sub-line mdp-subheader-line-bottom"></div>
                            <?php endif ?>

                        <?php endif ?>

                    </div>
                </div>

            <?php endif ?>

            <<?php esc_attr_e( $settings["header_tag"] ); ?>  class="mdp-header-text mdp-header-align">

            <?php if (!empty($url) ): ?>
            <a href="<?php esc_html_e( $url ); ?>"
               class="mdp-color-header" <?php esc_attr_e( $target ); ?> <?php esc_attr_e( $nofollow ); ?>>
                <?php endif; ?>

                <span class="mdp-sub-header-box">
                        <span <?php  echo $this->get_render_attribute_string( 'header_text' ); ?>>

		                    <?php
                            if ( in_array($settings["animation_header"], [
                                'motionger-rise',
                                'motionger-drop-in',
                                'motionger-slide-right',
                                'motionger-slide-left',
                                'motionger-slide-down',
                                'motionger-slide-up',
                                'motionger-bounce',
                                'motionger-fade',
                                'motionger-flip-x',
                                'motionger-flip-y',
                                'motionger-jello',
                                'motionger-zoom',
                                'motionger-crop',
                            ]) ) {
                                echo $this->get_customHeaderEasy( $settings["header_text"] );
                            } elseif ( in_array($settings["animation_header"], ['curl', 'jalousie', 'ripple', 'typing']) ){
                                echo $this->get_customHeaderHard( $settings["header_text"] );
                            }
                            ?>

                        </span>

                        <?php if ( $settings["header_line"] === 'yes' ): ?>

                            <?php if ( $settings["header_position"] === 'before' or $settings["header_position"] === 'after_before' ): ?>
                                <div class="line mdp-header-line-left mdp-typography-header"></div>
                            <?php endif ?>

                            <?php if ( $settings["header_position"] === 'after' or $settings["header_position"] === 'after_before' ): ?>
                                <div class="line mdp-header-line-right mdp-typography-header"></div>
                            <?php endif ?>

                            <?php if ( $settings["header_position"] === 'bottom' ): ?>
                                <div class="line mdp-header-line-bottom"></div>
                            <?php endif ?>

                        <?php endif ?>

                    </span>

                <?php if ( $settings["header_split"] === 'yes' ): ?>
                    <span <?php echo $this->get_render_attribute_string( 'split_text' ); ?>>
                            <?php esc_html_e( $settings["split_text"] ); ?>
                        </span>
                <?php endif ?>

                <?php if (!empty($url) ): ?>
            </a>
        <?php endif; ?>

        </<?php esc_attr_e( $settings["header_tag"] ); ?>>

        <?php if ( $settings["show_subheader"] === 'yes' and  $settings["header_bottom"] === 'yes' ): ?>

            <div class="mdp-sub-heading mdp-sub-header-align mdp-subheading-margin">
                <div class="mdp-sub-header-box">
                    <div <?php echo $this->get_render_attribute_string( 'sub_heading' ); ?>>
                        <?php esc_html_e( $settings["sub_heading"] ); ?>
                    </div>

                    <?php if ( $settings["subheader_line"] === 'yes' ): ?>

                        <?php if ( $settings["subheader_position"] === 'before' or $settings["subheader_position"] === 'after_before' ): ?>
                            <div class="sub-line mdp-subheader-line-left mdp-typography-subheader"></div>
                        <?php endif ?>

                        <?php if ( $settings["subheader_position"] === 'after' or $settings["subheader_position"] === 'after_before' ): ?>
                            <div class="sub-line mdp-subheader-line-right mdp-typography-subheader"></div>
                        <?php endif ?>

                        <?php if ( $settings["subheader_position"] === 'bottom' ): ?>
                            <div class="sub-line mdp-subheader-line-bottom"></div>
                        <?php endif ?>

                    <?php endif ?>

                </div>
            </div>

        <?php endif ?>

        </div>
        <script>
            <?php if ( in_array($settings["animation_header"], [
                'motionger-rise',
                'motionger-drop-in',
                'motionger-slide-right',
                'motionger-slide-left',
                'motionger-slide-down',
                'motionger-slide-up',
                'motionger-bounce',
                'motionger-fade',
                'motionger-flip-x',
                'motionger-flip-y',
                'motionger-jello',
                'motionger-zoom',
                'motionger-crop',
                'typing'
            ]) and strripos( $settings["header_text"], "{") !== false ) : ?>
            window.MotiongerInit<?php echo esc_attr( $this->get_id() ); ?> = function() {

                /** We begin to show the title from the first element. */
                let motiongerIndex = 1;

                /** Header container. */
                const wrapper = document.querySelector('.mdp-motionger-wrapper-<?php echo esc_attr( $this->get_id() ) ?>');

                /** Call the header animation function. */
                animationHeader(motiongerIndex);

                /** We are shifting to the next heading element. */
                function nextMotionger() {
                    animationHeader(motiongerIndex += 1);
                }

                /** Header animation function. */
                function animationHeader(n) {

                    /** Header items. */
                    let items = wrapper.querySelectorAll('.mdp-<?php echo esc_attr( $settings["animation_header"] ); ?>-active-header');

                    /** We determine the number of title items. */
                    let itemsLength = items.length;

                    /** Go back to the beginning of the list items. */
                    if (n > itemsLength) {
                        motiongerIndex = 1
                    }

                    /** Set the pointer to the item. */
                    if (n < 1) {
                        motiongerIndex = itemsLength;
                    }

                    /** Hide all items */
                    for (let i = 0; i < itemsLength; i++) {

                        // Hide un-active phrases
                        if ( items[i].classList.contains( 'mdp-<?php echo esc_attr( $settings["animation_header"] ); ?>-inactive' ) ) {
                            items[i].classList.add( 'mdp-motionger-hide' );
                        }

                        // Previous previous active phrases
                        if ( items[i].classList.contains( 'mdp-<?php echo esc_attr( $settings["animation_header"] ); ?>-active' ) ) {

                            items[i].classList.remove( 'mdp-<?php echo esc_attr( $settings["animation_header"] );  ?>-active' );
                            items[i].classList.add( 'mdp-<?php echo esc_attr( $settings["animation_header"] );  ?>-inactive' );

                            // Hide previous previous active phrases ofter -out animation played
                            setTimeout( function () {
                                items[i].classList.add( 'mdp-motionger-hide' );
                            }, 500);

                        }

                    }

                    /** Show current item */
                    setTimeout( function () {

                        items[ motiongerIndex - 1 ].classList.remove( 'mdp-motionger-hide' ); // Show active phrases
                        items[ motiongerIndex - 1 ].classList.remove( 'mdp-<?php echo esc_attr( $settings["animation_header"] ); ?>-inactive' );
                        items[ motiongerIndex - 1 ].classList.add( 'mdp-<?php echo esc_attr( $settings["animation_header"] ); ?>-active' );

                    }, 500);

                }

                setInterval(nextMotionger, <?php echo esc_attr( $settings["text_animation_speed"]["size"] ); ?> );
            }
            <?php endif ?>
            <?php if ( in_array($settings["animation_header"], [
                'curl',
                'jalousie',
                'ripple',
                'typing'
            ]) and strripos( $settings["header_text"], "{") !== false ) : ?>
            window.MotiongerInit<?php echo esc_attr( $this->get_id() ); ?> =  function() {

                /** We begin to show the title from the first element. */
                let motiongerIndex = 1;

                /** Header container. */
                const wrapper = document.querySelector('.mdp-motionger-wrapper-<?php echo esc_attr( $this->get_id() ) ?>');

                /** Call the header animation function. */
                animationHeader( motiongerIndex );

                /** We are shifting to the next heading element. */
                function nextMotionger() {
                    animationHeader( motiongerIndex += 1);
                }

                /** Header animation function. */
                function animationHeader( n ){

                    /** Header items. */
                    let items = wrapper.querySelectorAll('.mdp-<?php echo esc_attr( $settings["animation_header"] ); ?>-active-header');

                    /** We determine the number of title items. */
                    let itemsLength = items.length;

                    /** Go back to the beginning of the list items. */
                    if ( n > itemsLength ) {
                        motiongerIndex = 1
                    }

                    /** Set the pointer to the item. */
                    if ( n < 1 ) {
                        motiongerIndex = itemsLength;
                    }

                    /** Hide items. */
                    for (let i = 0; i < itemsLength; i++) {

                        let childItems = items[i].children;

                        // Hide un-active phrases
                        items[i].classList.add( 'mdp-motionger-hide' );

                        // Hide un-active words
                        for( let j = 0; j < childItems.length; j++ ){
                            if( childItems[j].className === 'mdp-split-word mdp-<?php echo esc_attr( $settings["animation_header"] ); ?>-text motionger-<?php echo esc_attr( $settings["animation_header"] ); ?>-active' ){
                                setTimeout(function (j) {
                                    childItems[j].classList.remove('motionger-<?php echo esc_attr( $settings["animation_header"] ); ?>-active');
                                    childItems[j].classList.add('motionger-<?php echo esc_attr( $settings["animation_header"] ); ?>-inactive');
                                }, 100 * j + 1, j);
                            }
                        }

                    }

                    /** Show item. */
                    // Hide active phrases
                    items[motiongerIndex - 1].classList.remove( 'mdp-motionger-hide' );

                    // Show active words
                    let childItemsBlock = items[motiongerIndex - 1].children;
                    let letterAnimationTime = <?php echo esc_attr( $settings["letter_animation_speed"]["size"] ); ?>;
                    for( let j = 0; j < childItemsBlock.length; j++ ){
                        setTimeout(function (j) {
                            childItemsBlock[j].classList.remove('motionger-<?php echo esc_attr( $settings["animation_header"] ); ?>-inactive');
                            childItemsBlock[j].classList.add('motionger-<?php echo esc_attr( $settings["animation_header"] ); ?>-active');
                        }, letterAnimationTime * j + 1, j);
                    }

                }

                setInterval( nextMotionger, <?php echo esc_attr( $settings["text_animation_speed"]["size"] ); ?> );

            }
            <?php endif ?>
            document.addEventListener( "DOMContentLoaded", window.MotiongerInit<?php echo esc_attr( $this->get_id() ); ?> ); //Motionger Front-end Init
            <?php if( is_admin() ) { ?>
            if ( document.readyState === "complete" ) { window.MotiongerInit<?php echo esc_attr( $this->get_id() ); ?>() } //Backend Init
            <?php } ?>
        </script>
        <?php

    }

    /**
     * Return link for documentation.
     *
     * Used to add stuff after widget.
     *
     * @since 1.0.0
     * @access public
     **/
    public function get_custom_help_url() {
        return 'https://docs.merkulov.design/category/motionger-elementor/';
    }

}
