<?php /** @noinspection PhpUndefinedClassInspection */
/**
 * Producter for Elementor
 * Customizable product fields for Elementor editor
 * Exclusively on https://1.envato.market/producter-elementor
 *
 * @encoding        UTF-8
 * @version         1.12.1
 * @copyright       (C) 2018 - 2023 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Dmitry Merkulov (dmitry@merkulov.design), Cherviakov Vlad (vladchervjakov@gmail.com)
 * @support         help@merkulov.design
 **/

namespace Merkulove\Selection;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
    header( 'Status: 403 Forbidden' );
    header( 'HTTP/1.1 403 Forbidden' );
    exit;
}

use Elementor\Core\Schemes\Color;
use Elementor\Core\Schemes\Typography;
use Elementor\Group_Control_Background;
use Elementor\Group_Control_Border;
use Elementor\Group_Control_Box_Shadow;
use Elementor\Group_Control_Text_Shadow;
use Elementor\Group_Control_Typography;
use Exception;
use Elementor\Widget_Base;
use Elementor\Controls_Manager;
use Merkulove\Selection\Unity\Plugin as UnityPlugin;

/** @noinspection PhpUnused */
/**
 * Product Rating - Custom Elementor Widget.
 **/
class product_rating_elementor extends Widget_Base {

    /**
     * Use this to sort widgets.
     * A smaller value means earlier initialization of the widget.
     * Can take negative values.
     * Default widgets and widgets from 3rd party developers have 0 $mdp_order
     **/
    public $mdp_order = 1;

    /**
     * Widget base constructor.
     * Initializing the widget base class.
     *
     * @access public
     * @throws Exception If arguments are missing when initializing a full widget instance.
     * @param array      $data Widget data. Default is an empty array.
     * @param array|null $args Optional. Widget default arguments. Default is null.
     *
     * @return void
     **/
    public function __construct( $data = [], $args = null ) {

        parent::__construct( $data, $args );

        wp_register_style( 'mdp-selection-elementor-admin', UnityPlugin::get_url() . 'css/elementor-admin' . UnityPlugin::get_suffix() . '.css', [], UnityPlugin::get_version() );
        wp_register_style( 'mdp-producter-elementor', UnityPlugin::get_url() . 'css/producter-elementor' . UnityPlugin::get_suffix() . '.css', [], UnityPlugin::get_version() );

    }

    /**
     * Return a widget name.
     *
     * @return string
     **/
    public function get_name() {

        return 'mdp-producter-product-rating-elementor';

    }

    /**
     * Return the widget title that will be displayed as the widget label.
     *
     * @return string
     **/
    public function get_title() {

        return esc_html__( 'Product Rating', 'selection' );

    }

    /**
     * Set the widget icon.
     *
     * @return string
     */
    public function get_icon() {

        return 'mdp-product-rating-elementor-widget-icon';

    }

    /**
     * Set the category of the widget.
     *
     * @return array with category names
     **/
    public function get_categories() {

        return [ 'selection-category' ];

    }

    /**
     * Get widget keywords. Retrieve the list of keywords the widget belongs to.
     *
     * @access public
     *
     * @return array Widget keywords.
     **/
    public function get_keywords() {

        return [ 'Merkulove', 'Producter', 'woocommerce' ];

    }

    /**
     * Get style dependencies.
     * Retrieve the list of style dependencies the widget requires.
     *
     * @access public
     *
     * @return array Widget styles dependencies.
     **/
    public function get_style_depends() {

        return [ 'mdp-producter-elementor', 'mdp-selection-elementor-admin', 'elementor-icons-fa-solid' ];

    }

	/**
	 * Get script dependencies.
	 * Retrieve the list of script dependencies the element requires.
	 *
	 * @access public
     *
	 * @return array Element scripts dependencies.
	 **/
	public function get_script_depends() {

		return [ 'mdp-producter' ];

    }

    /**
     * Add the widget controls.
     *
     * @access protected
     * @return void with category names
     **/
    protected function register_controls() {

        /** Content Tab. */
        $this->tab_content();

        /** Style Tab. */
        $this->tab_style();

    }

    /**
     * Add widget controls on Content tab.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function tab_content() {

        /** Content -> Product Rating Content Section. */
        $this->section_content_product_rating();

    }

    /**
     * Add widget controls on Style tab.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function tab_style() {

        /** Style -> Section Style Product Rating. */
        $this->section_style_product();

    }

    /**
     * Add widget controls: Content -> Product Rating Content Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_content_product_rating() {

        $this->start_controls_section( 'section_content_product_rating', [
            'label' => esc_html__( 'Product Rating', 'selection' ),
            'tab'   => Controls_Manager::TAB_CONTENT
        ] );


        if ( Caster::get_instance()->is_active_woocommerce() ) {
            $products_list = [];

            $products = wc_get_products( ['return' => 'ids', 'limit' => -1] );
            foreach ( $products as $product ) {
                $products_list[$product] = get_the_title( $product );
            }

            if ( empty( wc_get_product() ) ) {

                $this->add_control(
                    'products',
                    [
                        'label' => esc_html__('Products', 'selection'),
                        'type' => Controls_Manager::SELECT2,
                        'multiple' => false,
                        'options' => $products_list,
                        'default' => !empty( $products_list ) ? array_keys( $products_list )[0] : ''
                    ]
                );
            }

            if ( !empty( wc_get_product() ) ) {

                $this->add_control(
                    'show_quantity_link',
                    [
                        'label' => esc_html__( 'Show reviews link', 'selection' ),
                        'type' => Controls_Manager::SWITCHER,
                        'label_on' => esc_html__( 'Show', 'selection' ),
                        'label_off' => esc_html__( 'Hide', 'selection' ),
                        'return_value' => 'yes',
                        'default' => 'yes',
                    ]
                );
            }
        }

        $this->add_control(
            'rating_text_align',
            [
                'label' => esc_html__( 'Alignment', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'flex-start' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'fa fa-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'fa fa-align-center',
                    ],
                    'flex-end' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'fa fa-align-right',
                    ],
                    'space-between' => [
                        'title' => esc_html__( 'Space between', 'selection' ),
                        'icon' => 'fa fa-expand-arrows-alt',
                    ],
                ],
                'selectors' => [
                        '{{WRAPPER}} .woocommerce-product-rating' => 'justify-content: {{VALUE}}'
                ],
                'default' => 'center',
                'toggle' => true,
            ]
        );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Rating.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_product() {

        $this->start_controls_section( 'section_style_product', [
            'label' => esc_html__( 'Rating', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->add_responsive_control(
            'section_style_product_margin',
            [
                'label' => esc_html__( 'Margin', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'selectors' => [
                    "{{WRAPPER}} .woocommerce-product-rating" => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_responsive_control(
            'section_style_product_padding',
            [
                'label' => esc_html__( 'Padding', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'separator' => 'after',
                'selectors' => [
                    "{{WRAPPER}} .woocommerce-product-rating" => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );


        $this->add_control(
            'section_style_product_rating_star_size',
            [
                'label' => esc_html__( 'Rating star size', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', 'em' ],
                'range' => [
                    'px' => [
                        'min' => 1,
                        'max' => 500,
                        'step' => 1,
                    ],
                    'em' => [
                         'min' => 1,
                         'max' => 100,
                         'step' => 1
                    ]
                ],
                'selectors' => [
                    '{{WRAPPER}} .star-rating' => 'font-size: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->add_control(
            'section_style_product_star_size_divide',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name' => 'section_style_product_typography',
                'label' => esc_html__( 'Typography', 'selection' ),
                'scheme' =>  Typography::TYPOGRAPHY_1,
                'selector' => '{{WRAPPER}} .woocommerce-review-link',
            ]
        );

        $this->start_controls_tabs('control_style_tabs');

        $this->start_controls_tab( 'normal_style_tab', ['label' => esc_html__( 'NORMAL', 'selection' )] );

        $this->add_control(
            'section_style_product_rating_color',
            [
                'label' => esc_html__( 'Rating color', 'selection' ),
                'type' => Controls_Manager::COLOR,
                'scheme' => [
                    'type' => Color::get_type(),
                    'value' => Color::COLOR_3,
                ],
                'selectors' => [
                    '{{WRAPPER}} .woocommerce-product-rating' => 'color: {{VALUE}}',
                ],
            ]
        );

        $this->add_control(
            'section_style_product_text_color',
            [
                'label' => esc_html__( 'Review text color', 'selection' ),
                'type' => Controls_Manager::COLOR,
                'scheme' => [
                    'type' => Color::get_type(),
                    'value' => Color::COLOR_3,
                ],
                'selectors' => [
                    '{{WRAPPER}} .woocommerce-review-link' => 'color: {{VALUE}}',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Background::get_type(),
            [
                'name' => 'section_style_product_background',
                'label' => esc_html__( 'Background type', 'selection' ),
                'types' => [ 'classic', 'gradient', 'video' ],
                'selector' => '{{WRAPPER}} .woocommerce-product-rating',
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => 'section_style_product_border_normal',
                'label' => esc_html__( 'Border Type', 'selection' ),
                'separator' => 'before',
                'selector' => '{{WRAPPER}} .woocommerce-product-rating',
            ]
        );

        $this->add_responsive_control(
            'section_style_product_border_radius_normal',
            [
                'label' => esc_html__( 'Border radius', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    '{{WRAPPER}} .woocommerce-product-rating' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',

                ],
            ]
        );


        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => 'section_style_product_box_shadow_normal',
                'label' => esc_html__( 'Box Shadow', 'selection' ),
                'selector' => '{{WRAPPER}} .woocommerce-product-rating',
            ]
        );

        $this->add_group_control(
            Group_Control_Text_Shadow::get_type(),
            [
                'name' => 'section_style_product_text_shadow',
                'label' => esc_html__( 'Review Link Text Shadow', 'selection' ),
                'selector' => '{{WRAPPER}} .woocommerce-review-link',
            ]
        );

        $this->end_controls_tab();

        $this->start_controls_tab( 'hover_style_tab', ['label' => esc_html__( 'HOVER', 'selection' )] );

        $this->add_control(
            'section_style_product_rating_color_hover',
            [
                'label' => esc_html__( 'Text color', 'selection' ),
                'type' => Controls_Manager::COLOR,
                'scheme' => [
                    'type' => Color::get_type(),
                    'value' => Color::COLOR_3,
                ],
                'selectors' => [
                    '{{WRAPPER}} .woocommerce-product-rating:hover' => 'color: {{VALUE}}',
                ],
            ]
        );

        $this->add_control(
            'section_style_product_text_color_hover',
            [
                'label' => esc_html__( 'Text color', 'selection' ),
                'type' => Controls_Manager::COLOR,
                'scheme' => [
                    'type' => Color::get_type(),
                    'value' => Color::COLOR_3,
                ],
                'selectors' => [
                    '{{WRAPPER}} .woocommerce-product-rating:hover .woocommerce-review-link' => 'color: {{VALUE}}',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Background::get_type(),
            [
                'name' => 'section_style_product_background_hover',
                'label' => esc_html__( 'Background type', 'selection' ),
                'types' => [ 'classic', 'gradient', 'video' ],
                'selector' => '{{WRAPPER}} .woocommerce-product-rating:hover',
            ]
        );

        $this->add_control(
            'section_style_product_hover_transition',
            [
                'label' => esc_html__( 'Hover transition', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 's' ],
                'range' => [
                    's' => [
                        'min' => 0.1,
                        'max' => 5,
                        'step' => 0.1,
                    ],
                ],
                'default' => [
                    'unit' => 's',
                    'size' => 0,
                ],
                'selectors' => [
                    '{{WRAPPER}} .woocommerce-product-rating, {{WRAPPER}} .woocommerce-review-link' => 'transition: color {{SIZE}}{{UNIT}}, background {{SIZE}}{{UNIT}}, box-shadow {{SIZE}}{{UNIT}}, text-shadow {{SIZE}}{{UNIT}}, border-radius {{SIZE}}{{UNIT}}, border {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => 'section_style_product_border_hover',
                'label' => esc_html__( 'Border Type', 'selection' ),
                'selector' => '{{WRAPPER}} .woocommerce-product-rating:hover',
                'separator' => 'before',
            ]
        );

        $this->add_responsive_control(
            'section_style_product_border_radius_hover',
            [
                'label' => esc_html__( 'Border radius', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    '{{WRAPPER}} .woocommerce-product-rating:hover' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',

                ],
            ]
        );


        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => 'section_style_product_box_shadow_hover',
                'label' => esc_html__( 'Box Shadow', 'selection' ),
                'selector' => '{{WRAPPER}} .woocommerce-product-rating:hover',
            ]
        );


        $this->add_group_control(
            Group_Control_Text_Shadow::get_type(),
            [
                'name' => 'section_style_product_text_shadow_hover',
                'label' => esc_html__( 'Review Link Text Shadow', 'selection' ),
                'selector' => '{{WRAPPER}} .woocommerce-product-rating:hover .woocommerce-review-link',
            ]
        );


        $this->end_controls_tab();

        $this->end_controls_tabs();

        $this->end_controls_section();

    }


    /**
     * Render Frontend Output. Generate the final HTML on the frontend.
     *
     * @access protected
     *
     * @return void
     **/
    protected function render() {
        if ( !Caster::get_instance()->is_active_woocommerce() ) {
            esc_html_e( 'Install WooCommerce to display product rating!' );
            return;
        }

        $settings = $this->get_settings_for_display();

        global $product;
        $product = wc_get_product();

        if ( !post_type_supports( 'product', 'comments' ) ) {
            return;
        }

        if ( empty( $product ) ) {
            $product = wc_get_product( esc_sql( $settings['products'] ) );
            if ( empty( $product ) || $product->get_status() === 'trash' ) {
                return;
            }
        }

        ?>
        <!-- Start Producter for Elementor WordPress Plugin -->
        <div class="mdp-producter-elementor-product-rating-box">
            <div class="mdp-producter-elementor-product-wrapper
                <?php if ( empty( wc_get_product() ) ): ?> mdp-producter-elementor-non-product-page <?php endif; ?>
                <?php if ( isset( $settings['show_quantity_link'] ) && $settings['show_quantity_link'] !== 'yes' ): ?> mdp-producter-elementor-rating-hide-reviews-link <?php endif; ?>"
            >
                <?php
                    wc_get_template( 'single-product/rating.php' );
                ?>
            </div>
        </div>
        <!-- End Producter for Elementor WordPress Plugin -->
	    <?php

    }

    /**
     * Return link for documentation
     * Used to add stuff after widget
     *
     * @access public
     *
     * @return string
     **/
    public function get_custom_help_url() {

        return 'https://docs.merkulov.design/tag/producter';

    }

}
