<?php /** @noinspection PhpUndefinedClassInspection */
/**
 * Producter for Elementor
 * Customizable product fields for Elementor editor
 * Exclusively on https://1.envato.market/producter-elementor
 *
 * @encoding        UTF-8
 * @version         1.12.1
 * @copyright       (C) 2018 - 2023 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Dmitry Merkulov (dmitry@merkulov.design), Cherviakov Vlad (vladchervjakov@gmail.com)
 * @support         help@merkulov.design
 **/

namespace Merkulove\Selection;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
    header( 'Status: 403 Forbidden' );
    header( 'HTTP/1.1 403 Forbidden' );
    exit;
}

use Elementor\Core\Schemes\Color;
use Elementor\Core\Schemes\Typography;
use Elementor\Group_Control_Background;
use Elementor\Group_Control_Border;
use Elementor\Group_Control_Box_Shadow;
use Elementor\Group_Control_Typography;
use Exception;
use Elementor\Widget_Base;
use Elementor\Controls_Manager;
use Merkulove\Selection\Unity\Plugin as UnityPlugin;

/** @noinspection PhpUnused */
/**
 * Product Stock - Custom Elementor Widget.
 **/
class product_stock_elementor extends Widget_Base {

    /**
     * Use this to sort widgets.
     * A smaller value means earlier initialization of the widget.
     * Can take negative values.
     * Default widgets and widgets from 3rd party developers have 0 $mdp_order
     **/
    public $mdp_order = 1;

    /**
     * Widget base constructor.
     * Initializing the widget base class.
     *
     * @access public
     * @throws Exception If arguments are missing when initializing a full widget instance.
     * @param array      $data Widget data. Default is an empty array.
     * @param array|null $args Optional. Widget default arguments. Default is null.
     *
     * @return void
     **/
    public function __construct( $data = [], $args = null ) {

        parent::__construct( $data, $args );

        wp_register_style( 'mdp-selection-elementor-admin', UnityPlugin::get_url() . 'css/elementor-admin' . UnityPlugin::get_suffix() . '.css', [], UnityPlugin::get_version() );
        wp_register_style( 'mdp-producter-elementor', UnityPlugin::get_url() . 'css/producter-elementor' . UnityPlugin::get_suffix() . '.css', [], UnityPlugin::get_version() );

    }

    /**
     * Return a widget name.
     *
     * @return string
     **/
    public function get_name() {

        return 'mdp-producter-product-stock-elementor';

    }

    /**
     * Return the widget title that will be displayed as the widget label.
     *
     * @return string
     **/
    public function get_title() {

        return esc_html__( 'Product Stock', 'selection' );

    }

    /**
     * Set the widget icon.
     *
     * @return string
     */
    public function get_icon() {

        return 'mdp-product-stock-elementor-widget-icon';

    }

    /**
     * Set the category of the widget.
     *
     * @return array with category names
     **/
    public function get_categories() {

        return [ 'selection-category' ];

    }

    /**
     * Get widget keywords. Retrieve the list of keywords the widget belongs to.
     *
     * @access public
     *
     * @return array Widget keywords.
     **/
    public function get_keywords() {

        return [ 'Merkulove', 'Producter', 'woocommerce' ];

    }

    /**
     * Get style dependencies.
     * Retrieve the list of style dependencies the widget requires.
     *
     * @access public
     *
     * @return array Widget styles dependencies.
     **/
    public function get_style_depends() {

        return [ 'mdp-producter-elementor', 'mdp-selection-elementor-admin', 'elementor-icons-fa-solid' ];

    }

	/**
	 * Get script dependencies.
	 * Retrieve the list of script dependencies the element requires.
	 *
	 * @access public
     *
	 * @return array Element scripts dependencies.
	 **/
	public function get_script_depends() {

		return [ 'mdp-producter' ];

    }

    /**
     * Add the widget controls.
     *
     * @access protected
     * @return void with category names
     **/
    protected function register_controls() {

        /** Content Tab. */
        $this->tab_content();

        /** Style Tab. */
        $this->tab_style();

    }

    /**
     * Add widget controls on Content tab.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function tab_content() {

        /** Content -> Product Stock Content Section. */
        $this->section_content_product_stock();

    }

    /**
     * Add widget controls on Style tab.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function tab_style() {

        /** Style -> Section Style Product Stock. */
        $this->section_style_product_stock();

    }

    /**
     * Add widget controls: Content -> Product Stock Content Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_content_product_stock() {

        $this->start_controls_section( 'section_content_product_stock', [
            'label' => esc_html__( 'Product Stock', 'selection' ),
            'tab'   => Controls_Manager::TAB_CONTENT
        ] );

        if ( Caster::get_instance()->is_active_woocommerce() ) {
            $products_list = [];

            $products = wc_get_products( ['return' => 'ids', 'limit' => -1] );
            foreach ( $products as $product ) {
                $products_list[$product] = get_the_title( $product );
            }

            if ( empty( wc_get_product() ) ) {

                $this->add_control(
                    'products',
                    [
                        'label' => esc_html__('Products', 'selection'),
                        'type' => Controls_Manager::SELECT2,
                        'multiple' => false,
                        'options' => $products_list,
                        'default' => !empty( $products_list ) ? array_keys( $products_list )[0] : ''
                    ]
                );

            }
        }

        $this->add_control(
            'stock_text_align',
            [
                'label' => esc_html__( 'Alignment', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'fa fa-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'fa fa-align-center',
                    ],
                    'right' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'fa fa-align-right',
                    ],
                    'justify' => [
                        'title' => esc_html__( 'Justify', 'selection' ),
                        'icon' => 'fa fa-align-justify',
                    ],
                ],
                'selectors' => [

                ],
                'default' => 'center',
                'toggle' => true,
            ]
        );

        $this->add_control(
            'stock_text_before',
            [
                'label' => esc_html__( 'Before text', 'selection' ),
                'type' => Controls_Manager::WYSIWYG,
            ]
        );

        $this->add_control(
            'stock_text_after',
            [
                'label' => esc_html__( 'After text', 'selection' ),
                'type' => Controls_Manager::WYSIWYG,
            ]
        );

        $this->add_control(
            'stock_html_tag',
            [
                'label'     => esc_html__( 'HTML Tag', 'selection' ),
                'type'      => Controls_Manager::SELECT,
                'options'   => [
                    'h1'    => esc_html__( 'H1', 'selection' ),
                    'h2'    => esc_html__( 'H2', 'selection' ),
                    'h3'    => esc_html__( 'H3', 'selection' ),
                    'h4'    => esc_html__( 'H4', 'selection' ),
                    'h5'    => esc_html__( 'H5', 'selection' ),
                    'h6'    => esc_html__( 'H6', 'selection' ),
                    'div'    => esc_html__( 'div', 'selection' ),
                    'span'    => esc_html__( 'span', 'selection' ),
                    'p'    => esc_html__( 'p', 'selection' ),
                ],
                'default'   => 'div',
                'separator' => 'none',
            ]
        );


        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Product Title.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_product_stock() {

        $this->start_controls_section( 'section_style_product', [
            'label' => esc_html__( 'Product Stock', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->add_responsive_control(
            'section_style_product_margin',
            [
                'label' => esc_html__( 'Margin', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'selectors' => [
                    "{{WRAPPER}} .mdp-producter-elementor-product-wrapper" => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_responsive_control(
            'section_style_product_padding',
            [
                'label' => esc_html__( 'Padding', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'separator' => 'after',
                'selectors' => [
                    "{{WRAPPER}} .mdp-producter-elementor-product-wrapper" => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name' => 'section_style_product_typography',
                'label' => esc_html__( 'Typography', 'selection' ),
                'scheme' =>  Typography::TYPOGRAPHY_1,
                'selector' => '{{WRAPPER}} .stock',
            ]
        );

        $this->start_controls_tabs('control_style_tabs');

        $this->start_controls_tab( 'normal_style_tab', ['label' => esc_html__( 'NORMAL', 'selection' )] );

        $this->add_control(
            'section_style_product_text_color',
            [
                'label' => esc_html__( 'Text color', 'selection' ),
                'type' => Controls_Manager::COLOR,
                'scheme' => [
                    'type' => Color::get_type(),
                    'value' => Color::COLOR_3,
                ],
                'selectors' => [
                    '{{WRAPPER}} .stock' => 'color: {{VALUE}}',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Background::get_type(),
            [
                'name' => 'section_style_product_background',
                'label' => esc_html__( 'Background type', 'selection' ),
                'types' => [ 'classic', 'gradient', 'video' ],
                'selector' => '{{WRAPPER}} .mdp-producter-elementor-product-wrapper',
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => 'section_style_product_border_normal',
                'label' => esc_html__( 'Border Type', 'selection' ),
                'separator' => 'before',
                'selector' => '{{WRAPPER}} .mdp-producter-elementor-product-wrapper',
            ]
        );

        $this->add_responsive_control(
            'section_style_product_border_radius_normal',
            [
                'label' => esc_html__( 'Border radius', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-producter-elementor-product-wrapper' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',

                ],
            ]
        );


        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => 'section_style_product_box_shadow_normal',
                'label' => esc_html__( 'Box Shadow', 'selection' ),
                'selector' => '{{WRAPPER}} .mdp-producter-elementor-product-wrapper',
            ]
        );

        $this->end_controls_tab();

        $this->start_controls_tab( 'hover_style_tab', ['label' => esc_html__( 'HOVER', 'selection' )] );

        $this->add_control(
            'section_style_product_text_color_hover',
            [
                'label' => esc_html__( 'Text color', 'selection' ),
                'type' => Controls_Manager::COLOR,
                'scheme' => [
                    'type' => Color::get_type(),
                    'value' => Color::COLOR_3,
                ],
                'selectors' => [
                    '{{WRAPPER}} .stock:hover' => 'color: {{VALUE}}',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Background::get_type(),
            [
                'name' => 'section_style_product_background_hover',
                'label' => esc_html__( 'Background type', 'selection' ),
                'types' => [ 'classic', 'gradient', 'video' ],
                'selector' => '{{WRAPPER}} .mdp-producter-elementor-product-wrapper:hover',
            ]
        );

        $this->add_control(
            'section_style_product_hover_transition',
            [
                'label' => esc_html__( 'Hover transition', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 's' ],
                'range' => [
                    's' => [
                        'min' => 0.1,
                        'max' => 5,
                        'step' => 0.1,
                    ],
                ],
                'default' => [
                    'unit' => 's',
                    'size' => 0,
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-producter-elementor-product-wrapper, {{WRAPPER}} .stock' => 'transition: color {{SIZE}}{{UNIT}}, background {{SIZE}}{{UNIT}}, box-shadow {{SIZE}}{{UNIT}}, border-radius {{SIZE}}{{UNIT}}, border {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => 'section_style_product_border_hover',
                'label' => esc_html__( 'Border Type', 'selection' ),
                'selector' => '{{WRAPPER}} .mdp-producter-elementor-product-wrapper:hover',
                'separator' => 'before',
            ]
        );

        $this->add_responsive_control(
            'section_style_product_border_radius_hover',
            [
                'label' => esc_html__( 'Border radius', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-producter-elementor-product-wrapper:hover' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',

                ],
            ]
        );


        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => 'section_style_product_box_shadow_hover',
                'label' => esc_html__( 'Box Shadow', 'selection' ),
                'selector' => '{{WRAPPER}} .mdp-producter-elementor-product-wrapper:hover',
            ]
        );

        $this->end_controls_tab();

        $this->end_controls_tabs();

        $this->end_controls_section();

    }


    /**
     * Adding custom html for product stock information.
     *
     * @access public
     *
     * @param $html
     * @param $product
     * @return string
     */
    public function filter_wc_get_stock_html( $html, $product ) {
        $settings = $this->get_settings_for_display();

        if ( empty( $product ) ) { $product = wc_get_product( $settings['products'] ); }

        return '<'.$settings['stock_html_tag'].' class="stock '.$product->get_availability()['class'].'">' .
            wp_kses_post( $settings['stock_text_before'] ) .
            esc_html__( $product->get_availability()['availability'], "woocommerce" ) .
            wp_kses_post( $settings['stock_text_after'] )
        . '</'.$settings['stock_html_tag'].'>';
    }

    /**
     * Render Frontend Output. Generate the final HTML on the frontend.
     *
     * @access protected
     *
     * @return void
     **/
    protected function render() {
        if ( !Caster::get_instance()->is_active_woocommerce() ) {
            esc_html_e( 'Install WooCommerce to display product stock!' );
            return;
        }
        global $product;
        $settings = $this->get_settings_for_display();

        $product = wc_get_product();

        if ( empty( $product ) ) {
            $product = wc_get_product( esc_sql( $settings['products'] ) );
            if ( empty( $product ) || $product->get_status() === 'trash'  ) {
                return;
            }
        }

        add_filter( 'woocommerce_get_stock_html', [ $this, 'filter_wc_get_stock_html' ], 10, 2 );

        $product_stock_status = wc_get_stock_html( $product );

        remove_filter( 'woocommerce_get_stock_html', [ $this, 'filter_wc_get_stock_html' ] );

        ?>
        <!-- Start Producter for Elementor WordPress Plugin -->
        <div class="mdp-producter-elementor-product-stock-box mdp-producter-elementor-box mdp-producter-elementor-product-align-<?php esc_attr_e( $settings['stock_text_align'] ); ?>">
            <div class="mdp-producter-elementor-product-wrapper">
                <?php echo wp_kses_post( $product_stock_status ); ?>
            </div>
        </div>
        <!-- End Producter for Elementor WordPress Plugin -->
	    <?php

    }

    /**
     * Return link for documentation
     * Used to add stuff after widget
     *
     * @access public
     *
     * @return string
     **/
    public function get_custom_help_url() {

        return 'https://docs.merkulov.design/tag/producter';

    }

}
