<?php /** @noinspection PhpUndefinedClassInspection */
/**
 * Reviewer for Elementor
 * Testimonial carousel for Elementor editor
 * Exclusively on https://1.envato.market/reviewer-elementor
 *
 * @encoding        UTF-8
 * @version         1.12.1
 * @copyright       (C) 2018 - 2023 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Cherviakov Vlad (vladchervjakov@gmail.com), Dmitry Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

namespace Merkulove\Selection;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
    header( 'Status: 403 Forbidden' );
    header( 'HTTP/1.1 403 Forbidden' );
    exit;
}

use Elementor\Core\Schemes\Color;
use Elementor\Core\Schemes\Typography;
use Elementor\Group_Control_Background;
use Elementor\Group_Control_Border;
use Elementor\Group_Control_Box_Shadow;
use Elementor\Group_Control_Image_Size;
use Elementor\Group_Control_Typography;
use Elementor\Icons_Manager;
use Elementor\Repeater;
use Elementor\Utils;
use Exception;
use Elementor\Widget_Base;
use Elementor\Controls_Manager;
use Elementor\Plugin;
use Merkulove\Selection\Unity\Plugin as UnityPlugin;

/** @noinspection PhpUnused */
/**
 * Reviewer - Custom Elementor Widget.
 **/
class reviewer_elementor extends Widget_Base {

    /**
     * Use this to sort widgets.
     * A smaller value means earlier initialization of the widget.
     * Can take negative values.
     * Default widgets and widgets from 3rd party developers have 0 $mdp_order
     **/
    public $mdp_order = 1;

    /**
     * Widget base constructor.
     * Initializing the widget base class.
     *
     * @access public
     * @throws Exception If arguments are missing when initializing a full widget instance.
     * @param array      $data Widget data. Default is an empty array.
     * @param array|null $args Optional. Widget default arguments. Default is null.
     *
     * @return void
     **/
    public function __construct( $data = [], $args = null ) {

        parent::__construct( $data, $args );

        wp_register_style(
             'mdp-selection-elementor-admin',
                UnityPlugin::get_url() . 'css/elementor-admin' . UnityPlugin::get_suffix() . '.css',
                    [],
                    UnityPlugin::get_version()
        );
        wp_register_style(
            'mdp-reviewer-elementor',
               UnityPlugin::get_url() . 'css/reviewer-elementor' . UnityPlugin::get_suffix() . '.css',
                   [],
                   UnityPlugin::get_version()
        );
	    wp_register_script(
            'mdp-reviewer-elementor',
               UnityPlugin::get_url() . 'js/reviewer-elementor' . UnityPlugin::get_suffix() . '.js',
                   [ 'jquery' ],
                   UnityPlugin::get_version(),
          true
        );

    }

    /**
     * Return a widget name.
     *
     * @return string
     **/
    public function get_name() {

        return 'mdp-reviewer-elementor';

    }

    /**
     * Return the widget title that will be displayed as the widget label.
     *
     * @return string
     **/
    public function get_title() {

        return esc_html__( 'Reviews', 'selection' );

    }

    /**
     * Set the widget icon.
     *
     * @return string
     */
    public function get_icon() {

        return 'mdp-reviewer-elementor-widget-icon';

    }

    /**
     * Set the category of the widget.
     *
     * @return array with category names
     **/
    public function get_categories() {

        return [ 'selection-category' ];

    }

    /**
     * Get widget keywords. Retrieve the list of keywords the widget belongs to.
     *
     * @access public
     *
     * @return array Widget keywords.
     **/
    public function get_keywords() {

        return [ 'Merkulove', 'Reviews' ];

    }

    /**
     * Get style dependencies.
     * Retrieve the list of style dependencies the widget requires.
     *
     * @access public
     *
     * @return array Widget styles dependencies.
     **/
    public function get_style_depends() {

        return [ 'mdp-reviewer-elementor', 'mdp-selection-elementor-admin' ];

    }

	/**
	 * Get script dependencies.
	 * Retrieve the list of script dependencies the element requires.
	 *
	 * @access public
     *
	 * @return array Element scripts dependencies.
	 **/
	public function get_script_depends() {

		return [ 'mdp-reviewer-elementor' ];

    }

    /**
     * Add the widget controls.
     *
     * @access protected
     * @return void with category names
     **/
    protected function register_controls() {

        /** Content Tab. */
        $this->tab_content();

        /** Style Tab. */
        $this->tab_style();

    }

    /**
     * Add widget controls on Content tab.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function tab_content() {

        /** Content -> Testimonials Settings Section. */
        $this->section_testimonials();

        /** Content -> Testimonials Settings Section. */
        $this->section_layout_settings();

        /** Content -> Carousel Settings Section. */
        $this->section_carousel();

        /** Content -> Arrows Settings Section. */
        $this->section_arrows();

        /** Content -> Dots Settings Section. */
        $this->section_dots();

    }

    /**
     * Add widget controls on Style tab.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function tab_style() {

        /** Style -> Section Style Slide. */
        $this->section_style_slide();

        /** Style -> Section Style Description. */
        $this->section_style_description();

        /** Style -> Section Style Author. */
        $this->section_style_author();

        /** Style -> Section Style Company. */
        $this->section_style_company();

        /** Style -> Section Style Author Image. */
        $this->section_style_author_image();

        /** Style -> Section Style Rating. */
        $this->section_style_rating();

        /** Style -> Section Style Arrows. */
        $this->section_style_arrows();

        /** Style -> Section Style Dots. */
        $this->section_style_dots();

    }

    /**
     * Add widget controls: Content -> Testimonials Settings Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_testimonials() {

        $this->start_controls_section( 'section_testimonials', [
            'label' => esc_html__( 'Testimonials', 'selection' ),
            'tab'   => Controls_Manager::TAB_CONTENT
        ] );

       $repeater = new Repeater();

        $repeater->start_controls_tabs( 'control_tabs' );

        $repeater->start_controls_tab( 'author_company', ['label' => esc_html__( 'Author', 'selection' )] );

        $repeater->add_control (
            'author_name',
            [
                'label' => esc_html__( 'Author', 'selection' ),
                'type' => Controls_Manager::TEXT,
                'placeholder' => esc_html__( 'Type author name here', 'selection' ),
            ]
        );

        $repeater->add_control(
            'author_link',
            [
                'label' => esc_html__( 'Author link', 'selection' ),
                'type' => Controls_Manager::URL,
                'placeholder' => esc_html__( 'https://your-link.com', 'selection' ),
                'show_external' => true,
                'default' => [
                    'url' => '',
                    'is_external' => true,
                    'nofollow' => true,
                ],
            ]
        );


        $repeater->add_control(
            'author_tag',
            [
                'label'     => esc_html__( 'Author tag', 'selection' ),
                'type'      => Controls_Manager::SELECT,
                'options'   => [
                    'h1'    => esc_html__( 'h1', 'selection' ),
                    'h2'    => esc_html__( 'h2', 'selection' ),
                    'h3'    => esc_html__( 'h3', 'selection' ),
                    'h4'    => esc_html__( 'h4', 'selection' ),
                    'h5'    => esc_html__( 'h5', 'selection' ),
                    'h6'    => esc_html__( 'h6', 'selection' ),
                    'div'    => esc_html__( 'div', 'selection' ),
                    'span'    => esc_html__( 'span', 'selection' ),
                    'p'    => esc_html__( 'p', 'selection' ),
                ],
                'default'   => 'div',
                'separator' => 'none',
            ]
        );

        $repeater->add_control(
            'author_text_align',
            [
                'label' => esc_html__( 'Author align', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'fa fa-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'fa fa-align-center',
                    ],
                    'right' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'fa fa-align-right',
                    ],
                    'justify' => [
                        'title' => esc_html__( 'Justify', 'selection' ),
                        'icon' => 'fa fa-align-justify',
                    ],
                ],
                'default' => 'center',
                'toggle' => true,
                'selectors' => [
                    '{{WRAPPER}} {{CURRENT_ITEM}} .mdp-reviewer-elementor-slide--author' => 'text-align: {{VALUE}}'
                ]
            ]
        );

        $repeater->add_control (
            'author_separator',
            [
                'label' => esc_html__( 'Author separator', 'selection' ),
                'type' => Controls_Manager::TEXT,
                'placeholder' => esc_html__( 'Type your separator here', 'selection' ),
            ]
        );

        $repeater->add_control(
            'author_image',
            [
                'label' => esc_html__( 'Author Image', 'selection' ),
                'type' => Controls_Manager::MEDIA,
                'default' => [
                    'url' => Utils::get_placeholder_image_src(),
                ],
            ]
        );

        $repeater->add_control(
            'custom_author_image_size',
            [
                'label' => esc_html__( 'Custom size', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Yes', 'selection' ),
                'label_off' => esc_html__( 'No', 'selection' ),
                'return_value' => 'yes',
                'default' => 'no',
            ]
        );

        $repeater->add_control(
            'author_image_width',
            [
                'label' => esc_html__( 'Width', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 1000,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 0,
                        'max' => 100,
                    ],
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 150,
                ],
                'condition' => [
                    'custom_author_image_size' => 'yes'
                ],
            ]
        );

        $repeater->add_control(
            'author_image_height',
            [
                'label' => esc_html__( 'Height', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 1000,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 0,
                        'max' => 100,
                    ],
                ],
                'default' => [
                    'unit' => 'px',
                    'size' => 150,
                ],
                'condition' => [
                    'custom_author_image_size' => 'yes'
                ],
            ]
        );

        $repeater->add_group_control(
            Group_Control_Image_Size::get_type(),
            [
                'name' => 'author_image',
                'exclude' => [ 'custom' ],
                'include' => [],
                'default' => 'medium',
                'condition' => [
                    'custom_author_image_size!' => 'yes'
                ]
            ]
        );

        $repeater->add_control(
            'author_image_align',
            [
                'label' => esc_html__( 'Alignment', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'flex-start' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'fa fa-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'fa fa-align-center',
                    ],
                    'flex-end' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'fa fa-align-right',
                    ],
                ],
                'default' => 'center',
                'toggle' => true,
                'selectors' => [
                    '{{WRAPPER}} {{CURRENT_ITEM}} .mdp-reviewer-elementor-slide--author-image' => 'justify-content: {{VALUE}}'
                ]
            ]
        );

        $repeater->add_control (
            'company_name',
            [
                'label' => esc_html__( 'Company', 'selection' ),
                'type' => Controls_Manager::TEXT,
                'placeholder' => esc_html__( 'Type author name here', 'selection' ),
            ]
        );

        $repeater->add_control(
            'company_link',
            [
                'label' => esc_html__( 'Company link', 'selection' ),
                'type' => Controls_Manager::URL,
                'placeholder' => esc_html__( 'https://your-link.com', 'selection' ),
                'show_external' => true,
                'default' => [
                    'url' => '',
                    'is_external' => true,
                    'nofollow' => true,
                ],
            ]
        );

        $repeater->add_control(
            'company_tag',
            [
                'label'     => esc_html__( 'Company tag', 'selection' ),
                'type'      => Controls_Manager::SELECT,
                'options'   => [
                    'h1'    => esc_html__( 'h1', 'selection' ),
                    'h2'    => esc_html__( 'h2', 'selection' ),
                    'h3'    => esc_html__( 'h3', 'selection' ),
                    'h4'    => esc_html__( 'h4', 'selection' ),
                    'h5'    => esc_html__( 'h5', 'selection' ),
                    'h6'    => esc_html__( 'h6', 'selection' ),
                    'div'    => esc_html__( 'div', 'selection' ),
                    'span'    => esc_html__( 'span', 'selection' ),
                    'p'    => esc_html__( 'p', 'selection' ),
                ],
                'default'   => 'div',
                'separator' => 'none',
            ]
        );

        $repeater->add_control(
            'company_text_align',
            [
                'label' => esc_html__( 'Company align', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'fa fa-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'fa fa-align-center',
                    ],
                    'right' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'fa fa-align-right',
                    ],
                    'justify' => [
                        'title' => esc_html__( 'Justify', 'selection' ),
                        'icon' => 'fa fa-align-justify',
                    ],
                ],
                'default' => 'center',
                'toggle' => true,
                'selectors' => [
                    '{{WRAPPER}} {{CURRENT_ITEM}} .mdp-reviewer-elementor-slide--company' => 'text-align: {{VALUE}}'
                ]
            ]
        );

        $repeater->end_controls_tab();

        $repeater->start_controls_tab( 'review', ['label' => esc_html__( 'Review', 'selection' )] );

        $repeater->add_control(
            'testimonial_description',
            [
                'label' => esc_html__( 'Description', 'selection' ),
                'type' => Controls_Manager::WYSIWYG,
                'default' => esc_html__( 'Default description', 'selection' ),
                'placeholder' => esc_html__( 'Type your description here', 'selection' ),
            ]
        );

        $repeater->add_control(
            'description_text_align',
            [
                'label' => esc_html__( 'Description align', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'fa fa-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'fa fa-align-center',
                    ],
                    'right' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'fa fa-align-right',
                    ],
                    'justify' => [
                        'title' => esc_html__( 'Justify', 'selection' ),
                        'icon' => 'fa fa-align-justify',
                    ],
                ],
                'default' => 'center',
                'toggle' => true,
                'selectors' => [
                    '{{WRAPPER}} {{CURRENT_ITEM}} .mdp-reviewer-elementor-slide--description' => 'text-align: {{VALUE}}'
                ]
            ]
        );

        $repeater->end_controls_tab();

        $repeater->start_controls_tab( 'rating', ['label' => esc_html__( 'Rating', 'selection' )] );

        $repeater->add_control(
            'rating_count',
            [
                'label' => esc_html__( 'Rating count', 'selection' ),
                'type' => Controls_Manager::NUMBER,
                'min' => 1,
                'max' => 5,
                'step' => 1,
            ]
        );

        $repeater->add_control(
            'rating_icons',
            [
                'label' => esc_html__( 'Rating icon', 'selection' ),
                'type' => Controls_Manager::ICONS,
                'default' => [
                    'value' => 'fas fa-star',
                    'library' => 'solid',
                ],
            ]
        );

        $repeater->add_control(
            'rating_align',
            [
                'label' => esc_html__( 'Rating align', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'flex-start' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'fa fa-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'fa fa-align-center',
                    ],
                    'flex-end' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'fa fa-align-right',
                    ],
                ],
                'default' => 'center',
                'toggle' => true,
                'selectors' => [
                    '{{WRAPPER}} {{CURRENT_ITEM}} .mdp-reviewer-elementor-slide--rating-stars' => 'justify-content: {{VALUE}}'
                ]
            ]
        );

        $repeater->end_controls_tab();

        $this->add_control(
            'slides_list',
            [
                'label' => esc_html__( 'Testimonials', 'selection' ),
                'type' => Controls_Manager::REPEATER,
                'fields' => $repeater->get_controls(),
                'title_field' => '{{{ author_name }}}',
                'default' => [
                    [
                     'author_name' => 'author',
                     'company_name' => 'company',
                    ],

                    [
                        'author_name' => 'author',
                        'company_name' => 'company',
                    ],

                    [
                        'author_name' => 'author',
                        'company_name' => 'company',
                    ],
                ]
            ]
        );

        $repeater->end_controls_tabs();

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Content -> Layout Settings Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_layout_settings() {

        $this->start_controls_section( 'section_layout_settings', [
            'label' => esc_html__( 'Slide layout', 'selection' ),
            'tab'   => Controls_Manager::TAB_CONTENT
        ] );

        $repeater = new Repeater();

        $repeater->add_control(
            'layout',
            [
                'label'     => esc_html__( 'Layout', 'selection' ),
                'type'      => Controls_Manager::SELECT,
                'options'   => [
                    'author'    => esc_html__( 'Author', 'selection' ),
                    'company'    => esc_html__( 'Company', 'selection' ),
                    'description'    => esc_html__( 'Description', 'selection' ),
                    'rating'    => esc_html__( 'Rating', 'selection' ),
                    'author_image'    => esc_html__( 'Author image', 'selection' ),
                ],
                'default'   => 'author',
                'separator' => 'none',
            ]
        );

        $this->add_control(
            'layouts_list',
            [
                'label' => esc_html__( 'Layouts', 'selection' ),
                'type' => Controls_Manager::REPEATER,
                'fields' => $repeater->get_controls(),
                'default' => [
                    [
                        'layout' => esc_html__( 'author_image' )
                    ],
                    [
                        'layout' => esc_html__( 'description' )
                    ],
                    [
                        'layout' => esc_html__( 'author' )
                    ],
                    [
                        'layout' => esc_html__( 'company' )
                    ],
                    [
                        'layout' => esc_html__( 'rating' )
                    ],
                ],
                'title_field' => 'Layout - {{{ layout }}}',
            ]
        );
        $this->end_controls_section();

    }

    /**
     * Add widget controls: Content -> Carousel Settings Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_carousel() {
        $this->start_controls_section( 'section_carousel', [
            'label' => esc_html__( 'Carousel', 'selection' ),
            'tab'   => Controls_Manager::TAB_CONTENT
        ] );

        $this->add_control(
            'slides_switching_mode',
            [
                'label' => esc_html__( 'Slides switching mode', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'to-left',
                'options' => [
                    'to-left'  => esc_html__( 'To Left', 'selection' ),
                    'to-right' => esc_html__( 'To Right', 'selection' ),
                    'to-bottom' => esc_html__( 'To Bottom', 'selection' ),
                    'to-top' => esc_html__( 'To Top', 'selection' ),
                    'fade' => esc_html__( 'Fade', 'selection' ),
                ],
            ]
        );


        $this->add_control(
            'autoplay_speed',
            [
                'label' => esc_html__( 'Autoplay speed', 'selection' ),
                'type' => Controls_Manager::NUMBER,
                'min' => 1,
                'max' => 100,
                'step' => 1,
                'default' => 3,
            ]
        );

        $this->add_control(
            'transition_speed',
            [
                'label' => esc_html__( 'Transition speed', 'selection' ),
                'type' => Controls_Manager::NUMBER,
                'min' => 0.1,
                'max' => 10,
                'step' => 0.1,
                'default' => 0.3,
            ]
        );

        $this->add_responsive_control(
            'slides_to_show',
            [
                'label' => esc_html__( 'Slides to show', 'selection' ),
                'type' => Controls_Manager::NUMBER,
                'min' => 1,
                'max' => 10,
                'step' => 1,
                'default' => 1,
                'condition' => [
                        'slides_switching_mode!' => 'fade'
                ]
            ]
        );

        $this->add_control(
            'slides_to_scroll',
            [
                'label' => esc_html__( 'Slides to scroll', 'selection' ),
                'type' => Controls_Manager::NUMBER,
                'min' => 1,
                'max' => 100,
                'step' => 1,
                'default' => 1,
                'condition' => [
                    'slides_switching_mode!' => 'fade'
                ]
            ]
        );

        $this->add_control(
            'infinite_loop',
            [
                'label' => esc_html__( 'Infinite loop', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Yes', 'selection' ),
                'label_off' => esc_html__( 'No', 'selection' ),
                'return_value' => 'yes',
                'default' => 'yes',
            ]
        );

        $this->add_control(
            'show_navigation_arrows',
            [
                'label' => esc_html__( 'Show navigation arrows', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Yes', 'selection' ),
                'label_off' => esc_html__( 'No', 'selection' ),
                'return_value' => 'yes',
                'default' => 'yes',
            ]
        );

        $this->add_control(
            'center_mode',
            [
                'label' => esc_html__( 'Center mode', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Yes', 'selection' ),
                'label_off' => esc_html__( 'No', 'selection' ),
                'return_value' => 'yes',
                'default' => 'yes',
                'condition' => [
                    'slides_switching_mode!' => 'fade'
                ]
            ]
        );

        $this->add_control(
            'show_dots',
            [
                'label' => esc_html__( 'Show dots', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Yes', 'selection' ),
                'label_off' => esc_html__( 'No', 'selection' ),
                'return_value' => 'yes',
                'default' => 'yes',
            ]
        );

        $this->add_control(
            'autoplay',
            [
                'label' => esc_html__( 'Autoplay', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Yes', 'selection' ),
                'label_off' => esc_html__( 'No', 'selection' ),
                'return_value' => 'yes',
                'default' => 'yes',
            ]
        );

        $this->add_control(
            'pause_on_hover',
            [
                'label' => esc_html__( 'Pause on hover', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Yes', 'selection' ),
                'label_off' => esc_html__( 'No', 'selection' ),
                'return_value' => 'yes',
                'default' => 'yes',
            ]
        );


        $this->end_controls_section();

    }

    /**
     * Add widget controls: Content -> Arrows Settings Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_arrows()
    {
        $this->start_controls_section('section_arrows', [
            'label' => esc_html__('Arrows', 'selection'),
            'tab' => Controls_Manager::TAB_CONTENT
        ]);

        $this->add_control(
            'previous_arrow_icon',
            [
                'label' => esc_html__( 'Previous arrow', 'selection' ),
                'type' => Controls_Manager::ICONS,
                'default' => [
                    'value' => 'fas fa-chevron-left',
                    'library' => 'solid',
                ],
            ]
        );

        $this->add_control(
            'next_arrow_icon',
            [
                'label' => esc_html__( 'Next arrow', 'selection' ),
                'type' => Controls_Manager::ICONS,
                'default' => [
                    'value' => 'fas fa-chevron-right',
                    'library' => 'solid',
                ],
            ]
        );

        $this->end_controls_section();

    }



    /**
     * Add widget controls: Content -> Dots Settings Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_dots()
    {
        $this->start_controls_section('section_dots', [
            'label' => esc_html__('Dots', 'selection'),
            'tab' => Controls_Manager::TAB_CONTENT
        ]);

        $this->add_control(
            'dot_icon',
            [
                'label' => esc_html__( 'Dot icon', 'selection' ),
                'type' => Controls_Manager::ICONS,
                'default' => [
                    'value' => 'fas fa-circle',
                    'library' => 'solid',
                ],
            ]
        );

        $this->add_control(
            'dot_active_icon',
            [
                'label' => esc_html__( 'Active dot icon', 'selection' ),
                'type' => Controls_Manager::ICONS,
                'default' => [
                    'value' => 'fas fa-circle-notch',
                    'library' => 'solid',
                ],
            ]
        );


        $this->end_controls_section();

    }


    /**
     * Function for generating margin padding controls.
     *
     * @param $section_id
     * @param $html_class
     * @return void
     * @since 1.0.0
     * @access private
     *
     */
    private function generate_margin_padding_controls( $section_id, $html_class ) {
        $this->add_responsive_control(
            $section_id.'_margin',
            [
                'label' => esc_html__( 'Margin', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'selectors' => [
                    "{{WRAPPER}} .$html_class" => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_responsive_control(
            $section_id.'_padding',
            [
                'label' => esc_html__( 'Padding', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'selectors' => [
                    "{{WRAPPER}} .$html_class" => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_control(
            $section_id.'_margin_padding_divider',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );
    }


    /**
     * Function for generating typography and tabs controls.
     *
     * @param $section_id
     * @param $html_class
     * @param bool $include_color
     * @param bool $include_typography
     * @param bool $additional_color
     * @param string $additional_color_class
     * @param string $additional_color_name
     * @param bool $include_bg
     * @return void
     * @since 1.0.0
     * @access private
     */
    private function generate_typography_tabs_controls( $section_id, $html_class, $include_color = true, $include_typography = true, $additional_color = false, $additional_color_class = '', $additional_color_name = '', $include_bg = true ) {

        if ( $include_typography ) {
            $this->add_group_control(
                Group_Control_Typography::get_type(),
                [
                    'name' => $section_id . '_typography',
                    'label' => esc_html__('Typography', 'selection'),
                    'scheme' => Typography::TYPOGRAPHY_1,
                    'selector' => "{{WRAPPER}} .$html_class",
                ]
            );
        }

        $this->start_controls_tabs( $section_id.'_style_tabs' );

        $this->start_controls_tab( $section_id.'_normal_style_tab', ['label' => esc_html__( 'NORMAL', 'selection' )] );

        if ( $additional_color ) {

            $this->add_control(
                $section_id.'_'.$additional_color_name. '_normal',
                [
                    'label' => esc_html__("$additional_color_name color", 'selection'),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => Color::COLOR_3,
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .$additional_color_class" => 'color: {{VALUE}}',
                    ],
                ]
            );

        }

        if ( $include_color ) {

            $this->add_control(
                $section_id . '_normal_text_color',
                [
                    'label' => esc_html__('Color', 'selection'),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => Color::COLOR_3,
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .$html_class" => 'color: {{VALUE}}',
                    ],
                ]
            );

        }

        if ( $include_bg ) {

            $this->add_group_control(
                Group_Control_Background::get_type(),
                [
                    'name' => $section_id . '_normal_background',
                    'label' => esc_html__('Background type', 'selection'),
                    'types' => ['classic', 'gradient', 'video'],
                    'selector' => "{{WRAPPER}} .$html_class",
                ]
            );

            $this->add_control(
                $section_id . '_separate_normal',
                [
                    'type' => Controls_Manager::DIVIDER,
                ]
            );
        }




        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => $section_id.'_border_normal',
                'label' => esc_html__( 'Border Type', 'selection' ),
                'selector' => "{{WRAPPER}} .$html_class",
            ]
        );

        $this->add_responsive_control(
            $section_id.'_border_radius_normal',
            [
                'label' => esc_html__( 'Border radius', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    "{{WRAPPER}} .$html_class" => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => $section_id.'_box_shadow_normal',
                'label' => esc_html__( 'Box Shadow', 'selection' ),
                'selector' => "{{WRAPPER}} .$html_class",
            ]
        );


        $this->end_controls_tab();

        $this->start_controls_tab( $section_id.'_hover_style_tab', ['label' => esc_html__( 'HOVER', 'selection' )] );

        if ( $additional_color ) {

            $this->add_control(
                $section_id.'_'.$additional_color_name.'_hover_color',
                [
                    'label' => esc_html__("$additional_color_name color", 'selection'),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => Color::COLOR_3,
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .$html_class:hover .$additional_color_class" => 'color: {{VALUE}}',
                    ],
                ]
            );

        }

        if ( $include_color ) {
            $this->add_control(
                $section_id . '_hover_color',
                [
                    'label' => esc_html__('Color', 'selection'),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => Color::COLOR_3,
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .$html_class:hover" => 'color: {{VALUE}}',
                    ],
                ]
            );
        }

        if ( $include_bg ) {

            $this->add_group_control(
                Group_Control_Background::get_type(),
                [
                    'name' => $section_id . '_background_hover',
                    'label' => esc_html__('Background type', 'selection'),
                    'types' => ['classic', 'gradient', 'video'],
                    'selector' => "{{WRAPPER}} .$html_class:hover",
                ]
            );

            $this->add_control(
                $section_id.'_separate_hover',
                [
                    'type' => Controls_Manager::DIVIDER,
                ]
            );
        }


        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => $section_id.'_border_hover',
                'label' => esc_html__( 'Border Type', 'selection' ),
                'selector' => "{{WRAPPER}} .$html_class:hover",
            ]
        );

        $this->add_responsive_control(
            $section_id.'_border_radius_hover',
            [
                'label' => esc_html__( 'Border radius', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    "{{WRAPPER}} .$html_class:hover" => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => $section_id.'_box_shadow_hover',
                'label' => esc_html__( 'Box Shadow', 'selection' ),
                'selector' => "{{WRAPPER}} .$html_class:hover",
            ]
        );

        $this->end_controls_tab();

        $this->end_controls_tabs();
    }

    /**
     * Add widget controls: Style -> Section Style Slide.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_slide() {

        $this->start_controls_section( 'section_style_slide', [
            'label' => esc_html__( 'Slide', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->generate_margin_padding_controls( 'section_style_slide', 'mdp-reviewer-elementor-slide' );

        $this->add_responsive_control(
            'slide_height',
            [
                'label' => esc_html__( 'Slide height', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 1000,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 1,
                        'max' => 100,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-reviewer-elementor-slide' => 'height: {{SIZE}}{{UNIT}} !important; max-height: {{SIZE}}{{UNIT}} !important;',
                ],
            ]
        );

        $this->add_control(
             'separate_slide_height',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        $this->add_control(
            'spacing_between_slides_horizontal',
            [
                'label' => esc_html__( 'Spacing between slides', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 100,
                        'step' => 1,
                    ],
                ],
                'conditions' => [
                        'relation' => 'or',
                        'terms' => [
                            [
                                'name' => 'slides_switching_mode',
                                'operator' => '==',
                                'value' => 'to-left'
                            ],
                            [
                                'name' => 'slides_switching_mode',
                                'operator' => '==',
                                'value' => 'to-right'
                            ]
                        ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-reviewer-elementor-slide-wrapper' => 'padding: 0 {{SIZE}}{{UNIT}} !important;',
                ],
            ]
        );

        $this->add_control(
            'spacing_between_slides_vertical',
            [
                'label' => esc_html__( 'Spacing between slides', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 100,
                        'step' => 1,
                    ],
                ],
                'conditions' => [
                    'relation' => 'or',
                    'terms' => [
                        [
                            'name' => 'slides_switching_mode',
                            'operator' => '==',
                            'value' => 'to-top'
                        ],
                        [
                            'name' => 'slides_switching_mode',
                            'operator' => '==',
                            'value' => 'to-bottom'
                        ]
                    ],
                ],
            ]
        );

        $this->generate_typography_tabs_controls( 'section_style_slide', 'mdp-reviewer-elementor-slide', false, false );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Description.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_description() {

        $this->start_controls_section( 'section_style_description', [
            'label' => esc_html__( 'Description', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->generate_margin_padding_controls( 'section_style_description', 'mdp-reviewer-elementor-slide--description' );

        $this->generate_typography_tabs_controls( 'section_style_description', 'mdp-reviewer-elementor-slide--description' );

        $this->end_controls_section();

    }


    /**
     * Add widget controls: Style -> Section Style Author.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_author()
    {

        $this->start_controls_section('section_style_author', [
            'label' => esc_html__('Author', 'selection'),
            'tab' => Controls_Manager::TAB_STYLE
        ]);

        $this->generate_margin_padding_controls( 'section_style_author', 'mdp-reviewer-elementor-slide--author' );

        $this->add_control(
            'author_container_width',
            [
                'label' => esc_html__( 'Width', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 1000,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 0,
                        'max' => 100,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-reviewer-elementor-slide--author' => 'width: {{SIZE}}{{UNIT}};',
                ],
            ]
        );


        $this->generate_typography_tabs_controls( 'section_style_author', 'mdp-reviewer-elementor-slide--author', true, true, true, 'mdp-reviewer-elementor-slide--author-separator', 'Separator' );


        $this->end_controls_section();

    }


    /**
     * Add widget controls: Style -> Section Style Company.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_company()
    {

        $this->start_controls_section('section_style_company', [
            'label' => esc_html__('Company', 'selection'),
            'tab' => Controls_Manager::TAB_STYLE
        ]);

        $this->generate_margin_padding_controls( 'section_style_company', 'mdp-reviewer-elementor-slide--company' );

        $this->add_control(
            'company_container_width',
            [
                'label' => esc_html__( 'Width', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 1000,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 0,
                        'max' => 100,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-reviewer-elementor-slide--company' => 'width: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->generate_typography_tabs_controls( 'section_style_company', 'mdp-reviewer-elementor-slide--company' );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Author Image.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_author_image()
    {

        $this->start_controls_section('section_style_author_image', [
            'label' => esc_html__( 'Author image', 'selection' ),
            'tab' => Controls_Manager::TAB_STYLE
        ]);

        $this->generate_margin_padding_controls( 'section_style_author_image', 'mdp-reviewer-elementor-slide--author-image' );

        $this->generate_typography_tabs_controls( 'section_style_author_image', 'mdp-reviewer-elementor-slide--author-image img', false, false, false, '', '', false );

        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Rating.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_rating()
    {

        $this->start_controls_section('section_style_rating', [
            'label' => esc_html__('Rating', 'selection'),
            'tab' => Controls_Manager::TAB_STYLE
        ]);

        $this->generate_margin_padding_controls( 'section_style_rating', 'mdp-reviewer-elementor-slide--rating' );

        $this->add_control(
            'rating_icons_size',
            [
                'label' => esc_html__( 'Rating icons size', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 500,
                        'step' => 1,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-reviewer-elementor-slide--rating-star i' => 'font-size: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->add_control(
            'space_between_icons',
            [
                'label' => esc_html__( 'Space between icons', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 500,
                        'step' => 1,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-reviewer-elementor-slide--rating-star' => 'margin-right: calc({{SIZE}}{{UNIT}} / 2); margin-left: calc({{SIZE}}{{UNIT}} / 2);',
                ],
            ]
        );


        $this->start_controls_tabs( 'section_style_rating_style_tabs' );

        $this->start_controls_tab( 'section_style_rating_normal_style_tab', ['label' => esc_html__( 'NORMAL', 'selection' )] );


        $this->add_control(
            'section_style_rating_active_star_color_normal',
            [
                'label' => esc_html__("Active star color", 'selection'),
                'type' => Controls_Manager::COLOR,
                'scheme' => [
                    'type' => Color::get_type(),
                    'value' => Color::COLOR_3,
                ],
                'default' => 'rgba(252, 204, 0, 1)',
                'selectors' => [
                    "{{WRAPPER}} .mdp-reviewer-elementor-slide--rating-star.mdp-reviewer-elementor-slide--rating-star-active" => 'color: {{VALUE}} !important',
                ],
            ]
        );


        $this->add_control(
            'section_style_rating_normal_text_color',
            [
                'label' => esc_html__('Color', 'selection'),
                'type' => Controls_Manager::COLOR,
                'scheme' => [
                    'type' => Color::get_type(),
                    'value' => Color::COLOR_3,
                ],
                'default' => 'rgba(214, 213, 207, 1)',
                'selectors' => [
                    "{{WRAPPER}} .mdp-reviewer-elementor-slide--rating-star" => 'color: {{VALUE}}',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Background::get_type(),
            [
                'name' =>  'section_style_rating_normal_background',
                'label' => esc_html__('Background type', 'selection'),
                'types' => ['classic', 'gradient', 'video'],
                'selector' => "{{WRAPPER}} .mdp-reviewer-elementor-slide--rating",
            ]
        );

        $this->add_control(
            'section_style_rating_separate_normal',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );




        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => 'section_style_rating_border_normal',
                'label' => esc_html__( 'Border Type', 'selection' ),
                'selector' => "{{WRAPPER}} .mdp-reviewer-elementor-slide--rating",
            ]
        );

        $this->add_responsive_control(
            'section_style_rating_border_radius_normal',
            [
                'label' => esc_html__( 'Border radius', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    "{{WRAPPER}} .mdp-reviewer-elementor-slide--rating" => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => 'section_style_rating_box_shadow_normal',
                'label' => esc_html__( 'Box Shadow', 'selection' ),
                'selector' => "{{WRAPPER}} .mdp-reviewer-elementor-slide--rating",
            ]
        );


        $this->end_controls_tab();

        $this->start_controls_tab( 'section_style_rating_hover_style_tab', ['label' => esc_html__( 'HOVER', 'selection' )] );


        $this->add_control(
            'section_style_rating_active_star_hover_color',
            [
                'label' => esc_html__("Active star color", 'selection'),
                'type' => Controls_Manager::COLOR,
                'scheme' => [
                    'type' => Color::get_type(),
                    'value' => Color::COLOR_3,
                ],
                'default' => 'rgba(252, 204, 0, 1)',
                'selectors' => [
                    "{{WRAPPER}} .mdp-reviewer-elementor-slide--rating-stars:hover .mdp-reviewer-elementor-slide--rating-star.mdp-reviewer-elementor-slide--rating-star-active" => 'color: {{VALUE}} !important',
                ],
            ]
        );


        $this->add_control(
            'section_style_rating_hover_color',
            [
                'label' => esc_html__('Color', 'selection'),
                'type' => Controls_Manager::COLOR,
                'scheme' => [
                    'type' => Color::get_type(),
                    'value' => Color::COLOR_3,
                ],
                'default' => 'rgba(214, 213, 207, 1)',
                'selectors' => [
                    "{{WRAPPER}} .mdp-reviewer-elementor-slide--rating-stars:hover .mdp-reviewer-elementor-slide--rating-star" => 'color: {{VALUE}}',
                ],
            ]
        );


        $this->add_group_control(
            Group_Control_Background::get_type(),
            [
                'name' =>  'section_style_rating_background_hover',
                'label' => esc_html__('Background type', 'selection'),
                'types' => ['classic', 'gradient', 'video'],
                'selector' => "{{WRAPPER}} .mdp-reviewer-elementor-slide--rating:hover",
            ]
        );


        $this->add_control(
            'section_style_rating_separate_hover',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => 'section_style_rating_border_hover',
                'label' => esc_html__( 'Border Type', 'selection' ),
                'selector' => "{{WRAPPER}} .mdp-reviewer-elementor-slide--rating:hover",
            ]
        );

        $this->add_responsive_control(
            'section_style_rating_border_radius_hover',
            [
                'label' => esc_html__( 'Border radius', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    "{{WRAPPER}} .mdp-reviewer-elementor-slide--rating:hover" => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => 'section_style_rating_box_shadow_hover',
                'label' => esc_html__( 'Box Shadow', 'selection' ),
                'selector' => "{{WRAPPER}} .mdp-reviewer-elementor-slide--rating:hover",
            ]
        );

        $this->end_controls_tab();

        $this->end_controls_tabs();

        $this->end_controls_section();
    }


    /**
     * Add widget controls: Style -> Section Style Arrows.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_arrows()
    {

        $this->start_controls_section('section_style_arrows', [
            'label' => esc_html__('Arrows', 'selection'),
            'tab' => Controls_Manager::TAB_STYLE
        ]);

        $this->generate_margin_padding_controls( 'section_style_arrows', 'mdp-reviewer-elementor-arrow' );

        $this->add_control(
            'arrows_size',
            [
                'label' => esc_html__( 'Arrows size', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 500,
                        'step' => 1,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-reviewer-elementor-arrow' => 'font-size: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->generate_typography_tabs_controls( 'section_style_arrows', 'mdp-reviewer-elementor-arrow', true, false );


        $this->end_controls_section();

    }

    /**
     * Add widget controls: Style -> Section Style Arrows.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_dots()
    {

        $this->start_controls_section('section_style_dots', [
            'label' => esc_html__('Dots', 'selection'),
            'tab' => Controls_Manager::TAB_STYLE
        ]);

        $this->generate_margin_padding_controls( 'section_style_dots', 'mdp-reviewer-elementor-dots' );

        $this->add_control(
            'dots_size',
            [
                'label' => esc_html__( 'Dots size', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 500,
                        'step' => 1,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-reviewer-elementor-dots .mdp-reviewer-elementor-dot-icon, {{WRAPPER}} .mdp-reviewer-elementor-dots .mdp-reviewer-elementor-dot-active-icon' => 'font-size: {{SIZE}}{{UNIT}};',
                ],
            ]
        );


        $this->add_control(
            'dots_spacing',
            [
                'label' => esc_html__( 'Space between dots', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 500,
                        'step' => 1,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-reviewer-elementor-dots .mdp-reviewer-elementor-dot' => 'margin-right: calc({{SIZE}}{{UNIT}} / 2); margin-left: calc({{SIZE}}{{UNIT}} / 2);',
                ],
            ]
        );

        $this->start_controls_tabs( 'section_style_dots_style_tabs' );

        $this->start_controls_tab( 'section_style_dots_normal_style_tab', ['label' => esc_html__( 'NORMAL', 'selection' )] );


        $this->add_control(
            'section_style_dot_color_normal',
            [
                'label' => esc_html__("Color", 'selection'),
                'type' => Controls_Manager::COLOR,
                'scheme' => [
                    'type' => Color::get_type(),
                    'value' => Color::COLOR_3,
                ],
                'selectors' => [
                    "{{WRAPPER}} .mdp-reviewer-elementor-dot-icon i, {{WRAPPER}} .mdp-reviewer-elementor-dot-active-icon i" => 'color: {{VALUE}} !important',
                ],
            ]
        );


        $this->add_group_control(
            Group_Control_Background::get_type(),
            [
                'name' =>  'section_style_dot_normal_background',
                'label' => esc_html__('Background type', 'selection'),
                'types' => ['classic', 'gradient', 'video'],
                'selector' => "{{WRAPPER}} .mdp-reviewer-elementor-dots",
            ]
        );

        $this->add_control(
            'section_style_dot_separate_normal',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );




        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => 'section_style_dot_border_normal',
                'label' => esc_html__( 'Border Type', 'selection' ),
                'selector' => "{{WRAPPER}} .mdp-reviewer-elementor-dots",
            ]
        );

        $this->add_responsive_control(
            'section_style_dot_border_radius_normal',
            [
                'label' => esc_html__( 'Border radius', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    "{{WRAPPER}} .mdp-reviewer-elementor-dots" => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => 'section_style_dot_box_shadow_normal',
                'label' => esc_html__( 'Box Shadow', 'selection' ),
                'selector' => "{{WRAPPER}} .mdp-reviewer-elementor-dots",
            ]
        );


        $this->end_controls_tab();

        $this->start_controls_tab( 'section_style_dot_hover_style_tab', ['label' => esc_html__( 'HOVER', 'selection' )] );

        $this->add_control(
            'section_style_dot_hover_color',
            [
                'label' => esc_html__("Color", 'selection'),
                'type' => Controls_Manager::COLOR,
                'scheme' => [
                    'type' => Color::get_type(),
                    'value' => Color::COLOR_3,
                ],
                'selectors' => [
                    "{{WRAPPER}} .mdp-reviewer-elementor-dot-icon:hover i, {{WRAPPER}} .mdp-reviewer-elementor-dot-active-icon:hover i" => 'color: {{VALUE}} !important',
                ],
            ]
        );


        $this->add_group_control(
            Group_Control_Background::get_type(),
            [
                'name' =>  'section_style_dot_background_hover',
                'label' => esc_html__('Background type', 'selection'),
                'types' => ['classic', 'gradient', 'video'],
                'selector' => "{{WRAPPER}} .mdp-reviewer-elementor-dots:hover",
            ]
        );


        $this->add_control(
            'section_style_dot_separate_hover',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => 'section_style_dot_border_hover',
                'label' => esc_html__( 'Border Type', 'selection' ),
                'selector' => "{{WRAPPER}} .mdp-reviewer-elementor-dots:hover",
            ]
        );

        $this->add_responsive_control(
            'section_style_dot_border_radius_hover',
            [
                'label' => esc_html__( 'Border radius', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    "{{WRAPPER}} .mdp-reviewer-elementor-dots:hover" => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => 'section_style_dot_box_shadow_hover',
                'label' => esc_html__( 'Box Shadow', 'selection' ),
                'selector' => "{{WRAPPER}} .mdp-reviewer-elementor-dots:hover",
            ]
        );

        $this->end_controls_tab();

        $this->end_controls_tabs();


        $this->end_controls_section();

    }

    /**
     * Render Frontend Output. Generate the final HTML on the frontend.
     *
     * @access protected
     *
     * @return void
     **/
    protected function render() {
        $settings = $this->get_settings_for_display();
        ?>
        <!-- Start Reviewer for Elementor WordPress Plugin -->
        <div class="mdp-reviewer-elementor-box">
            <div class="mdp-reviewer-elementor-carousel-wrapper">
                <div class="mdp-reviewer-elementor-carousel"
                     data-slides-switching-mode="<?php esc_attr_e( $settings['slides_switching_mode'] ); ?>"
                     data-autoplay="<?php esc_attr_e( $settings['autoplay'] ); ?>"
                     data-pause-hover="<?php esc_attr_e( $settings['pause_on_hover'] ); ?>"
                     data-autoplay-speed = "<?php esc_attr_e( $settings['autoplay_speed'] ); ?>"
                     data-transition-speed = "<?php esc_attr_e( $settings['transition_speed'] ); ?>"
                     data-slides-to-scroll = "<?php esc_attr_e( $settings['slides_to_scroll'] ); ?>"
                     data-slides-to-show = "<?php esc_attr_e( $settings['slides_to_show'] ); ?>"
                     data-infinite-loop = "<?php esc_attr_e( $settings['infinite_loop'] ); ?>"
                     data-active-dot-class = "<?php esc_attr_e( $settings['dot_active_icon']['value'] ); ?>"
                     data-dot-class = "<?php esc_attr_e( $settings['dot_icon']['value'] ); ?>"
                     data-center-mode = "<?php esc_attr_e( $settings['center_mode'] ); ?>"
                    <?php if ( array_key_exists( 'widescreen', Plugin::$instance->breakpoints->get_active_breakpoints() ) && isset( $settings['slides_to_show_widescreen'] ) ): ?> data-slides-to-show-widescreen="<?php esc_attr_e( $settings['slides_to_show_widescreen'] ); ?>" <?php endif; ?>
                    <?php if ( array_key_exists( 'laptop', Plugin::$instance->breakpoints->get_active_breakpoints() ) && isset( $settings['slides_to_show_laptop'] ) ): ?> data-slides-to-show-laptop="<?php esc_attr_e( $settings['slides_to_show_laptop'] ); ?>" <?php endif; ?>
                    <?php if ( array_key_exists( 'tablet_extra', Plugin::$instance->breakpoints->get_active_breakpoints() ) && isset( $settings['slides_to_show_tablet_extra'] ) ): ?> data-slides-to-show-tablet-extra="<?php esc_attr_e( $settings['slides_to_show_tablet_extra'] ); ?>" <?php endif; ?>
                    <?php if ( array_key_exists( 'mobile_extra', Plugin::$instance->breakpoints->get_active_breakpoints() ) && isset( $settings['slides_to_show_mobile_extra'] ) ): ?> data-slides-to-show-mobile-extra="<?php esc_attr_e( $settings['slides_to_show_mobile_extra'] ); ?>" <?php endif; ?>
                    <?php if ( isset( $settings['slides_to_show_tablet'] ) ): ?> data-slides-to-show-tablet = "<?php esc_attr_e( $settings['slides_to_show_tablet'] ); ?>" <?php endif; ?>
                    <?php if ( isset( $settings['slides_to_show_mobile'] ) ): ?> data-slides-to-show-mobile = "<?php esc_attr_e( $settings['slides_to_show_mobile'] ); ?>" <?php endif; ?>
                >
                    <?php foreach ( $settings['slides_list'] as $key => $slide ): ?>
                        <div class="mdp-reviewer-elementor-slide-wrapper" data-index="<?php esc_attr_e( $key ); ?>" <?php if ( $settings['slides_switching_mode'] === 'to-top' || $settings['slides_switching_mode'] === 'to-bottom' ): ?> style=" padding: <?php esc_attr_e( $settings['spacing_between_slides_vertical']['size'] ); ?>px 0;" <?php endif; ?>>
                            <div class="mdp-reviewer-elementor-slide elementor-repeater-item-<?php esc_attr_e( $slide['_id'] ) ?>" data-rating="<?php esc_attr_e( $slide['rating_count'] ); ?>">
                                <?php foreach ( $settings['layouts_list'] as $layout ): ?>
                                    <?php switch ( $layout['layout'] ):
                                        case 'description': ?>
                                            <div class="mdp-reviewer-elementor-slide--description">
                                                <?php echo wp_kses_post( $slide['testimonial_description'] ) ?>
                                            </div>
                                            <?php break; ?>
                                        <?php case 'author_image': ?>
                                            <div class="mdp-reviewer-elementor-slide--author-image">
                                                <?php if( $slide['custom_author_image_size'] === 'yes' ): ?>
                                                    <img src="<?php echo esc_url( $slide['author_image']['url'] )?>" style="width: <?php esc_attr_e( $slide['author_image_width']['size'].$slide['author_image_width']['unit'] ); ?> !important; height: <?php esc_attr_e( $slide['author_image_height']['size'].$slide['author_image_height']['unit'] ); ?> !important" alt="">
                                                <?php else: ?>
                                                <?php echo wp_kses_post( Group_Control_Image_Size::get_attachment_image_html( $slide, 'author_image' ) ); ?>
                                                <?php endif; ?>
                                            </div>
                                            <?php break; ?>
                                        <?php case 'author': ?>
                                            <?php if ( !empty( $slide['author_link']['url'] ) ): ?>
                                                <a href="<?php echo esc_url( $slide['author_link']['url'] ) ?>" >
                                                    <?php echo wp_kses_post( '<'.$slide['author_tag'].' class="mdp-reviewer-elementor-slide--author">'
                                                        .$slide['author_name'].' <span class="mdp-reviewer-elementor-slide--author-separator">'.$slide['author_separator'].
                                                        '</span> </'.$slide['author_tag'].'>' ) ?>
                                                </a>
                                            <?php else: ?>
                                                <?php echo wp_kses_post( '<'.$slide['author_tag'].' class="mdp-reviewer-elementor-slide--author">'
                                                    .$slide['author_name'].' <span class="mdp-reviewer-elementor-author-separator">'.$slide['author_separator'].
                                                    '</span> </'.$slide['author_tag'].'>' ) ?>
                                            <?php endif; ?>
                                            <?php break; ?>
                                        <?php case 'company': ?>
                                            <?php if ( !empty( $slide['company_link']['url'] ) ): ?>
                                                <a href="<?php echo esc_url( $slide['company_link']['url'] ) ?>">
                                                    <?php echo wp_kses_post( '<'.$slide['company_tag'].' class="mdp-reviewer-elementor-slide--company">'
                                                        .$slide['company_name'].
                                                        '</'.$slide['company_tag'].'>' ) ?>
                                                </a>
                                            <?php else: ?>
                                                <?php echo wp_kses_post( '<'.$slide['company_tag'].' class="mdp-reviewer-elementor-slide--company">'
                                                    .$slide['company_name'].
                                                    '</'.$slide['company_tag'].'>' ) ?>
                                            <?php endif; ?>
                                            <?php break; ?>
                                        <?php case 'rating': ?>
                                            <div class="mdp-reviewer-elementor-slide--rating">
                                                <ul class="mdp-reviewer-elementor-slide--rating-stars">
                                                    <li class="mdp-reviewer-elementor-slide--rating-star"><?php Icons_Manager::render_icon( $slide['rating_icons'], [ 'aria-hidden' => 'true' ] ); ?></li>
                                                    <li class="mdp-reviewer-elementor-slide--rating-star"><?php Icons_Manager::render_icon( $slide['rating_icons'], [ 'aria-hidden' => 'true' ] ); ?></li>
                                                    <li class="mdp-reviewer-elementor-slide--rating-star"><?php Icons_Manager::render_icon( $slide['rating_icons'], [ 'aria-hidden' => 'true' ] ); ?></li>
                                                    <li class="mdp-reviewer-elementor-slide--rating-star"><?php Icons_Manager::render_icon( $slide['rating_icons'], [ 'aria-hidden' => 'true' ] ); ?></li>
                                                    <li class="mdp-reviewer-elementor-slide--rating-star"><?php Icons_Manager::render_icon( $slide['rating_icons'], [ 'aria-hidden' => 'true' ] ); ?></li>
                                                </ul>
                                            </div>
                                            <?php break; ?>
                                        <?php endswitch; ?>
                                <?php endforeach; ?>
                            </div>
                        </div>
                    <?php endforeach; ?>
                </div>
                <?php if ( $settings['show_navigation_arrows'] === 'yes' ): ?>
                <div class="mdp-reviewer-elementor-arrow mdp-reviewer-elementor-previous-arrow">
                    <?php Icons_Manager::render_icon( $settings['previous_arrow_icon'], [ 'aria-hidden' => 'true' ] ); ?>
                </div>
                <div class="mdp-reviewer-elementor-arrow mdp-reviewer-elementor-next-arrow">
                    <?php Icons_Manager::render_icon( $settings['next_arrow_icon'], [ 'aria-hidden' => 'true' ] ); ?>
                </div>
                <?php endif; ?>
            </div>
            <?php if ( $settings['show_dots'] === 'yes' ): ?>
            <div class="mdp-reviewer-elementor-dots"></div>
            <?php endif; ?>
        </div>
        <!-- End Reviewer for Elementor WordPress Plugin -->
        <?php
        if ( is_admin() ) {
            $widget_hash = substr( hash( 'ripemd160', date('l jS \of F Y h:i:s A') ), rand( 0 , 20 ), 3 ) . rand( 11 , 99 );
            ?>

            <!--suppress JSUnresolvedFunction -->
            <script>
                try {
                    reviewerReady<?php esc_attr_e( $widget_hash ); ?>( mdpReviewer.addReviewer.bind( mdpReviewer) );
                } catch ( msg ) {
                    const reviewerReady<?php esc_attr_e( $widget_hash ); ?> = ( callback ) => {
                        'loading' !== document.readyState ?
                            callback() :
                            document.addEventListener( 'DOMContentLoaded', callback );
                    };
                    reviewerReady<?php esc_attr_e( $widget_hash ); ?>( mdpReviewer.addReviewer.bind( mdpReviewer ) );
                }
            </script>
            <?php
        }
    }


    /**
     * JS Render.
     *
     * @access protected
     *
     * @return void
     **/
    function content_template() {
        ?>
        <#
        var image = '';
        var image_url = '';
        #>
        <!-- Start Reviewer for Elementor WordPress Plugin -->
        <div class="mdp-reviewer-elementor-box">
            <div class="mdp-reviewer-elementor-carousel-wrapper">
                <div class="mdp-reviewer-elementor-carousel"
                     data-slides-switching-mode="{{{ settings.slides_switching_mode }}}"
                     data-autoplay="{{{ settings.autoplay }}}"
                     data-pause-hover="{{{ settings.pause_on_hover }}}"
                     data-autoplay-speed = "{{{ settings.autoplay_speed }}}"
                     data-transition-speed = "{{{ settings.transition_speed }}}"
                     data-slides-to-scroll = "{{{ settings.slides_to_scroll }}}"
                     data-slides-to-show = "{{{ settings.slides_to_show }}}"
                     data-infinite-loop = "{{{ settings.infinite_loop }}}"
                     data-active-dot-class = "{{{ settings.dot_active_icon.value }}}"
                     data-dot-class = "{{{ settings.dot_icon.value }}}"
                     data-center-mode = "{{{ settings.center_mode }}}"
                    <# if ( elementorFrontend.config.responsive.activeBreakpoints.hasOwnProperty( 'widescreen' ) && settings.slides_to_show_widescreen ) { #> data-slides-to-show-widescreen="{{{ settings.slides_to_show_widescreen }}}" <# } #>
                    <# if ( elementorFrontend.config.responsive.activeBreakpoints.hasOwnProperty( 'laptop' ) && settings.slides_to_show_laptop ) { #> data-slides-to-show-laptop="{{{ settings.slides_to_show_laptop }}}" <# } #>
                    <# if ( elementorFrontend.config.responsive.activeBreakpoints.hasOwnProperty( 'tablet_extra' ) && settings.slides_to_show_tablet_extra ) { #> data-slides-to-show-tablet-extra="{{{ settings.slides_to_show_tablet_extra }}}" <# } #>
                    <# if ( elementorFrontend.config.responsive.activeBreakpoints.hasOwnProperty( 'mobile_extra' ) && settings.slides_to_show_mobile_extra ) { #> data-slides-to-show-mobile-extra="{{{ settings.slides_to_show_mobile_extra }}}" <# } #>
                    <# if ( settings.slides_to_show_tablet ) { #> data-slides-to-show-tablet = "{{{ settings.slides_to_show_tablet }}}" <# } #>
                    <# if ( settings.slides_to_show_mobile ) { #> data-slides-to-show-mobile = "{{{ settings.slides_to_show_mobile }}}" <# } #>
                >
                    <# _.each( settings.slides_list, function( slide, index ) { #>
                        <div class="mdp-reviewer-elementor-slide-wrapper" data-index="{{{ index }}}"  <# if( settings.slides_switching_mode === 'to-top' || settings.slides_switching_mode === 'to-bottom' ) { #> style="padding: {{{ settings.spacing_between_slides_vertical.size }}}px 0;" <# } #>>
                            <div class="mdp-reviewer-elementor-slide elementor-repeater-item-{{{ slide._id }}}" data-rating="{{{ settings.rating_count }}}">
                                <# _.each( settings.layouts_list, function( layout, index ) {  #>
                                        <# if( layout.layout === 'description' ) { #>
                                            <div class="mdp-reviewer-elementor-slide--description">
                                                {{{ slide.testimonial_description }}}
                                            </div>
                                        <# } #>
                                        <# if( layout.layout === 'author_image' ) { #>
                                            <#
                                                image = {
                                                    id: slide.author_image.id,
                                                    url: slide.author_image.url,
                                                    size: slide.author_image_size,
                                                    dimension: slide.author_image_custom_dimension,
                                                    model: view.getEditModel()
                                                };

                                                image_url = elementor.imagesManager.getImageUrl( image );
                                            #>
                                            <div class="mdp-reviewer-elementor-slide--author-image">
                                                <# if ( slide.custom_author_image_size === 'yes' ) { #>
                                                    <img src="{{{ slide.author_image.url }}}" alt="" style="width: {{{ slide.author_image_width.size }}}{{{ slide.author_image_width.unit }}} !important; height: {{{ slide.author_image_height.size }}}{{{ slide.author_image_height.unit }}} !important">
                                                <# } else { #>
                                                    <img src="{{{ image_url }}}" alt="" />
                                                <# } #>
                                            </div>
                                        <# } #>
                                        <# if( layout.layout === 'author' ) { #>
                                            <# if( slide.author_link.url !== '' ) { #>
                                                <a href="{{{ slide.author_link.url }}}" >
                                                    <{{{ slide.author_tag }}} class="mdp-reviewer-elementor-slide--author">
                                                        {{{ slide.author_name }}} <span class="mdp-reviewer-elementor-slide--author-separator">{{{ slide.author_separator }}}</span>
                                                    <{{'/'}}{{{ slide.author_tag }}}>
                                                </a>
                                            <# } else { #>
                                                    <{{{ slide.author_tag }}} class="mdp-reviewer-elementor-slide--author">
                                                        {{{ slide.author_name }}} <span class="mdp-reviewer-elementor-author-separator">{{{ slide.author_separator }}}</span>
                                                    <{{{ '/' }}}{{{ slide.author_tag }}}>
                                            <# } #>
                                        <# } #>
                                        <# if( layout.layout === 'company' ) { #>
                                            <# if( slide.company_link.url ) { #>
                                                <a href="{{ slide.company_link.url }}">
                                                    <{{{ slide.company_tag }}} class="mdp-reviewer-elementor-slide--company">
                                                        {{{ slide.company_name }}}
                                                    <{{'/'}}{{{ slide.company_tag }}}>
                                                </a>
                                            <# } else { #>
                                                    <{{{ slide.company_tag }}} class="mdp-reviewer-elementor-slide--company">
                                                        {{{ slide.company_name }}}
                                                    <{{'/'}}{{{ slide.company_tag }}}>
                                            <# } #>
                                        <# } #>
                                        <# if( layout.layout === 'rating' ) { #>
                                            <div class="mdp-reviewer-elementor-slide--rating">
                                                <ul class="mdp-reviewer-elementor-slide--rating-stars">
                                                    <li class="mdp-reviewer-elementor-slide--rating-star"><i class="{{{ slide.rating_icons.value }}}"></i></li>
                                                    <li class="mdp-reviewer-elementor-slide--rating-star"><i class="{{{ slide.rating_icons.value }}}"></i></li>
                                                    <li class="mdp-reviewer-elementor-slide--rating-star"><i class="{{{ slide.rating_icons.value }}}"></i></li>
                                                    <li class="mdp-reviewer-elementor-slide--rating-star"><i class="{{{ slide.rating_icons.value }}}"></i></li>
                                                    <li class="mdp-reviewer-elementor-slide--rating-star"><i class="{{{ slide.rating_icons.value }}}"></i></li>
                                                </ul>
                                            </div>
                                        <# } #>
                                <# } ) #>
                            </div>
                        </div>
                    <# } ) #>
                </div>
                <# if( settings.show_navigation_arrows ) { #>
                <div class="mdp-reviewer-elementor-arrow mdp-reviewer-elementor-previous-arrow">
                    <i class="{{{ settings.previous_arrow_icon.value }}}"></i>
                </div>
                <div class="mdp-reviewer-elementor-arrow mdp-reviewer-elementor-next-arrow">
                    <i class="{{{ settings.next_arrow_icon.value }}}"></i>
                </div>
                <# } #>
            </div>
            <# if( settings.show_dots === 'yes' ) { #>
                <div class="mdp-reviewer-elementor-dots"></div>
            <# } #>
        </div>
        <!-- End Reviewer for Elementor WordPress Plugin -->
        <?php
        if ( is_admin() ) {
            $widget_hash = substr( hash( 'ripemd160', date('l jS \of F Y h:i:s A') ), rand( 0 , 20 ), 3 ) . rand( 11 , 99 );
            ?>

            <!--suppress JSUnresolvedFunction -->
            <script>
                try {
                    reviewerReady<?php esc_attr_e( $widget_hash ); ?>( mdpReviewer.addReviewer.bind( mdpReviewer) );
                } catch ( msg ) {
                    const reviewerReady<?php esc_attr_e( $widget_hash ); ?> = ( callback ) => {
                        'loading' !== document.readyState ?
                            callback() :
                            document.addEventListener( 'DOMContentLoaded', callback );
                    };
                    reviewerReady<?php esc_attr_e( $widget_hash ); ?>( mdpReviewer.addReviewer.bind( mdpReviewer ) );
                }
            </script>
            <?php
        }
    }

    /**
     * Return link for documentation
     * Used to add stuff after widget
     *
     * @access public
     *
     * @return string
     **/
    public function get_custom_help_url() {

        return 'https://docs.merkulov.design/tag/reviewer';

    }

}
