<?php /** @noinspection PhpUndefinedClassInspection */
/**
 * Widgeter for Elementor
 * Customizable WordPress widgets for Elementor editor
 * Exclusively on https://1.envato.market/widgeter-elementor
 *
 * @encoding        UTF-8
 * @version         1.12.1
 * @copyright       (C) 2018 - 2023 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Cherviakov Vlad (vladchervjakov@gmail.com), Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Dmytro Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

namespace Merkulove\Selection;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
    header( 'Status: 403 Forbidden' );
    header( 'HTTP/1.1 403 Forbidden' );
    exit;
}

use Elementor\Core\Schemes\Color;
use Elementor\Core\Schemes\Typography;
use Elementor\Group_Control_Background;
use Elementor\Group_Control_Border;
use Elementor\Group_Control_Box_Shadow;
use Elementor\Group_Control_Typography;
use Exception;
use Elementor\Widget_Base;
use Elementor\Controls_Manager;
use Merkulove\Selection\Unity\Plugin as UnityPlugin;

/** @noinspection PhpUnused */
/**
 * Widgeter - Custom Elementor Widget.
 **/
class tags_elementor extends Widget_Base {

    /**
     * Use this to sort widgets.
     * A smaller value means earlier initialization of the widget.
     * Can take negative values.
     * Default widgets and widgets from 3rd party developers have 0 $mdp_order
     **/
    public $mdp_order = 1;

    /**
     * Widget base constructor.
     * Initializing the widget base class.
     *
     * @access public
     * @throws Exception If arguments are missing when initializing a full widget instance.
     * @param array      $data Widget data. Default is an empty array.
     * @param array|null $args Optional. Widget default arguments. Default is null.
     *
     * @return void
     **/
    public function __construct( $data = [], $args = null ) {

        parent::__construct( $data, $args );

        wp_register_style(
  'mdp-selection-elementor-admin',
     UnityPlugin::get_url() . 'css/elementor-admin' . UnityPlugin::get_suffix() . '.css',
         [],
         UnityPlugin::get_version()
        );

    }

    /**
     * Return a widget name.
     *
     * @return string
     **/
    public function get_name() {

        return 'mdp-widgeter-tags-elementor';

    }

    /**
     * Return the widget title that will be displayed as the widget label.
     *
     * @return string
     **/
    public function get_title() {

        return esc_html__( 'Tag cloud', 'selection' );

    }

    /**
     * Set the widget icon.
     *
     * @return string
     */
    public function get_icon() {

        return 'mdp-tags-elementor-widget-icon';

    }

    /**
     * Set the category of the widget.
     *
     * @return array with category names
     **/
    public function get_categories() {

        return [ 'selection-category' ];

    }

    /**
     * Get widget keywords. Retrieve the list of keywords the widget belongs to.
     *
     * @access public
     *
     * @return array Widget keywords.
     **/
    public function get_keywords() {

        return [ 'Merkulove', 'Widgeter' ];

    }

    /**
     * Get style dependencies.
     * Retrieve the list of style dependencies the widget requires.
     *
     * @access public
     *
     * @return array Widget styles dependencies.
     **/
    public function get_style_depends() {

        return [ 'mdp-widgeter', 'mdp-selection-elementor-admin' ];

    }

	/**
	 * Get script dependencies.
	 * Retrieve the list of script dependencies the element requires.
	 *
	 * @access public
     *
	 * @return array Element scripts dependencies.
	 **/
	public function get_script_depends() {

		return [ 'mdp-widgeter' ];

    }

    /**
     * Add the widget controls.
     *
     * @access protected
     * @return void with category names
     **/
    protected function register_controls() {

        /** Content Tab. */
        $this->tab_content();

        /** Style Tab. */
        $this->tab_style();

    }

    /**
     * Add widget controls on Content tab.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function tab_content() {

        /** Content -> General Content Section. */
        $this->section_content_general();

    }

    /**
     * Add widget controls on Style tab.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function tab_style() {

        /** Style -> Section Style Title. */
        $this->section_style_title();

        /** Style -> Section Style Tag Item. */
        $this->section_style_tag_item();

    }

    private function get_taxonomies_options() {
        $taxonomies = get_taxonomies( [ 'show_tagcloud' => true ], object );
        $options = [];

        foreach ( $taxonomies as $tax_key => $taxonomy ) {
            $options[$tax_key] = $taxonomy->label;
        }

        return $options;
    }

    /**
     * Add widget controls: Content -> General Content Section.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_content_general() {

        $this->start_controls_section( 'section_content_general', [
            'label' => esc_html__( 'General', 'selection' ),
            'tab'   => Controls_Manager::TAB_CONTENT
        ] );

        $this->add_control(
            'tag_cloud_title',
            [
                'label' => esc_html__( 'Title', 'selection' ),
                'type' => Controls_Manager::TEXT,
                'placeholder' => esc_html__( 'Type your title here', 'selection' ),
            ]
        );

        $this->add_control(
            'tag_cloud_title_tag',
            [
                'label' => esc_html__( 'Title HTML Tag', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'options' => [
                    'h1' => 'H1',
                    'h2' => 'H2',
                    'h3' => 'H3',
                    'h4' => 'H4',
                    'h5' => 'H5',
                    'h6' => 'H6',
                    'div' => 'div',
                    'span' => 'span',
                    'p' => 'p',
                ],
                'default' => 'h5'
            ]
        );

        $this->add_control(
            'enable_tag_cloud_title_icon',
            [
                'label' => esc_html__( 'Enable title icon', 'selection' ),
                'type' => Controls_Manager::SWITCHER,
                'label_on' => esc_html__( 'Yes', 'selection' ),
                'label_off' => esc_html__( 'No', 'selection' ),
                'return_value' => 'yes',
                'default' => '',
            ]
        );

        $this->add_control(
            'tag_cloud_title_icon',
            [
                'label' => esc_html__( 'Title icon', 'selection' ),
                'type' => Controls_Manager::ICONS,
                'default' => [
                    'value' => 'fas fa-star',
                    'library' => 'solid',
                ],
                'condition' => [
                    'enable_tag_cloud_title_icon' => 'yes'
                ]
            ]
        );

        $this->add_control(
            'tag_cloud_icon_position',
            [
                'label' => esc_html__( 'Icon position', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'before',
                'options' => [
                    'before'  => esc_html__( 'Left', 'selection' ),
                    'after' => esc_html__( 'Right', 'selection' ),
                    'above-left' => esc_html__( 'Top left', 'selection' ),
                    'above-center' => esc_html__( 'Top center', 'selection' ),
                    'above-right' => esc_html__( 'Top right', 'selection' ),
                    'under-left' => esc_html__( 'Bottom left', 'selection' ),
                    'under-center' => esc_html__( 'Bottom center', 'selection' ),
                    'under-right' => esc_html__( 'Bottom right', 'selection' )
                ],
                'condition' => [
                    'enable_tag_cloud_title_icon' => 'yes'
                ]
            ]
        );

        $this->add_control(
            'tag_cloud_alignment',
            [
                'label' => esc_html__( 'Alignment', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'eicon-text-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'eicon-text-align-center',
                    ],
                    'right' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'eicon-text-align-right',
                    ],
                ],
                'default' => 'left',
                'toggle' => true,
            ]
        );

        $this->add_control(
            'tag_cloud_title_alignment',
            [
                'label' => esc_html__( 'Title alignment', 'selection' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'selection' ),
                        'icon' => 'eicon-text-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'selection' ),
                        'icon' => 'eicon-text-align-center',
                    ],
                    'right' => [
                        'title' => esc_html__( 'Right', 'selection' ),
                        'icon' => 'eicon-text-align-right',
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-widgeter-elementor-title-wrapper' => 'text-align: {{VALUE}};'
                ],
                'default' => 'left',
                'toggle' => true,
            ]
        );

        $this->add_control(
            'tag_cloud_taxonomy',
            [
                'label' => esc_html__( 'Taxonomy', 'selection' ),
                'type' => Controls_Manager::SELECT,
                'default' => 'post_tag',
                'options' => $this->get_taxonomies_options(),
            ]
        );



        $this->add_control(
		    'show_tag_count',
		    [
			    'label' => esc_html__( 'Show tag count', 'selection' ),
			    'type' => Controls_Manager::SWITCHER,
			    'return_value' => 'yes',
			    'default' => 'no',
		    ]
	    );


        $this->end_controls_section();

    }

    /**
     * Function for generating margin padding controls.
     *
     * @param $section_id
     * @param $html_class
     * @param array $default_padding
     * @param array $default_margin
     * @return void
     * @since 1.0.0
     * @access private
     */
    private function generate_margin_padding_controls( $section_id, $html_class, $default_padding = [], $default_margin = [] ) {
        $this->add_responsive_control(
            $section_id.'_margin',
            [
                'label' => esc_html__( 'Margin', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'default' => $default_margin,
                'selectors' => [
                    "{{WRAPPER}} .$html_class" => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}} !important;',
                ],
            ]
        );

        $this->add_responsive_control(
            $section_id.'_padding',
            [
                'label' => esc_html__( 'Padding', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'separator' => 'after',
                'default' => $default_padding,
                'selectors' => [
                    "{{WRAPPER}} .$html_class" => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}} !important;',
                ],
            ]
        );
    }

    /**
     * Function for generating typography and tabs controls.
     *
     * @param $section_id
     * @param $opts
     * @return void
     * @since 1.0.0
     * @access private
     */
    private function generate_typography_tabs_controls( $section_id, $opts = [] ) {
        $style_opts = [
            'html_class' => array_key_exists( 'html_class', $opts ) ?
                $opts['html_class'] : '',
            'active_class' => array_key_exists( 'active_class', $opts ) ?
                $opts['active_class'] : '',
            'include_color' => array_key_exists( 'include_color', $opts ) ?
                $opts['include_color'] : true,
            'include_border' => array_key_exists( 'include_border', $opts ) ?
                $opts['include_border'] : true,
            'include_bg' => array_key_exists( 'include_bg', $opts ) ?
                $opts['include_color'] : true,
            'include_typography' => array_key_exists( 'include_typography', $opts ) ?
                $opts['include_typography'] : true,
            'include_transition' => array_key_exists( 'include_transition', $opts ) ?
                $opts['include_transition'] : true,
            'additional_color' => array_key_exists( 'additional_color', $opts ) ?
                $opts['additional_color'] : false,
            'additional_background' => array_key_exists( 'additional_background', $opts ) ?
                $opts['additional_background'] : false,
            'exclude_font_size' => array_key_exists( 'exclude_font_size', $opts ) ?
                $opts['exclude_font_size'] : false,
            'typography_name' => array_key_exists( 'typography_name', $opts ) ?
                $opts['typography_name'] : 'Typography',
            'additional_background_name' => array_key_exists( 'additional_background_name', $opts ) ?
                $opts['additional_background_name'] : '',
            'additional_background_class' => array_key_exists( 'additional_background_class', $opts ) ?
                $opts['additional_background_class'] : '',
            'additional_border_radius_class' => array_key_exists( 'additional_border_radius_class', $opts ) ?
                $opts['additional_border_radius_class'] : '',
            'additional_border_radius_hover_class' => array_key_exists( 'additional_border_radius_hover_class', $opts ) ?
                $opts['additional_border_radius_hover_class'] : '',
            'include_active_tab' => array_key_exists( 'include_active_tab', $opts ) ?
                $opts['include_active_tab'] : false,
            'active_tab_name' => array_key_exists( 'active_tab_name', $opts ) ?
                $opts['active_tab_name'] : 'FOCUS',
            'color_prefix' => array_key_exists( 'color_prefix', $opts ) ?
                $opts['color_prefix'] : '',
            'color_class' => array_key_exists( 'color_class', $opts ) ?
                $opts['color_class'] : '',
            'color_hover_class' => array_key_exists( 'color_hover_class', $opts ) ?
                $opts['color_hover_class'] : '',
            'color_active_class' => array_key_exists( 'color_active_class', $opts ) ?
                $opts['color_active_class'] : '',
            'color_hover_selector' => array_key_exists( 'color_hover_selector', $opts ) ?
                $opts['color_hover_selector'] : '',
            'additional_color_name' => array_key_exists( 'additional_color_name', $opts ) ?
                $opts['additional_color_name'] : '',
            'additional_color_class' => array_key_exists( 'additional_color_class', $opts ) ?
                $opts['additional_color_class'] : '',
            'additional_color_hover_class' => array_key_exists( 'additional_color_hover_class', $opts ) ?
                $opts['additional_color_hover_class'] : '',
            'additional_color_active_class' => array_key_exists( 'additional_color_active_class', $opts ) ?
                $opts['additional_color_active_class'] : '',
            'additional_transition_selector' => array_key_exists( 'additional_transition_selector', $opts ) ?
                $opts['additional_transition_selector'] : '',
            'typography_class' => array_key_exists( 'typography_class', $opts ) ?
                $opts['typography_class'] : '',
            'color_scheme_default' => array_key_exists( 'color_scheme_default', $opts ) ?
                $opts['color_scheme_default'] : Color::COLOR_3,
            'additional_color_scheme_default' => array_key_exists( 'additional_color_scheme_default', $opts ) ?
                $opts['additional_color_scheme_default'] : Color::COLOR_3
        ];


        if ( $style_opts['include_typography'] ) {
            $typography_settings = [
                'name' => $section_id . '_typography',
                'label' => esc_html__( $style_opts['typography_name'], 'selection' ),
                'scheme' => Typography::TYPOGRAPHY_1,
                'selector' => "{{WRAPPER}} .".$style_opts['typography_class'],
            ];

            if ( $style_opts['exclude_font_size'] ) { $typography_settings['exclude'] = [ 'font_size' ]; }

            $this->add_group_control( Group_Control_Typography::get_type(), $typography_settings );
        }

        $this->start_controls_tabs( $section_id.'_style_tabs' );

        $this->start_controls_tab(
            $section_id.'_normal_style_tab',
            ['label' => esc_html__( 'NORMAL', 'selection' )]
        );

        if ( $style_opts['include_color'] ) {
            $this->add_control(
                $section_id . '_normal_text_color',
                [
                    'label' => esc_html__($style_opts['color_prefix'].'Color', 'selection'),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => $style_opts['color_scheme_default'],
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .".$style_opts['color_class'] => 'color: {{VALUE}} !important;',
                    ],
                ]
            );

        }

        if ( $style_opts['additional_color'] ) {
            $this->add_control(
                $section_id . '_' . $style_opts['additional_color_name'] . '_normal_text_color',
                [
                    'label' => esc_html__( $style_opts['additional_color_name'], 'selection' ),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => $style_opts['additional_color_scheme_default'],
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .".$style_opts['additional_color_class'] => 'color: {{VALUE}} !important;',

                    ],
                ]
            );
        }

        if ( $style_opts['include_bg'] ) {

            $this->add_group_control(
                Group_Control_Background::get_type(),
                [
                    'name' => $section_id . '_normal_background',
                    'label' => esc_html__('Background type', 'selection'),
                    'types' => ['classic', 'gradient', 'video'],
                    'selector' => "{{WRAPPER}} .".$style_opts['html_class'],
                ]
            );

        }

        if ( $style_opts['additional_background'] ) {

            $this->add_control(
                $section_id . '_' . $style_opts['additional_background_name'] . '_normal_text_color',
                [
                    'label' => esc_html__( $style_opts['additional_background_name'], 'selection' ),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => $style_opts['additional_color_scheme_default'],
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .".$style_opts['additional_background_class'] => 'background: {{VALUE}} !important;',

                    ],
                ]
            );

        }

        $this->add_control(
            $section_id . '_separate_normal',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        if ( $style_opts['include_border'] ) {

            $this->add_group_control(
                Group_Control_Border::get_type(),
                [
                    'name' => $section_id . '_border_normal',
                    'label' => esc_html__('Border Type', 'selection'),
                    'selector' => "{{WRAPPER}} ." . $style_opts['html_class'],
                ]
            );

        }

        $this->add_responsive_control(
            $section_id.'_border_radius_normal',
            [
                'label' => esc_html__( 'Border radius', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    "{{WRAPPER}} .".$style_opts['html_class'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                    $style_opts['additional_border_radius_class'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};'
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => $section_id.'_box_shadow_normal',
                'label' => esc_html__( 'Box Shadow', 'selection' ),
                'selector' => "{{WRAPPER}} .".$style_opts['html_class'],
            ]
        );


        $this->end_controls_tab();

        $this->start_controls_tab(
            $section_id.'_hover_style_tab',
            ['label' => esc_html__( 'HOVER', 'selection' )]
        );

        if ( $style_opts['include_color'] ) {
            $this->add_control(
                $section_id . '_hover_color',
                [
                    'label' => esc_html__($style_opts['color_prefix'].'Color', 'selection'),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => $style_opts['color_scheme_default'],
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .".$style_opts['color_hover_class'] => 'color: {{VALUE}} !important;',
                    ],
                ]
            );
        }

        if ( $style_opts['additional_color'] ) {
            $this->add_control(
                $section_id . '_' . $style_opts['additional_color_name'] . '_hover_text_color',
                [
                    'label' => esc_html__( $style_opts['additional_color_name'], 'selection'),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => $style_opts['additional_color_scheme_default'],
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .".$style_opts['additional_color_hover_class'] => 'color: {{VALUE}} !important;',
                    ],
                ]
            );
        }

        if ( $style_opts['include_bg'] ) {
            $this->add_group_control(
                Group_Control_Background::get_type(),
                [
                    'name' => $section_id . '_background_hover',
                    'label' => esc_html__('Background type', 'selection'),
                    'types' => ['classic', 'gradient', 'video'],
                    'selector' => "{{WRAPPER}} .".$style_opts['html_class'].":hover",
                ]
            );
        }

        if ( $style_opts['additional_background'] ) {

            $this->add_control(
                $section_id . '_' . $style_opts['additional_background_name'] . '_hover_text_color',
                [
                    'label' => esc_html__( $style_opts['additional_background_name'], 'selection' ),
                    'type' => Controls_Manager::COLOR,
                    'scheme' => [
                        'type' => Color::get_type(),
                        'value' => $style_opts['additional_color_scheme_default'],
                    ],
                    'selectors' => [
                        "{{WRAPPER}} .".$style_opts['additional_background_class'].':hover' => 'background: {{VALUE}} !important;',

                    ],
                ]
            );

        }


        if ( $style_opts['include_transition'] ) {
            $this->add_control(
                $section_id.'_hover_transition',
                [
                    'label' => esc_html__( 'Hover transition duration', 'selection' ),
                    'type' => Controls_Manager::SLIDER,
                    'size_units' => [ 's' ],
                    'range' => [
                        's' => [
                            'min' => 0.1,
                            'max' => 5,
                            'step' => 0.1,
                        ],
                    ],
                    'default' => [
                        'unit' => 's',
                        'size' => 0,
                    ],
                    'selectors' => [
                        '{{WRAPPER}} .'.$style_opts['html_class'] => 'transition: color {{SIZE}}{{UNIT}}, background {{SIZE}}{{UNIT}}, box-shadow {{SIZE}}{{UNIT}}, border-radius {{SIZE}}{{UNIT}}, border {{SIZE}}{{UNIT}}, filter {{SIZE}}{{UNIT}}, stroke {{SIZE}}{{UNIT}};',
                        $style_opts['additional_transition_selector'] => 'transition: color {{SIZE}}{{UNIT}}, background {{SIZE}}{{UNIT}}, box-shadow {{SIZE}}{{UNIT}}, border-radius {{SIZE}}{{UNIT}}, border {{SIZE}}{{UNIT}}, filter {{SIZE}}{{UNIT}}, stroke {{SIZE}}{{UNIT}};;'
                    ],
                ]
            );
        }

        $this->add_control(
            $section_id.'_separate_hover',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        if ( $style_opts['include_border'] ) {

            $this->add_group_control(
                Group_Control_Border::get_type(),
                [
                    'name' => $section_id . '_border_hover',
                    'label' => esc_html__('Border Type', 'selection'),
                    'selector' => "{{WRAPPER}} ." . $style_opts['html_class'] . ":hover",
                ]
            );

        }

        $this->add_responsive_control(
            $section_id.'_border_radius_hover',
            [
                'label' => esc_html__( 'Border radius', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'selectors' => [
                    "{{WRAPPER}} .".$style_opts['html_class'].":hover" => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                    $style_opts['additional_border_radius_hover_class'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};'
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(),
            [
                'name' => $section_id.'_box_shadow_hover',
                'label' => esc_html__( 'Box Shadow', 'selection' ),
                'selector' => "{{WRAPPER}} .".$style_opts['html_class'].":hover",
            ]
        );

        $this->end_controls_tab();

        if ( $style_opts['include_active_tab'] ) {

            $this->start_controls_tab(
                $section_id . '_focus_style_tab',
                ['label' => esc_html__( $style_opts['active_tab_name'], 'selection' )]
            );

            if ( $style_opts['include_color'] ) {
                $this->add_control(
                    $section_id . '_active_color',
                    [
                        'label' => esc_html__( $style_opts['color_prefix'] . 'Color', 'widgeter-elementor '),
                        'type' => Controls_Manager::COLOR,
                        'scheme' => [
                            'type' => Color::get_type(),
                            'value' => $style_opts['color_scheme_default'],
                        ],
                        'selectors' => [
                            "{{WRAPPER}} ." . $style_opts['color_active_class'] => 'color: {{VALUE}} !important;',
                        ],
                    ]
                );
            }

            if ( $style_opts['additional_color'] ) {
                $this->add_control(
                    $section_id . '_' . $style_opts['additional_color_name'] . '_active_text_color',
                    [
                        'label' => esc_html__( $style_opts['additional_color_name'], 'selection' ),
                        'type' => Controls_Manager::COLOR,
                        'scheme' => [
                            'type' => Color::get_type(),
                            'value' => $style_opts['additional_color_scheme_default'],
                        ],
                        'selectors' => [
                            "{{WRAPPER}} ." . $style_opts['additional_color_active_class'] => 'color: {{VALUE}} !important;',
                        ],
                    ]
                );
            }

            if ( $style_opts['include_bg'] ) {
                $this->add_group_control(
                    Group_Control_Background::get_type(),
                    [
                        'name' => $section_id . '_background_active',
                        'label' => esc_html__( 'Background type', 'selection' ),
                        'types' => ['classic', 'gradient', 'video'],
                        'selector' => "{{WRAPPER}} ." . $style_opts['active_class'],
                    ]
                );
            }

            $this->add_control(
                $section_id . '_separate_active',
                [
                    'type' => Controls_Manager::DIVIDER,
                ]
            );

            if ( $style_opts['include_border'] ) {

                $this->add_group_control(
                    Group_Control_Border::get_type(),
                    [
                        'name' => $section_id . '_border_active',
                        'label' => esc_html__('Border Type', 'selection'),
                        'selector' => "{{WRAPPER}} ." . $style_opts['active_class'],
                    ]
                );

            }

            $this->add_responsive_control(
                $section_id . '_border_radius_active',
                [
                    'label' => esc_html__( 'Border radius', 'selection' ),
                    'type' => Controls_Manager::DIMENSIONS,
                    'size_units' => ['px', '%', 'em'],
                    'selectors' => [
                        "{{WRAPPER}} ." . $style_opts['active_class'] => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                    ],
                ]
            );

            $this->add_group_control(
                Group_Control_Box_Shadow::get_type(),
                [
                    'name' => $section_id . '_box_shadow_active',
                    'label' => esc_html__( 'Box Shadow', 'selection' ),
                    'selector' => "{{WRAPPER}} ." . $style_opts['active_class'],
                ]
            );

            $this->end_controls_tab();

        }


        $this->end_controls_tabs();
    }

    /**
     * Add widget controls: Style -> Section Style Title.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_title() {

        $this->start_controls_section( 'section_style_title', [
            'label' => esc_html__( 'Title', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->generate_margin_padding_controls(
            'section_style_title',
            'mdp-widgeter-tag-cloud-title'
        );

        $this->add_control(
            'title_width',
            [
                'label' => esc_html__( 'Width', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'range' => [
                    'px' => [
                        'min' => 1,
                        'max' => 1000,
                        'step' => 1,
                    ],
                    '%' => [
                        'min' => 1,
                        'max' => 100,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .mdp-widgeter-tag-cloud-title' => 'width: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->add_responsive_control(
            'tag_cloud_title_icon_spacing',
            [
                'label' => esc_html__( 'Icon spacing', 'selection' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'em' ],
                'devices' => ['desktop', 'tablet', 'mobile'],
                'condition' => [
                    'enable_tag_cloud_title_icon' => 'yes'
                ],
                'separator' => 'before',
                'selectors' => [
                    "{{WRAPPER}} .mdp-widgeter-elementor-title-icon" => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_control(
            'tag_cloud_title_icon_size',
            [
                'label' => esc_html__( 'Icon size', 'selection' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 0,
                        'max' => 500,
                        'step' => 1,
                    ]
                ],
                'condition' => [
                    'enable_tag_cloud_title_icon' => 'yes'
                ],
                'separator' => 'after',
                'selectors' => [
                    '{{WRAPPER}} .mdp-widgeter-elementor-title-icon' => 'height: {{SIZE}}{{UNIT}}; width: {{SIZE}}{{UNIT}};',
                    '{{WRAPPER}} .mdp-widgeter-elementor-title-icon i' => 'font-size: {{SIZE}}{{UNIT}};',
                    '{{WRAPPER}} .mdp-widgeter-elementor-title-icon svg' => 'height: {{SIZE}}{{UNIT}}; width: {{SIZE}}{{UNIT}};'
                ],
            ]
        );

        $this->generate_typography_tabs_controls( 'section_style_title', [
            'html_class' => 'mdp-widgeter-tag-cloud-title',
            'typography_class' => 'mdp-widgeter-tag-cloud-title',
            'additional_color' => true,
            'additional_color_name' => 'Icon color',
            'additional_color_class' => 'mdp-widgeter-elementor-title-icon',
            'additional_color_hover_class' => 'mdp-widgeter-tag-cloud-title:hover .mdp-widgeter-elementor-title-icon',
            'additional_transition_selector' => '{{WRAPPER}} .mdp-widgeter-elementor-title-icon',
            'color_class' => 'mdp-widgeter-tag-cloud-title',
            'color_hover_class' => 'mdp-widgeter-tag-cloud-title:hover',
        ] );

        $this->end_controls_section();

    }


    /**
     * Add widget controls: Style -> Section Style Tag Item.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function section_style_tag_item() {

        $this->start_controls_section( 'section_style_tag_item', [
            'label' => esc_html__( 'Tag item', 'selection' ),
            'tab'   => Controls_Manager::TAB_STYLE
        ] );

        $this->generate_margin_padding_controls(
            'section_style_tag_item',
            'tagcloud a'
        );

        $this->generate_typography_tabs_controls( 'section_style_tag_item', [
            'html_class' => 'tagcloud a',
            'typography_class' => 'tagcloud a',
            'exclude_font_size' => true,
            'color_class' => 'tagcloud a',
            'color_hover_class' => 'tagcloud a:hover',
        ] );

        $this->end_controls_section();

    }

    /**
     * Render Frontend Output. Generate the final HTML on the frontend.
     *
     * @access protected
     *
     * @return void
     **/
    protected function render() {
       $settings = $this->get_settings_for_display();
       $title_tag = esc_attr( $settings['tag_cloud_title_tag'] );
       $icon_enabled = $settings['enable_tag_cloud_title_icon'] === 'yes';

        $before_widget = sprintf(
             '<div class="mdp-widgeter-elementor-box mdp-widgeter-tag-cloud-elementor-box
                    mdp-widgeter-elementor-tag-alignment-%s">',
                    esc_attr( $settings['tag_cloud_alignment'] ) );


        $default_widget_args = Caster::get_instance()->widgeter_get_widget_default_args(
       'tag-cloud',
            $title_tag,
            $icon_enabled,
            $settings['tag_cloud_title_icon'],
            $settings['tag_cloud_icon_position'],
            '',
            $before_widget
        );

         the_widget( 'WP_Widget_Tag_Cloud', [
             'title' => esc_html__( $settings['tag_cloud_title'], 'selection' ),
             'taxonomy' => esc_attr( $settings['tag_cloud_taxonomy'] ),
             'count' => esc_attr( $settings['show_tag_count'] ) === 'yes'
         ], $default_widget_args  );

    }

    /**
     * Return link for documentation
     * Used to add stuff after widget
     *
     * @access public
     *
     * @return string
     **/
    public function get_custom_help_url() {

        return 'https://docs.merkulov.design/tag/widgeter';

    }

}
