<?php
/**
 * Selection
 * Carefully selected Elementor addons bundle, for building most awesome websites
 * Exclusively on https://1.envato.market/selection
 *
 * @encoding        UTF-8
 * @version         1.12.1
 * @copyright       (C) 2018 - 2023 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Dmitry Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

namespace Merkulove\Selection;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
	header( 'Status: 403 Forbidden' );
	header( 'HTTP/1.1 403 Forbidden' );
	exit;

}

use Elementor\TemplateLibrary\Source_Local;
use Merkulove\Selection\Unity\Plugin;

final class TemplateShortcode {

	/**
	 * The one true TemplateShortcode.
	 *
     * @since 1.0.0
     * @access private
	 * @var TemplateShortcode
	 **/
	private static $instance;

    /**
     * Setup the plugin.
     *
     * @since 1.0.0
     * @access public
     *
     * @return void
     **/
    public function __construct()
    {
	    /** Styles and scripts Elementor templates list */
	    add_action( 'current_screen', [ $this, 'enqueue_templates_list' ] );

	    /** Elementor templates list column */
	    if ( is_plugin_active( 'elementor/elementor.php' ) ) {

		    /** Add shortcode column header name in templates */
		    add_action( 'manage_' . Source_Local::CPT . '_posts_columns', [$this, 'admin_columns_headers'] );

		    /** Add shortcode column in templates */
		    add_action( 'manage_' . Source_Local::CPT . '_posts_custom_column', [$this, 'admin_columns_content'], 10, 2 );

	    }
    }

	/**
	 * Add copy shortcode button on the Elementor templates list
	 * @return void
	 */
	public function enqueue_templates_list()
	{
		$screen = get_current_screen();

		// Run only on Elementor Templates List
		if ( null === $screen ) { return; }
		if ( 'edit-elementor_library' !== $screen->id ) { return; }

		wp_enqueue_style( 'mdp-selection-templates-edit', Plugin::get_url() . 'css/selection/templates-edit' . Plugin::get_suffix() . '.css', [], Plugin::get_version() );

		wp_enqueue_script( 'mdp-selection-templates-edit', Plugin::get_url() . 'js/selection/templates-edit' . Plugin::get_suffix() . '.js', [], Plugin::get_version(), true );
	}

	/**
	 * Add column header in templates page for shortcode
	 *
	 * @param $defaults
	 * @return string
	 * @since 1.0.0
	 * @access public
	 */
	public function admin_columns_headers( $defaults )
	{
		$defaults['shortcode'] = esc_html__( 'Shortcode', 'selection' );

		return $defaults;
	}

	/**
	 * Add shortcode column in templates page for shortcode
	 *
	 * @param $column_name
	 * @param $post_id
	 * @since 1.0.0
	 * @access public
	 */
	public function admin_columns_content( $column_name, $post_id )
	{
		if ( 'shortcode' === $column_name ) {
			// %s = shortcode, %d = post_id
			$shortcode = esc_attr( sprintf( '[%s id="%d"]', 'selection-template', $post_id ) );
			echo wp_sprintf(
				'<div class="%1$s"><button class="%1$s-copy eicon-code-bold" title="%2$s"></button><input class="%1$s-input" type="text" readonly onfocus="this.select()" value="%3$s" /></div>',
				'selection-shortcode',
				esc_html__( 'Copy Template Shortcode', 'selection' ),
				$shortcode
			);
		}
	}

	/**
	 * Main TemplateShortcode Instance.
	 * Insures that only one instance of TemplateShortcode exists in memory at any one time.
	 *
	 * @static
     * @since 1.0.0
     * @access public
     *
	 * @return TemplateShortcode
	 **/
	public static function get_instance() {

		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof self ) ) {

			self::$instance = new self;

		}

		return self::$instance;

	}

}
